c-----------------------------------------------------------------------

c RLIB - a library of subprograms called by VERTEX, FRENDLY, ACTCOR
c BUILD, ADIABAT, and ISOCHOR.

c Copyright (c) 1990 by James A. D. Connolly, Institute for Mineralogy
c & Petrography, Swiss Federal Insitute of Technology, CH-8092 Zurich,
c SWITZERLAND. All rights reserved.
 
c Please do not distribute this source.

c-----------------------------------------------------------------------

      subroutine gexces (id,dg)
c-----------------------------------------------------------------------
c gexces evaluates the contribution to the gibbs energy of a pseudocompound
c arising from configurational entropy, excess properties, and dqf corrections
c for special cases (internal EoS's or Van Laar gexces does not return
c the excess energy, see routines fexces, gvlaar, and toop).
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
c-----------------------------------------------------------------------
      
      dg = exces(1,id) + t * exces(2,id) + p * exces(3,id)

      end 


      subroutine fexces (id,dg)
c-----------------------------------------------------------------------
c gexces evaluates the contribution to the gibbs energy of a pseudocompound
c arising from dqf corrections, and excess properties computed from an
c internal fluid EoS
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst11 /f(2)
     *      / cst12 /cp(k5,k1)
c-----------------------------------------------------------------------

      dg = exces(1,id) + t * exces(2,id) + p * exces(3,id)

      xco2 = cp(2,id)

      call cfluid (fo2,fs2)

      dg = dg + r*t*(cp(1,id)*f(1) + cp(2,id)*f(2))

      end 

      subroutine gvlaar (jd,id,dg)
c-----------------------------------------------------------------------
c gvlaar evaluates the contribution to the gibbs energy of a pseudocompound
c arising from van laar excess properties. 
c
c input:
c      jd - solution pointer
c      id - compound pointer
c      dg - ideal free energy
c output:
c      dg - ideal free energy + excess energy     
c-----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ind,ideps,hsub,inid,lterm,ksp,msite,
     *        mstot,ifp,jsub,jterm,i,id,ixp,jend,jm,im2jm,ndep,jd,knsp

      double precision phi(m4), alpha(m4), tphi, vlar, t, p, sxs, y2pg,
     *                 dg, exces, wgl, qmult, d0, dcoef,
     *                 w, x, sdzdy, dvnu, deph, dydy, xco2, u1, u2, tr,
     *                 pr, r, ps

      common/ cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)

c-----------------------------------------------------------------------
c                            first compute "volumes"
      tphi = 0d0
      jm = im2jm(jd)

      do i = 1, jend(jd,1)

         ind = knsp(jm,i) 

         alpha(ind) = vlar(jm,i,1) + t * vlar(jm,i,2) + p * vlar(jm,i,3)
c                            these phi's are the numerator of
c                            holland & powells "phi's"
         phi(ind) =  alpha(ind)* sxs(ixp(id)+i)
c                            tphi is the denominator of holland & powell's
c                            phi's
         tphi = tphi + phi(ind) 
      end do 
c                            rearrange tphi
      tphi = 2d0/tphi
c                            dg is initialized as gph in the calling 
c                            program
      do i = 1, jterm(jm)
c                            assume holland powell form, all terms regular
         dg = dg + (wgl(jm,i,1) + t * wgl(jm,i,2) + p * wgl(jm,i,3)) 
     *           * tphi * phi(jsub(jm,i,1)) * phi(jsub(jm,i,2))
     *           / (alpha(jsub(jm,i,1)) + alpha(jsub(jm,i,2)))   

      end do  

      end 

      function gzero (id)
c-----------------------------------------------------------------------
c gzero computes the reference pressure free energy of a compound 
c with no transitions identified by the argument 'id'. see gcpd. 
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)/ cst12 /cp(k5,k1)
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps/ cst44 /dlogt,dsqrtt
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct
 
      gzero = thermo(1,id)
c                                    -sdt
     *      + t * (thermo(2,id) - thermo(4,id) * dlogt
     *      - t * (thermo(5,id) + thermo(7,id) * t))
     *      - (thermo(6,id) + thermo(10,id) / t) / t
     *      + thermo(8,id) * dsqrtt + thermo(9,id)*dlogt
 
      do j = 1, jmct      
c                                    -ndu   
         gzero = gzero - cp(jprct+j,id) * um(j)
      end do

      end 

      subroutine gcpd (id,gval)
c-----------------------------------------------------------------------
c gcpd computes the gibbs free energy of a compound identified by
c the arguement 'id' from the thermochemical parameters stored
c in the array 'thermo' which is located in common block 'cst1'.
c the parameters are: g(pr,tr) [thermo(1,id)], s(pr,tr) [thermo
c (2,id)], v(pr,tr) [thermo(3,id)], the coefficients of the extended
c maier-kelley heat capacity equation:
 
c           cp(pr,t)=a+b*t+c/(t*t)+d*t*t+e/t**(1/2)+f/t+g/t**3
 
c [thermo(4-10,id)], and the coefficients of the volumetric 
c equations given in the program documentation (Eqs 2.1-2.3):

c        b8 = 0 =>
 
c           v(p,t) = v(pr,tr) + b2*(t-tr) + b4*(p-pr)
c                           + b6*(p-pr)**2 + b7*(t-tr)**2

c        -3d0 < b8 < 0 =>

c           v(p,t) = v(pr,tr) * exp[ b3*(t-tr) + b8*(p-pr) ]

c        b8 < -3d0 =>

c           p = 3 * k * f * (2f + 1)^(5/2) * (1 - 3/2 * f * (4 + b8))
c           f = 1/2 * ((v0/v)^(2/3) - 1)
c           v0 = v(pr,tr) + b1 * (t - tr)
c           k = -v0 / (b2 + b3 * (t - tr))

c        b8 > 0 =>

c           v(p,t) = v(pr,t) * [1 + b8*p/(b8*p+Kt)]^(1/b8)
c               Kt = b6 + b7 * t
c            alpha = b1 + b2*t + b3/t - b4/t^2 + b5/t^(1/2)      

c [thermo(11-18,id)].

c the variables in cst44 must be set before a call to gcpd.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'

      double precision gmurn, gsixtr, gstxgi
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)/ cst11 /f(2)
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst113 /ifproj
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)/ cst44 /dlogt,dsqrtt
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct

      save rkt

      data rkt/0d0/
c                                    identify sixtrude's EoS by negative
c                                    entropy term
      if (thermo(3,id).lt.0d0) then
c                                    identify the version of stxrude's
c                                    EoS by sign on number of atoms
         if (thermo(2,id).gt.0) then 
c                                    use this call to evalute the 
c                                    debye T as in stixrude JGR '05
            gval = gsixtr (id)
         else 
c                                    and this call to evalute the 
c                                    debye T as in stixrude JGI '05
            gval = gstxgi (id) 
         end if 

         goto 999

      end if 
c                                    all other EoS's with Cp function
      gval = thermo(1,id)
c                                    -sdt
     *       + t * (thermo(2,id) - thermo(4,id) * dlogt
     *       - t * (thermo(5,id) + thermo(7,id) * t))
     *       - (thermo(6,id) + thermo(10,id) / t) / t
     *       + thermo(8,id) * dsqrtt + thermo(9,id)*dlogt
c                                     vdp-ndu term:
      if (thermo(18,id).eq.0d0) then 
c                                     normal polynomial:
         vdp =  p * (thermo(3,id) 
     *               + (thermo(17,id) * t + thermo(12,id)) * t
     *               + (thermo(16,id) * p + thermo(14,id)) * p)

      else if (thermo(18,id).gt.0d0) then
c                                     HP-murnaghan EoS:
c                                     vdp = V(1,T)*KT**(1/K')/(K'-1)
c                                           * [(KT+K'*p)**(1-1/K') -
c                                              (KT+K'*pr)**(1-1/K')]
         rkt = thermo(16,id) + thermo(17,id)*t
c                                     a fuckwit has entered a ridiculous
c                                     temperature
         if (rkt.lt.0d0) call error (46,t,id,'GCPD') 

         vdp = (thermo(3,id) + (thermo(11,id) + thermo(12,id)*t)*t +
     *          thermo(13,id)*dlogt + thermo(14,id)/t + 
     *          thermo(15,id)*dsqrtt) * rkt**thermo(21,id)/thermo(22,id) 
     *          *((rkt+thermo(18,id)*p)**thermo(19,id)
     *           -(rkt+thermo(20,id))**thermo(19,id))

      else if (thermo(18,id).lt.-3d0) then
c                                     Ghiorso-murnaghan

         vdp = gmurn (id)

      else 
c                                     gottschalk.
         vdp = thermo(11,id)*dexp(thermo(13,id)*t)*
     *               (1d0 - dexp(thermo(18,id)*(p - pr)))

      end if
c                                    -ndu term
      du = 0d0
      do j = 1, jmct      
         du = du - cp(jprct+j,id) * um(j)
      end do

      gval = gval + vdp + du
c                                     check for lambda transitions:
      if (ltyp(id).ne.0) then
 
         if (ltyp(id).lt.4) then
c                                     ubc-type transitions
            call lamubc (p,t,dg,lmda(id),ltyp(id))
            gval = gval + dg
 
         else if (ltyp(id).lt.7) then
c                                     standard transitions
            call lamhel (p,t,gval,vdp+du,lmda(id),ltyp(id))
 
         else if (ltyp(id).lt.10) then
c                                     supcrt q/coe lambda transition
            call lamqtz (p,t,gval,du,lmda(id),id)
 
         else if (ltyp(id).gt.9 .and. ltyp(id).le.12 ) then
c                                     holland and powell (landau model)
            call lamlan (dg,rkt,lmda(id),ltyp(id))
            gval = gval + dg
 
         end if
      end if
c                                     check for temperature dependent
c                                     order/disorder:
      if (idis(id).ne.0) then
         jd = idis(id)
         call disord (t,therdi(1,jd),therdi(2,jd),therdi(3,jd),
     *                  therdi(4,jd),therdi(5,jd),therdi(6,jd),
     *                  therdi(7,jd),therdi(8,jd),therdi(9,jd),gss)
         gval = gval + gss
      end if
c                                     modification for fluid
c                                     projections:
      if (ifproj.gt.0.and.thermo(3,id).eq.0d0) then
c                                     if pr or tr = 0 then 
c                                     this is a special calculation
c                                     and we can skip this part.
         if (pr.eq.0d0.or.tr.eq.0d0) goto 999

         if (cp(1,id).eq.0d0.and.cp(2,id).eq.0d0) goto 999

         if (ifproj.eq.1) then
            xco2 = 0d0
            call cfluid (fo2,fs2)
            gval = gval + r*t*cp(1,id)*f(1)
         else if (ifproj.eq.2) then
            xco2 = 1d0
            call cfluid (fo2,fs2)
            gval = gval + r*t*cp(1,id)*f(2)
         else 
            xco2 = cp(2,id)
            call cfluid (fo2,fs2)
            gval = gval + r*t*(cp(1,id)*f(1) + cp(2,id)*f(2))
         end if  
      end if
 
999   end


      subroutine zeroi (iarray,index,ivalue,n)

      integer iarray(n), n, index, ivalue

      do i = 1, index
         iarray(i) = ivalue
      end do 

      end

      subroutine snorm0 (tname)
c------------------------------------------------------------------------
c compute endmember configurational entropies
c site fractions expressed as a function of bragg-williams site fractions
c see snorm1 for endmember fraction version.
c------------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*10 tname, mname*8

      double precision x(mst,msp)

      common/ cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst5 /p,t,xc,u1,u2,tr,pr,r,ps/ cst18a /mname(m4)
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst142 /jmsol(m4,mst),kdsol(m4)

c                                     get normalization constants
c                                     for each retained endmember
      do h = 1, jstot
c                                     zero x-array
         do i = 1, isite
            do j = 1, isp(i)
               x(i,j) = 0d0
            end do
c                                     now assign endmember fractions
            x(i,jmsol(insp(h),i)) = 1d0 
         end do          
c                                    x's now set, evaluate ln W:
         call omega0 (x,dlnw,tname)
         ecoef(insp(h)) = dlnw

      end do 

      write (*,2000) tname

      write (*,2010) (mname(insp(h)), r*ecoef(insp(h)), h = 1, jstot)

2000  format (/,'Endmember configurational entropies (',
     *          'doc. eq. 8.2) for ',a,' are:')
2010  format (3(2x,a,' - ',f8.5,2x))
      end 

      subroutine snorm1 (tname)
c------------------------------------------------------------------------
c compute endmember configurational entropies
c site fractions expressed as a function of end-member proportions.
c see snorm0 for bragg-williams site fraction version.
c------------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*10 tname, mname*8

      double precision x(m4)

      common/ cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst5 /p,t,xc,u1,u2,tr,pr,r,ps/ cst18a /mname(m4)
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst142 /jmsol(m4,mst),kdsol(m4)

c                                     get normalization constants
c                                     for each retained endmember
      do h = 1, jstot
c                                     zero x-array
         do j = 1, istot + 1
            x(j) = 0d0
         end do

         x(insp(h)) = 1d0        
c                                    x's now set, evaluate ln W:
         call omega1 (x,dlnw,tname)
         ecoef(insp(h)) = dlnw

      end do 

      write (*,2000) tname

      write (*,2010) (mname(insp(h)), r*ecoef(insp(h)), h = 1, jstot)

2000  format (/,'Endmember configurational entropies (',
     *          'doc. eq. 8.2) for ',a,' are:')
2010  format (3(2x,a,' - ',f8.5,2x))
      end 

      subroutine omega0 (x,dlnw,tname)
c----------------------------------------------------------------------
c subroutine to evaluate the log of the number of site configurations
c per mole of a solution (dlnw) with composition x. bragg-williams 
c version.
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      character*10 tname

      double precision x(mst,msp), z, zt, dlnw, zad

      integer i,j,k,l,nsub1,nsub2,nttyp,nterm,nspm1,nsite

      double precision ecoef,acoef,smult,a0

      common/ cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)

      dlnw = 0d0
c                                     for each site
      do i = 1, nsite
         zt = 0d0
c                                     get site fractions
         do j = 1, nspm1(i)
            z = a0(i,j)
            if (nterm(i,j).ne.0) then 
c                                     for each term:
               do k = 1, nterm(i,j)
                  zad = acoef(i,j,k)
                  do l = 1, nttyp(i,j,k)
                     zad = zad * x(nsub1(i,j,k,l),nsub2(i,j,k,l))
                  end do 
                  z = z + zad
               end do 
            end if 

            call zchk (z,tname)
            zt = zt + z
            if (z.gt.0d0.and.z.lt.1d0) then 
               dlnw = dlnw - smult(i) * z * dlog (z)
            end if 
            
         end do 
         z = 1d0 - zt
         call zchk (z,tname)
         if (z.gt.0d0.and.z.lt.1d0) dlnw = dlnw - smult(i) * z * dlog(z)
      end do 

      end

 
      subroutine subdiv (tname)
c---------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer ms1

      parameter (ms1=msp-1)
 
      character*10 tname

      double precision z(mst,msp), zpt

      integer istot,jstot,iend,isub,imd,jmsol,kdsol,ntot,
     *        ist,i,j,iord,iterm,isite,isp,insp,kstot,npairs,np1,
     *        h,k,l,m

      double precision wg,xmn,xmx,xnc,y,xy,zt

      common/ cst86 /xy(mdim,k1),y(k1,mst,ms1),ntot,npairs
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),
     *      insp(m4),ist(mst),isp(mst),isite,iterm,iord,
     *                istot,jstot,kstot
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
c---------------------------------------------------------------------
c                                 do the first site:
      call cart (1,tname)

      if (npairs.gt.k1) call error (41,xy(1,1),k1,'SUBDIV_1')

      do h = 1, npairs
         do i = 1, isp(1) - 1
            y(h,1,i) = xy(i,h)
         end do 
      end do 

      np1 = npairs
      ntot = np1 

      if (isite.eq.1) goto 99
c                                 do the second site:
      call cart (2,tname)

c                                 there will be a total of
c                                 (npairs-1)*np1 compositions,
c                                 copy the first site 2 distribution
c                                 into the first np1 compositions:
      do h = 1, np1 
         do i = 1, isp(2) - 1
c                                 this could be an invalid compostion for
c                                 a 3 site model.
            y(h,2,i) = xy(i,1)
         end do 
      end do 

      do h = 2, npairs
c                                 for each site 2 composition,
c                                 duplicate the range of site 1
c                                 compositions.
         do 10 i = 1, np1

            ntot = ntot + 1

            if (ntot.gt.k1) call error (41,xy(1,1),k1,'SUBDIV_2')

            do j = 1, isp(1) - 1
               y(ntot,1,j) = y(i,1,j)
            end do 
c                                 put in the new site 2 compositions:
            do j = 1, isp(2) - 1 
               y(ntot,2,j) = xy(j,h)
            end do 
c                                 check if valid composition:
c                                 this is a lot of trouble to save 
c                                 generating unused compositions
            if (isite.eq.2) then 

               do k = 1, 2
                  zt = 0d0
                  do j = 1, isp(k) - 1
                     z(k,j) = y(ntot,k,j)
                     zt = zt + z(k,j)
                  end do 
                  z(k,isp(k)) = 1d0 - zt
               end do 

               do l = 1, istot
 
                  zpt = 1d0
                  do m = 1, 2
c                              check for invalid compositions
                     call zchk (z(m,jmsol(l,m)),tname)
                     zpt = zpt*z(m,jmsol(l,m))
                  end do
c                              reject the compositions if missing
c                              endmember or invalid value
                  if (kdsol(l).eq.0.and.zpt.ne.0d0.or.
     *                                  zpt.gt.0.9999) then 
                     ntot = ntot - 1
c                    if (ntot.lt.npairs) ntot = npairs
                     goto 10
                  end if 
               end do 
            end if 
10       continue 
      end do 
c                                 do the third site:
c                                 note this hardwires the array dimensions
c                                 to "mst"
      if (isite.eq.2) goto 99

      np1 = (npairs-1) * np1

      call cart (3,tname)
c                                 copy the first site 3 distribution
c                                 into the first np1*np2 compositions:
      do h = 1, ntot
         do i = 1, isp(mst) - 1
            y(h,mst,i) = xy(i,1)
         end do 
      end do 
c                                 for each site 3 composition,
c                                 duplicate the range of site 1 and 
c                                 site 2 compositions.
      do h = 2, npairs

         do i = 1, np1
            ntot = ntot + 1
            do j = 1, isp(1) - 1
               y(ntot,1,j) = y(i,1,j)
            end do 
            do j = 1, isp(2) - 1
               y(ntot,2,j) = y(i,1,j)
            end do 
            do j = 1, isp(mst) - 1
               y(ntot,mst,j) = xy(j,h)
            end do 
         end do 
      end do 
 
99    end


      subroutine isok (ilast,ksite)
c---------------------------------------------------------------------
c check if a composition generated by cart is a duplicate.
c    the composition is in xy(j,npairs), npairs is decremented
c    if the composition matches an earlier value.

c this routine is only here for debugging.
c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ms1
 
      parameter (ms1=msp-1)
 
      integer ksite,ind

      integer istot,jstot,iend,isub,imd,ntot,npairs,ilast,
     *        ist,i,j,iord,iterm,isite,isp,insp,kstot

      double precision wg,xmn,xmx,xnc,xy,yy
 
      common/ cst86 /xy(mdim,k1),yy(k1,mst,ms1),ntot,npairs
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),
     *      insp(m4),ist(mst),isp(mst),isite,iterm,iord,
     *                istot,jstot,kstot

      ind = isp(ksite) - 1

      do 10 i = ilast, npairs - 1

         do j = 1, ind
            if (xy(j,i).ne.xy(j,npairs)) goto 10
         end do 
c                             composition i matches npairs

c         write (*,1999) npairs,(xy(j,npairs),j=1,ind)
c1999     format (i6,1x,8(f6.4,1x))

         npairs = npairs - 1
         exit 

10    continue

      end 

      subroutine cart (ksite,sname)
c---------------------------------------------------------------------

c      xmax2   |\              cartesian subdivision onto 1-, 2- and 3-
c              |  \            dimensional simplexes. isp(ksite) is number of
c              |    \          vertices, in 1-dimensional case 
c      xinc2   |      \        ordinates of vertices are xmin1 & xmax1,
c              |        \      in 2-dimensional case coordinates
c              |          \    are (xmin1, xmin2), (xmax1, xmin2) & 
c      xmin2   |            \  (xmin1, xmax2).
c              ---------------
c           xmin1    xinc1    xmax1

c      ksite - the site of the solution sname to be represented.
c      iscp  - site dimension

c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ms1
 
      parameter (ms1=msp-1)
 
      double precision vco1(673), vco2(673), dx1, dx2
 
      integer ibary(4), loop(mdim),ksite

      integer istot,jstot,iend,isub,imd,jmsol,kdsol,ntot,npairs,
     *        ist,i,j,iord,iterm,isite,isp,insp,kstot,jpairs

      double precision wg,xmn,xmx,xnc,delx,xy,yy,x0,b,bm1,t1,dx,
     *                 bp1,x

      character sname*10
 
      save vco1, vco2, ibary
 
      common/ cst86 /xy(mdim,k1),yy(k1,mst,ms1),ntot,npairs
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),
     *      insp(m4),ist(mst),isp(mst),isite,iterm,iord,
     *                istot,jstot,kstot
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
c                               ibary(j) is the number of stars after
c                               jth barycentric subdivision.
      data ibary/7,25,121,673/
c                               (vco1(i), vco2(i)) are the simplicial
c                               coordinates of the stars generated by
c                               barycentric subdivsion.
      data (vco1(i), i = 1, 3)/ 0., 1., 0./
      data (vco2(i), i = 1, 3)/ 1., 0., 0./

      data (vco1(i), i = 4,121)/.500,.500,.0  ,.333,
c                      4  after subdivision:            1
     *.750,.611,.250,.417,.278,.0  ,.167,.111,.0  ,.167,.111,.250,
     *.167,.278,.750,.667,.417,.611,
c                     22  after subdivision:            2
     *.875,.787,.625,.681,.620,.556,.509,.514,.454,.472,.537,.806,
     *.639,.759,.375,.343,.125,.264,.176,.139,.148,.222,.259,.375,
     *.306,.343,.458,.389,.347,.398,.0  ,.037,.0  ,.056,.037,.056,
     *.093,.139,.204,.250,.222,.204,.083,.056,.139,.093,.0  ,.037,
     *.0  ,.056,.037,.056,.093,.139,.204,.250,.222,.204,.083,.056,
     *.139,.093,.125,.176,.375,.264,.343,.389,.398,.347,.343,.250,
     *.306,.259,.083,.139,.222,.148,.875,.787,.625,.681,.620,.458,
     *.556,.509,.375,.514,.454,.500,.472,.537,.833,.806,.639,.759/
c                    118  after subdivision:            3
      data (vco1(i), i = 122,337)/
     *.938,.887,.813,.831,.804,.769,.739,.734,.693,.699,.735,.894,
     *.796,.864,.688,.665,.563,.623,.582,.560,.559,.588,.596,.646,
     *.616,.637,.715,.685,.650,.684,.489,.484,.461,.463,.480,.512,
     *.545,.583,.560,.559,.528,.505,.532,.522,.415,.414,.387,.394,
     *.420,.463,.512,.562,.532,.526,.465,.435,.484,.461,.457,.519,
     *.568,.602,.614,.588,.596,.542,.574,.540,.403,.435,.505,.448,
     *.864,.796,.753,.653,.713,.688,.625,.699,.670,.708,.685,.725,
     *.903,.880,.782,.855,.438,.406,.313,.359,.323,.296,.285,.303,
     *.295,.310,.336,.421,.366,.410,.188,.184,.063,.150,.100,.088,
     *.105,.157,.198,.271,.227,.239,.257,.213,.220,.230,.077,.116,
     *.133,.157,.179,.185,.216,.208,.213,.188,.069,.074,.144,.096,
     *.225,.255,.281,.296,.299,.282,.281,.250,.269,.253,.194,.213,
     *.241,.216,.354,.350,.396,.359,.378,.380,.369,.345,.323,.292,
     *.310,.309,.319,.338,.324,.327,.479,.452,.437,.428,.424,.382,
     *.407,.387,.312,.373,.341,.333,.338,.355,.444,.449,.394,.429,
     *.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.074,.068,.019,
     *.046,.031,.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.083,
     *.074,.068,.028,.019,.046,.031,.059,.088,.114,.130,.133,.116/
      data (vco1(i), i = 338,553)/
     *.114,.083,.102,.086,.028,.046,.074,.049,.207,.227,.262,.269,
     *.253,.213,.179,.125,.157,.151,.153,.185,.171,.170,.292,.262,
     *.208,.227,.207,.185,.170,.171,.151,.167,.157,.179,.278,.269,
     *.213,.253,.042,.059,.125,.088,.114,.153,.130,.133,.125,.116,
     *.114,.083,.102,.086,.028,.046,.074,.049,.0  ,.012,.0  ,.019,
     *.012,.019,.031,.046,.068,.074,.068,.019,.046,.031,.0  ,.012,
     *.0  ,.019,.012,.019,.031,.046,.068,.083,.074,.068,.028,.019,
     *.046,.031,.059,.088,.114,.130,.133,.116,.114,.083,.102,.086,
     *.028,.046,.074,.049,.207,.227,.262,.269,.253,.213,.179,.125,
     *.157,.151,.153,.185,.171,.170,.292,.262,.208,.227,.207,.185,
     *.170,.171,.151,.167,.157,.179,.278,.269,.213,.253,.042,.059,
     *.125,.088,.114,.153,.130,.133,.125,.116,.114,.083,.102,.086,
     *.028,.046,.074,.049,.063,.100,.188,.150,.184,.213,.230,.220,
     *.239,.227,.198,.088,.157,.105,.313,.323,.438,.359,.406,.421,
     *.410,.366,.336,.271,.310,.295,.257,.296,.303,.285,.452,.428,
     *.424,.407,.387,.373,.341,.333,.338,.355,.444,.449,.394,.429,
     *.378,.359,.350,.338,.327,.324,.309,.312,.310,.323,.382,.380,
     *.345,.369,.292,.281,.208,.255,.225,.213,.216,.241,.253,.292/
      data (vco1(i), i = 554,673)/
     *.269,.281,.319,.296,.282,.299,.042,.077,.125,.116,.133,.194,
     *.157,.179,.250,.185,.216,.208,.213,.188,.069,.074,.144,.096,
     *.938,.887,.813,.831,.804,.769,.739,.734,.693,.699,.735,.894,
     *.796,.864,.688,.665,.563,.623,.582,.560,.559,.588,.596,.646,
     *.616,.637,.715,.685,.650,.684,.479,.489,.437,.484,.461,.463,
     *.480,.512,.545,.583,.560,.559,.528,.505,.532,.522,.396,.415,
     *.354,.414,.387,.394,.420,.463,.512,.562,.532,.526,.465,.435,
     *.484,.461,.417,.457,.583,.519,.568,.602,.614,.588,.596,.542,
     *.574,.540,.403,.435,.505,.448,.917,.864,.750,.796,.753,.653,
     *.713,.688,.625,.699,.670,.708,.685,.725,.903,.880,.782,.855/
c                    670  after subdivision:            4
      data (vco2(i), i = 4,121)/.500,.0  ,.500,.333,
c                      4  after subdivision:            1
     *.250,.278,.750,.417,.611,.750,.667,.611,.250,.417,.278,.0  ,
     *.167,.111,.0  ,.167,.167,.111,
c                     22  after subdivision:            2
     *.125,.176,.375,.264,.343,.389,.398,.347,.343,.306,.259,.139,
     *.222,.148,.625,.620,.875,.681,.787,.806,.759,.639,.537,.375,
     *.472,.454,.458,.556,.514,.509,.875,.787,.625,.681,.620,.556,
     *.509,.514,.454,.500,.472,.537,.833,.806,.639,.759,.375,.343,
     *.125,.264,.176,.139,.148,.222,.259,.375,.306,.343,.458,.389,
     *.347,.398,.0  ,.037,.0  ,.056,.037,.056,.093,.139,.204,.250,
     *.222,.204,.083,.056,.139,.093,.0  ,.037,.0  ,.056,.037,.083,
     *.056,.093,.250,.139,.204,.250,.222,.204,.083,.056,.139,.093/
c                    118  after subdivision:            3
      data (vco2(i), i = 122,337)/
     *.063,.100,.188,.150,.184,.213,.230,.220,.239,.227,.198,.088,
     *.157,.105,.313,.323,.438,.359,.406,.421,.410,.366,.336,.271,
     *.310,.295,.257,.296,.303,.285,.452,.428,.424,.407,.387,.373,
     *.341,.333,.338,.355,.444,.449,.394,.429,.378,.359,.350,.338,
     *.327,.324,.309,.312,.310,.323,.382,.380,.345,.369,.281,.255,
     *.225,.213,.216,.241,.253,.292,.269,.281,.319,.296,.282,.299,
     *.077,.116,.133,.194,.157,.179,.250,.185,.216,.208,.213,.188,
     *.069,.074,.144,.096,.563,.582,.688,.623,.665,.685,.684,.650,
     *.637,.616,.596,.560,.588,.559,.813,.804,.938,.831,.887,.894,
     *.864,.796,.735,.646,.699,.693,.715,.769,.734,.739,.864,.796,
     *.753,.713,.688,.699,.670,.708,.685,.725,.903,.880,.782,.855,
     *.568,.519,.457,.435,.448,.505,.540,.625,.574,.596,.653,.602,
     *.588,.614,.354,.387,.396,.414,.415,.435,.461,.484,.526,.542,
     *.532,.512,.403,.394,.463,.420,.479,.489,.437,.484,.461,.465,
     *.463,.480,.562,.512,.545,.583,.560,.559,.528,.505,.532,.522,
     *.938,.887,.813,.831,.804,.769,.739,.734,.693,.699,.735,.894,
     *.796,.864,.688,.665,.563,.623,.582,.560,.559,.588,.596,.646,
     *.616,.637,.715,.685,.650,.684,.489,.484,.461,.463,.480,.512/
      data (vco2(i), i = 338,553)/
     *.545,.583,.560,.559,.528,.505,.532,.522,.415,.414,.387,.394,
     *.420,.463,.512,.562,.532,.526,.465,.435,.484,.461,.417,.457,
     *.583,.519,.568,.602,.614,.588,.596,.542,.574,.540,.403,.435,
     *.505,.448,.917,.864,.750,.796,.753,.653,.713,.688,.625,.699,
     *.670,.708,.685,.725,.903,.880,.782,.855,.438,.406,.313,.359,
     *.323,.296,.285,.303,.295,.310,.336,.421,.366,.410,.188,.184,
     *.063,.150,.100,.088,.105,.157,.198,.271,.227,.239,.257,.213,
     *.220,.230,.077,.116,.133,.157,.179,.185,.216,.208,.213,.188,
     *.069,.074,.144,.096,.225,.255,.281,.296,.299,.282,.281,.250,
     *.269,.253,.194,.213,.241,.216,.354,.350,.396,.359,.378,.380,
     *.369,.345,.323,.292,.310,.309,.319,.338,.324,.327,.479,.452,
     *.437,.428,.424,.382,.407,.387,.312,.373,.341,.333,.338,.355,
     *.444,.449,.394,.429,.0  ,.012,.0  ,.019,.012,.019,.031,.046,
     *.068,.074,.068,.019,.046,.031,.0  ,.012,.0  ,.019,.012,.019,
     *.031,.046,.068,.083,.074,.068,.028,.019,.046,.031,.059,.088,
     *.114,.130,.133,.116,.114,.083,.102,.086,.028,.046,.074,.049,
     *.207,.227,.262,.269,.253,.213,.179,.125,.157,.151,.153,.185,
     *.171,.170,.292,.262,.208,.227,.207,.185,.170,.171,.151,.167/
      data (vco2(i), i = 554,673)/
     *.157,.179,.278,.269,.213,.253,.042,.059,.125,.088,.114,.153,
     *.130,.133,.125,.116,.114,.083,.102,.086,.028,.046,.074,.049,
     *.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.074,.068,.019,
     *.046,.031,.0  ,.012,.0  ,.019,.012,.019,.031,.046,.068,.083,
     *.074,.068,.028,.019,.046,.031,.042,.059,.125,.088,.114,.130,
     *.133,.116,.114,.083,.102,.086,.028,.046,.074,.049,.208,.207,
     *.292,.227,.262,.269,.253,.213,.179,.125,.157,.151,.153,.185,
     *.171,.170,.292,.262,.208,.227,.207,.185,.170,.171,.151,.167,
     *.157,.179,.278,.269,.213,.253,.042,.059,.125,.088,.114,.153,
     *.130,.133,.125,.116,.114,.083,.102,.086,.028,.046,.074,.049/
c                    670  after subdivision:            4
c---------------------------------------------------------------------
      npairs = 0

      if (imd(ksite).eq.0) then 

         call cartes (ksite,sname)
         goto 99
 
      end if

      if (isp(ksite).ne.2) goto 30
c                              one-dimensional simplex:
      if (imd(ksite).eq.1) then
         if (xmx(ksite,1).eq.0d0.or.dabs(xmx(ksite,1)).gt.1d0) then
            delx = 1d0
            x0 = 0d0
         else if (xmx(ksite,1).gt.0d0) then
            delx = xmx(ksite,1)
            x0 = 0d0
         else if (xmx(ksite,1).lt.0d0) then
            delx = dabs(xmx(ksite,1))
            x0 = 1d0 - delx
         end if 
c                              symmetric stretching imod = 1:
         b = xmn(ksite,1)
         if (b.lt.1d0) call error (207,b,j,sname)
         bm1 = b - 1d0
         bp1 = b + 1d0
 
         dx = 1d0 / (xnc(ksite,1) - 1d0)
         x = dx
         loop(1) = idint (xnc(ksite,1))  - 1
c                              first loop generates phases
c                              with xx(1)  =  0 through 0.5 .
         do j = 1, loop(1)
            npairs = npairs + 1
c                              generate compositions:
            t1 = (bp1/bm1)**(1d0-x)
            xy(1,npairs) = x0 + (0.5 * (bp1-bm1*t1)/(1d0 +t1))*delx
            x = x + dx
         end do 
c                              second loop generates phases
c                              with y = 0.5 through 1 - xmin.
         loop(1) = loop(1) - 1
         jpairs = loop(1)
         do j = 1, loop(1)
            npairs = npairs + 1
c                              generate compositions:
            xy(1,npairs) = 2d0*x0 + delx - xy(1,jpairs)
            jpairs  = jpairs - 1
         end do 

      else if (imd(ksite).eq.2) then
c                              assymmetric stretching imod = 2:
         b = xmn(ksite,1)
         if (b.lt.1d0) call error (207,b,j,sname)
         bm1 = b - 1d0
         bp1 = b + 1d0
 
         dx = 1d0 / (xnc(ksite,1) - 1d0)
         x = dx
         loop(1) = idint (xnc(ksite,1)) - 1
c                              with x = xmin through xmax
         do j = 1, loop(1)
            npairs = npairs + 1
c                              generate compositions:
            t1 = (bp1/bm1)**(1d0-x)
            xy(1,npairs) = xmx(ksite,1) * (bp1-bm1*t1)/(1d0+t1)
            x = x + dx
         end do 
c                              value of imod is illegal, 
c                              write error msg:
      else if (imd(ksite).gt.3)  then
 
         call error (169,b,imd(ksite),'CART')
 
      end if
 
      goto 99

30    if (isp(ksite).ne.3) goto 20 
c                              two dimensional simplex:
      if (imd(ksite).gt.0.and.imd(ksite).lt.5) then
c                               barycentric subdivision of ternary
c                               solutions, imod = # of iterations.
         npairs = ibary(imd(ksite))
         if (npairs.gt.k1) call error (180,b,k1,'CART')
         dx1 = xmx(ksite,1) - xmn(ksite,1) 
         dx2 = xmx(ksite,2) - xmn(ksite,2)
         do j = 1, npairs
            xy(1,j) = xmn(ksite,1) + ( vco1(j) - xmn(ksite,1) ) * dx1
            xy(2,j) = xmn(ksite,2) + ( vco2(j) - xmn(ksite,2) ) * dx2
         end do 
 
      else if (imd(ksite).gt.4) then
c                              illegal value of imod, write error msg:
           call error (169,b,imd(ksite),'CART')
 
      end if

      goto 99 

20    call error (169,b,ksite,'CART')

99    end

      subroutine xchk (xmin, xmax, xinc, tname) 

      implicit double precision (a-g,o-z),integer (h-n)

      character tname*10

      if (xmax.gt.1d0) then 
         call warn (21,xmax,1,tname)
         xmax = 1d0
      end if 
      if (xmin.lt.0d0) then 
         call warn (22,xmin,1,tname) 
         xmin = 0d0
      end if 
      if (xmax.lt.xmin) then
         call warn (23,xmax,1,tname)
         xmax = 1d0
         xmin = 0d0
      end if 
      if (xinc.le.0d0) then 
         call warn (23,xinc,1,tname)
         xinc = 1d0       
      end if
      end 

      subroutine zchk (z, tname)

      implicit none

      double precision z

      character tname*10

      if (z.gt.0d0.and.z.le.1d-3) then
         z = 0d0
      else if (z.ge.0.999.and.z.lt.1d0) then
         z = 1d0
      else if (z.ge.0d0.and.z.le.1d0) then 
         return
      else if (z.gt.1d0.and.z.lt.1.001) then
         z = 1d0
      else if (z.gt.-.01) then
         z = 0d0
      else 
         call error (125,z,1,tname)
      end if
      end 

      subroutine assptx
c---------------------------------------------------------------------
c assptx assigns the current value of v1 and v2 to the array ptx and
c increments the ordinate counter ipt2.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst32 /ptx(l5),ipt2
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst5 /v(l2),tr,pr,r,ps/ cst63 /delv(l2)
  
      ipt2 = ipt2 + 2

      if (ipt2.gt.l5) ipt2 = l5

      ptx(ipt2-1) = v(iv1)
      ptx(ipt2)   = v(iv2)
 
      end
 
      subroutine concrt
c---------------------------------------------------------------------
c concrt determines convergence criteria and limits for reasonable
c solutions for the routine univeq. the array delt is also used in
c the routine slope.
 
c input:  dv - array of default search increments.
c         vmax,vmin - arrays containing the maximum and minimum values
c           of the independent and dependent intensive variables.
c         iv - array indexes variables in vmax, vmin, and output arrays
c output: ulim, blim - arrays with upper and lower limits for reasonable
c           solutions (vmax,vmin+/-4*dv)
c         delt - array containing the finite difference increments
c           (vmax-vmin/10**5), delt also serves as the test value
c           for convergence.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst62 /blim(l2),ulim(l2),dgr
     *      / cst9 /vmax(l2),vmin(l2),dv(l2)
c---------------------------------------------------------------------
      do i = 1, 5
         if (dv(i).lt.0d0) call error (34,dv(i),i,'CONCRT')
         if (i.eq.3) then 
            ulim(i) = vmax(i)
            blim(i) = vmin(i)
         else if (i.eq.1.or.i.eq.2) then
            ulim(i) = vmax(i)+dv(i)
            blim(i) = vmin(i)-dv(i)
            if (blim(i).lt.0d0) blim(i) = 1d0 
         else 
            ulim(i) = vmax(i)+dv(i)
            blim(i) = vmin(i)-dv(i)
         end if 
         ddv = vmax(i)-vmin(i)
         if (ddv.lt.0d0) call error (35,ddv,i,'CONCRT')
      end do 
 
      end
 
      subroutine conver (g,s,v,a,b,c,d,e,f,gg,b1,b2,b3,b4,b5,b6,b7,b8,
     *                   b9,b10,b11,b12,tr,pr)
c---------------------------------------------------------------------
c convert thermodynamic equation of state from a pr tr reference state
c to get polynomial coefficients
c---------------------------------------------------------------------
 
      implicit double precision (a-g,o-z),integer (h-n)

c                              this routine uses special characteristics
c                              of the data to identify the different types
c                              of EoS's in use within perple_x
      if (v.lt.0d0) then 
c                              negative "volume" signals
c                              the stixrude EoS, don't do anything in this
c                              case.

c                              Otherwise:

c                              b8 is K' in the HP-murnaghan EoS
c                              b8 is -K' in the Ghiorso-murnaghan EoS
c                              or -beta in exponential volumetric
c                              functions.
      else if (b8.eq.0d0) then
c                              normal polynomial vdp term:
 
c                              first compute G*
         g  = g
c                              add the integral of sdt at tr,pr:
     *       + s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *       - d * tr**3 / 3d0 - 2d0 * e * dsqrt(tr)
     *       - f * dlog(tr) + gg / tr / tr / 2d0 + f
c                              add the constant components of the
c                              integral -vdp at (t,pr):
     *       - v * pr + b2 * tr * pr + b4 * pr * pr / 2d0
c                              sign on b7 corrected June 16, 2004. PJ Gorman. 
     *       - b6 * pr**3/3d0 - b7 * tr * tr * pr
c                              S* (T)
         s  = a - b2*pr - s + a*dlog(tr) + b*tr - c/tr/tr/2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f/tr
     *        - gg / tr**3 / 3.d0
c                              b7 term added June 16, 2004. PJ Gorman.
     *        + b7 * 2d0 * pr * tr
c                              V* (P)
         v  = v - b2 * tr - b4 * pr  
c                              b6 term added June 16, 2004. PJ Gorman.
     *          + b6 * pr * pr
c                              sign on b7 corrected June 16, 2004. PJ Gorman.
     *          + b7 * tr * tr
c                              a*  (-T log T)
c        a  = a
c                              b*  (-T*T)
         b  = b7 * pr + b / 2d0
c                              c*  (-1/T)
         c  = c / 2d0
c                              d*  (-T**3)
         d  = d / 6d0
c                              e*  (sqrt T)
         e  = 4d0 * e
c                              f*  (log T)
c        f  = f
c                              gg* (-1/T**2)
         gg = gg/6d0
c                              b2* (PT)
         b2 = b2
c                              b7 term added June 16, 2004. PJ Gorman.
     *          - 2d0 * b7 * tr 
c                              b3* ((P-Pr) exp (T / c1))
c        b3 = b3
c                              b4* (P**2)
         b4 = b4/2d0
c                              b6 term added June 16, 2004. PJ Gorman.
     *          - b6 * pr 
c                              b5* (exp (P/ c2))
c        b5 = c2 * b5
c                              b6* (P**3)
         b6 = b6/3d0
c                              b7* (P*T**2)
c        b7 = b7
      else
c                              nonlinear EoS's:

c                              thermal terms:
         g  = g
c                              add the integral of sdt at tr,pr:
     *       + s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *       - d * tr**3 / 3d0 - 2d0 * e * dsqrt(tr)
     *       - f * dlog(tr) + gg / tr / tr / 2d0 + f
c                              S* (T)
         s  = a - s + a * dlog(tr) + b * tr - c / tr / tr /2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f / tr
     *        - gg / tr**3 / 3d0
c                              these terms are as above
         b  = b / 2d0
         c  = c / 2d0
         d  = d / 6.d0
         e  = 4d0 * e
         gg = gg/6.d0
c                              volume terms:
         if (b8.gt.0d0) then
c                              HP-murnaghan vdp
            b1 = v*b1 
            b2 = v*b2/2d0 
            b3 = v*b3 
            b4 = -v*b4 
            b5 = 2d0*v*b5 
            v  = v - (b1-b2*tr)*tr - b3*dlog(tr) - b4/tr - b5*dsqrt(tr) 
c                                 these are unnecessary, but may cut time?
            b9 = (1d0-1d0/b8) 
            b10 = b8*pr 
            b11 = 1d0/b8 
            b12 = b8 - 1d0

         else if (b8.lt.-3d0) then
c                              Ghiorso Murnaghan



         else 
c                              gottschalk, b3 = alpha, -b8 = beta: 
            b1 = -v / b8 / dexp (b3*tr)
      
         end if 

      end if 
 
      end

      subroutine disord (tk,d0,d1,d2,d3,d4,d5,d6,tr,tmax,g)
c----------------------------------------------------------------------
c compute t-dependent disorder contribution, g is the
c gibbs energy of disordering, tr is the t of onset of
c disordering, tmax is the t of complete disorder.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      common/ cst5 /p,v(4),pr,z(3)
 
      g = 0d0
      if (tk .lt. tr) return
      tt  = tk
      if (tk.gt.tmax) tt = tmax
 
      dh = d0 * (tt - tr)
     *     + 2d0 * d1 * (tt**0.5 - tr**0.5)
     *     - d2 * (1d0 / tt - 1d0 / tr)
     *     + d4 * dlog(tt/tr)
     *     + d5 * (tt*tt - tr*tr) / 2d0
     *     + d6 * (tt**3 - tr**3) / 3d0
 
      ds = d0 * dlog(tt/tr)
     *     - 2d0 * d1 * (tt**(-0.5) - tr**(-0.5))
     *     - d2 * (1d0/tt/tt - 1d0/tr/tr) / 2d0
     *     - d4 * (1d0/tt - 1d0 / tr)
     *     + d5 * (tt - tr)
     *     + d6 * (tt*tt - tr*tr) / 2d0
 
      gss = dh - (tk * ds)
      if (d3.ne.0d0) gss = gss + dh/d3 * (p - pr)
      end
 
      subroutine getphi (name,ibase,icmpn,*)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 name, oname, record*1
 
      common/ cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst43  /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst202 /tm(m7,m6),td(m8),ilam,idiso,lamin,idsin
     *      / cst207 /ctrans(k0,k5),ictr(k5),itrans/ cst318 /emodu(k15)
 
      save oname
 
      data oname/' '/
 
30    read (n2,1000,end=90,err=98) record
      if (record.eq.' ') then
c                          check for comments, i.e., data 
c                          with a blank 1st character 
         goto 30
      else  
         backspace (n2)
         read (n2,1000,end=90,err=98)
     *                     name, ibase, ikind, ilam, idiso
      end if 

      read (n2,*,err=98) (comp(i), i=1, icmpn), 
     *                   (therm(i), i= 1, k14)
c                               do component transformation if
c                               itrans is not zero
      if (itrans.gt.0) then
 
         do i = 1, itrans
            it = ictr(i)
            if (comp(it).ne.0d0) then
c                                ct is how much of the new
c                                component is in the phase.
               ct =  comp(it) / ctrans(it,i)
 
               do j = 1, icmpn
                  comp(j) = comp(j) - ct * ctrans(j,i)
               end do 
 
               comp(it) = ct
            end if 
         end do
      end if
 
      if (ilam.ne.0) then
c                               determine number of transitions from
c                               flag ilam:
         jlam = ilam

         if (ilam.gt.3) jlam = ilam - 3
         if (ilam.gt.6) jlam = ilam - 6
         if (ilam.gt.9) jlam = ilam - 9
 
         do i= 1, jlam
            read (n2,*,err=98) (tm(j,i), j = 1, m7 - 2)
         end do 
      end if
 
      if (idiso.ne.0) read (n2,*,err=98) td

      if (ikind.ne.0) read (n2,*,err=98) emodu
  
      oname = name
 
      return
 
90    return 1

98    if (oname.ne.' ') then
         call error (23,ct,i,oname)
       else
         call error (23,ct,i,'  none  ')
       end if

1000  format (a,i2,i2,i2,i2)
 
      end
 
      subroutine chkphi (ichk,name,tot,icout,ibase,icmpn,*)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 name, exname
 
      integer icout(k0)
 
      common/ cst43 /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst6 /icomp,istct,iphct,icp/ cst36 /exname(h8)
     *      / cst37 /ixct,iexyn/ cst42 /ic(k5),idbase
 
c                               reject the data if wrong data base:
      if (ibase.ne.idbase) goto 90
c                               reject the data if excluded in input:
      do i = 1, ixct
         if (name.eq.exname(i)) goto 90
      end do
c                               reject phases which have a component
c                               not included in the input:
      do i= 1, icmpn
         if (icout(i).eq.0.and.comp(i).ne.0d0) goto 90
      end do
c                               reject phases with negative compositions
      do j = 1, icmpn
         if (comp(j).lt.0d0) then
            if (ichk.eq.0) call warn (13,r,j,name)
            goto 90
         end if
      end do 
c                               the following is not executed by build:
c                               if ichk ne 0 reject phases that consist entirely
c                               of saturated components: 
      if (ichk.eq.0) return
c                               reject phases with null composition, in case
c                               a user puts one in by accident
      tot = 0d0
      do j = 1, icp
         tot = tot + comp(ic(j))
      end do 

      if (tot.eq.0d0) goto 90

      goto 99

90    return 1
 
99    end
 
      subroutine lamlan (dg,rkt,ld,ltype)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using  the
c     Landau model (Holland and Powell).
 
c     input variables
 
c                        t = temperature in k
c                        p = pressure in bar
c                        ld = pointer to phase in therlm
c                        rkt = bulk modulus
 
c     returned - dg - difference in free energy due to the transition
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst1 /thermo(k4,k10),uf(2),us(h5)

      lct = ltype - 9
c                              don't do this stuff until TJBH
c                              has a consistent policy:

c                              create a constant which contains
c                              the 0 to Tr contribution to the
c                              free energy:
c               q = (1.-(tr/tc))**0.25
c               if (tr.gt.tc) q = 0.
c                              maximum enthalpy of disorder:
c               therlm (3,j,lamin) = 2./3.*smax*tc
c                              enthalpy of disorder at tr:
c               therlm (4,j,lamin) = 2.*smax*tc * 
c     *                              (q**6/6. - q*q/2. + 1./3.)
c                              entropy of disorder at tr:
c               therlm (5,j,lamin) = 

 
      dg = 0d0

      do i = 1, lct
 
         tc = therlm(1,i,ld) + therlm(3,i,ld)*(p-pr)
 
         smax = therlm (2,i,ld)

         if (t.gt.tc) then
c                                  fully disordered:
            dg = dg + therlm(4,i,ld) - t*therlm(5,i,ld)
 
         else
c                                  partially disordered:
            q2 = dsqrt(1d0-t/tc)
c                                  bulk modulus:
            dg = dg   
c                                  hr-t*sr
     *           + therlm(4,i,ld) - t*therlm(5,i,ld)   
c                                  G(landau)
     *           + smax * (q2*(t-tc) + tc*q2**3/3d0)
 
         end if
c                                  if pressure dependent
c                                  add the vdp term:
c                                  pressure dependent 
c                                  landau transitions are
c                                  only in the murnaghan 
c                                  versions of HP data.
         if (therlm(3,i,ld).ne.0d0) then
            dg = dg 
c                                  vt*kt/3
     *           + (therlm(6,i,ld) + therlm(7,i,ld)*t
     *              + therlm(8,i,ld)*dsqrt(t))*rkt/3d0
c                                  *[(1+4p/kt)^(3/4) - 1]
c                                  this approximation is only
c                                  good for pr near 0.
     *             *((1d0 + 4d0*p/rkt)**(0.75d0) - 1d0)
         end if 

 
      end do
 
      end

      function gtrans (id,j)
c-----------------------------------------------------------------------
c gtrans computes the reference pressure free energy of a compound 
c aboves its jth transition. 
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst44 /dlogt,dsqrtt
 
      gtrans = therlm(12,j,id)
c                                    -sdt
     *       + t * (therlm(3,j,id) - therlm(5,j,id) * dlogt
     *       - t * (therlm(6,j,id) + therlm(8,j,id) * t))
     *       - (therlm(7,j,id) + therlm(11,j,id) / t) / t
     *       + therlm(9,j,id) * dsqrtt + therlm(10,j,id)*dlogt

      end 
 
      subroutine lamhel (p,t,g,vdp,ld,ltype)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using model
c     of helgeson et al 1978 (AJS).
 
c     there is something seriously wrong in this routine!!!!
 
c     input variables
c                        p = pressure in bars
c                        t = temperature in k
c                        ld = pointer to phase in therlm
c                        g = initial free energy
 
c     returned - g - modified phase free energy
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)/ cst5 /v(6),pr,u(2)
 
      lct = ltype - 3
c                                 T<T lowest transition, ignore
c                                 possibility of < clapeyron slope
c                                 and exit:
      if (t.lt.therlm(1,1,ld)) goto 99

      do 10 i = 1, lct 
         if (t.lt.therlm(1,i,ld)) then 

            if (i.eq.1) then
c                                 T<T lowest transition, ignore
c                                 possibility of < clapeyron slope
c                                 and exit:
                goto 99
             else
c                                 at the i-1 th transition:
                jtran = i - 1
                goto 20
             end if 
          else 
             jtran = i 
          end if 
10    continue

20    g = gtrans(ld,jtran) + vdp 
c                                 add vtrans*dp terms, this is
c                                 only set up for one transition
c                                 with a non-zero clapeyron.
c                                 should write warning. 
      if (therlm(2,1,ld).eq.0d0) goto 99

      trtp = therlm(1,1,ld) + (p-pr)/therlm(2,1,ld)
      dt = t - therlm(1,1,ld)

      if (t .gt. trtp) then
c                                     1 bar polymorph is stable at p,
c                                     p*thermo(3,id) is 0 bar standard
c                                     state pdv polymorph contribution &
c                                     therlm(4,j,ld) is Delta V of trans
c                                     contribution.
         pstar = dt*therlm(2,1,ld) + pr
 
         g = g + (p-pstar)*therlm(4,1,ld)
 
      else
c                                     the 1 bar polymorph isn't stable.
         g = g + dt * therlm(2,1,ld) * therlm(4,1,ld)
 
      end if
 
99    end
 
      subroutine lamqtz (p,t,g,du,ld,id)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using model
c     of helgeson et al 1978 (AJS). eq. (114) (corrected for the
c     misplaced parenthesis and the sign on c sub alpha), this is
C     probably bogus for coesite note that only one transition
c     is allowed.
 
c     input variables
c                        p = pressure in bars
c                        t = temperature in k
c                        ld = pointer to phase in therlm
c                        g = initial free energy
 
c     constants
c                Tr = 298.15 K
c                Pr = 1. bar
c                S(lope) = 38.5 bar/K
c                ba = 0.066 j/bar
c                aa = 549.82 K
c                ca = -4.973d-6 j/bar/bar
c                trt = 848. K
 
c     returned - g - modified phase free energy
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst1   /thermo(k4,k10),uf(2),us(h5)
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
 
      save trt, tr, pr, s, aa, ba, ca , vb
 
      data trt, tr, pr, s, aa, ba, ca, vb / 848., 298.15, 1d0,
     *                  38.5, 549.82, 0.066, -4.973d-6, 2.372 /
 
c      trtp = trt + (p-pr) / s
 
      if (t.gt.trt) then
         ps = (t-trt) * therlm(2,1,ld) + pr
      else
         ps = pr
      end if
c                                      if above the ref P transition T
c                                      do the cp integration:
      if (t.gt.trt) g = therlm(8,1,ld) + (p-ps) * thermo(3,id)
     *                - therlm(3,1,ld) * (t-trt) + du
     *     + therlm(5,1,ld) * (t - trt - t * dlog(t/trt))
     *     - (therlm(7,1,ld) + therlm(6,1,ld) * t * trt * trt)
     *     * (t - trt)**2 / 2d0 / t / trt / trt
 
c                                      now add in pdv terms to
c                                      the free energy, note that
c                                      pdv term for the ref polymorph
c                                      is already in:
 
      pdv  =  vb * (ps-pr)
     *      - ca * ( (2. * pr * (p-ps) - (p * p - ps * ps) ) / 2.
     *              + s * (t-tr) * (p-ps) )
     *      + s * (ba + aa*ca*s) * (t-tr)
     *                           * dlog ((aa+p/s) / (aa + ps/s))
 
      g = g + pdv
 
      end
 
      subroutine lamubc (p,t,gspk,k,ltype)
c---------------------------------------------------------------------
c     calculate the extra energy of a lamdba transition using model
c     of berman and brown (1985, contribs. miner. petro.)
 
c     input variables
c                        p = pressure in bars
c                        t = temperature in k
c                        k = pointer to phase in therlm
 
c     returned - gspk - energy because of lambda spike
 
c     modified from perkins et al 1986 (computers and geosciences).
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
c---------------------------------------------------------------------
      gspk=0d0
 
      do 10 j = 1, ltype
 
         tq1bar = therlm(3,j,k)
 
         if (tq1bar.eq.0d0) goto 10
 
            tr    = therlm(7,j,k)
            teq   = therlm(4,j,k) * (p-1d0) + tq1bar
            ctrans = tq1bar - teq
            tr9    = tr - ctrans
 
            if (t.lt.tr9) goto 10
 
            aspk2 = therlm(1,j,k)
            bspk2 = therlm(2,j,k)
            dvdtr = therlm(5,j,k)
            dvdp  = therlm(6,j,k)
            dstr  = therlm(8,j,k) / tq1bar
            abspk = therlm(9,j,k)
 
            if (t .gt. teq) then
               t9 = teq
            else
               t9 = t
            end if
 
            ct2 = ctrans * ctrans
            ct3 = ct2 * ctrans
 
            a1 = aspk2 * ctrans + 2. * abspk * ct2 + bspk2 * ct3
            b1 = aspk2 + 4. * abspk * ctrans + 3. * bspk2 * ct2
            c1 = 2. * abspk + 3. * ctrans * bspk2
 
            t92 = t9 * t9
            t93 = t9 *t92
            tr92 = tr9 * tr9
            tr93 = tr92 * tr9
 
            dhspk = a1 * (t9 - tr9)
     *            + b1 * (t92 - tr92) / 2.
     *            + c1 * (t93 - tr93) / 3.
     *            + bspk2 * (t9*t93 - tr93*tr9) / 4.
 
            dsspk = a1 * (dlog(t9) - dlog(tr9))
     *            + b1 * (t9 - tr9)
     *            + c1 * (t92 - tr92) / 2.
     *            + bspk2 * (t93 - tr93) / 3.
 
            gspk = gspk - (t9 * dsspk) + dhspk
 
            if (t.gt.teq) gspk = gspk - (dstr + dsspk ) * (t-teq)
 
            gspk = gspk + dvdtr * (p-1d0) * (t9-tr)
     *                  + dvdp * ((p*p-1d0)/2d0 - (p-1d0))
 
10    continue
      end
 
      subroutine loadit (id)
c---------------------------------------------------------------------
c loadit loads descriptive data for phases and species (name,comp,
c and therm) into the appropriate arrays (names,comps,thermo,vf,
c and vs).  the arguement 'id' indexes the phase in the arrays.
c note that loadit also computes reference state constants which
c are dependent on the state function being used and on its
c analytical expression.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 names, name, cmpnt*5, dname*40
      double precision z(13)
 
      common/ cst60  /ipoint,imyn/ cst319 /emod(k15,k10),iemod(k1),kmod
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst1   /thermo(k4,k10),uf(2),us(h5)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst8   /names(k1)/ cst318 /emodu(k15)
     *      / cst5   /p,t,xco2,u1,u2,tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst6   /icomp,istct,iphct,icp/ csta6 /name
     *      / cst40  /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / csta5  /dname(8),cmpnt(k0)/ cst42 /ic(k5),idbase
     *      / cst202 /tm(m7,m6),td(m8),ilam,idiso,lamin,idsin
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst43  /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
c---------------------------------------------------------------------
      if (iphct.gt.k10) call error (180,r,k10,'LOADIT')
      ipoint = iphct
      
c                               load name and phase flag
      names(id) = name
c                               shear modulus flag
      if (therm(3).lt.0) then 
         iemod(id) = 2
      else 
c                               this is gonna be bad news
c                               if anyone ever take the 
c                               seimic velocity tutorial too
c                               earnestly.
         iemod(id) = ikind
      end if 

      ikp(id) = 0
      atw(id) = 0d0
c                               set fluid flag:
      if (therm(3).eq.0d0.and.ifyn.eq.1) then
         ifp(id) = 1       
      else
         ifp(id) = 0
      end if
c                               load stoichiometry of components.
      do i = 1, icomp
         atw(id) = comp(ic(i)) * atwt(i) + atw(id)
         cp(i,id) = comp(ic(i))
      end do 
c                               load elastic props if present
      if (iemod(id).ne.0) then 
c                               kmod initialized 0 in main.
         kmod = 1        
         do i = 1, k15
            emod(i,id) = emodu(i)
         end do
      end if 
c                               compute reference state constants,
c                               etc..
      do i = 1, k4
         thermo(i,id) = therm(i)
      end do 
 
      call conver (thermo(1,id),thermo(2,id),thermo(3,id),thermo(4,id),
     *             thermo(5,id),thermo(6,id),thermo(7,id),thermo(8,id),
     *             thermo(9,id),thermo(10,id),thermo(11,id),
     *             thermo(12,id),thermo(13,id),thermo(14,id),
     *             thermo(15,id),thermo(16,id),thermo(17,id),
     *             thermo(18,id),thermo(19,id),thermo(20,id),
     *             thermo(21,id),thermo(22,id),tr,pr)
 
      if (tr.eq.0d0) then
         thermo(1,id) = therm(1)
         thermo(2,id) = therm(2)
      end if
c                              lmda transitions:
      idis(id) = 0
      lmda(id) = 0
      if (ilam.ne.0) then
         lamin=lamin+1
 
         if (ilam.ge.10.and.ilam.le.12) then
c                              holland and powell, landau model:
            lct = ilam - 9
            do 140 j = 1, lct       

               smax = tm(2,j)
               t0 = tm(1,j)
               vmax = tm(3,j)
               a0 = therm(11)

               therlm(1,j,lamin) = t0
               therlm(2,j,lamin) = smax
c                              this makes therlm(3) dt/dp
               therlm(3,j,lamin) = tm(3,j)/tm(2,j)
c                              qr2,qr4
               
               qr2 = dsqrt (1d0 - tr/t0)
               qr4 = (1d0 - tr/t0)**(1.5)/3.d0
c                              href:
               therlm(4,j,lamin) = smax*t0*(qr2-qr4)
c                              sref:
               therlm(5,j,lamin) = smax*qr2
c                              vT coefficients:  
c                              constant:
               therlm(6,j,lamin) = vmax*qr2*
     *                             (1d0-a0*tr+20d0*a0*dsqrt(tr))
c                              *T coefficient:
               therlm(7,j,lamin) = vmax*qr2*a0
c                              *sqrt(T) coeff:
               therlm(8,j,lamin) = -20d0*vmax*qr2*a0
c                              TJBH said that he is not yet
c                              consistent in correcting the 
c                              reference state props for the
c                              landau effect, accordingly I
c                              assume here that the effect is
c                              not incorporated. 

c                              this stuff is no longer possible with
c                              hollands variable tc formulation:

c                              create a constant which contains
c                              the 0 to Tr contribution to the
c                              free energy:
c               q = (1.-(tr/tc))**0.25
c               if (tr.gt.tc) q = 0.
c                              maximum enthalpy of disorder:
c               therlm (3,j,lamin) = 2./3.*smax*tc
c                              enthalpy of disorder at tr:
c               therlm (4,j,lamin) = 2.*smax*tc * 
c     *                              (q**6/6. - q*q/2. + 1./3.)
c                              entropy of disorder at tr:
c               therlm (5,j,lamin) = smax * (1. - q*q)
140         continue

 
 
         else if (ilam.le.3) then
c                              ubc:
            do 90 j = 1, ilam
 
               therlm(1,j,lamin)=tm(1,j)*tm(1,j)
               therlm(2,j,lamin)=tm(2,j)*tm(2,j)
               therlm(9,j,lamin)=tm(1,j)*tm(2,j)
 
               do 80 k = 3, 8
80                therlm(k,j,lamin)=tm(k,j)
90          continue
 
         else if (ilam.gt.3.and.ilam.lt.8) then
c                              helgeson:
            p = pr
c                              now convert paramters:
            do 100 k = 1, ilam - 3
c                              load into therlm:
               therlm(1,k,lamin) = tm(1,k)
               therlm(2,k,lamin) = tm(2,k)

               do 120 j = 4, 10
120               therlm(j+1,k,lamin) = tm(j,k)

               t = tm(1,k)
               if (k.eq.1) then
c                              set transition type to null
c                              for call to gphase
                  lmda(id) = 0
               else 
                  lmda(id) = lamin
                  ltyp(id) = 2 + k
               end if
c                              g at trt:
               call updumy
               call gphase (id,gph)
               therlm(12,k,lamin) = gph
c                              delta v trans:
               therlm(4,k,lamin) = 0d0
               if (tm(2,k).ne.0.) therlm(4,k,lamin) = tm(3,k)/tm(2,k)
c                              s + st at trt:
               t = t + 1d-3
               call updumy
               call gphase (id,gph)
               therlm(3,k,lamin) = tm(3,k) -
     *                             (gph - therlm(12,k,lamin))/1d-3
c                              streamline the eos:
               do j = 1, 13
                  z(j) = 0d0 
               end do 

100            call conver (therlm(12,k,lamin),therlm(3,k,lamin),
     *          z(1),therlm(5,k,lamin),therlm(6,k,lamin),
     *          therlm(7,k,lamin),therlm(8,k,lamin),therlm(9,k,lamin),
     *          therlm(10,k,lamin),therlm(11,k,lamin),
     *          z(2),z(3),z(4),z(5),z(6),z(7),z(8),
     *          z(9),z(10),z(11),z(12),z(13),tm(1,k),pr)

         end if
 
         lmda(id)=lamin
         ltyp(id)=ilam
 
      end if
c                              t dependent order: load berman and brown
c                              parameters, (this should be cleaned up)
      if (idiso.ne.0) then
         idsin = idsin + 1
         idis(id) = idsin
         do 70 j = 1, m8
70          therdi(j,idsin) = td(j)
      end if
 
      end

      subroutine difgx (x,fact,index,iord,isub)
c---------------------------------------------------------------------
c subroutine to evaluate the function P(X)+(1-xi)*diff(P(X),xi)
c where P(X) is the product x(isub(1))*x(isub(2))*...,
c isub contains the indexes of x in P(X), and index is the
c value of i, the value is returned as difgx. iord is the maximum
c order possible for P(X).
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      double precision x(4)

      integer isub(4), jnot(3), iord
 
      if (x(index).eq.1d0.or.x(index).eq.0d0) 
     *              call error (27,x(1),iord,'IMBECILLE!')
c                               find the order of P(X) = jord,
c                               and count the number of occurences
c                               of x(index) = ixi.
      ixi = 0 
      ini = 0 
      do 10 i = 1, iord
         if (isub(i).ne.0) then 
c                               assume 0 subscripts at end of isub
            jord = i
            if (isub(i).eq.index) then 
               ixi = ixi + 1
            else 
               ini = ini + 1
               jnot(ini) = isub(i)
            end if 
         end if 
10    continue
c                               2nd order parameter:
      if (jord.eq.2) then

         xm = 1. - x(index)
         
         if (ixi.eq.0) then 

            fact = -x(isub(1))*x(isub(2))

         else if (ixi.eq.1) then 

            fact = x(jnot(1))*x(index) 
     *           + (xm*x(jnot(1))-2.*x(index)*x(jnot(1))) / xm

         else 

            call error (999,fact,jord,'difgx')

         end if 
c                               3rd order parameter:
      else if (jord.eq.3) then

         if (ixi.eq.0) then 

            fact = -2.*x(isub(1))*x(isub(2))*x(isub(3))

         else if (ixi.eq.1) then 

            fact = -2.*x(isub(1))*x(isub(2))*x(isub(3)) 
     *           + x(jnot(1))*x(jnot(2))

         else if (ixi.eq.2) then 
       
            fact = 2.*(x(index)*x(jnot(1)) - x(index)**2*x(jnot(1)))

         else 

            call error (999,fact,jord,'difgx')

         end if 

      else if (jord.eq.4) then 
c                               4th order parameter:
         if (ixi.eq.0) then 

            call error (999,fact,jord,'difgx')

         else if (ixi.eq.1) then 

             fact = -2.*x(index)**2*x(jnot(1))*x(jnot(2))*x(jnot(3))
     *            + 2. * x(isub(1))*x(isub(2))*x(isub(3))*x(isub(4))

         else if (ixi.eq.2) then 

             fact = 2.*x(index)*x(jnot(1))*x(jnot(2))
     *            - 3.*x(index)**2*x(jnot(1))*x(jnot(2))

         else if (ixi.eq.3) then 

             fact = 3.*(x(jnot(1))*x(index)**2 
     *            - x(jnot(1))*x(index)**3)
 
         end if 
      end if 
      end

      subroutine univeq (i,ier)
c---------------------------------------------------------------------
c univeq computes the equilibrium condition of a univariant reaction.
c using a finite difference newton-raphson approximation. the method
c will fail when the derivative of the state function, with respect to
c the state variable v(i), goes to infinity.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst62 /blim(l2),ulim(l2),dgr/ cst5  /v(l2),tr,pr,r,ps
     *      / cst87 /delt(l2),dtol,utol,ptol
     *      / cst9 /vmax(l2),vmin(l2),ddv(l2)
 
      ier = 0
 
      vi = v(i)
      del = delt(i)
      b = blim(i)
      u = ulim(i)
c                                  phase composition special case:
      if (i.eq.3) then 
         if (vi.lt.1d1*del) then 
            del = dabs(vi)/1d1
         else if (1d0-vi.lt.1d1*del) then 
            del = dabs(1d0-vi)/1d1
         end if 
      end if 
 
      if (vi+dabs(del).gt.u.or.vi-dabs(del).lt.b) goto 30
 
5     do j = 1, 100

         call grxn (gr)

         v(i) = vi + del
         call incdep (i,v(i))

         call grxn (dgr) 
         dgr = dgr - gr

         if (dgr.eq.0d0) exit
 
         delv = gr*del/dgr

         if (delv/ddv(i).gt.1d0) then 
            v(i) = vi - del
            del = del*ddv(i)/delv/2d0
            if (del/delt(i).lt.1d-6) goto 30 
            cycle
         end if 

         vi = vi - delv

         if (vi+dabs(del).gt.u.or.vi-dabs(del).lt.b) goto 30

         v(i) = vi

         call incdep (i,v(i))
c                                 used to be on the basis of utol,
c                                 but this allows uniform relative error.
         if (dabs(delv).lt.del) goto 99

      end do 
c                                  error code 1: iv and dv must be
c                                  switched
      ier = 1
      return
c                                  error code 2: value too far out of
c                                  range, refine iv increment.
30    ier = 2
99    end

      subroutine unver (g,s,v,a,b,c,d,e,f,gg,
     *                  b1,b2,b3,b4,b5,b6,b7,b8,tr,pr)
c---------------------------------------------------------------------
c convert thermodynamic equation of state from a 0-0 reference state
c to a pr-tr reference state.

c corrections made corresponding to PJ's corrections in conver, 
c June 16, 2004.
c---------------------------------------------------------------------
 
      implicit double precision (a-g,o-z),integer (h-n)
c                               stixrudes EoS, exit without
c                               doing anything
      if (v.lt.0d0) goto 99

      gg = 6d0 * gg
      e  = e / 4d0
      d  = 6d0 * d
      c  = 2d0 * c

      if (b8.eq.0d0) then 
c                                normal vdp term:
         b6 = 3d0 * b6
         b4 = 2d0 * (b4 + b6 * pr)
         b2 = b2 + 2d0 * b7 * tr
         b  = 2d0 * (b - b7*pr)
         v  = v + b2 * tr + b4 * pr - b6 * pr * pr - b7 * tr * tr
         s  = -1d0 * ( s - ( a - b2 * pr  + a * dlog(tr) + b * tr
     *        - c / tr / tr /2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f / tr
     *        - gg / tr**3 / 3d0 + b7 * 2d0 * pr * tr ) )
         g  = g - ( s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *        - d * tr**3 / 3d0 - 2d0 * e * dsqrt(tr)
     *        - f * dlog(tr) + gg / tr / tr / 2d0 + f
     *        - v * pr + b2 * tr * pr + b4 * pr * pr / 2d0 
     *        - b6 * pr**3/3d0 - b7 * tr * tr * pr )
      else 

         b  = 2d0 * b

         s  = -1d0 * ( s - ( a + a * dlog(tr) + b * tr
     *        - c / tr / tr /2d0
     *        + d * tr * tr / 2d0 - 2d0 * e / dsqrt(tr) - f / tr
     *        - gg / tr**3 / 3d0 ) )
         g  = g - ( s * tr - a * tr - b * tr * tr / 2d0 + c / tr
     *        - d * tr**3 / 3d0 - 2d0 * e * dsqrt(tr)
     *        - f * dlog(tr) + gg / tr / tr / 2d0 + f)

         if (b8.gt.0d0) then 
c                                 b8 is K' 
c                                 HP-murnaghan vdp:
            v  = v + (b1+b2*tr)*tr + b3*dlog(tr) + b4/tr + b5*dsqrt(tr)
            b1 = b1 / v
            b2 = 2d0 * b2/v
            b3 = b3 / v
            b4 = -b4 / v
            b5 = b5 / v / 2d0  

         else if (b8.lt.-3d0) then
c                                 b8 is -K' 
c                                 Ghirso-murnaghan

         else 
c                                 v = f(exponential(beta))
c                                 zero b1, so users don't get confused.
            b1 = 0d0
         end if  

      end if 
 
99    end

 
      subroutine updumy
c---------------------------------------------------------------------
c dummy call to replace uproj in loadit
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      common/ cst5  /p,t,xco2,u1,u2,tr,pr,r,ps/ cst44 /dlogt,dsqrtt
 
c                           compute isobaric isothermal constants.
      if (t.lt.0d0.or.tr.eq.0d0) then
        dlogt=0d0
        dsqrtt=0d0
      else
        dlogt=dlog(t)
        dsqrtt=dsqrt(t)
      end if
 
      end
 
c-----------------------------------------------------------------------
 
c PLIB -  subroutines common to FRENDLY and VERTEX.
 
c-----------------------------------------------------------------------
 
      subroutine slope (iv1,iv2,s)
c---------------------------------------------------------------------
c slope computes the slope (div(1)/div(2)) of a univariant
c equilibria by approximating the derivatives dg/div(1) and
c dg/div(2) by finite differences.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      integer iv(2)

      double precision dg(2)
 
      common/ cst5 /v(l2),tr,pr,r,ps/ cst62 /blim(l2),ulim(l2),dgr
     *      / cst87 /delt(l2),dtol,utol,ptol
 
      iv(1) = iv1
      iv(2) = iv2
      call grxn (gr)

      do i = 1, 2
         v(iv(i)) = v(iv(i)) + delt(iv(i))
         call incdep (iv(i),v(iv(i)))
         call grxn (gval)
         dg(i) = (gval-gr)/delt(iv(i))
c                                    note the possibility exists here
c                                    for the value of an intensive
c                                    parameter to excede allowed
c                                    limits.  although this is very
c                                    unlikely a limit test like the
c                                    one done in 'univeq' could be
c                                    used.
         v(iv(i)) = v(iv(i)) - delt(iv(i))
         call incdep (iv(i),v(iv(i)))
      end do 

      s = -dg(2) / dg(1)
 
      end
 
      subroutine switch (div,ivi,ivd,jer)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst5 /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv(l2)/ cst31 /vn(k2,k5+1),irct,ird
c---------------------------------------------------------------------
c                                  reset intensive variables
      call reptx 
      vo = v(ivi)
c                                  determine the sign for the
c                                  ivd increment
      call slope(ivd,ivi,s)
      jer = 0
c
      if (s.eq.0d0) then
c                                  a zero slope shouldn't occur
c                                  someplace is a bug
         jer = 1
         return
      end if
c
      odiv = div
      div = dv(ivd)
      if (odiv.gt.0d0) goto 10
      if (s.gt.0d0) div = -dv(ivd)
      goto 20
10    if (s.lt.0d0) div = -dv(ivd)
c                                 estimate a new value for v(ivi)
20    v(ivi) = v(ivi)+div/s
      call incdep (ivi,v(ivi)) 
c                                 switch variables
      if ((v(ivi).gt.vmin(ivi)).and.(v(ivi).lt.vmax(ivi))) goto 30
      div = div/5d0

      if (dabs(div).lt.dv(ivd)/1d6) then
         jer = 1
         return
      end if 

      v(ivi) = vo
      call incdep (ivi,v(ivi)) 
      goto 20

30    iovd = ivd
      ivd = ivi
      ivi = iovd

      end

      subroutine unblnk (text)
c------------------------------------------------------------------- 
c unblnk - subroutine to remove blanks from text
 
c     text - character string 
c     jchar - length of unblanked character string, 0 on input 
c             if unknown.
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1 

      nchar = len(text)
 
      read (text,1000) (bitsy(i), i = 1, nchar)
c                                 scan for blanks:
      ict = 0

      do 10 i = 1, nchar
         if (bitsy(i).eq.' ') goto 10
         ict = ict + 1
         bitsy(ict) = bitsy(i)
10    continue

      write (text,1000) (bitsy(i), i = 1, ict)
 
1000  format (400a1)
      end

      subroutine enblnk (text)
c------------------------------------------------------------------- 
c enblnk - scan text to first blank and cut remaining text.
 
c     text - character string 
c     jchar - length of unblanked character string, 0 on input 
c             if unknown.
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = 1, nchar
         if (bitsy(i).eq.' ') goto 10
         ict = ict + 1
      end do 

10    write (text,1000) (bitsy(i), i = 1, ict),(' ',i = ict+1, nchar)
 
1000  format (400a1)
      end

      subroutine deblnk (text)
c------------------------------------------------------------------- 
c deblnk - scan text and delete multiple blank characters
 
c     text - character string 
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = 2, nchar
         if (bitsy(i).eq.' '.and.bitsy(i-1).eq.' ') goto 10 
         ict = ict + 1
         bitsy(ict) = bitsy(i)
10    end do 

      write (text,1000) (bitsy(i), i = 1, ict),(' ',i = ict+1, nchar)
 
1000  format (400a1)
      end

      subroutine mertxt (text,text1,text2)
c------------------------------------------------------------------- 
c mertxt - merge text strings cutting trailing blanks, if text1 eq 
c          blank then pad by an initial 40 blanks.
 
c     text - character string 
c-------------------------------------------------------------------

      implicit none

      integer i, nchar1, nchar2, nchar3
 
      character text*(*), text1*(*), text2*(*), bitsy(400)*1 

      if (text1.eq.' ') then
         write (text,1010) text2
      else 
         nchar1 = len(text1) 
         read (text1,1000) (bitsy(i), i = 1, nchar1)  
         do i = 2, nchar1
            if (bitsy(i).eq.' '.and.bitsy(i-1).eq.' ') then 
               nchar1 = i - 1
               goto 10 
            end if 
         end do 
c                                  put 5 blanks between 1st and 
c                                  2nd strings
10       do i = nchar1, nchar1 + 5
            bitsy(i) = ' '
         end do 

         nchar1 = nchar1 + 5 
         nchar2 = nchar1 + len(text2) 
         nchar3 = len(text)

         if (nchar2.gt.nchar3) then 
            nchar2 = nchar3 - nchar1
         end if 

         if (nchar2.gt.400) then 
            nchar2 = 400 - nchar1
         end if 

         read (text2,1000) (bitsy(i), i = nchar1, nchar2)  
         write (text,1000) (bitsy(i), i = 1, nchar2),
     *                     (' ',i = nchar2+1, nchar3)
      end if 

1010  format (40x,a)
1000  format (400a1)

      end



      subroutine inblnk (text,char)
c------------------------------------------------------------------- 
c inblnk - scan text to last '/' or '\' and insert char after.
 
c     text - character string 
c-------------------------------------------------------------------
 
      character text*(*), bitsy(400)*1, char*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0
      nchar = len(text)

      do i = nchar,1,-1
c                                 this line may cause problems
c                                 on some operating systems that 
c                                 recognize the backslash as an escape
c                                 character.
         if (bitsy(i).eq.'/') goto 10
         bitsy(i+1) = bitsy(i)
      end do 

      i = 0

10    bitsy(i+1) = char

      write (text,1000) (bitsy(i), i = 1, nchar)
 
1000  format (400a1)
      end

      subroutine reblnk (text)
c------------------------------------------------------------------- 
c reblnk - subroutine to replace blanks followed by a character
c          with an underscore.
 
c     text - character string 
c     jchar - length of unblanked character string, 0 on input 
c             if unknown.
c-------------------------------------------------------------------
 
      character*8 text, bitsy(8)*1 
 
      read (text,1000) bitsy
c                                 scan for blanks:
      ict = 0

      do 10 i = 1, 7

         if (i.eq.1.and.bitsy(i).eq.' ') goto 10 

         if (bitsy(i).eq.' '.and.bitsy(i+1).ne.' ') then
            ict = ict + 1
            bitsy(ict) = '_'
         else if (bitsy(i).eq.' ') then 
            goto 10
         else
            ict = ict + 1
            bitsy(ict) = bitsy(i)
         end if

10    continue 
 
      bitsy(ict+1) = bitsy(8)

      write (text,1000) (bitsy(i), i = 1, ict + 1)
 
1000  format (8a1)
      end


      subroutine reptx
c-----------------------------------------------------------------------
c reset - resets indendent potentials to the last known stable condition 
c along a univariant curve 
c----------------------------------------------------------------------- 

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
    
      common/ cst5  /v(l2),tr,pr,r,ps/ cst32 /ptx(l5),ipt2
     *      / cst24 /ipot,jv(l2),iv(l2)

      v(iv(1)) = ptx(ipt2-1)
      call incdep (iv(1),v(iv(1)))
      v(iv(2)) = ptx(ipt2)
      call incdep (iv(2),v(iv(2)))
   
      end

      subroutine toop (id,gex)
c-----------------------------------------------------------------------
c giulio, here is a dummy routine to serve as a template, be careful
c of the type conventions that i use (i.e., all reals are implicit
c double) and also not to change any of the variables in the common
c blocks. you have to keep the three common blocks cts6, cst5 and cst12
c for input, the variables that will concern you are 

c on input:
c           r - the gas constant, J/mol-K
c           t - temperature (K)
c           p - pressure (bar)
c           tr - reference t, as in the thermodynamic data file header.
c           pt - reference p, dido.
c           icp - the number of components.
c           id - a pointer to the phase composition
c           cp(i,id) - the mol fraction of the ith component in composition id.
c on output:
c           gex - the excess free energy in J (i.e., in excess of a mechanical
c                 mixture of the endmembers) per mol of components.

c N.B. the components appear in the same order that they are entered in 
c the input file and this must also be identical to the order in the 
c solution model data file. i think build will always make SIO2 the 
c first component, overriding your choice. so to be safe anywhere you
c enter components expect SIO2 to be the 1st composition, then enter 
c the remaining components in order of the cation atomic weight, e.g., 
c cp(1,id) will always be X(SIO2) in the id'th composition, cp(2,id) will
c be X(CA0) if there are no other components.

c----------------------------------------------------------------------- 

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst6   /icomp,istct,iphct,icp

      gex = 0d0

      do i = 1, icp
         if (cp(i,id).ne.0d0) gex = gex + r*t*cp(i,id)*log(cp(i,id))
      end do 

      end 
         
      subroutine unlam (tm,id,lct)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-y),integer (h-m)
 
      include 'perplex_parameters.h'
  
      double precision tm(m7,m6),z(9)

      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst100 /dlnfo2,elag,gz,gy,gx,ibuf,hu,hv,hw,hx
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
c-----------------------------------------------------------------------
                                 
      ilam = ltyp(id)
      jd = lmda(id)
      lct = 0 

      do i = 1, 10
         do j = 1, 3
            tm(i,j) = 0d0
         end do 
      end do 

      if (ilam.ne.0.and.(ilam.lt.4.or.ilam.gt.7)) then
   
         ilam = ltyp(id)
         jd = lmda(id)
         if (ilam.le.3) then
            lct = ilam
            do j = 1, ilam
               tm(1,j) = dsqrt (therlm(1,j,jd))
               tm(2,j) = dsqrt (therlm(2,j,jd)) 
            end do         
         else if (ilam.gt.9.and.ilam.lt.13) then
            lct = ilam - 9
            do j = 1, ilam - 9
               tm(1,j) = therlm(1,j,jd)
               tm(2,j) = therlm(2,j,jd)
               tm(3,j) = therlm(3,j,jd) *therlm(2,j,jd) 
            end do 
         end if
      end if

      if (ilam.gt.3.and.ilam.lt.7) then

         lct = ilam - 3
         p = pr
         do i = lct, 1 , -1
c                              get s transition:
c                              load into therlm:
            tm(1,i) = therlm(1,i,jd) 
            tm(2,i) = therlm(2,i,jd) 

            do j = 5, 11
               tm(j-1,i) = therlm(j,i,jd)  
            end do 
            t = tm(1,i)

            if (i.eq.1) then
c                              set transition type to null
c                              for call to gphase 
               ltyp(id) = 0
            else 
               ltyp(id) = 2 + i
            end if
c    c                         -s at trt:
            call updumy
            call gphase (id,g1)
            t = t + 1d-3
            call updumy
            call gphase (id,gph)
            tm(3,i) =  (gph - g1)/1d-3

            g0 = therlm(12,i,jd)
            s0 = therlm(3,i,jd)
            do k = 1, 9
               z(k) = 0d0
            end do 

            call unver (g0,s0,z(1),tm(4,i),tm(5,i),tm(6,i),tm(7,i),
     *                  tm(8,i),tm(9,i),tm(10,i),z(2),z(3),z(4),z(5),
     *                  z(6),z(7),z(8),z(9),tm(1,i),pr) 

            tm(3,i) = s0 + tm(3,i)
         end do 

         ltyp(id) = ilam
         lmda(id) = jd

      end if
      
      end 

      subroutine outlam (tm,jd,lct)
c---------------------------------------------------------------------
      implicit double precision (a-g,o-y),integer (h-m)
 
      include 'perplex_parameters.h'
  
      double precision tm(m7,m6)

      common/ cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
c-----------------------------------------------------------------------
      if (lct.ne.0) write (n2,1000) ((tm(i,j),i=1,10),j= 1,lct)

      if (jd.ne.0) write (n2,1000) (therdi(j,jd), j = 1, m8)

1000  format (5(g13.7,1x))

      end 
 
      subroutine incdep (ind,var) 
c--------------------------------------------------------------------
c incdep computes the dependent variable if defined (idep).

c  ind  - is the index of the current variable. 
c  iind - is the index of the independent variable that v(idep) is a 
c         function of.
c  var  - is the value of the current variable.
c--------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst316 /c0,c1,c2,c3,c4,iind,idep

      if (ind.eq.iind) then 

         v(idep) = c0 + var*(c1 + var*(c2 + var*(c3 + c4*var)))
 
      end if

      end 

      double precision function depvar (var) 
c--------------------------------------------------------------------
c depvar computes the dependent variable from the independent variable
c var
c--------------------------------------------------------------------
      implicit none

      double precision c0,c1,c2,c3,c4,var

      integer iind,idep

      common/ cst316 /c0,c1,c2,c3,c4,iind,idep

      depvar = c0 + var*(c1 + var*(c2 + var*(c3 + c4*var)))

      end 

      integer function match (idim,ier,name)
c----------------------------------------------------------------------
c find the index of endmember identified by name
c----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer idim, ier 

      character*8 mname, name

      common / cst18a /mname(m4)

      ier = 0

      do match = 1, idim
         if (name.eq.mname(match)) exit
      end do  

      if (match.gt.idim) ier = 1
      
      end 

      subroutine readn (idim,tname)
c----------------------------------------------------------------------
c readn - read idim endmember names expected format

c  name_1 name_2 ... | comment
c  name_n+1....      | comment

c end_of_data is either a "|" or the end of the record.
c----------------------------------------------------------------------
 
      implicit none

      include 'perplex_parameters.h'

      integer ibeg, iend, len, ier, idim, ict, i

      character chars(240)*1, mname*8, name*8, tname*10 

      common / cst18a /mname(m4)

      ier = 0 

      call readcd (len,ier,chars)
      if (ier.ne.0) goto 90

      ibeg = 1
      iend = len
      ict = 0  

      do while (ict.lt.idim) 
c                                    find the name
         call readnm (ibeg,iend,len,ier,name,chars)
         if (ier.ne.0) goto 90
         ict = ict + 1
         mname(ict) = name

         if (ibeg.ge.len.and.ict.lt.idim) then
            call readcd (len,ier,chars)
            ibeg = 1
            if (ier.ne.0) goto 90
         end if 

      end do

      goto 99      

90    write (*,1000) tname,(chars(i),i=1,len)
      pause
      stop
      
1000  format (' **error ver200** READN bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a,/)

99    end 


      subroutine readda (rnums,idim,tname)
c----------------------------------------------------------------------
c readda - reads idim numbers, discarding all data in records 
c          beyond the comment marker "|" or the end of the record.
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer len, idim, kdim, jdim, i, isnum, ier

      character chars(240)*1, tname*10, nums*240

      double precision rnums(idim)
c                                   read card scans for non blank data
c                                   card:
      kdim = 1
      jdim = 0
      isnum = 0 

      do while (jdim.lt.idim)

         call readcd (len,ier,chars)
         if (ier.ne.0) goto 91
c                                   got data, count how many "numbers"
         do i = 1, len
            if (chars(i).ne.' '.and.isnum.eq.0) then
               jdim = jdim + 1
               isnum = 1
            else if (chars(i).eq.' ') then
               isnum = 0
            end if 
         end do
c                                    if there is no comment
c                                    marker there can be more 
c                                    data than anticipated:
         if (jdim.gt.idim) jdim = idim
c                                    write numbers to string
         write (nums,*) (chars(i), i = 1, len),' '
         
         read (nums,*,err=90) (rnums(i), i = kdim, jdim)

         kdim = jdim + 1

      end do 

      goto 99 

90    write (*,1000) tname, (chars(i),i = 1, len)

      goto 99

91    write (*,1010) tname

      goto 99 

1000  format (' **error ver209** READDA bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a,/,
     *        ' READDA was expecting numeric data but found',
     *        ' character data.',/)

1010  format (' **error ver210** READDA read to end of file',
     *        ' reading solution model: ',a,/,
     *        ' READDA was expecting numeric data.',/)

99    end 

      subroutine readx (idim,tname)
c----------------------------------------------------------------------
c readx - read excess function for a solution model, assumes
c data on one line of less than 240 characters, the expected format

c        W(name-name-...) number number number

c end_of_data is either a "|" or the end of the record.
c----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ibeg, jend, len, ier, iscan, lord, imax, match, idim

      character chars(240)*1, name*8, begin*5, eod*3, tname*10

      integer istot,jstot,iend,isub,imd,
     *        ist,i,j,iord,iterm,isite,isp,insp,kstot,
     *        n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9

      double precision wg,xmn,xmx,xnc

      common/ cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9

      iterm = 0 
      iord = 0 

      call readcd (len,ier,chars)
      write (begin,'(5a)') (chars(i),i=1,5)

      if (begin.eq.'ideal') then
         goto 99
      else if (begin.ne.'begin') then 
         goto 90
      end if 

      do i = 1, m1
         do j = 1, m2
            isub(i,j,1) = 0 
         end do 
      end do 
 
      eod = ''

      do while (eod.ne.'end')  

         call readcd (len,ier,chars)
         if (ier.ne.0) goto 90

         write (eod,'(3a)') (chars(i),i=1,3) 
c                                     find brackets
         ibeg = iscan (1,len,chars,'(') + 1
         imax = iscan (1,len,chars,')') - 1

         if (ibeg.gt.len.or.imax.gt.len) cycle
c                                     data found
         iterm = iterm + 1
         if (iterm.gt.m1) call error (48,wg(1,1),m1,tname)

         lord = 0

         do while (ibeg.lt.imax)

            jend = imax

            call readnm (ibeg,jend,imax,ier,name,chars)
            if (ier.ne.0) goto 90

            lord = lord + 1
            if (lord.gt.m2) call error (49,wg(1,1),m2,tname)

            isub(iterm,lord,1) = match (idim,ier,name)  

            if (ier.ne.0) goto 90         

         end do 

         if (lord.gt.iord) iord = lord

         ibeg = imax + 2
         jend = len

         do i = 1, 3

            call readfr (wg(iterm,i),ibeg,jend,len,ier,chars)
     
            if (ier.ne.0) goto 90 
    
         end do 
      end do

      goto 99

90    write (*,1000) tname,(chars(i),i=1,len)
      pause
      stop
      
1000  format (' **error ver200** READX bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a,/,
     *        ' usually this error is due to a mispelled',
     *        ' endmember name.',/)

99    end 


      subroutine readop (idim,jlaar,tname)
c----------------------------------------------------------------------
c readop - tail of solution model to find optional dqf and
c          van laar size parameters

c readop - reads data until it finds an     "end_of_model" record
c          van laar data is identified by a "begin_van_la" record
c          dqf data is identified by a      "begin_dqf_co" record

c readop returns:

c          jlaar = 1 if van laar data found (else 0).
c          idqf  > 0 if dqf data found (else 0)

c----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer len, ier, idim, jlaar

      character chars(240)*1, begin*12, tname*10

      integer i,indq,idqf

      double precision dqf

      common/ cst222 /dqf(m3,m4),indq(m4),idqf

      jlaar = 0 
      idqf = 0 

10    call readcd (len,ier,chars)

      write (begin,'(12a)') (chars(i),i=1,12)

      if (begin.eq.     'end_of_model') then 
         goto 99
      else if (begin.eq.'begin_model ') then
c                              found new model, current
c                              model does not of end_of_model keyword
         goto 90
      else if (begin.eq.'begin_van_la') then 
c                              read van laar data: 
         jlaar = 1
         call readvl (idim,tname)
         goto 10 

      else if (begin.eq.'begin_dqf_co') then 
c                              read dqf data:
         call readdq (idim,tname)
         goto 10

      else 

         goto 91

      end if 

      goto 99

90    write (*,1000) tname,(chars(i),i=1,len)
      pause
      stop

91    write (*,1010) tname,(chars(i),i=1,len)
      write (*,1020)
      pause
      stop

1000  format (/,' **error ver200** READOP missing "end_of_model"',
     *          ' keyword at end',' of solution model:',a,/)
      
1010  format (/,' **error ver210** READOP bad data, currently',
     *          ' reading solution model: ',a,' data was:',/,240a)

1020  format (/,' This error is most probably due to an out-of-date',
     *          ' solution model file.',//,
     *          ' Copy the current version from:',//,
     *          ' www.perplex.ethz.ch/perplex/datafiles/newest_format_',
     *          'solut.dat',//)

99    end 

      subroutine readvl (idim,tname)
c----------------------------------------------------------------------
c readvl - read van laar volumes for a solution models endmembers, assumes
c data on one line of less than 240 characters, the expected format

c        alpha(name) number number number

c end_of_data is either a "|" or the end of the record.
c----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ibeg, len, ier, iscan, imax, match, idim, index

      character chars(240)*1, name*8, eod*3, tname*10

      integer jend,i,ict,jsmod

      double precision vlaar

      common/ cst221 /vlaar(m3,m4),jsmod

      ict = 0 
 
      eod = ''

      do while (eod.ne.'end')  

         call readcd (len,ier,chars)
         if (ier.ne.0) goto 90

         write (eod,'(3a)') (chars(i),i=1,3) 
c                                     find brackets
         ibeg = iscan (1,len,chars,'(') + 1
         imax = iscan (1,len,chars,')') - 1

         if (ibeg.gt.len.or.imax.gt.len) cycle
c                                     data found
         ict = ict + 1
         if (ict.gt.m4) goto 91
 
         jend = imax

         call readnm (ibeg,jend,imax,ier,name,chars)
         if (ier.ne.0) goto 90

         index = match (idim,ier,name)  

         if (ier.ne.0) goto 90         

         ibeg = imax + 2
         jend = len

         do i = 1, m3

            call readfr (vlaar(i,index),ibeg,jend,len,ier,chars)
     
            if (ier.ne.0) goto 90 
    
         end do 

      end do

      goto 99

90    write (*,1000) tname,(chars(i),i=1,len)
      write (*,1001)
      pause
      stop

91    write (*,1010) tname,ict
      pause
      stop
      
1000  format (' **error ver200** READVL bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a)

1001  format (/,' usually this error is caused by a mispelled',
     *          ' endmember name.',/)

1010  format (' **error ver201** READVL bad data, currently',
     *        ' reading solution model: ',a,/,
     *        ' this model requires 1 size parameter for',
     *        ' each endmember, READVL found ',i2,' parameters.',/)

99    end 

      subroutine readdq (idim,tname)
c----------------------------------------------------------------------
c readvl - read dqf corrections for solution models endmembers, assumes
c data on one line of less than 240 characters, the expected format

c        dqf(name) number number number

c        output:

c          idqf           - the number of endmembers with dqf corrections
c          indq(idqf)     - pointer to corrected endmember in the phase name array
c          dqf(1..3,idqf) - the dqf parameters for the correction

c end_of_data is either a "|" or the end of the record.
c----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ibeg, len, ier, iscan, imax, match, idim

      character chars(240)*1, name*8, eod*3, tname*10

      integer jend,i,idqf,indq

      double precision dqf

      common/ cst222 /dqf(m3,m4),indq(m4),idqf
 
      eod = ''

      do while (eod.ne.'end')  

         call readcd (len,ier,chars)
         if (ier.ne.0) goto 90

         write (eod,'(3a)') (chars(i),i=1,3) 
c                                     find brackets
         ibeg = iscan (1,len,chars,'(') + 1
         imax = iscan (1,len,chars,')') - 1

         if (ibeg.gt.len.or.imax.gt.len) cycle
c                                     data found
         idqf = idqf + 1
 
         jend = imax

         call readnm (ibeg,jend,imax,ier,name,chars)
         if (ier.ne.0) goto 90

         indq(idqf) = match (idim,ier,name)  

         if (ier.ne.0) goto 90         

         ibeg = imax + 2
         jend = len

         do i = 1, m3

            call readfr (dqf(i,idqf),ibeg,jend,len,ier,chars)
     
            if (ier.ne.0) goto 90 
    
         end do 

      end do

      goto 99

90    write (*,1000) tname,(chars(i),i=1,len)
      write (*,1001)
      pause
      stop
      
1000  format (' **error ver200** READDQ bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a)

1001  format (/,' usually this error is caused by a mispelled',
     *          ' endmember name.',/)

99    end 




      subroutine readr (coeffs,enth,inds,idim,nreact,tname)
c----------------------------------------------------------------------
c readr - read stoichiometric reaction data for a solution model, assumes
c data on one line of less than 240 characters, the expected format

c        name "=" (acoef(i), mame(i), i= 2..nreact) Enthalpy_value

c nreact = 4 for reciprocal reactions
c nreact = 3 for ordered species
c enthalpy_value is only read if nreact = 2

c end_of_data is either a "|" or the end of the record.
c----------------------------------------------------------------------
 
      implicit none

      include 'perplex_parameters.h'

      integer ibeg, iend, len, ier, iscan, i, nreact, inds(nreact), 
     *        idim, match

      double precision coeffs(nreact), enth

      character chars(240)*1, mname*8, name*8, tname*10 

      common / cst18a /mname(m4)

      ier = 0 

      call readcd (len,ier,chars)
      if (ier.ne.0) goto 90

      ibeg = 1
      iend = len
c                                    first name
      call readnm (ibeg,iend,len,ier,name,chars)
      if (ier.ne.0) goto 90

      if (nreact.eq.3) then 
c                                    if nreact = 3, new name
         idim = idim + 1
         mname(idim) = name
         inds(1) = idim

      else 
c                                    reciprocal sol, get index
         inds(1) = match(idim,ier,name)
         if (ier.ne.0) goto 90
              
      end if 
c                                    find marker '='
      ibeg = iscan (1,len,chars,'=') + 1
      iend = len

      do i = 2, nreact 
c                                    find a stoichiometric coeff
         call readfr (coeffs(i),ibeg,iend,len,ier,chars)
         if (ier.ne.0) goto 90
c                                    find the name
         call readnm (ibeg,iend,len,ier,name,chars)
         if (ier.ne.0) goto 90
c
         inds(i) = match(idim,ier,name)
         if (ier.ne.0) goto 90
           
      end do

      if (nreact.eq.3) then      
c                                     ordered compound, read
c                                     enthalpy, find marker '='
         ibeg = iscan (ibeg,len,chars,'=') + 1

         call readfr (enth,ibeg,iend,len,ier,chars)
         if (ier.ne.0) goto 90

      end if 

      goto 99      

90    write (*,1000) tname,(chars(i),i=1,len)
      pause
      stop
      
1000  format (' **error ver200** READR bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a,/)

99    end 

      subroutine readz (coeffs,inds,ict,idim,nmax,tname)
c----------------------------------------------------------------------
c readz - read site fraction data for a solution model, assumes
c data on one line of less than 240 characters, the expected format

c        comments "=" a0 (acoef(i), mame(i), i= 1...end_of_data)

c end_of_data is either a "|" or the end of the record.
c----------------------------------------------------------------------
      implicit none

      integer ibeg, iend, len, ier, iscan, ict, nmax, inds(nmax),
     *        match, idim, i

      double precision rnum, coeffs(nmax)

      character chars(240)*1, name*8, tname*10

      call readcd (len,ier,chars)
      if (ier.ne.0) goto 90 
c                                    find start of data marker '='
      ibeg = iscan (1,len,chars,'=') + 1
c                                    find a number
      call readfr (rnum,ibeg,iend,len,ier,chars)
      if (ier.ne.0) goto 90 
c                                    this is the a0 contant
      coeffs(1) = rnum
c                                    the rest of the data should
c                                    consist of coefficients followed
c                                    by names
      ict = 1 

      do while (ibeg.lt.len) 
c                                    find the number
         call readfr (rnum,ibeg,iend,len,ier,chars)
         if (ier.ne.0) goto 90
c                                    find the name
         call readnm (ibeg,iend,len,ier,name,chars)
         if (ier.ne.0) goto 90

c        if (ibeg.ge.len) exit

         ict = ict + 1
         coeffs(ict) = rnum 
         inds(ict) = match(idim,ier,name)
         if (ier.ne.0) goto 90
           
      end do

      goto 99

90    write (*,1000) tname,(chars(i),i=1,len)
      pause
      stop
      
1000  format (' **error ver200** READZ bad data, currently',
     *        ' reading solution model: ',a,' data was:',/,240a)

99    end 
 
      integer function iscan (ibeg,iend,chars,char)
c----------------------------------------------------------------------
      implicit none

      character chars(240)*1, char*1

      integer ibeg, iend

      do iscan = ibeg, iend

         if (chars(iscan).eq.char) exit

      end do 

      end 

      integer function iscnlt (ibeg,iend,chars,char)
c----------------------------------------------------------------------
      implicit none

      character chars(240)*1, char*1

      integer ibeg, iend


      do iscnlt = ibeg, iend

         if (chars(iscnlt).gt.char) exit

      end do 

      end 

      subroutine readfr (rnum,ibeg,iend,len,ier,chars)
c----------------------------------------------------------------------
c readfr looks for a number or two numbers separated by a backslash / in
c that array chars(iend:ibeg), the latter case is interpreted as a ratio. 
c the result is returned as num
c-----------------------------------------------------------------------
      implicit none

      double precision rnum, rnum1 

      integer ibeg, iend, len, iback, ier, iscan, iscnlt, i

      character chars(240)*1, num*30

      ier = 0 
c                                    now find start of a number
      ibeg = iscnlt (ibeg,len,chars,' ')  
c                                    find backslash
      iback = iscan (ibeg,len,chars,'/') - 1
c                                    find next blank
      iend = iscan (ibeg,len,chars,' ') - 1
c                                    three cases:
      if (iend.ge.len) then
         ier = 1
         goto 99 

      else if (iback.gt.iend) then
c                                    no fraction
         if (iend-ibeg+1.gt.30) goto 90
c                                    first constant
         write (num,'(30a)') (chars(i),i=ibeg,iend)
         read (num,*,err=90) rnum

      else 
c                                    fraction write numerator
         if (iback+1-ibeg.gt.30) goto 90
c                                    first number
         write (num,'(30a)') (chars(i),i=ibeg,iback)       
         read (num,*,err=90) rnum
c                                    second number 

         if (iend-iback-1.gt.30) goto 90
         write (num,'(30a)') (chars(i),i=iback+2,iend)      
         read (num,*,err=90) rnum1

         rnum = rnum/rnum1

      end if 

      ibeg = iend + 1

      goto 99

90    ier = 2

99    end 

      subroutine readnm (ibeg,iend,len,ier,name,chars)
c----------------------------------------------------------------------
c readfr looks for a number or two numbers separated by a backslash / in
c that array chars(iend:ibeg), the latter case is interpreted as a ratio. 
c the result is returned as num
c-----------------------------------------------------------------------
      implicit none

      integer ibeg, iend, len, iscan, iscnlt, ier, i

      character chars(240)*1, name*8

      ier = 0 
c                                    now find start of name
      ibeg = iscnlt (ibeg,len,chars,' ') 
c                                    find next blank
      iend = iscan (ibeg,len,chars,' ') - 1
c                                    or assume data goes to len
      if (iend-ibeg.gt.7) goto 90
c                                    initialize to be safe:
      name = '        '

      write (name,'(8a1)') (chars(i),i=ibeg,iend)

      ibeg = iend + 1

      goto 99

90    ier = 4

99    end 

      subroutine readcd (len,ier,chars)
c----------------------------------------------------------------------
c readcd - read 240 column card image from unit 9, strip out unwanted
c characters. ier = 1 no card found.
c----------------------------------------------------------------------    
      implicit none

      integer len, ier, iscan, ict, i, iscnlt, ibeg, 
     *        n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9

      character chars(240)*1, card*240

      common / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
 
      ier = 0 

      ibeg = 0
  
      len = 0 

      card = ''

      do while (ibeg.ge.len) 

         read (n9,'(a)',end=90) card

         if (card.ne.'') then 
            read (card,'(240a)') chars
c                                    find end of data marker '|'
            len = iscan (1,240,chars,'|') - 1
c                                    find a non blank character
            ibeg = iscnlt (1,len,chars,' ')
         end if 

      end do 

      ict = 1

      do i = 2, len 
c                                    strip out '+' and '*' chars
         if (chars(i).eq.'+'.or.chars(i).eq.'*') chars(i) = ' '
c                                    eliminate blanks after '/' and '-'
c                                    and double blanks
         if ((chars(ict).eq.'/'.and.chars(i  ).ne.' ') .or. 
     *       (chars(ict).eq.'-'.and.chars(i  ).ne.' ') .or.
     *       (chars(ict).eq.' '.and.chars(i  ).ne.' ') .or.
     *       (chars(ict).ne.'-'.and.chars(ict).ne.'/'.and.
     *        chars(ict).ne.' ') ) then
             ict = ict + 1
             chars(ict) = chars(i)
         end if

      end do 

      len = ict

      goto 99

90    ier = 3

99    end


      subroutine xrmode (tname)
c---------------------------------------------------------------------
c xrmode - reads old style solution models from LUN n9.
c---------------------------------------------------------------------
      implicit none
  
      include 'perplex_parameters.h'
 
      character*10 mname*8,tname,fname,names*8

      integer istot,jstot,iend,isub,imd,msub,mord,
     *        ist,i,j,k,l,m,inew,idummy,mdep,idep,jdep,
     *        iord,iterm,isite,isp,nsub1,nsub2,nttyp,
     *        h,insp,isoct,jmsol,nterm,nspm1,nsite,
     *        kstot,iddeps,jsmod,kdsol,n1,n2,n3,n4,n5,n6,n7,
     *        n8,n9,io3,io4,io5,io9

      double precision wg,xmn,xmx,xnc,ecoef,acoef,smult,vlaar,depvnu,
     *                 depenth,a0,y2p,nu

      integer ijk(mst)

      common/ cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),ecoef(m4)
     *      ,nsite,nspm1(m10),nterm(m10,m11),nsub1(m10,m11,m0,m12),
     *      nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst8 /names(k1)/ cst221 /vlaar(m3,m4),jsmod
     *      / csta7 /fname(h9)/ cst18a /mname(m4)/ cst79 /isoct
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst140 /inew
     *      / cst141 /depvnu(2),depenth,iddeps(2)
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
     *      / cst143 /mord(m4,m13),msub(m4,m13,m14)
     *      / cst146 /nu(m15,3),y2p(m4,m15),mdep,jdep(m15),idep(m15,3)

      mdep = 0 
      istot = 0
c                               read number of independent sites:
      read (n9,*,err=90) isite

      do i = 1, mst
         isp(i) = 1
         ist(i) = 1
      end do 
c                               read number of species on each site:
      read (n9,*,err=90) (isp(i),ist(i),i=1,isite)
c                               total number of endmembers:
      istot = 1
      do i = 1, isite
         istot = istot*isp(i)
      end do 
c                               read endmember names:
      read (n9,1000) (mname(i), i = 1, istot)
c                               read endmember flags:
      read (n9,*,err=90) (iend(i), i= 1, istot)
c                               read composition limits:
      do i = 1, isite

         read (n9,*,err=90) (xmn(i,j), xmx(i,j), xnc(i,j), 
     *                           j = 1, isp(i) - 1), imd(i)
c                               valid subdivision choice?:
         if (imd(i).gt.10) call error (323,a0(1,1),i,'XRMODE')
      end do 
c                                create bragg-williams indexes
      do i = 2, isite
         ijk(i) = 1
      end do 
 
      ijk(1) = 0  

      do 20 l = 1, istot

         do m = 1, isite

            if (ijk(m).lt.isp(m)) then 

               ijk(m) = ijk(m) + 1
c                                increment only one index per endmember
               do i = 1, isite
                  jmsol(l,i) = ijk(i)
               end do 

               if (ijk(m).eq.isp(m).and.l.lt.istot-isp(1)+1) then
c                                saturated counter, increment first
c                                unsaturated counter and reset all
c                                lower site counters to first index,
                  do j = 2, isite
                     if (ijk(j).lt.isp(j)) then
                        ijk(j) = ijk(j) + 1
                        do k = 2, j-1
                           ijk(k) = 1
                        end do 
                        ijk(1) = 0
                        goto 20 
                     end if
                  end do  
               end if 

               goto 20
            end if
         end do 
20    continue 
c                               read total number of terms, and order
c                               of highest order term, in mixing model
c                               ideal models: iterm = iord = 0.
      read (n9,*,err=90) iterm, iord
      if (iterm.ne.0) then 
         if (iterm.gt.m1) call error (48,wg(1,1),m1,tname)
         if (iord.gt.m2) call error (49,wg(1,1),m2,tname)
c                               read parameters for margules
c                               expression of the excess free
c                               energy of the solution.
         do h = 1, iterm
c                               weigthing by bragg-williams fractions
            read (n9,*,err=90) ((isub(h,i,j), j = 1, 2),i = 1, iord)
            read (n9,*,err=90) (wg(h,k), k = 1, m3)
         end do 
      end if 
c                              expansion for S(configurational)
      read (n9,*,err=90) nsite

      if (nsite.ne.0) call smodel (tname) 
c                              read p-t dependent size parameter for 
c                              each endmember
      if (inew.ne.0) read (n9,*,err=90) idummy

      goto 99

90    call error (20,wg(1,1),i,tname)

1000  format (3(a,1x))

99    end 


      subroutine rmodel (tname)
c---------------------------------------------------------------------
c rmodel - reads solution models from LUN n9.
c---------------------------------------------------------------------
      implicit none
  
      include 'perplex_parameters.h'
 
      character*10 mname*8,tname,fname,names*8,dnames*8

      integer istot,jstot,iend,isub,imd,msub,mord,
     *        ist,i,j,k,l,m,inew,mdep,idep,jdep,jlaar,
     *        iord,iterm,isite,isp,nsub1,nsub2,nttyp,
     *        insp,isoct,jmsol,nterm,nspm1,nsite,
     *        kstot,iddeps,jsmod,kdsol,n1,n2,n3,n4,n5,n6,n7,
     *        n8,n9,io3,io4,io5,io9,idim,idnam

      double precision wg,xmn,xmx,xnc,ecoef,acoef,smult,vlaar,depvnu,
     *                 depenth,a0,y2p,nu,coeffs(m4),
     *                 rnums(100),dqf

      integer ijk(mst),inds(m4),ict,indq,idqf

      common/ cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),ecoef(m4)
     *      ,nsite,nspm1(m10),nterm(m10,m11),nsub1(m10,m11,m0,m12),
     *      nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst8 /names(k1)/ cst221 /vlaar(m3,m4),jsmod
     *      / csta7 /fname(h9)/ cst18a /mname(m4)/ cst79 /isoct
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst140 /inew/ cst141 /depvnu(2),depenth,iddeps(2)
     *      / cst142 /jmsol(m4,mst),kdsol(m4)/ cst149 /idnam
     *      / cst143 /mord(m4,m13),msub(m4,m13,m14)
     *      / cst146 /nu(m15,3),y2p(m4,m15),mdep,jdep(m15),idep(m15,3)
     *      / cst149a /dnames(h8) / cst222 /dqf(m3,m4),indq(m4),idqf

      mdep = 0 
      istot = 0
      ist(1) = 0 

10    read (n9, 1000, end = 99) tname
c                               if blank, assume comment.
      if (tname.eq.' '.or.tname.eq.'begin_mode') goto 10

      if (inew.eq.0) then 
         call readda (rnums,1,tname)
c                               model type flag
         jsmod = idint(rnums(1))   
c                               correct jsmod for old versions    
         if (jsmod.eq.3) jsmod = 2                 
      else
         jsmod = 1
      end if 

      if (jsmod.eq.1) then 
c                               old style solution models
         call xrmode (tname)
c                               look for the end of model marker
c                               added in perplex04  
         if (inew.eq.0) call readop (idim,jlaar,tname)

         goto 99 

      else 
c                               new style:

c                               read number of independent sites:
      call readda (rnums,1,tname)    
      isite = idint(rnums(1)) 
c                               read number of species on each site:
      call readda (rnums,isite,tname)  
      do i = 1, isite
         isp(i) = idint(rnums(i))
      end do 
c                               total number of endmembers:
      istot = 1
      do i = 1, isite
         istot = istot*isp(i)
      end do 
c                               counter for character read routines
      idim = istot
c                               read endmember names:
      call readn (idim,tname)
c                               compound formation models
      if (jsmod.eq.6.or.jsmod.eq.8) then 
c                               get ordering reaction and name
c                               of ordered species:        
         call readr (coeffs,depenth,inds,idim,3,tname)

         do j = 1, 2
            depvnu(j) = coeffs(j+1)
            iddeps(j) = inds(j+1)
         end do 

      end if 
c                               read dependent endmembers
      if (jsmod.eq.7.or.jsmod.eq.8) then
c                               number of dependent endmembers
         call readda (rnums,1,tname)    
         mdep = idint(rnums(1))
         if (mdep.gt.m15) call error (1,xmn(1,1),mdep,'m15')
         do i = 1, mdep

            call readr (coeffs,depenth,inds,idim,4,tname)

            nu(i,1) = 1d0
            jdep(i) = inds(1)
c                               save dependent endmember names for 
c                               build
            do j = 1, idnam
               if (mname(jdep(i)).eq.dnames(j)) goto 110
            end do 

            idnam = idnam + 1
            if (idnam.eq.h8) then 
               call warn (8,depenth,h8,'RMODEL')
               idnam = h8
            else 
               dnames(idnam) = mname(jdep(i))
            end if 

110         do j = 1, 3
               nu(i,j) = coeffs(j+1)
               idep(i,j) = inds(j+1)
            end do 

         end do 
      end if 
c                               read endmember flags:
      call readda (rnums,istot,tname)  
      do i = 1, istot
         iend(i) = idint(rnums(i))
      end do 
c                               read composition limits, subdivision type:
      m = 0
      do i = 1, isite
c                               number of ranges to be read
         m = m + isp(i) - 1
      end do 
c                               get the numbers
      call readda (rnums,3*m+isite,tname) 
c                               assign the numbers:
      l = 1

      do i = 1, isite
         
         do j = 1, isp(i) - 1
            xmn(i,j) = rnums(l)
            xmx(i,j) = rnums(l+1)
            xnc(i,j) = rnums(l+2)
            l = l + 3
         end do 
         
         imd(i) = idint(rnums(l))

         l = l + 1

      end do 
c                                create bragg-williams indexes
      do i = 2, isite
         ijk(i) = 1
      end do 
 
      ijk(1) = 0  

      do 20 l = 1, istot

         do m = 1, isite

            if (ijk(m).lt.isp(m)) then 

               ijk(m) = ijk(m) + 1
c                                increment only one index per endmember
               do i = 1, isite
                  jmsol(l,i) = ijk(i)
               end do 

               if (ijk(m).eq.isp(m).and.l.lt.istot-isp(1)+1) then
c                                saturated counter, increment first
c                                unsaturated counter and reset all
c                                lower site counters to first index,
                  do j = 2, isite
                     if (ijk(j).lt.isp(j)) then
                        ijk(j) = ijk(j) + 1
                        do k = 2, j-1
                           ijk(k) = 1
                        end do 
                        ijk(1) = 0
                        goto 20 
                     end if
                  end do  
               end if 

               goto 20
            end if
         end do 
20    continue 
c                              read excess function
      call readx (idim,tname)
c                              expansion for S(configurational)
      call readda (rnums,1,tname)    
      nsite = idint(rnums(1))

      if (nsite.gt.m10) call error (31,a0(1,1),isite,tname)
c                                     for each site
      do i = 1, nsite 
c                                     read # of species, and site 
c                                     multiplicty.
         call readda (rnums,2,tname)    
         nspm1(i) = idint(rnums(1)) - 1
         smult(i) = rnums(2)
c                                     for each species, read
c                                     function to define the
c                                     site fraction of the species:
         do j = 1, nspm1(i)
c                                     read expression for site
c                                     fraction of species j on 
c                                     site i.
            call readz (coeffs,inds,ict,idim,m4,tname)

            a0(i,j) = coeffs(1)
            nterm(i,j) = ict - 1
            if (nterm(i,j).gt.m0) call error (33,a0(1,1),m0,tname)
c                                     for each term:
            do k = 2, ict
c                                     all terms 1 order type, this
c                                     saved for compatability with 
c                                     old versions:
               nttyp(i,j,k-1)   = 1
               acoef(i,j,k-1)   = coeffs(k)
               nsub1(i,j,k-1,1) = inds(k)
            end do 
         end do 
      end do 
c                              look for van laar and/or dqf parameters
c                              or the end of model marker
      call readop (idim,jlaar,tname)

      if (jlaar.ne.0) then 
         if (jsmod.eq.2) then 
            jsmod = 3
         else if (jsmod.eq.7) then
            write (*,1010) tname
            jlaar = 0 
         else
            write (*,*) 'ASF Model option not implemented'
            pause
            stop
         end if 
      end if 

      goto 99

      end if 

90    call error (20,wg(1,1),i,tname)

1000  format (3(a,1x))
1010  format (/,'WARNING! WARNING! WARNING! WARNING! WARNING!',//,a,
     *       ' is a Van Laar reciprocal solution model and will be',
     *       ' reformulated as',/,'a regular solution model. Use ',
     *       'Perple_X 06 to treat this model correctly.',//,
     *       'WARNING! WARNING! WARNING! WARNING! WARNING!',/)

99    end 

      subroutine smodel (tname)
c-----------------------------------------------------------------
c smodel reads models for configurational entropy
c of solutions with dependent site mixing or disordered
c endmembers, see documentation section 1.3.1, equation (8b)
c-----------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      character*10 tname

      integer istot,jstot,iend,isub,imd,ist,i,j,knsp,
     *        iord,iterm,isite,isp,nsub1,nsub2,nttyp,
     *        insp,nterm,nspm1,nsite,mord,msub,k,l,
     *        kstot,jsmod,n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9

      double precision wg,xmn,xmx,xnc,ecoef,acoef,smult,vlaar,a0

      common/ cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),isp(mst),
     *      isite,iterm,iord,istot,jstot,kstot
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst221 /vlaar(m3,m4),jsmod
     *      / cst143 /mord(m4,m13),msub(m4,m13,m14)
c-----------------------------------------------------------------

      if (nsite.gt.m10) call error (31,a0(1,1),isite,tname)
c
      if (nsite.lt.isite) call error (30,a0(1,1),isite,tname)
c                                     for each site
      do i = 1, nsite 
c                                     read # of species, and site 
c                                     multiplicty.
         read (n9,*,err=90) knsp,smult(i)
         nspm1(i) = knsp - 1
c                                     for each species, read
c                                     function to define the
c                                     site fraction of the species:
         do j = 1, nspm1(i)
c                                     read # of terms in the 
c                                     site fraction function and a0.
            read (n9,*,err=90) nterm(i,j), a0(i,j)
            if (nterm(i,j).gt.m0) call error (33,a0(1,1),m0,tname)
c                                     for each term:
            do k = 1, nterm(i,j)
c                                     read term type:
               read (n9,*,err=90) nttyp(i,j,k)
               if (nttyp(i,j,k).lt.m12) then
                  if (jsmod.eq.1) then 
                     read (n9,*,err=90) acoef(i,j,k),(nsub1(i,j,k,l),
     *                            nsub2(i,j,k,l),l = 1, nttyp(i,j,k))
                  else
                     read (n9,*,err=90) acoef(i,j,k),(nsub1(i,j,k,l),
     *                                  l = 1, nttyp(i,j,k))                     
                  end if 
               else
                  call error (29,a0(1,1),nttyp(i,j,k),tname)
               end if
            end do 

         end do 
      end do 

      return

90    call error (20,a0(1,1),i,tname)

      end 

      subroutine cmodel (im,idsol,tname,ibuild,x1,x2)
c---------------------------------------------------------------------
c cmodel - checks to see if solution models contain valid endmembers
c and, if not, whether they can be recast as simpler models.
c---------------------------------------------------------------------
      implicit none
  
      include 'perplex_parameters.h'
 
      character*10 mname*8,tname,fname,names*8,missin(m4)*8,x1*8,x2*8

      integer imiss,im,idsol,istot,jstot,ibuild,iend,isub,imd,mdep,jdep,
     *        ltyp,lmda,idis,ikp,ist,ivarrx,isudo,ivar,i,j,idep,
     *        imyn,ivarip,iord,iterm,isite,isp,nsub1,nsub2,nttyp,
     *        ipoint,h,insp,isoct,jmsol,nterm,nspm1,nsite,ndep,
     *        kstot,jord,iddeps,jterm,jsmod,kdsol,k

      double precision wg,xmn,xmx,xnc,ecoef,acoef,smult,vlaar,depvnu,
     *                 depenth,a0,y2p,nu
 
      common/ cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *                iend(m4),isub(m1,m2,2),imd(mst),insp(m4),
     *                ist(mst),isp(mst),isite,iterm,iord,
     *                istot,jstot,kstot
     *      / cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst18a /mname(m4)
     *      / cst8 /names(k1)/ cst60 /ipoint,imyn/ cst79 /isoct
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)/ csta7 /fname(h9)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar    
     *      / cst141 /depvnu(2),depenth,iddeps(2)
     *      / cst221 /vlaar(3,m4),jsmod
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
     *      / cst146 /nu(m15,3),y2p(m4,m15),mdep,jdep(m15),idep(m15,3)

      jstot = 0
c                              if called by build (ibuild = 1) skip the
c                              name check:
      if (ibuild.eq.1) goto 60
c                              check that solution name matches a 
c                              name requested in input from n1
      do i = 1, isoct

         if (tname.eq.fname(i)) then 
c                              got a match, goto 60:
            idsol = i 
            im = im + 1
            goto 60 
         end if 
c
      end do 
c                              didn't find a match, read a new name:
      goto 99 
c                              check that the endmembers match with data
c                              from n2:   
60    do 70 i = 1, istot

         kdsol(i) = 0

         if (jsmod.eq.7.or.jsmod.eq.8) then
c                              reciprocal solution, check if endmember i
c                              is a dependent endmember, if it is flag
c                              it by setting kdsol(i) = -2
            do j = 1, mdep
               if (jdep(j).eq.i) then
                  kdsol(i) = -2
                  goto 70
               end if 
            end do 
         end if 

         do h = 1, ipoint
c                               in build, check for forbidden choices:
            if (ibuild.eq.1) then
               if (mname(i).eq.x1.or.mname(i).eq.x2) then
                  write (*,1010) tname, x1, x2
                  jstot = 0
                  goto 99
               end if
            end if

            if (names(h).eq.mname(i)) then
c                                  got a valid endmember, count and                     
               jstot = jstot + 1 
               kstot = jstot
c                                  create arrays of convenience, where j = 1, jstot
               kdsol(i) = h 
c                                  insp(j)   - points to the original
c                                              index of the endmember
c                                              in the model.
               insp(jstot) = i                                   
c                                  set kill flag: 
               if (iend(i).ne.0.and.ikp(h).eq.0) then
c                                  the endmember should be killed
                  ikp(h) = -1
               else if (iend(i).eq.0.and.ikp(h).eq.-1) then
c                                  the endmember was killed by a previous
c                                  model, but should be unkilled here
                  ikp(h) = im
               end if 
c                                found all endmembers:
               if (jstot.eq.istot) goto 99

               goto 70
            end if 
         end do
70    continue
c                                check for dependent endmembers, necessary?
      if (jsmod.eq.7.or.jsmod.eq.8) then

         ndep = 0  

         do 100 i = 1, mdep
            do j = 1, 3
               if (idep(i,j).le.istot) then
c                                looking for a dependent endmember component: 
                  if (kdsol(idep(i,j)).eq.0) then
c                                a component is missing, reset kdsol to 0
                     kdsol(jdep(i)) = 0
                     goto 100
                  end if 
               else
c                                looking for "the" ordered species
                  do k = 1, 2
                     if (kdsol(iddeps(k)).eq.0) goto 100
                  end do 

               end if 
            end do 
c                                dependent endmember is ok
            ndep = ndep + 1
            jdep(ndep) = jdep(i)

            do j = 1, 3
               idep(ndep,j) = idep(i,j)
               nu(ndep,j) = nu(i,j)
            end do 

            jstot = jstot + 1
c                                necessary? could cause ordering problems
            insp(jstot) = jdep(i)

100      continue 

         mdep = ndep

         if (mdep.eq.0) then
            if (jsmod.eq.7) then
               jsmod = 2
            else 
               jsmod = 6
            end if 
         end if  
      end if 
c                                  reformulate ordering models  
c                                  this consists of creating
c                                  a pointer to the index of the original
c                                  endmember (used in the internal solution
c                                  model) from the index of the endmember
c                                  fractions stored for the pseudocompounds
      if (jsmod.eq.6.or.jsmod.eq.8) then 
c                                  first check if the ordered endmember
c                                  may be stable       
         do j = 1, 2
            if (kdsol(iddeps(j)).eq.0) then 
               kdsol(istot+i) = 0
c                                  reset jsmod to indicate a model without
c                                  speciation
               if (jsmod.eq.8) then
                  jsmod = 7
               else 
                  jsmod = 2
               end if 

               goto 80 

            end if 
         end do 

         kdsol(istot+1) = -1

      end if 

80    if (jsmod.eq.7.or.jsmod.eq.8) then 
c                                this saves time later
c                                make y2p array
         h = kstot

         if (jsmod.eq.8) then 
            insp(jstot+1) = istot + 1          
            h = jstot + 1
         end if   

         do i = 1, h
            do j = 1, mdep

               y2p(i,j) = 0d0

               do k = 1, 3
                  if (idep(j,k).eq.insp(i)) y2p(i,j) = nu(j,k)
               end do 

            end do
         end do
      end if 
c                                done if nothing is missing:
      if (jstot.eq.istot) goto 99
c                                missing endmember warnings:
      if (jstot.lt.2) then
         im = im - 1
         call warn (25,wg(1,1),jstot,tname)
         jstot = 0 
      else 
         imiss = 0 
         do i = 1, istot
            if (kdsol(i).eq.0) then
               imiss = imiss + 1
               missin(imiss) = mname(i)
            end if 
         end do 
         write (*,1000) tname,(missin(i), i = 1, imiss)
      end if 
c                                  now check for terms that contain ivalid indexes
      if (jsmod.ne.1) then 

         jterm = 0
         jord = 0  

         do 90 i = 1, iterm
            do j = 1, iord
               if (isub(i,j,1).eq.0.or.kdsol(isub(i,j,1)).eq.0) goto 90
            end do 
c                                  if here the term contains no missing
c                                  endmembers
            jterm = jterm + 1

            do j = 1, iord
               isub(jterm,j,1) = isub(i,j,1)
            end do

            do j = 1, m3
               wg(jterm,j) = wg(i,j)
            end do 

90       continue

         iterm = jterm

      end if 

1000  format (/,'**warning ver114** the following endmembers',
     *          ' are missing for ',a,/,4(8(2x,a),/),/)
1010  format (/,'**warning ver113** ',a,' is not a valid model',
     *        ' because component ',a,' or ',a,/,
     *        ' is constrained or missing.',/)
99    end


      subroutine pmodel (im,jm)
c---------------------------------------------------------------------
c pmodel - stores solution models paramters in global arrays
c---------------------------------------------------------------------
      implicit none
  
      include 'perplex_parameters.h'

      integer im,istot,jstot,iend,isub,imd,mdep,jdep,
     *        ist,i,j,idep,mord,ind,im2jm,
     *        iord,iterm,isite,isp,nsub1,nsub2,nttyp,
     *        insp,jmsol,nterm,nspm1,nsite,ndep,
     *        kstot,iddeps,jterm,jsmod,kdsol,k,jsub,l,knsp,
     *        mstot,ideps,msite,lterm,ksp,itic,inid,
     *        hsub,ii,msub,jm

      double precision wg,xmn,xmx,xnc,ecoef,acoef,smult,vlaar,depvnu,
     *                 depenth,a0,y2p,nu,wgl,r,vlar,deph,dydy,y2pg,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu

      common/ cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),ecoef(m4)
     *      ,nsite,nspm1(m10),nterm(m10,m11),nsub1(m10,m11,m0,m12),
     *      nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst221 /vlaar(m3,m4),jsmod
     *      / cst141 /depvnu(2),depenth,iddeps(2)
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
     *      / cst143 /mord(m4,m13),msub(m4,m13,m14)
     *      / cst146 /nu(m15,3),y2p(m4,m15),mdep,jdep(m15),idep(m15,3)
c                                global arrays:
     *      / cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)

c                                make pointer from im (solution index) to
c                                storage index
      jm = jm + 1
c
      im2jm(im) = jm
c                                save the excess terms.                              
      jterm(jm) = iterm

      do i = 1, iterm 
         do j = 1, m3
            wgl(jm,i,j) = wg(i,j)
         end do 
c                                      high order terms not allowed
         if (iord.gt.2.and.jsmod.eq.3) call error (999,r,800,'PMODEL') 
         do j = 1, iord
            jsub(jm,i,j) = isub(i,j,1)
         end do 
      end do 

      do i = 1, jstot
c                               if the solution is van laar save
c                               the "volume" function.
         if (jsmod.eq.3) then  
            do l = 1, m3
               vlar(jm,i,l) = vlaar(l,insp(i))
            end do 
         end if 
c                                insp points to the original position 
c                                of endmember i in the solution model
c                                input:
         knsp(jm,i) = insp(i)
      end do   
c                                saved data with all endmembers regardless
c                                of whether they are present in the calculation:
      if (jsmod.eq.6.or.jsmod.eq.8) then 

         deph(jm) = depenth

         do i = 1, 2
            ideps(jm,i) = iddeps(i)
         end do 
c                                first create derivatives of endmember
c                                fractions with respect to the ordered
c                                species:
         do i = 1, istot
            dydy(jm,i) = 0d0
         end do
 
         dydy(jm,istot+1) = 1d0

         do i = 1, 2
            dydy(jm,iddeps(i)) = -depvnu(i)
         end do

         if (jsmod.eq.8) then 
c                                pointer to ordered cpd
            knsp(jm,jstot+1) = insp(jstot+1)
c                                save y2p array 
            ndep(jm) = mdep
            do i = 1, kstot 
               do j = 1, ndep(jm)
                  y2pg(jm,i,j) = y2p(i,j)
               end do
            end do
c                                make an array to get the the fractions
c                                of the disordered species in the ordered
c                                species indexed by kstot counter rather
c                                than the istot counter
            do i = 1, kstot
               dvnu(jm,i) = 0d0

               do j = 1, 2
                  if (insp(i).ne.iddeps(j)) cycle
                  dvnu(jm,i) = depvnu(j)
               end do 
            end do  

         end if

         mstot(jm) = istot
c                                end-member fractions as a function of bulk
c                                y's and dependent species y:
         do i = 1, istot
            inid(jm,i) = kdsol(i)
         end do 
c                                site fractions as a function of bulk
c                                y's and dependent species y:
         msite(jm) = nsite
         do i = 1, nsite 
c                                     # of species, and site 
c                                     multiplicty.
            qmult(jm,i) = smult(i)
            ksp(jm,i) = nspm1(i) 
c                                     for each species, read
c                                     function to define the
c                                     site fraction of the species:
            do j = 1, nspm1(i)
c                                     # of terms in the 
c                                     site fraction function and a0.
               lterm(jm,i,j) = nterm(i,j)
               d0(jm,i,j) = a0(i,j)
               sdzdy(jm,i,j) = 0d0
c                                     for each term:
               do k = 1, nterm(i,j)
c                                     term type and coefficient:
                  dcoef(jm,i,j,k) = acoef(i,j,k)

                  itic = 0 

c                                     endmember indexes  
                  ind = nsub1(i,j,k,1) 
                  hsub(jm,i,j,k) = ind
c                                     get derivatives, this
c                                     may not always work:
                  do ii = 1, 2
                     if (ind.eq.iddeps(ii)) then
                        sdzdy(jm,i,j) = sdzdy(jm,i,j) 
     *                                + dydy(jm,iddeps(ii))*acoef(i,j,k)  
                        itic = itic + 1
c                                      high order terms not allowed
                        if (itic.gt.1) call error (999,r,801,'PMODEL') 
                     end if                       
                  end do

                  if (ind.eq.istot+1) 
     *               sdzdy(jm,i,j) = sdzdy(jm,i,j) + acoef(i,j,k)  
                
               end do 
            end do 
         end do 
      end if 

      end 

      subroutine specis (dg,id,jd)
c----------------------------------------------------------------------
c subroutine to speciation of a solution with one ordering parameter
c and composition x. id identifies the solution, jd the ordered species.
c dg is the change in G for the stable speciation relative to a mechanical
c mixture of the endmembers. this version uses endmember fractions, see
c specip for the equivalent problem in terms of endmember proportions.
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i,jterm,knsp,ndep,jsub,im2jm,
     *        mstot,ideps,msite,lterm,ksp,itic,inid,
     *        hsub,jd,id

      double precision wgl,r,vlar,deph,dydy,y2pg,y,pa,p0a,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 dg,v,tr,pr,ps,ymax,yt,odgdy,
     *                 dgdy,sdgdy,sign,sconfg,dy,tol

      common/ cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /v(l2),tr,pr,r,ps
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)
     *      / cst148 /p0a(m4),pa(m4),y(m4)

      save tol

      data tol/1d-3/
c                                       first find the maximum and 
c                                       minimum values of the ordered
c                                       species:
      ymax = 1d0
c                                       these could be set earlier,
c                                       only 2 of these fractions 
c                                       can change
      do i = 1, jd-1
         y(i) = x(i)
      end do

      do i = 1, 2
         yt = -x(ideps(id,i))/dydy(id,ideps(id,i))
         if (yt.lt.ymax) ymax = yt
      end do 
c                                       no ordering possible if yt = 0
      if (ymax.lt.tol) then 
         y(jd) = 0d0
         goto 90
      end if 

      ymax = ymax - tol/1d3

      itic = 0
c                                       solve by halving

c                                       set fractions
      y(jd) = ymax 
      do i = 1, 2
         y(ideps(id,i)) = x(ideps(id,i)) + dydy(id,ideps(id,i))*y(jd)
      end do 

      odgdy = dgdy(id)
c                                       if derivative of
c                                       g is positive must
c                                       be fully ordered
      if (odgdy.lt.0) goto 90

      dy = 0.5d0*ymax 
      y(jd) = y(jd) - dy 

10    y(ideps(id,1)) = x(ideps(id,1)) + dydy(id,ideps(id,1))*y(jd)
      y(ideps(id,2)) = x(ideps(id,2)) + dydy(id,ideps(id,2))*y(jd)

      itic = itic + 1

      sdgdy = dgdy(id)

      sign = odgdy*sdgdy

      if (sign.gt.0d0) then 
         odgdy = sdgdy
         if (y(jd).eq.tol) goto 90
         y(jd) = y(jd) - dy
         if (y(jd).lt.tol) y(jd) = tol
         goto 10 
      else if (dy.gt.tol) then 
         dy = dy / 2d0
         y(jd) = y(jd) + dy
         goto 10
      end if 

90    dg = y(jd)*deph(id)
c                                     get dexcess energy
      do i = 1, jterm(id)
         dg = dg + w(i)*y(jsub(id,i,1))*y(jsub(id,i,2))
      end do
c                                     now sconf contribution
      dg =  dg - v(2)*sconfg(id)
     
      end 


      function dgdy (id)
c----------------------------------------------------------------------
c subroutine to compute dg/dy for a solution with one ordering parameter
c and speciation x. 
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i,jterm,knsp,i1,i2,im2jm,
     *        mstot,ideps,msite,lterm,ksp,inid,
     *        hsub,id,ndep,jsub

      double precision wgl,r,vlar,deph,dydy,y2pg,dgex,dsconf,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 dgdy,v,tr,pr,ps,y,pa,p0a

      common/ cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /v(l2),tr,pr,r,ps/ cst148 /p0a(m4),pa(m4),y(m4)
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)
c                                     get derivative of excess function
      dgex = 0d0

      do i = 1, jterm(id)
         i1 = jsub(id,i,1)
         i2 = jsub(id,i,2)
         dgex = dgex + w(i)*(y(i1)*dydy(id,i2)+y(i2)*dydy(id,i1))
      end do
c                                     now get dg/dy(jd)
      dgdy = deph(id) + dgex - v(2)*dsconf(id)

      end                          


      function dsconf (id)
c----------------------------------------------------------------------
c subroutine to the derivative of the configurational entropy of a 
c solution with respect to the proportion of a dependent species.
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i,jterm,knsp,j,k,mstot,ideps,msite,lterm,ksp,inid,
     *        hsub,id,jsub

      double precision wgl,r,vlar,deph,dydy,dsconf,dlnz,pa,p0a,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 v,tr,pr,ps,y,dscon,zt,dzdyt,dzdy,z

      common/ cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /v(l2),tr,pr,r,ps/ cst148 /p0a(m4),pa(m4),y(m4)

      dscon = 0d0
c                                     for each site
      do i = 1, msite(id)

         zt = 0d0
         dzdyt = 0d0
c                                     get site fractions
         do j = 1, ksp(id,i)

            z = d0(id,i,j)
c                                     for each term:
            do k = 1, lterm(id,i,j)
               z = z + dcoef(id,i,j,k) * y(hsub(id,i,j,k))
            end do 

            zt = zt + z

            dzdy = sdzdy(id,i,j)

            if (dzdy.ne.0d0) then
               dzdyt = dzdyt + dzdy
               dlnz = 1d0
               if (z.gt.0d0) dlnz = 1d0 + dlog(z)
               dscon = dscon - qmult(id,i) * dzdy * dlnz
            end if 

         end do 

         z = 1d0 - zt 
         dzdy = -dzdyt
         dlnz = 1d0
         if (z.gt.0d0) dlnz = 1d0 + dlog(z)
         dscon = dscon - qmult(id,i) * dzdy * dlnz

      end do 

      dsconf = dscon*r

      end

      subroutine gphase (id,gph)
c-----------------------------------------------------------------------
c gphase computes the gibbs free energy of a compound identified by index
c id, it is assumed that the variables in cst44 are set prior to the call.
c gphase does not assume that the properties of a pseudocompound endmembers
c are known (unlike gall) it is thus less efficient than gall.
c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer jterm,knsp,ipoint,j,imyn,im2jm,jm,
     *        mstot,ideps,msite,lterm,ksp,inid,ifp,jend,k,ixp,jd,
     *        hsub,id,ndep,jsub,ixpt,l,
     *        ids,ikp,ivarrx,ivarip,isudo,ivar

      double precision wgl,r,vlar,deph,dydy,y2pg,y,pa,p0a,gph,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 tr,pr,ps,sxs,gzero,p0ord,
     *                 dg,exces,p,t,xco2,u1,u2

      common/ cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
      common/ cst60  /ipoint,imyn/ cst148 /p0a(m4),pa(m4),y(m4)
     *      / cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst5   /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)

      if (id.le.ipoint) then
c                                 phase is an endmember compound
         call gcpd (id,gph)

      else if (ifp(id).eq.6) then
c                                 speciation model with 1 order parameter

         ids = ikp(id)
         jm = im2jm(ids)
         jd = mstot(jm) + 1

         do j = 1, mstot(jm)
            x(j) = 0d0
         end do 
c                                 initialize gph and get any dqf corrections
         call gexces (id,gph)
c                                 get gmech
         do k = 1, jend(ikp(id),1) 
            call gcpd (jend(ikp(id),2+k),dg)
            x(knsp(jm,k)) = sxs(ixp(id)+k)
            gph = gph + dg*sxs(ixp(id)+k)
         end do 
c                                 compute margules coefficients
         do j = 1, jterm(jm)
            w(j) = wgl(jm,j,1) + t * wgl(jm,j,2) + p * wgl(jm,j,3)
         end do 

c                                 get the speciation energy effect
         call specis (dg,jm,jd)
c                                 subtract endmember sconfg
c                                 modified march 7, 2006. JADC.
c        gph = gph + dg + t * exces(2,id)
         gph = gph + dg 


      else if (ifp(id).eq.8) then 
c                                 reciprocal solution speciation model 
c                                 with 1 order parameter
         ids = ikp(id)
         jm = im2jm(ids)
         jd = mstot(jm) + 1
c                                 initialize x's and p0's
         do j = 1, mstot(jm)
            x(j) = 0d0
            p0a(j) = 0d0
         end do 
c                                 load x's from sxs array.                              
         do j = 1, jend(ids,1) 
            x(knsp(jm,j)) = sxs(ixp(id)+j)
         end do 
c                                 convert y's to p's, this loop 
c                                 could be eliminated by saving the
c                                 p's.
         ixpt = ixp(id) + jend(ids,1) - ndep(jm)
         p0ord = 0d0

         do k = 1, jend(ids,1) - ndep(jm)

            p0a(knsp(jm,k)) =  sxs(ixp(id)+k)

            do l = 1, ndep(jm)
               p0a(knsp(jm,k)) =  p0a(knsp(jm,k)) 
     *                           + y2pg(jm,k,l) * sxs(ixpt+l)
            end do 

            p0ord = p0ord + p0a(knsp(jm,k))

         end do 
c                                 the initial fraction of the ordered species
         p0ord = 1d0 - p0ord
         p0a(knsp(jm,jend(ids,1)+1)) = p0ord
c                                 compute margules coefficients
         do j = 1, jterm(jm)
            w(j) = wgl(jm,j,1) + t * wgl(jm,j,2) + p * wgl(jm,j,3)
         end do 
c                                 get the speciation energy effect
         call specip (gph,jm,jd)
c                                 get endmember sconfg and dqf's
         call gexces (id,dg) 

         gph = gph + dg
c                                 get gmech
         do k = 1, jend(ids,1) - ndep(jm)
            call gcpd (jend(ids,2+k),dg)
            gph = gph + dg * (p0a(knsp(jm,k)) + dvnu(jm,k)*p0ord)
         end do 

      else 
c                              a pseudocompound without speciation:
         if (ifp(id).ne.1) then
c                              get the excess and/or ideal mixing effect
c                              and/or dqf corrections:
            if (ifp(id).eq.23) then 
               call toop(id,gph)
            else if (ifp(id).eq.26) then 
               call hcneos (gph,sxs(ixp(id)+1),
     *                      sxs(ixp(id)+2),sxs(ixp(id)+3))
            else
               call gexces (id,gph)
            end if 
c                              compute mech mix G for 
c                              all models except fluid 
            do k = 1, jend(ikp(id),1) 
               call gcpd (jend(ikp(id),2+k),dg)
               gph = gph + dg*sxs(ixp(id)+k)
            end do 

         else 
c                              get the excess and/or ideal mixing effect
c                              and/or dqf corrections:
            call fexces (id,gph)
c                              excess props don't include vdp:
            do k = 1, jend(ikp(id),1) 
               gph = gph + gzero(jend(ikp(id),2+k))*sxs(ixp(id)+k)
            end do 
         end if 
c                              for van laar get fancier excess function         
         if (ifp(id).eq.3) call gvlaar (ikp(id),id,gph)
         
      end if 

      end

      subroutine specip (dg,id,jd)
c----------------------------------------------------------------------
c subroutine to speciation of a solution with one ordering parameter
c and composition x. id identifies the solution, jd the ordered species.
c dg is the change in G for the stable speciation relative to a mechanical
c mixture of the endmembers. this version uses endmember proportions, see
c specis for the equivalent problem in terms of endmember fractions.
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i,jterm,knsp,ndep,jsub,hsub,jd,id,
     *        mstot,ideps,msite,lterm,ksp,itic,inid,im2jm

      double precision wgl,r,vlar,deph,dydy,y2pg,pa,p0a,y,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 dg,v,tr,pr,ps,ymax,yt,odgdy,dgdyp,
     *                 sdgdy,sign,sconfg,dy,tol,p0ord

      common/ cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /v(l2),tr,pr,r,ps/ cst148 /p0a(m4),pa(m4),y(m4)
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9) 

      save tol

      data tol/1d-3/
c                                       first find the maximum and 
c                                       minimum values of the ordered
c                                       species:
      ymax = 1d0
      p0ord = p0a(jd)
c                                       these could be set earlier,
c                                       only 2 of these fractions 
c                                       can change
      do i = 1, jd-1
         y(i) = x(i)
         pa(i) = p0a(i)
      end do
c                                       get stoichiometric limit on 
c                                       fraction of the ordered species
      do i = 1, 2
         yt = -x(ideps(id,i))/dydy(id,ideps(id,i))
         if (yt.lt.ymax) ymax = yt
      end do 
c                                       no ordering possible if yt = 0
      if (ymax.lt.tol) then 
         y(jd) = 0d0
         pa(jd) = p0ord
         goto 90
      end if 

      ymax = ymax - tol/1d3

      itic = 0
c                                       solve by halving

c                                       set fractions and proportions, here
c                                       it is assumed that the ordered species
c                                       is written only as a combination of the
c                                       idependent endmembers
      y(jd) = ymax 
      pa(jd) = p0ord + ymax

      do i = 1, 2
         y(ideps(id,i)) = x(ideps(id,i)) + dydy(id,ideps(id,i))*y(jd)
         pa(ideps(id,i)) = p0a(ideps(id,i)) + dydy(id,ideps(id,i))*y(jd)
      end do 

      odgdy = dgdyp(id)
c                                       if derivative of
c                                       g is positive must
c                                       be fully ordered
      if (odgdy.lt.0) goto 90

      dy = 0.5d0*ymax 
      y(jd) = y(jd) - dy 
      pa(jd) = p0ord + y(jd)

10    do i = 1, 2
         y(ideps(id,i)) = x(ideps(id,i)) + dydy(id,ideps(id,i))*y(jd)
         pa(ideps(id,i)) = p0a(ideps(id,i)) + dydy(id,ideps(id,i))*y(jd)
      end do 

      itic = itic + 1

      sdgdy = dgdyp(id)

      sign = odgdy*sdgdy

      if (sign.gt.0d0) then 
         odgdy = sdgdy
         if (y(jd).eq.tol) goto 90
         y(jd) = y(jd) - dy
         pa(jd) = p0ord + y(jd)

         if (y(jd).lt.tol) then
            y(jd) = tol
            pa(jd) = p0a(jd) + tol
         end if 

         goto 10 
      else if (dy.gt.tol) then 
         dy = dy / 2d0
         y(jd) = y(jd) + dy
         pa(jd) = p0ord + y(jd)
         goto 10
      end if 

90    dg = pa(jd)*deph(id)
c                                     get dexcess energy
      do i = 1, jterm(id)
         dg = dg + w(i)*pa(jsub(id,i,1))*pa(jsub(id,i,2))
      end do
c                                     now sconf contribution
      dg =  dg - v(2)*sconfg(id)
     
      end 


      function dgdyp (id)
c----------------------------------------------------------------------
c subroutine to compute dg/dy for a solution with one ordering parameter
c and speciation x where G(p), see dgdy for G(z)
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i,jterm,knsp,i1,i2,mstot,ideps,msite,lterm,ksp,inid,
     *        hsub,id,jsub

      double precision wgl,r,vlar,deph,dydy,dgex,dsconf,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 dgdyp,v,tr,pr,ps,y,pa,p0a

      common/ cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /v(l2),tr,pr,r,ps/ cst148 /p0a(m4),pa(m4),y(m4)

c                                     get derivative of excess function
      dgex = 0d0

      do i = 1, jterm(id)
         i1 = jsub(id,i,1)
         i2 = jsub(id,i,2)
         dgex = dgex + w(i)*( pa(i1)*dydy(id,i2)
     *                      + pa(i2)*dydy(id,i1) )
      end do
c                                     now get dg/dy(jd)
      dgdyp = deph(id) + dgex - v(2)*dsconf(id)

      end

      function sconfg (id)
c----------------------------------------------------------------------
c subroutine to evaluate configurational entropy of a solution
c with composition y. 
c----------------------------------------------------------------------

      implicit none 

      include 'perplex_parameters.h'


      integer i,jterm,knsp,mstot,ideps,msite,lterm,ksp,inid,
     *        hsub,id,jsub,j,k

      double precision wgl,r,vlar,deph,dydy,z,pa,p0a,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 v,tr,pr,ps,y,sconfg,dlnw,zt

      common/ cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst5   /v(l2),tr,pr,r,ps
     *      / cst148 /p0a(m4),pa(m4),y(m4)

      dlnw = 0d0
c                                     for each site
      do i = 1, msite(id)

         zt = 0d0
c                                     get site fractions
         do j = 1, ksp(id,i)
            z = d0(id,i,j)
c                                     for each term:
            do k = 1, lterm(id,i,j)
               z = z + dcoef(id,i,j,k) * y(hsub(id,i,j,k))
            end do 

            zt = zt + z
            if (z.gt.0d0) dlnw = dlnw - qmult(id,i) * z * dlog (z)

         end do 
c                                     the last species is by difference:
         z = 1d0 - zt 
         if (z.gt.0d0) dlnw = dlnw - qmult(id,i) * z * dlog(z)

      end do 

      sconfg = dlnw*r

      end


      subroutine hpmelt (y,dlnw)
c----------------------------------------------------------------------
c subroutine to evaluate the configurational entropy of Holland & Powell's
c haplogranite melt model, dlnw is S/R.


c this routine expects:

c                         y(2) = y(fo) 
c                         y(3) = y(fa) 
c                         y(1) = y(H2O) 
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i

      double precision y(m4), dlnw, ytot

      dlnw = 0d0
c                                    the basic entropy
      if (y(1).ne.1d0) then 

         do i = 1, 8

            if (y(i).eq.0d0.or.i.eq.1) cycle
            dlnw = dlnw - y(i) * dlog(y(i)*(1d0-y(1)))
 
         end do 

      end if 
c                                    the quadratic water term
      if (y(1).ne.0d0) dlnw = dlnw - 2d0 * y(1) * dlog(y(1)) 
c                                    the fe-mg fudge factor
      ytot = y(2) + y(3)

      if (ytot.ne.0d0) then 

         do i = 2, 3 

            if (y(i).eq.0d0) cycle
            dlnw = dlnw - 4d0 * y(i) * dlog(y(i)/ytot)

         end do 
      end if 

      end 


      subroutine gmelt (y,dlnw)
c----------------------------------------------------------------------
c subroutine to evaluate the configurational entropy of Ghiorso's
c pMELTS/MELTS model, dlnw is S/R.


c this routine expects:

c                         y(1) = y(H2O) 
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      integer i

      double precision y(m4), dlnw

      dlnw = 0d0
c                                    the basic entropy
      do i = 1, 8
         if (y(i).eq.0d0) cycle
         dlnw = dlnw - y(i) * dlog(y(i))
      end do 
c                                    i wouldn't expect to count
c                                    water here yet again, but that
c                                    is what eqs 1 & 2 of Ghiorso et al. (2002)
c                                    imply

c                                    the quadratic water term
      if (y(1).ne.0d0) dlnw = dlnw - 2d0 * 
     *   (y(1) * dlog(y(1)) + (1d0-y(1))*dlog(1d0-y(1)))

      end 


      subroutine omega1 (x,dlnw,tname)
c----------------------------------------------------------------------
c subroutine to evaluate the log of the number of site configurations
c per mole of a solution (dlnw) with composition x. end-member proportion 
c version.
c----------------------------------------------------------------------

      implicit none

      include 'perplex_parameters.h'

      character*10 tname

      double precision x(m4),z,zt,dlnw

      integer i,j,k,nsub1,nsub2,nttyp,nterm,nspm1,nsite

      double precision ecoef,acoef,smult,a0

      common/ cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)

      dlnw = 0d0
c                                     for each site
      do i = 1, nsite
         zt = 0d0
c                                     get site fractions
         do j = 1, nspm1(i)
            z = a0(i,j)
            if (nterm(i,j).ne.0) then 
c                                     for each term:
               do k = 1, nterm(i,j)
                  z = z + acoef(i,j,k) * x(nsub1(i,j,k,1))
               end do 
            end if 

            call zchk (z,tname)
            zt = zt + z
            if (z.gt.0d0.and.z.lt.1d0) then 
               dlnw = dlnw - smult(i) * z * dlog (z)
            end if 
            
         end do 
         z = 1d0 - zt
         call zchk (z,tname)
         if (z.gt.0d0.and.z.lt.1d0) dlnw = dlnw - smult(i) * z * dlog(z)
      end do 

      end

      double precision function gsixtr (id)
c-----------------------------------------------------------------------
c gsixtr computes G from the EoS formulated by Sixtrude & Bukowski '90, 
c JGR 95:19311-19325.

c Sixtrude parameters:

c    F0    n  -v(J/bar) K0(bar) K0'   theta0 gamma0 q etaS0 Smag

c Perple_X parameters (1st index in array thermo):

c    1     2     3        4     5      6      7    8    9    10

c and for shear modulii

c    G0(bar)   G0'

c are in the Perple_X array emod(i,id) with i

c      1        2
c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer id, itic, kmod, iemod, izap, izap1

      character*8 names 

      double precision thermo, uf, us, p, t, xco2, um, r, turd2, a0,
     *                 tr, pr, ps, v0, v, df, f, dv, k0p, a, ethv,
     *                 gamma0, k00, plg, c1, c2, c3, f1, gvq, dt,
     *                 n, q, vq, v23, theta0, smu, emod, fac, tol

      double precision nr9, qm1, d2f, tht, tht0, etht, etht0, ltht, df1,
     *                 ltht0, dtht, dtht0, d2tht, d2tht0, g, g0, dg, 
     *                 dg0, d2g, d2g0, dfc, d2fc, dft, d2ft, dft0, d2ft0
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)/ cst323 /smu
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps/ cst8   /names(k1)
     *      / cst319 /emod(k15,k10),iemod(k1),kmod

      save turd2, izap, izap1 

      data turd2, izap, izap1 /0.6666666666666666667, 0, 0/

c                                       assign local variables:
      a0     =  thermo(1,id)
      n      =  thermo(2,id)
      v0     = -thermo(3,id)
      k00    =  thermo(4,id)
      k0p    =  thermo(5,id)
      theta0 =  thermo(6,id)
      gamma0 =  thermo(7,id)
      q      =  thermo(8,id)

      nr9 = 9d0*n*r
      c1 = 9d0*k00*v0
      c2 = k0p/2d0-2d0
      c3 = 3d0*c1*c2
      qm1 = q - 1d0
      dt = t - tr
c                                       initial guess for volume:
c                                       uses a 4 order expansion on
c                                       the polylog terms before 
c                                       differentiation and a second
c                                       order series on the resulting
c                                       derivative.
      v = v0*(3d0*((2d0 - q)*dt*gamma0*nr9 - 3d0*p*v0) + c1)/
     *       (c1 - qm1*dt*gamma0*nr9)
      if (v.lt.0d0) v = v0

      f1 = 1d9
      fac = 1d0
      itic = 0 
c                                       change to use relative tolerance
c                                       JADC March 1, 2005. formerly 1d-1 bar.
      tol = 1d-6*p

      do while (dabs(f1).gt.tol)

         itic = itic + 1

         vq = (v/v0)**q
         gvq = gamma0*vq
         v23 = (v0/v)**turd2
         f = 0.5d0*v23 - 0.5d0
         df = -v23/v/3d0
         d2f = 5d0/9d0*v23/v**2

         tht  =  theta0*dexp(-gamma0*(vq-1d0)/q)/t
         tht0 =  tht*t/tr
         etht  = dexp(-tht )
         if (1d0-etht.le.0) goto 90 
         etht0 = dexp(-tht0)
         ltht  = dlog(1d0 - etht )
         ltht0 = dlog(1d0 - etht0)
c                                      diff(theta/T,v)
         dtht  = -gvq/v*tht
         dtht0 = -gvq/v*tht0
c                                      diff(theta/T,v,v)
         d2tht  = gvq*tht /v**2*(gvq - qm1)
         d2tht0 = gvq*tht0/v**2*(gvq - qm1)

         g   = plg(tht )
         g0  = plg(tht0)
         dg  = tht **2*ltht *dtht
         dg0 = tht0**2*ltht0*dtht0
         d2g  = ((2d0*ltht  + tht *etht /(1d0-etht ))*dtht **2 + 
     *          tht *ltht *d2tht )*tht
         d2g0 = ((2d0*ltht0 + tht0*etht0/(1d0-etht0))*dtht0**2 + 
     *          tht0*ltht0*d2tht0)*tht0

         dfc = (c3*f+c1)*f*df
         d2fc = (2d0*c3*f+c1)*df**2+(c3*f+c1)*f*d2f

         dft  = nr9*t /tht **3*(dg  -3d0/tht *g *dtht )
         dft0 = nr9*tr/tht0**3*(dg0 -3d0/tht0*g0*dtht0)

         d2ft =  nr9*t /tht **3*(3d0/tht *(dtht *(4d0/tht *g *dtht  
     *                        - 2d0*dg ) - g *d2tht ) + d2g )
         d2ft0 = nr9*tr/tht0**3*(3d0/tht0*(dtht0*(4d0/tht0*g0*dtht0 
     *                        - 2d0*dg0) - g0*d2tht0) + d2g0)

         f1  = -dfc - dft + dft0 - p

         df1 = -d2fc - d2ft + d2ft0 

         dv = f1/df1
         v = v - dv

         if (v.le.0d0.or.itic.gt.40.or.dabs(f1).gt.1d40) goto 90
 
      end do
c                                 if everything is ok, now get 
c                                 helmoltz energy:
      goto 10 
c                                 if we get here, failed to converge
90    if (izap.lt.100) then
         write (*,1000) t,p,names(id)
      else if (izap.eq.100) then 
         write (*,1010)
      end if 

      izap = izap + 1
c                                 use series expansion on failure
c      v = v0*(3d0*((2d0 - q)*dt*gamma0*nr9 - 3d0*p*v0) + c1)/
c     *       (c1 - qm1*dt*gamma0*nr9)
c      if (v.lt.0d0) v = v0
c                                 destabilize the phase:
      gsixtr = 1d30
      goto 99 
         
10    vq = (v/v0)**q
      f = 0.5d0*(v0/v)**turd2 - 0.5d0
      tht  =  theta0*dexp(-gamma0*(vq-1d0)/q)/t
      tht0 =  tht*t/tr 

      a = a0 + c1*f**2*(0.5d0 + c2*f) 
     *  + nr9*(t/tht**3*plg(tht ) -tr/tht0**3*plg(tht0))

      gsixtr = a + p*v - t*thermo(10,id)
c                                 thermal energy/v
      ethv = (dft0-dft)/gamma0/vq
c                                 etas0 = thermo(9,id)
c                                 g0 = emod(1,id)
c                                 g0p = emod(2,id)
c                                 etas = thermo(9,id)*v/v0
c                                 adiabatic shear modulus
      smu = (1d0 + 2d0*f)**(2.5d0)*
     *      (emod(1,id)*(1d0 - 5d0*f) + f*emod(2,id)*3d0*k00)
     *    -  thermo(9,id)*v/v0*ethv

      if (tht.gt.5d0.or.tht0.gt.5d0.and.izap1.lt.100) then
         write (*,*) 'danger will robinson, danger, danger!!',
     *               id,tht0,tht
         izap1 = izap1 + 1
      end if 
                   
1000  format (/,'**warning ver369** failed to converge at T= ',f8.2,' K'
     *       ,' P=',f9.1,'bar',/,' Using Sixtrude EoS.',
     *        ' Phase ',a,' will be destabilized.',/)
1010  format (/,'**warning ver369** will not be reapeated for future ',
     *          'instances of this problem',/)

99    end 

      double precision function gstxgi (id)
c-----------------------------------------------------------------------
c gsixtr computes G from the EoS formulated by Sixtrude & Bukowski '90, 
c JGR 95:19311-19325.

c with the debye T evaluated as in Sixtrude GJI '05
c

c Sixtrude GJI parameters:

c    F0    -n  -v(J/bar) K0(bar) K0'   theta0 gamma0 q etaS0 Smag

c Perple_X parameters (1st index in array thermo):

c    1     2     3        4     5      6      7    8    9    10

c and for shear modulii

c    G0(bar)   G0'

c are in the Perple_X array emod(i,id) with i

c      1        2
c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer id, itic, kmod, iemod, izap, izap1

      character*8 names 

      double precision thermo, uf, us, p, t, xco2, um, r, turd2, a0,
     *                 tr, pr, ps, v0, v, df, f, dv, k0p, a, ethv,
     *                 gamma0, k00, plg, c1, c2, c3, c5, f1,  
     *                 gvq, dt, root, d0, d1, c7, d2, d3,
     *                 n, q, vq, v23, theta0, smu, emod, fac, tol

      double precision nr9, qm1, d2f, tht, tht0, etht, etht0, ltht, df1,
     *                 ltht0, dtht, dtht0, d2tht, d2tht0, g, g0, dg, 
     *                 dg0, d2g, d2g0, dfc, d2fc, dft, d2ft, dft0, d2ft0
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)/ cst323 /smu
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps/ cst8   /names(k1)
     *      / cst319 /emod(k15,k10),iemod(k1),kmod

      save turd2, izap, izap1 

      data turd2, izap, izap1 /0.6666666666666666667, 0, 0/

c                                       assign local variables:
      a0     =  thermo(1,id)
      n      = -thermo(2,id)
      v0     = -thermo(3,id)
      k00    =  thermo(4,id)
      k0p    =  thermo(5,id)
      theta0 =  thermo(6,id)
      gamma0 =  thermo(7,id)
      q      =  thermo(8,id)

      nr9 = 9d0*n*r
      c1 = 9d0*k00*v0
      c2 = k0p/2d0-2d0
      c3 = 3d0*c1*c2
      c5 = 3d0*gamma0
      c7 = c5*(-2d0+6d0*gamma0-3d0*q)
      qm1 = q - 1d0
      dt = t - tr
c                                       initial guess for volume:
c                                       uses a 4 order expansion on
c                                       the polylog terms before 
c                                       differentiation and a second
c                                       order series on the resulting
c                                       derivative.
      v = v0*(3d0*((2d0 - q)*dt*gamma0*nr9 - 3d0*p*v0) + c1)/
     *       (c1 - qm1*dt*gamma0*nr9)
      if (v.lt.0d0) v = v0

      f1 = 1d9
      fac = 1d0
      itic = 0 
c                                       change to use relative tolerance
c                                       JADC March 1, 2005. formerly 1d-1 bar.
      tol = 1d-6*p

      do while (dabs(f1).gt.tol)

         itic = itic + 1

         vq = (v/v0)**q
         gvq = gamma0*vq
         v23 = (v0/v)**turd2
         f = 0.5d0*v23 - 0.5d0
         df = -v23/v/3d0
         d2f = 5d0/9d0*v23/v**2
c                                      debye T/T
         root = 1d0+c7*f**2+2d0*c5*f
         if (root.lt.0d0) goto 90 

         d0  = dsqrt(root)
         tht = theta0/t*d0
         tht0 =  tht*t/tr

         etht  = dexp(-tht )
         if (1d0-etht.le.0) goto 90 
         etht0 = dexp(-tht0)
         ltht  = dlog(1d0 - etht )
         ltht0 = dlog(1d0 - etht0)
c                                      diff(theta/T,v)
         d2 = (c7*f+c5)/d0
         d1 = theta0*df*d2
         dtht = d1/t
         dtht0 = d1/tr
c                                      diff(theta/T,v,v)
         d3 = theta0*(df**2*(c7-c5**2)/d0**3 + d2f*d2)
         d2tht  = d3/t
         d2tht0 = d3/tr

         g   = plg(tht )
         g0  = plg(tht0)
         dg  = tht **2*ltht *dtht
         dg0 = tht0**2*ltht0*dtht0
         d2g  = ((2d0*ltht  + tht *etht /(1d0-etht ))*dtht **2 + 
     *          tht *ltht *d2tht )*tht
         d2g0 = ((2d0*ltht0 + tht0*etht0/(1d0-etht0))*dtht0**2 + 
     *          tht0*ltht0*d2tht0)*tht0

         dfc = (c3*f+c1)*f*df
         d2fc = (2d0*c3*f+c1)*df**2+(c3*f+c1)*f*d2f

         dft  = nr9*t /tht **3*(dg  -3d0/tht *g *dtht )
         dft0 = nr9*tr/tht0**3*(dg0 -3d0/tht0*g0*dtht0)

         d2ft =  nr9*t /tht **3*(3d0/tht *(dtht *(4d0/tht *g *dtht  
     *                        - 2d0*dg ) - g *d2tht ) + d2g )
         d2ft0 = nr9*tr/tht0**3*(3d0/tht0*(dtht0*(4d0/tht0*g0*dtht0 
     *                        - 2d0*dg0) - g0*d2tht0) + d2g0)

         f1  = -dfc - dft + dft0 - p

         df1 = -d2fc - d2ft + d2ft0 

         dv = f1/df1
         v = v - dv
c                                 the limit on f1 should be 
c                                 machine dependent
         if (v.le.0d0.or.itic.gt.40.or.dabs(f1).gt.1d40) goto 90
 
      end do
c                                 if everything is ok, now get 
c                                 helmoltz energy:
      goto 10 
c                                 if we get here, failed to converge
90    if (izap.lt.100) then
         write (*,1000) t,p,names(id)
      else if (izap.eq.100) then 
         write (*,1010)
      end if 

      izap = izap + 1
c                                 use series expansion on failure
c      v = v0*(3d0*((2d0 - q)*dt*gamma0*nr9 - 3d0*p*v0) + c1)/
c     *       (c1 - qm1*dt*gamma0*nr9)
c      if (v.lt.0d0) v = v0
c                                 destabilize the phase:
      gstxgi  = 1d30
      goto 99 
         
10    vq = (v/v0)**q
      f = 0.5d0*(v0/v)**turd2 - 0.5d0 
      d0  = dsqrt(1d0+c7*f**2+2d0*c5*f)
      tht = theta0/t*d0
      tht0 =  tht*t/tr

      a = a0 + c1*f**2*(0.5d0 + c2*f) 
     *  + nr9*(t/tht**3*plg(tht ) -tr/tht0**3*plg(tht0))

      gstxgi = a + p*v - t*thermo(10,id)
c                                 thermal energy/v
      ethv = (dft0-dft)/gamma0/vq
c                                 etas0 = thermo(9,id)
c                                 g0 = emod(1,id)
c                                 g0p = emod(2,id)
c                                 etas = thermo(9,id)*v/v0
c                                 adiabatic shear modulus
      smu = (1d0 + 2d0*f)**(2.5d0)*
     *      (emod(1,id)*(1d0 - 5d0*f) + f*emod(2,id)*3d0*k00)
     *    -  thermo(9,id)*v/v0*ethv

      if (tht.gt.5d0.or.tht0.gt.5d0.and.izap1.lt.100) then
         write (*,*) 'danger will robinson, danger, danger!!',
     *               id,tht0,tht
         izap1 = izap1 + 1
      end if 

c      if (isnan(gstxgi).or.isnan(smu)) then
c         write (*,*) 'ugly pugly'
c      end if 
                   
1000  format (/,'**warning ver369** failed to converge at T= ',f8.2,' K'
     *       ,' P=',f9.1,'bar',/,' Using Sixtrude EoS.',
     *        ' Phase ',a,' will be destabilized.',/)
1010  format (/,'**warning ver369** will not be reapeated for future ',
     *          'instances of this problem',/)

99    end 



      double precision function plg (t)
c-----------------------------------------------------------------------
c 29th order series expansion of polylog terms in sixtrude's EoS about
c t = 0, good to 0.0001 accuracy to t = 5.
c f := int((ln(1-exp(-t))*t^2),t=0..TD);
c gg := convert(series(f,TD=0,29),polynom);
c-----------------------------------------------------------------------

      implicit none

      double precision c2, t

      save c2 

      data c2 /0.1111111111111111111/

c                              13th order expansion good to t ~ 3
c       c1 = 1/3
c     plg  = t**3*((dlog(t)*c1-c2) - t/8d0 + t**2/120d0     
c    *     - t**4/20160d0 + t**6/1632960d0 - t**8/106444800d0)

      plg  = (t**3*((dlog(t)/3d0-c2) - t/8d0 + t**2/120d0     
     *     - t**4/20160d0 + t**6/1632960d0 - t**8/106444800d0
     *     + t**10/6227020800d0 - 0.2935661189d-11*t**12 
     *     + 0.562291451d-13*t**14 - 0.1115026413d-14*t**16
     *     + 0.2271444989d-16*t**18 - 0.4727975432d-18*t**20
     *     + 0.1001636878d-19*t**22 - 0.2153466772d-21*t**24))

      end 


      double precision function gmurn (id)
c-----------------------------------------------------------------------
c gmurn computes the vdp integral of a compound identified by id
c that is described by ghiorso's pmelt murnaghan EoS.

c Ghiorso parameters:

c v(J/bar) dv/dt(J/K/bar) dv/dp(J/bar^2) dv/dp/dt(J/bar^2/K) -K'

c Perple_X parameters (1st index in array thermo):

c    3            11           12             13              18

c N.B. for some reason Ghiorso chooses a reference T = 1673 for tbe
c mechanical EoS parameters, this is hardwired here as variable trv.

c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer id, itic

      double precision thermo, uf, us, p, t, xco2, um, r, turd2, trv,
     *                 tr, pr, ps, v0, k, turd, rat, rat2, c0, c1, 
     *                 c2, c3, c4, c5, a0, a1, v, df, f, dv, kprime
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)
     *      / cst5 /p,t,xco2,um(2),tr,pr,r,ps

      save turd, turd2, trv 

      data turd, turd2, trv/0.3333333333333333333,0.6666666666666666667,
     *                      1673.15/

c                                       v0 = Volume at 1 bar, T
      v0 = thermo(3,id) + thermo(11,id) * (t - trv)

      k = -v0 / (thermo(12,id) + thermo(13,id) * (t - trv) )
c                                       NOTE kprime = -K'
      kprime = thermo(18,id)
c                                       constants:
      a0 = 0.375d0 * v0 * k
      a1 = -0.125d0 * v0**2 * k
      c0 = (-28d0 -6d0 * kprime) * v0 * a0
      c1 = (12d0 + 3d0 * kprime) * v0**2 * a0
      c2 = (16d0 + 3d0 * kprime) * a0
      c3 = a1 * v0 * (-196d0 - 42d0 * kprime)
      c4 = a1 * (80d0 + 15d0 * kprime)
      c5 = a1 * v0 * (108d0 + 27d0 * kprime)
c                                       initial guess for volume:
      v = k * v0 / (p + k)
      dv = 1d0
      itic = 0 

      do while (dabs(dv).gt.1d-5)

         itic = itic + 1
         rat = (v0/v)**turd
         rat2 = rat**2
         f = p  + ((c0*v*rat+c1+c2*v**2*rat2)/v**3)
         df = (c3/rat2+c4*v/rat+c5)/v**4
         dv = f/df
         v = v - dv
         if (v.le.0d0.or.itic.gt.20) then

            write (*,1000) t,p

            if (k*(k-2d0*p*(1d0+kprime)).gt.0d0) then 

               v = 0.5d0/((3d0+kprime)*k+2d0*p)*((2d0+kprime)*2d0*k
     *             +2d0*dsqrt(k*(k-2d0*p*(1d0+kprime))))*v0 

               if (v.lt.0d0) v = 0.5d0/((3d0+kprime)*k+2d0*p)*
     *                       ((2d0+kprime)*2d0*k
     *                       -2d0*dsqrt(k*(k-2d0*p*(1d0+kprime))))*v0 
            else 

               v = k * v0 / (p + k)

            end if 

            exit

         end if 

1000  format (/,'**warning ver369** failed to converge at T= ',f8.2,' K'
     *       ,' P=',f9.1,'bar',/,' Using Murnaghan ',
     *        'EoS, probably for Ghiorso et al. MELTS/PMELTS endmember',
     *        ' data.',/,
     *        ' Volume estimated using 3rd order taylor series.',/)

      end do
c                                       and the vdp integral is:
      f = 0.5*((v0/v)**turd2-1d0)

      gmurn = p*v - v0*(pr-4.5d0*k*f**2*(1d0-f*(4d0+kprime))) 

      end 


      subroutine cartes (ksite,sname)
c---------------------------------------------------------------------
c      xmax2   |\              cartesian subdivision onto an n-dimensional
c              |  \            simplexe. isp(ksite) is number of
c              |    \          vertices.
c      xinc2   |      \        
c              |        \      
c              |          \    
c      xmin2   |            \  .
c              ---------------
c           xmin1    xinc1    xmax1

c      ksite - the site of solution "sname" to be represented.
c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ms1, mres
 
      parameter (ms1=msp-1,mres=3000)

      integer istot,jstot,iend,isub,imd,kdsol,ntot,npairs,
     *        ist,i,iord,iterm,isite,isp,insp,kstot,jmsol

      double precision wg, xmn, xmx, xnc, xy, yy, tol

      double precision y(ms1,mres), ycum, ymax, dy, ync

      integer ind(ms1), iy(ms1), jsp, ksite, indx, iexit, ieyit, izero

      character sname*10

      common/ cst86 /xy(mdim,k1),yy(k1,mst,ms1),ntot,npairs
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),isp(mst),
     *      isite,iterm,iord,istot,jstot,kstot

      save tol

      data tol/1d-6/
c---------------------------------------------------------------------
 
      ycum = 0d0
      izero = 1
      jsp = isp(ksite) - 1

      do i = 1, jsp

         if (isite.eq.1.and.kdsol(i).eq.0) then 
c                                  set counter to 0
            izero = izero + 1
            iy(i) = 0
            y(i,1) = 0d0 
         else 
c                                   generate coordinates for i'th component
            iy(i) = 1
            y(i,1) = xmn(ksite,i)
            ync = xnc(ksite,i)
c                                   avoid impossible compositions 'cause a min > 0
            if (i.gt.izero) then 

               ycum = ycum + xmn(ksite,i)
c                                   1-ycum is the smallest fraction possible
               if (1d0-ycum.lt.0) then 
c                                   user has specified inconsistent limits
                  call error (999,ycum,jsp,'cartes')
               else if (ycum.gt.xmx(ksite,i)) then  
c                                   the smallest fraction is gt than xmax
                  ymax = 1d0 - ycum 
               else
c                                   the smallest fraction possible is lt
c                                   than xmax
                  ymax = xmx(ksite,i)
               end if 
            else 
               ymax = xmx(ksite,i)
            end if 

            do while (y(i,iy(i)).lt.ymax)
               iy(i) = iy(i) + 1
               if (iy(i).gt.mres) call error (999,ync,mres,sname)
               y(i,iy(i)) = y(i,iy(i)-1) + ync
               if (dabs(y(i,iy(i))-ymax).lt.tol) y(i,iy(i)) = ymax
            end do
c                                   add last point if necessary
            if (y(i,iy(i)).lt.ymax) then
               iy(i) = iy(i) + 1
               if (iy(i).gt.mres) call error (999,ync,mres,sname)
               y(i,iy(i)) = ymax
            else if (y(i,iy(i)).gt.ymax.and.xmn(ksite,i).ne.ymax) then 
               y(i,iy(i)) = ymax
            end if          

         end if 
      end do
c                                  
      do i = 1, jsp
         ind(i) = 1
      end do 
c                                   assign the first point
      npairs = 1

      do i = 1, jsp
         xy(i,1) = y(i,ind(i))
      end do
c                                   now make the array index run over all
c                                   values increasing the last index fastest
      iexit = 0 
      ieyit = 0 
      dy = 0d0

      do while (iexit.eq.0)
c                                   figure out which index to increment
         do i = jsp, 1, -1
            if (ind(i).lt.iy(i).and.ieyit.eq.0) then
c                                   this is the one to increment
               ind(i) = ind(i) + 1
               indx = i 
               exit 
            else if (i.gt.1) then 
c                                  saturated the index
               ind(i) = 1
               ieyit = 0 
               
            else
c                                  saturated first index, done.
                goto 99 

            end if 
         end do 
c                                  ok now we have the indices, check
c                                  the composition
         ycum = 0d0

         do i = 1, jsp 
            ycum = ycum + y(i,ind(i))  
         end do 

         if (ycum.gt.1d0) then

            ieyit = 1
c                                  here is where it gets messy:
            if (indx.eq.1) then
c                                  we're at the first point, and already
c                                  over the top
               iexit = 1
               cycle

            else if ( y(indx,ind(indx)) - y(indx,ind(indx)-1)
     *               - ycum + 1d0    .gt. tol ) then          
c                                  reset the current variable
c                                  and max loop index
               dy =  1d0 - ycum 

            else
c                                   must have just hit on the last increment
               cycle 

            end if 
         end if 

         npairs = npairs + 1
         if (npairs.gt.k1) call error (180,ycum,k1,'CARTES')

         do i = 1, jsp
            xy(i,npairs) = y(i,ind(i))
         end do

         xy(indx,npairs) = xy(indx,npairs) + dy

         dy = 0d0 

      end do 

99    end


      subroutine blanko (text,chars,nchar,ilen)
c------------------------------------------------------------------- 
c unblnk - find the last non-blank character in text, 
c     text - character string 
c-------------------------------------------------------------------
      implicit none

      integer nchar, i, ilen

      character text*(*), chars(*)*1
 
      read (text,1000) (chars(i),i=1,ilen)
c                                 scan for blanks:
      do nchar = ilen, 1, -1
         if (chars(nchar).gt.' ') exit
      end do 

1000  format (400a1)

      end
