
c Please do not distribute any part of this source.
 
c Copyright (c) 1998 by James A. D. Connolly, Institute for Mineralogy
c & Petrography, Swiss Federal Insitute of Technology, CH-8092 Zurich,
c SWITZERLAND. All rights reserved.

      program vertex        
c----------------------------------------------------------------------
c                       ************************
c                       *                      *
c                       *  vertex.sep.10.1998  *
c                       *                      *
c                       ************************
c----------------------------------------------------------------------
 
c references:
 
c       connolly jad, kerrick dm (1987) bounds:an algorithm and program
c         for calculating composition diagrams, calphad 11, 1-57.
c       connolly jad (1990) multivariable phase diagrams: an algorithm
c         based on generalized thermodynamics, am j sci 290, p 666-718.
 
c-----------------------------------------------------------------------
c file structure:
 
c vertex does i/o from at least three and as many as nine logical units:
 
c  l.u.n  i/o       purpose
c  -----  ---   --------------------------------------------------------
c   n1     i    input to define computations/options.
c   n2     i    data file containing the names, compositional vectors,
c               and standard state thermodynamic data for potentially
c               stable condensed phases and fluid species.
c   n3     o    optional (io3) printer output.
c   n4     o    optional (io4) data file for graphics applications.
c   n5     o    optional (io5) data file for graphics applications.
c   n6    i/o   temporary work file required to generate data for n4,
c               contains the names and stable conditions of invariant
c               assemblages which are written to n4.
c   n7     i    unused
c   n8          unused.
c   n9     i    optional data file which contains data on the
c               compositional equations of state and subdivision schemes
c               to be used for solution phases.
c-----------------------------------------------------------------------
c this version of vertex has been modified to be compatible with cdc
c fortran 5 compilers.  memory requirements can be reduced on most
c other compilers by changing the implicit variable type integer to
c integer*2, on ibm compilers integer variables beginning with z must
c be integer*4, i.e., for ibm compilers the current implicit type
c statement 'implicit integer (h-n,z)' should be changed to 'implicit
c integer (z), integer*2 (h-n).  the necessity for double precision real
c variables has not been established, users interested in reducing 
c memory requirements may wish to experiment with single precision 
c variables.
c-----------------------------------------------------------------------
c functional(1) tolerance(2) constant(3) or zero (4) dependent
c     subprograms:                         (this list is not current)

c                 testit (4)      sreset (2)      asschk (4)
c itestc   (4)    loadit (1)      flipit (2)      wway   (2)
c                 solod  (1,3)    schk   (4)      concrt (2,3)
c miscib   (4)    newass (4)      pchk   (2,4)    univeq (2,3)
c balanc   (4)    uproj  (1,2,3)  grxn   (1)      gphase (1)
c mrk      (1,3)  hsmrk  (1,3)    newrap (1,3)    fug    (1,3)

c units, tolerances, and constants:

c function specific
c routines for fluid properties (mrk, trkmrk, hsmrk) require units of
c kelvins (v(2)) and bars (v(3)) and joules.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      character*8 vname,xname,title*72
c-----------------------------------------------------------------------

c parameters are assigned in "perplex_parameter.h"
 
c-----------------------------------------------------------------------
      include 'perplex_parameters.h'
 
      common/ cst5   /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),dv(l2)
     *      / cst29  /vip(l2,k2),ipid(k2,k5+2),ipct/ csta8 /title
     *      / cst24  /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst81  /ipchk(k2),ip2(k2),icp2,ifake/ cst80 /irchk(k2)
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst6   /icomp,istct,iphct,icp/ cst31 /vn(k2,k7),irct,ird
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / csta2  /xname(k5),vname(l2)
     *      / cst319 /emod(k15,k10),iemod(k1),kmod
c----------------------------------------------------------------------- 
c                              initialize invariant and univariant
c                              field counters:
      ipct = 0
      irct = 0
c                              elastic modulii flag
      kmod = 0 
c                              get runtime parameters
      call redop1
c                              be funny
c      call money
c                              and flags:
      do i = 1, k2 
         ipchk(i) = 0
      end do
c                            -------------------------------------------
c                              open statements for units n1-n6 and n9
c                              are in subroutine input1
 
c                              read input from unit n1 (terminal/disk).
c                              subroutine F also initializes:
c                              equilibrium counters; units n2 n4 and n6;
c                              and the limits for numerical results.
      call input1 
c                              read thermodynamic data on unit n2:
      call input2 (icmpn)
c                              read data for solution phases on n9:
      call input9 (icmpn)
c                              adaptive minimization, no return from 
c                              here.
c     if (icopt.eq.10) call xeron (icmpn)

      write (*,*) 
c                              write header to graphics files (n4, n5):
      if (io4.ne.1) then 
         if (icopt.eq.1.and.jbulk.ge.icp) then
c                              computing full constrained pseudo-section
            call header (6)
         else if (icopt.eq.5.or.icopt.eq.7.or.icopt.eq.9) then 
c                              5 - minimization on a 2-d grid
c                              7 - fractionation on a 1-d path
c                              9 - fractionation on a 2-d path
            call header (icopt)
         else if (icopt.eq.1) then 
c                              computing schreinemakers projection
            call header (icopt)
         else 
c                              computing chemography or mixed-var.
            call outhed
         end if
      end if 
c                              initialize variables.
      do i = 1, 5
         v(i) = vmin(i)
      end do 

      call incdep (iv1,v(iv1)) 
      call incdep (iv2,v(iv2)) 
      call incdep (iv3,v(iv3)) 

      if (icopt.eq.0) then
c                              calculate composition phase diagrams:
         call chmcal
         goto 99
      else
c                              output title page to unit n3:
         call outtit (n3)
c                              read stability field assemblages on unit n1:
         if (icopt.eq.2) call error (32,v(1),k2,'MAIN')
      end if

      if (icopt.eq.5) then 
c                              optimization on a 2-d grid, control does
c                              not return to main program.
         call wav2d1 
c        call wavgrd
      else if (icopt.eq.7) then 
c                              fractionation on a 1-d path, control does
c                              not return to main program.
         call dummy0

      else if (icopt.eq.9) then 
c                              fractionation on a 2-d path (space-time), control
c                              does not return to the main program 
         call dummy1

      end if          
c                              calculate phase diagram/stability field
      call newhld 
c                              summarize output for each tier
c                              on units n3 and n4:
      call outier

      if (io3.eq.1) goto 200

      write (n3,1160)
c                              summarize calculations:
200   if (icopt.ne.3) call outlst
c                              done:
1160  format (/,80('-'),/)
99    end


      subroutine dummy0
c-----------------------------------------------------------------------
c a main program template to illustrate how to call the minimization 
c procedure
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      double precision dcomp

      integer ipath, loopx, jlow, jtest

      character*8 cname*5, y*1, xname, vname

      common/ csta4 /cname(k5)/ cst6  /icomp,istct,iphct,icp
     *      / cst12 /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst313 /a(k5,k1),b(k5)/ cst312 /imode,jlow,jlev,loopx
     *      / debug  /jtest,jpot/ frct1 /ifrct,ifr(5),jfr(5)
     *      / cst5  /v(l2),tr,pr,r,ps/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / csta2 /xname(k5),vname(l2)/ cst311/igrd(l7,l7)
     *      / frct2 /dcomp(k5)
c-----------------------------------------------------------------------
c                                 set jtest = 3 to overide tests for
c                                 redundancy, this forces vertex to output
c                                 data for each optimization
      jtest = 3
c                                 get phases to be fractionated
      ifrct = 0 
      loopy = jlow
      ipath = loopx

      write (*,1040) 
      read (*,1050) y
      if (y.eq.'y'.or.y.eq.'Y') call frname
c                                 call initlp to initialize arrays 
c                                 for optimization.
      call initlp (jphct,istart)
c                                 initialize sectioning variables
      call setvar (loopx,loopy)
c                                 loopx = 99 indicates interactive
c                                 mode, otherwise traverse on  
      if (ipath.eq.99) then 

         write (*,1290) 
         read (*,*) ipath

         if (ipath.eq.1) then 
            do while (v(jv(1)).gt.0)
               write (*,1070) (vname(jv(i)), i = 1, ipot)
               read (*,*) (v(jv(i)), i = 1, ipot)
               if (v(jv(1)).eq.0d0) goto 99
c                                 the systems molar composition is in 
c                                 the array cblk.
               ctotal = 0d0
c                                 get total moles to compute mole fractions             
               do i = 1, icp
                  ctotal = ctotal + cblk(i)
               end do

               do i = 1, icp 
                  b(i) = cblk(i)/ctotal
               end do
c                                 lpopt does the minimization and outputs
c                                 the results to the print file.
               call lpopt (1,j,jphct,istart,idead)

               write (*,1000)
               do i = 1, jbulk
                  write (*,1010) cname(i), cblk(i)
               end do 
c                                 the bulk composition can be modified here, 
c                                 e.g., to add back previously fractionated
c                                 phases.
               write (*,1060) 
               read (*,1050) y
               if (y.eq.'y'.or.y.eq.'Y') then 
                  write (*,1020) 
                  read (*,*) (dcomp(i), i = 1, jbulk)
                  do i = 1, jbulk
                     cblk(i) = cblk(i) + dcomp(i)
                  end do 
               end if 

            end do
         else 
c                                 coordinates from file
            write (*,1300)
            write (*,1280) (vname(jv(i)), i = 1, ipot)
            write (*,1310)
            write (*,*) ' this option is not implemented.' 
            stop

         end if 
      else
c                                 automated mode:
         do j = 1, loopy

            ctotal = 0d0
c                                 get total moles to compute mole fractions             
            do i = 1, icp
               ctotal = ctotal + cblk(i)
            end do

            do i = 1, icp 
               b(i) = cblk(i)/ctotal
            end do
c                                 lpopt does the minimization and outputs
c                                 the results to the print file.
            call lpopt (1,j,jphct,istart,idead)

c           igrd(1,j) = jgrd(1,j)

c                                 increment v(jv(1))
            call incvr2 (j-1)

         end do 

      end if 
c                             no more hot points or at highest level
10    if (io4.eq.0) then 

         write (n4,1080) 1, loopy 
         do j = 1, loopy
            write (n4,1080) 0, igrd(1,j)
         end do 
c                                             compressed form below
c                                             output graphics data
c         kst = 1

c20       kd = igrd(1,kst)
c         ltic = -1

c         do j = kst, loopy

c            if (igrd(1,j).eq.kd) then
c               ltic = ltic + 1

c               if (j.eq.loopy) then 
c                  write (n4,1080) ltic,kd
c                  goto 99
c               end if 

c            else 
c               write (n4,1080) ltic,kd
c               kst = j
c               goto 20 
c            end if 
c         end do        
      end if

1000  format (/,'Composition is now:',/)
1010  format (1x,a,1x,g14.6)
1020  format (/,'Enter molar amounts of the components to be added ',
     *        '(ordered as above:')
1040  format ('Fractionate phases (y/n)?')
1050  format (a)
1060  format (/,'Modify composition (y/n)?')
1070  format (/,'Enter (zeroes to quit) ',7(a,1x))
1080  format (20(i7,1x))
1280  format (7(a,1x))
1290  format (/,'Path coordinates are to be entered:',/,
     *          '   1 - interactively',/,
     *          '   2 - a file containing a list of x-y points',/,
     *          'Enter 1 or 2 (default=2):'/) 
1300  format (/,'Enter the name of the file containing the:',/)
1310  format (/,'path coordinates (one coordinate per line)',/)

99    stop

      end 

      subroutine dummy1
c-----------------------------------------------------------------------
c a subprogram template to illustrate how to do 2-d (space-time) 
c fractionation. 

c modified oct 15 for more rational input and variable gradients.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (maxbox=1760) 

      double precision gblk(maxbox,k5), zlayer

      character*8 cname*5, y*1, xname, vname, happy*3, n1name*100

      common/ csta4 /cname(k5)/ cst6  /icomp,istct,iphct,icp
     *      / cst12 /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst313 /a(k5,k1),b(k5)/ cst312 /imode,jlow,jlev,loopx
     *      / debug  /jtest,jpot/ frct1 /ifrct,ifr(5),jfr(5)
     *      / cst5  /v(l2),tr,pr,r,ps/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / csta2 /xname(k5),vname(l2)/ cst311/igrd(l7,l7)
     *      / frct2 /dcomp(k5)/ cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst228 /n1name
c-----------------------------------------------------------------------
c                                 set jtest = 3 to overide tests for
c                                 redundancy, this forces vertex to output
c                                 data for each optimization
      jtest = 3
c                                 get phases to be fractionated
      ifrct = 0 
c                                 look for input data from a file 
c                                 named by prefixing the computational
c                                 option file with 'a'
      call inblnk (n1name,'a')
c
      open (666,file=n1name,status='old',iostat=ierr)

      if (ierr.ne.0) call error (51,zbox,loopy,n1name) 
c                                 set the number of independent variables
c                                 to 1 (the independent path variable must
c                                 be variable jv(1), and the dependent path
c                                 variable must be jv(2), the path variables
c                                 can only be pressure and temperature
      ipot = 1
c                                 number of steps along the path
      read (666,*) loopy
c                                 thickness of a box in column
      read (666,*) zbox 
c                                 gradient in variable jv(1) with z, jv(1)
c                                 is the independent variable, for subduction
c                                 this is logically pressure, i.e.,
c                                 dp(bar)/dz(m)
      read (666,*) dv1dz 
c                                 now we need a path function for the dependent
c                                 variable, here we take a function defined in
c                                 terms of the absolute depth of the top of the
c                                 column (z0) and the relative depth (dz) within
c                                 the column
c                                   
c                                       v2 = a(z0)*dz^2 + b(z0)*dz + c(z0)

c                                 e.g., T(K) =  a(z0)*dz^2 + b(z0)*dz + c(z0)

c                                 where a(z0) = a0 + a1*z0 + a2*z0^2 + a3*z0^3 + ...
c                                       b(z0) = b0 + b1*z0 + b2*z0^2 + b3*z0^3 + ...
c                                       c(z0) = c0 + c1*z0 + c2*z0^2 + c3*z0^3 + ...
      read (666,*) a0, a1, a2, a3
      read (666,*) b0, b1, b2, b3
      read (666,*) c0, c1, c2, c3

c                                 get the initial global composition array
c                                 consisting of ibox compositions defined 
c                                 in terms of icp components. this read
c                                 statement assumes that H2O an CO2 (if 
c                                 thermodynamic components) are the 1st and
c                                 2nd components (if present). 
      ibox = 0 
c                                 number of nodes with appended composition
c                                 end of data indicated by zero 
      do 
         read (666,*) zlayer
	   if (zlayer.eq.0) exit 
	   read (666,*) (cblk(i),i=1,icp)

         irep = zlayer/zbox

	   do i = 1, irep
	      ibox = ibox + 1
            if (ibox.gt.maxbox) then
               write (*,*) 'increase maxbox in routine DUMMY1'
               pause 
               stop
            end if 
	      do j = 1, icp 
	         gblk(ibox,j) = cblk(j)
	      end do 
         end do
	end do 

      if (ibox*loopy.gt.k2) then
         write (*,*) ' parameter k2 must be >= ibox*loopy'
         write (*,*) ' increase parameter k2 for routine DUMMY1'
         write (*,*) ' or increase box size (zbox) or decrease'
         write (*,*) ' number of path increments (loopy) or try'
         write (*,*) ' the large parameter version of VERTEX'
         write (*,*) ' k2 = ',k2 
         write (*,*) ' ibox * loopy = ',ibox*loopy
         pause
         stop
      end if 

      close (666)

      write (*,1040) 
      read (*,1050) y
      if (y.eq.'y'.or.y.eq.'Y') call frname
c                                 call initlp to initialize arrays 
c                                 for optimization.
      call initlp (jphct,istart)
c                                 initialize path variables
      call setvar (loopx,loopy)

	dv(jv(1)) = (vmax(jv(1)) - vmin(jv(1)))/(loopy-1)
c                                 loopy is the number of steps along the subduction
c                                 path:
      do j = 1, loopy

         happy = 'no'
c                                 remember the starting variables
         v01 = v(jv(1))
         v02 = v(jv(2))
c                                 convert v01 to z0
         z0 = v01/dv1dz
c                                 get coefficients for dependent variable
c                                 as a function of dz below z0:
         az0 = a0 + a1*z0 + a2*z0**2 + a3*z0**3 
         bz0 = b0 + b1*z0 + b2*z0**2 + b3*z0**3 
         cz0 = c0 + c1*z0 + c2*z0**2 + c3*z0**3 
c                                 happy is set to yes when we've finally finished 
c                                 equilibrating at point j on the path
         do while (happy.eq.'no') 

	      happy = 'yes'
c                                 for each box in our column compute phase 
c                                 relations
            do k = 1, ibox

c                                 set the independent variable
               v(jv(1)) = v01 + (ibox - k) * zbox * dv1dz
c                                 we are moving up from the base of the 
c                                 column towards z0. because our function is
c                                 in terms of z, get z from v1 (p) as:
               dz = (ibox - k) * zbox
c                                 set the dependent variable
               v(jv(2)) = az0*dz**2 + bz0*dz + cz0
c                                 now but the bulk composition from the global
c                                 array into the local array and get the total
c                                 number of moles (ctotal)
               ctotal = 0d0
c                                 get total moles to compute mole fractions             
               do i = 1, icp
	            dcomp(i) = 0 
	            cblk(i) = gblk(k,i)
                  ctotal = ctotal + cblk(i)
               end do

               do i = 1, icp 
                  b(i) = cblk(i)/ctotal
               end do
c                                 lpopt does the minimization and outputs
c                                 the results to the print file.
               if (io3.eq.0) then
c                                 then write position
                  write (n3,*) 'step ',j,' on path, for box ',k
	         end if 

               call lpopt (j,k,jphct,istart,idead)
c                                 a flag that identifies the assemblage at
c                                 node j,k
               igrd(j,k) = jgrd(j,k)
c                                 at this point we've computed the stable
c                                 assemblage at each point in our column
c                                 and could do mass transfer, etc etc 
c                                 here we'll simply fractionate the fluid 
c                                 phase 
                do i = 1, icp 
c                                 subtract the fluid from the current composition
                   gblk(k,i) = gblk(k,i) - dcomp(i) 
                   if (gblk(k,i).lt.0d0) then
                      write (*,*) 'negative - bulk at k,i',k,i,gblk(k,i)
                       gblk(k,i) = 0d0
                   end if 
c                                 and add it to the overlying composition
                   if (k.lt.ibox) then
                       gblk(k+1,i) = gblk(k+1,i) + dcomp(i) 
                       if (gblk(k,i).lt.0d0) then
                      write (*,*) 'negative + bulk at k,i',k,i,gblk(k,i)
                          gblk(k,i) = 0d0
                       end if 
                   end if 
                end do
c                                 end of the k index loop
	      end do 
c                                 end of the happy loop. 
         end do 
c                                 now we increment the subduction path
c                                 variable v(jv(1)), by dv(jv(1))
         v(jv(1)) = v01 + dv(jv(1))
c                                 end of j index loop
      end do 
c                                 no more hot points or at highest level
10    if (io4.eq.0) then 

         write (n4,*) loopy, ibox, zbox
c                                             output graphics data
c                                             loop for column data
         do i = 1, loopy 
c                                             initialize column counter 
            kst = 1

20          kd = igrd(i,kst)
            ltic = -1

            do j = kst, ibox

               if (igrd(i,j).eq.kd) then
                  ltic = ltic + 1

                  if (j.eq.ibox) then 
                     write (n4,1080) ltic, kd
                     goto 30
                  end if 

               else 

                  write (n4,1080) ltic, kd
                  kst = j
	            if (j.eq.ibox) then
                     write (n4,1080) ltic, igrd(i,j)  
                     goto 30
                  end if  
                  goto 20 
               end if 

            end do 
30       end do        
      end if

1040  format ('Fractionate phases (y/n)?')
1050  format (a)
1080  format (20(i7,1x))

99    pause 
      stop

      end 


      subroutine dummy2
c-----------------------------------------------------------------------
c a subprogram template to illustrate how to do 2-d (space-time) 
c fractionation.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (maxbox=1) 

      double precision gblk(maxbox,k5)

      character*8 cname*5, y*1, xname, vname, happy*3

      common/ csta4 /cname(k5)/ cst6  /icomp,istct,iphct,icp
     *      / cst12 /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst313 /a(k5,k1),b(k5)/ cst312 /imode,jlow,jlev,loopx
     *      / debug  /jtest,jpot/ frct1 /ifrct,ifr(5),jfr(5)
     *      / cst5  /v(l2),tr,pr,r,ps/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / csta2 /xname(k5),vname(l2)/ cst311/igrd(l7,l7)
     *      / frct2 /dcomp(k5)/ cst9  /vmax(l2),vmin(l2),dv(l2)
c-----------------------------------------------------------------------
c                                 set jtest = 3 to overide tests for
c                                 redundancy, this forces vertex to output
c                                 data for each optimization
      jtest = 3
c                                 get phases to be fractionated
      ifrct = 0 
c
      open (666,file='input.dat')
c                                 set the number of independent variables
c                                 to 1 (the independent path variable must
c                                 be variable jv(1), and the dependent path
c                                 variable must be jv(2), the path variables
c                                 can only be pressure and temperature
      ipot = 1
c                                 number of steps along the path
      read (666,*) loopy
c                                 thickness of a box
      read (666,*) zbox 
c                                 gradient in variable jv(1) with z
      read (666,*) dv1dz 
c                                 gradient in variable jv(2) with z
      read (666,*) c5,c4,c3,c2,c1,c0 
c                                 get the initial global composition array
c                                 consisting of ibox compositions defined 
c                                 in terms of icp components. this read
c                                 statement assumes that H2O an CO2 (if 
c                                 thermodynamic components) are the 1st and
c                                 2nd components (if present). 
      ibox = 0 
c                                 number of nodes with appended composition
c                                 end of data indicated by zero 
      do 
         read (666,*) irep
	   if (irep.eq.0) exit 
	   read (666,*) (cblk(i),i=1,icp)
	   do i = 1, irep
	      ibox = ibox + 1
            if (ibox.gt.maxbox) then 
               write (*,*) ' increase parameter maxbox in routine OPSYS'
               stop
            end if 
	      do j = 1, icp 
	         gblk(ibox,j) = cblk(j)
	      end do 
         end do
	end do 

      if (ibox*loopy.gt.k2) then
         write (*,*) ' increase parameter k2 for routine OPSYS'
         write (*,*) ' k2 = ',k2 
         write (*,*) ' ibox * loopy = ',ibox*loopy
         stop
      end if 

      close (666)

      write (*,1040) 
      read (*,1050) y
      if (y.eq.'y'.or.y.eq.'Y') call frname
c                                 call initlp to initialize arrays 
c                                 for optimization.
      call initlp (jphct,istart)
c                                 initialize path variables
      call setvar (loopx,loopy)

	dv(jv(1)) = (vmax(jv(1)) - vmin(jv(1)))/(loopy-1)
c                                 loopy is the number of steps along the subduction
c                                 path:
      do j = 1, loopy

         happy = 'no'
c                                 remember the starting variables
         v01 = v(jv(1))
         v02 = v(jv(2))
         dv2dz = c0 + c1*v01 + c2*v01**2 + c3*v01**3 + c4*v01**4
     *              + c5*v01**5 
c                                 happy is set to yes when we've finally finished 
c                                 equilibrating at point j on the path
         do while (happy.eq.'no') 

	      happy = 'yes'
c                                 for each box in our column compute phase 
c                                 relations
            do k = 1, ibox

c                                 set the independent variable
               v(jv(1)) = v01 + (ibox - k) * zbox * dv1dz
c                                 set the dependent variable
               v(jv(2)) = v02 + (ibox - k) * zbox * dv2dz
c                                 now but the bulk composition from the global
c                                 array into the local array and get the total
c                                 number of moles (ctotal)
               ctotal = 0d0
c                                 get total moles to compute mole fractions             
               do i = 1, icp
	            dcomp(i) = 0 
	            cblk(i) = gblk(k,i)
                  ctotal = ctotal + cblk(i)
               end do

               do i = 1, icp 
                  b(i) = cblk(i)/ctotal
               end do
c                                 lpopt does the minimization and outputs
c                                 the results to the print file.
               if (io3.eq.0) then
c                                 then write position
                  write (n3,*) 'step ',j,' on path, for box ',k
	         end if 

               call lpopt (j,k,jphct,istart,idead)
c                                 a flag that identifies the assemblage at
c                                 node j,k
               igrd(j,k) = jgrd(j,k) 
c                                 at this point we've computed the stable
c                                 assemblage at each point in our column
c                                 and could do mass transfer, etc etc 
c                                 here we'll simply fractionate the fluid 
c                                 phase 
                do i = 1, icp 
c                                 if idead is not zero, the optimization 
c                                 failed and dcomp is inherited from an
c                                 earlier node. in this case we don't modify
c                                 the local composition, but we do pass dcomp
c                                 on to the overlying node. 
                   if (idead.eq.0) then 
c                                 subtract the fluid from the current composition
                      gblk(k,i) = gblk(k,i) - dcomp(i) 
                      if (gblk(k,i).lt.0d0) then
                         write (*,*) 'negative - bulk at k,i',
     *                               k,i,gblk(k,i)
                         gblk(k,i) = 0d0
                      end if 
                   end if 
c                                 and add it to the overlying composition
                   if (k.lt.ibox) then
                       gblk(k+1,i) = gblk(k+1,i) + dcomp(i) 
                       if (gblk(k,i).lt.0d0) then
                      write (*,*) 'negative + bulk at k,i',k,i,gblk(k,i)
                          gblk(k,i) = 0d0
                       end if 
                   end if 
                end do
c                                 end of the k index loop
	      end do 
c                                 end of the happy loop. 
         end do 
c                                 now we increment the subduction path
c                                 variable v(jv(1)), by dv(jv(1))
         v(jv(1)) = v01 + dv(jv(1))
         call incdep (jv(1),v(jv(1)))
c                                 end of j index loop
      end do 



c                             no more hot points or at highest level
10    if (io4.eq.0) then 

         write (n4,*) loopy, ibox, zbox
c                                             output graphics data
c                                             loop for column data
         do i = 1, loopy 
c                                             initialize column counter 
            kst = 1

20          kd = igrd(i,kst)
            ltic = -1

            do j = kst, ibox

               if (igrd(i,j).eq.kd) then
                  ltic = ltic + 1

                  if (j.eq.ibox) then 
                     write (n4,1080) ltic, kd
                     goto 30
                  end if 

               else 

                  write (n4,1080) ltic, kd
                  kst = j
	            if (j.eq.ibox) then
                     write (n4,1080) ltic, igrd(i,j)  
                     goto 30
                  end if  
                  goto 20 
               end if 

            end do 
30       end do        
      end if

1000  format (/,'Composition is now:',/)
1010  format (1x,a,1x,g14.6)
1020  format (/,'Enter molar amounts of the components to be added ',
     *        '(ordered as above:')
1040  format ('Fractionate phases (y/n)?')
1050  format (a)
1060  format (/,'Modify composition (y/n)?')
1070  format (/,'Enter (zeroes to quit) ',7(a,1x))
1080  format (20(i6,1x))

99    pause 
      stop

      end 

      subroutine chmcal
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst5   /v(l2),tr,pr,r,ps/ cst19 /jpoly
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst24  /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
c-----------------------------------------------------------------------
      call outtit (n3)

      jpoly = 0 
 
90    if (icopt.eq.0) then 
         read (n1,*,end=99) v
c                                  for old file formats:
         if (v(1).lt.0d0) goto 99
         jpoly = jpoly + 1
         write (*,1010) jpoly
      else 
c                                  must be in polygon mode:
         read (n7,*,end=99) jpoly,v(iv1),v(iv2)
         write (*,1000) jpoly
      end if 
c                                  compute phase diagram
      call gall
      call combin
c                                  generate permanent graphics file
c                                  if requested
      if (io4.ne.1) call outgrf
c                                  printer output:
      if (io3.ne.1) call outchm
      goto 90

1000  format ('Computing the assemblage in polygon ',i6)
1010  format ('Computing the compositional phase relations at',
     *        ' condition ',i2)
 
99    end
  
      subroutine input1 
c-----------------------------------------------------------------------
c input1 reads data from a file on unit n1, this data controls the
c computational options and is modified frequently.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 fname*10,blank*1,exname,n5name*100,n7name*100,
     *            cname*5,rname*5,name*8,sname*10,n1name*100,
     *            dname*40,cmpnt*5,vname,xname,strg*80,
     *            title*72,n2name*100,n3name*100,n4name*100,
     *            n9name*100,tcname*5,y*1,zname*5
 
      common/ cst6  /icomp,istct,iphct,icp/ cst79 /isoct
     *      / csta7 /fname(h9) / cst24 /ipot,jv(l2),iv(l2)
     *      / cst81 /ipchk(k2),ip2(k2),icp2,ifake/ cst112 /buf(5)
     *      / csta2 /xname(k5),vname(l2)
     *      / cst5  /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),dv(l2)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst31 /vn(k2,k7),irct,ird/ cst38 /iedge
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst39 /ichem
     *      / csta8 /title/ cst12 /cp(k5,k1)
     *      / cst36 /exname(h8)/ cst63 /delv(l2)/ debug /jtest,jpot
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst37 /ixct,iexyn/ csta4 /cname(k5)
      common/ cst209 /iwt/ cst209a /zname
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b/ cst14 /ipoly
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / csta5 /dname(8),cmpnt(k0)/ cst42 /ic(k5),idbase
     *      / cst207 /ctrans(k0,k5),ictr(k5),itrans
     *      / cst208 /ifct,idfl/ csta9 /tcname(k5)/ cst102 /ivfl
     *      / cst100 /dlnfo2,elag,gz,gy,gx,ibuf,hu,hv,hw,hx     
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct
     *      / cst312 /imode,jlow,jlev,loopx
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont
     *      / cst316 /c0,c1,c2,c3,c4,iind,idep/ cst228 /n1name
 
      data blank/' '/
c-----------------------------------------------------------------------
c                             read computational option file name
1     write (*,1050)
      read (*,1000) n1name
      call enblnk (n1name)

      open (n1, file = n1name, iostat = ierr, status = 'old')

      if (ierr.ne.0) then
c                             system could not find the file
         write (*,1040) n1name
         read (*,1000) y
         if (y.ne.'Y'.and.y.ne.'y')
     *                        call error (120,r,n1,n1name)
         goto 1
c                             try again
      end if
c                             begin reading input:
c                             read name of thermodynamic data file
      read (n1,1000) n2name
      call enblnk (n2name)
c                             open thermodynamic data file
      open (n2, file = n2name, iostat = ierr, status = 'old')
      if (ierr.ne.0) call error (119,r,n2,n2name) 
      write (*,1170) n2name
c                             read print and graphic file names
      read (n1,1000) n3name
      call enblnk (n3name)
      read (n1,1000) n4name
      call enblnk (n4name)
      read (n1,1000) n9name
      call enblnk (n9name)
c                             open files if requested
      if (n3name.ne.blank) then 
         io3 = 0 
         open (n3, file = n3name)
         write (*,1180) n3name
      else
         io3 = 1
         write (*,1180) 'none requested'
      end if

      if (n4name.ne.blank) then
         io4 = 0
         open (n4, file = n4name)
         write (*,1190) n4name
      else
         io4 = 1
         write (*,1190) 'none requested'
      end if

      if (n9name.ne.blank) then
         io9 = 0 
c                             open solution model file
         open (n9,file = n9name,iostat = ierr,status = 'old')
         if (ierr.ne.0) call error (121,r,n9,n9name)
         write (*,1200)
         write (*,1210) n9name
       else
         io9 = 1
         write (*,1210) 'none requested'
       end if
c                             read title for the calculation:
      read (n1,1000) title
      read (n1,*,err=998) icopt 
      read (n1,*,err=998) isec
      read (n1,*,err=998) jpot 
      read (n1,*,err=998) jtest 
      read (n1,*,err=998) ifull
      read (n1,*,err=998) imsg 
      read (n1,*,err=998) io3p
      read (n1,*,err=998) imode
      read (n1,*,err=998) jlow
      read (n1,*,err=998) jlev
      read (n1,*,err=998) itrans
      read (n1,*,err=998) icmpn
c                             read new component definitions:
      do i = 1, itrans
         read (n1,1120) tcname(i), ictr(i)
         read (n1,*) (ctrans(j,i), j = 1, icmpn)
      end do

      read (n1,*,err=998) iwt
      read (n1,*,err=998) io3b
c                             isudo is a flag which determines if
c                             pseudo-univariant equilibria are defined
c                             for icopt .eq. 1, 2, or 3.
      read (n1,*,err=998) isudo  
      if (icopt.ne.1) isudo = 0
c                             read data base code for the calculation.
c                             allowed codes are input by the user in the
c                             the data file (21).  see read's below.
      read (n1,*,err=998) idbase
c                             read code for choice of fluid equation
c                             of state from terminal. 
      read (n1,*,err=998) ifug
      if (ifug.ge.7.and.ifug.le.12.and.ifug.ne.9.or.ifug.eq.19.or.
     *    ifug.eq.14.or.ifug.eq.16.or.ifug.eq.17.or.ifug.eq.24.or.
     *    ifug.eq.20.or.ifug.eq.25) 
     *                  read (n1,*,err=998) ibuf,hu,dlnfo2,elag
      if (ibuf.eq.5) read (n1,*,err=998) buf     
c                             ichem is a flag which determines
c                             whether chemographies are output
c                             when evaluated by the subroutines
c                             vertex and check. (yes = 0,no = 1).
c     read (n1,*,err=998) ichem
      ichem = 0
c                             iedge is a variable which indicates
c     12222222222222222222    which edges of a phase diagram
c     1                  3    coordinate frame are to be searched
c  i  1                  3    for univariant equilibria.
c  v  1                  3    in this version of vertex (w/newhld)
c  1  1                  3    the traverses are done conterclockwise
c     1                  3    as indicated. the traverses must be done
c     1                  3    sequentially, and only relatively small
c     44444444444444444443    savings are obtained by not searching all
c              iv2            four edges, so iedge is not input.
c     read (n1,*,err=998) iedge
c                             iedge = 1 do traverse 1
c                             iedge = 2 do traverses 1 and 2, etc
      iedge = 4
c                             read solid pressure (bars), if solid
c                             pressure equals fluid pressure set ps =
c                             0d0.
c     read (n1,*,err=998) ps
      ps = 0d0
c                             no dependent variable
      iind = 0 
      loopx = (jlow-1) * 2**(jlev-1) + 1 
c                             decode thermodynamic components
c                             and quick fix for idep, and coeffs:
10    read (n1,1000) rname,strg
      if (rname.eq.blank) then 
         goto 10
      else if (rname.ne.'begin') then 
c                             must be a file generated by new build
c                             read extra parameters as required:
         read (rname,*,err=998) loop
         if (loop.gt.0.and.loop.lt.l7) loopx = loop

         read (n1,*,err=998) idep
         read (n1,*,err=998) c0,c1,c2,c3,c4

         if (idep.eq.1) then 
            iind = 2
         else if (idep.eq.2) then 
            iind = 1
         end if 
         
         goto 10 
      end if 
c                             count (icp) and save names (cname)
      icp = 0
      jbulk = 0 
      if (rname.ne.'begin') goto 998

20    read (n1,1000) rname,strg

      if (rname.eq.'end t') then 
c                             finished, could check for no components
         if (icp.eq.0) then
            write (*,*) 'No thermodynamic components'
            goto 998
         else if (icopt.eq.5.and.jbulk.lt.icp) then 
            write (*,*) 'All thermodynamic components must be ',
     *                  'constrained.'
            goto 998
         end if 
         icp1 = icp + 1
         icp2 = icp + 2
         goto 40
      else if (rname.eq.blank) then 
         goto 20 
      end if 

      icp = icp + 1
      cname(icp) = rname
c                             encode a graphics names for the
c                             compositional variables, this is kind of
c                             pointless, but it looks good.
      write (xname(icp),1010) 'x(',rname,')'
c                             unblank the name
      call unblnk (xname(icp))
      if (icp.gt.k5) call error (197,r,k5,'INPUT1')
c                             check for compositional constraints
      read (strg,*,err=998) icont
      if (icont.ne.0) then 
         jbulk = jbulk + 1
         read (strg,*,err=998) j, (dblk(i,jbulk), i = 1, icont)    
c                             override variance flag choice, why here?
         isudo = 0    
      end if 
      goto 20            
c                             decode saturated components
40    read (n1,1000) rname,strg
      if (rname.eq.blank) goto 40
c                             isat is the saturated component counter,
c                             isyn = 1 if isat = 0, else isyn = 0
      isyn = 1
      isat = 0
      io2  = 0 

      if (rname.ne.'begin') goto 998

60    read (n1,1000) rname,strg

      if (rname.eq.'end s') then 
         if (isat.ne.0) isyn = 0
         icomp = icp + isat
         goto 70
      else if (rname.eq.blank) then 
         goto 60 
      end if 
c                             check for compositional constraints
      read (strg,*,err=998) jcont
      if (jcont.ne.0) then 
         jbulk = jbulk + 1
         read (strg,*,err=998) j, (dblk(i,jbulk), i = 1, icont)    
c                             override variance flag choice, why here?
         isudo = 0    
      end if  

      isat = isat + 1
      if (isat.gt.h5) call error (15,r,i,'BUILD')
      cname(icp+isat) = rname
      if (rname.eq.'O2') io2 = isat
      goto 60 
c                             decode saturated phase components
70    read (n1,1000) rname
      if (rname.eq.blank) goto 70
c                             ifct is the fluid component counter,
c                             ifyn = 1 if ifct = 0.
      ifyn = 1
      ifct = 0

      if (rname.ne.'begin') goto 998

80    read (n1,1000) rname

      if (rname.eq.'end s') then 
         if (ifct.ne.0) ifyn = 0
         icomp = icomp + ifct
         goto 90
      else if (rname.eq.blank) then 
         goto 80 
      end if 
      
      ifct = ifct + 1
      if (ifct.gt.2) call error (44,r,i,'BUILD')
c                             save the component if only one
c                             for use in input2.
      if (ifct.eq.1) zname = rname
      cname(icomp+ifct) = rname
      goto 80
c                             decode mobile components
90    read (n1,1000) rname
      if (rname.eq.blank) goto 90
c                             jmct - mobile component counter
      jmct = 0 

      if (rname.ne.'begin') goto 998

100   read (n1,1000) rname

      if (rname.eq.'end m') then 
         icomp = icomp + jmct
         goto 110
      else if (rname.eq.blank) then 
         goto 100 
      end if 
      
      jmct = jmct + 1
      if (jmct.gt.2) call error (45,r,i,'BUILD')
      cname(icomp+jmct) = rname
      vname(3+jmct) = blank
c                             encode a text and graphics code name for
c                             each mobile component:
      write (vname(3+jmct),1010) 'u(',rname,')'
      call unblnk (vname(3+jmct))
      goto 100

110   jsat = 0
c                             pointers to components in cp and cname:
c                             1..icp  -> thermodynamic components
      if (isat.ne.0) then 
c                             jsat..ksat -> (jsat.ne.0) saturated components
         jsat = icp + 1
         ksat = icp + isat
      end if 

      if (ifct.ne.0) then 
c                             the ifct flag can probably be set later if fluid
c                             is in the thermodynamic composition space.

c                             jfct+1..kfct -> (ifct.ne.0) saturated phase components
         jfct = icp + isat 
         kfct = icp + isat + ifct
      end if 
c                             jprct+1..icomp -> (jmct.ne.0) mobile components
      if (jmct.ne.0) then 
         jprct = icomp - jmct 
      end if 
c                             initialize special plot files for constrained
c                             bulk composition calculations.
      if (jbulk.ge.icp.and.io4.ne.1.and.icopt.ne.0) then
c                                 create special plot output file
c                                 name = b + plotfile name
         n5name = n4name
         call inblnk (n5name,'b')
         open (n5, file = n5name)
         write (*,1220) n5name
         
         if (icopt.ne.5) then 
c                                 initialize polygon file
            n7name = n4name
            call inblnk (n7name,'p')
            open (n7, file = n7name)
            write (n7,*) 0
            close (n7)
            write (*,1240) n7name

         end if 

      else if (jbulk.ge.icp.and.io4.eq.1.and.icopt.ne.0) then
         write (*,1220) 'none requested'
      end if
c                             excluded phases
      iexyn = 1
      ixct = 0
c                             decode excluded phases
115   read (n1,1000) name
      if (name.eq.blank) goto 115

      if (name.ne.'begin ex') goto 998

120   read (n1,1000) name

      if (name.eq.'end excl') then 
         if (ixct.ne.0) iexyn = 0 
         goto 130
      else if (name.eq.blank) then 
         goto 120 
      end if 

      ixct = ixct + 1
      if (ixct.gt.h8) call error (13,r,i,'BUILD')
      exname(ixct) = name
      goto 120 
c                             solution phases:
130   read (n1,1000) sname
      if (sname.eq.blank) goto 130
c                             isoct - solution phase counter,
c                             io9 is a flag = 0 no solution file
      isoct = 0
      if (sname.ne.'begin solu') goto 998

140   read (n1,1000) sname

      if (sname.eq.'end soluti') then 
         if (io9.eq.1) isoct = 0 
         goto 150
      else if (name.eq.blank) then 
         goto 140 
      end if 

      isoct = isoct + 1
      if (isoct.gt.h9) call error (25,r,i,'BUILD')
      fname(isoct) = sname
      goto 140 
c                             read the maximum pressure, temper-
c                             ature, xco2, u1, and u2; the minimum
c                             pressure temperature, xco2, u1, and u2;
c                             and the default pressure, temperature,
c                             xco2, and chemical
c                             potential increments use kelvins, bars and
c                             joules as units (if no mobile components
c                             enter two zeroes for each read).
150   read (n1,*,err=998) vmax
      read (n1,*,err=998) vmin
      read (n1,*,err=998) dv
c                             read the default indices of the
c                             dependent, independent, and secondary
c                             independent intensive variables, p = 1,
c                             t = 2, and xco2 = 3, respectively.
      read (n1,*,err=998) (iv(i), i = 1, 5)
c                             check to make sure input requests are
c                             consistent:
      if (icopt.ne.0.and.icopt.ne.4) then
c                             first check iv(1):
         if (iv(1).eq.3.and.ifyn.eq.1) call error (110,r,i,'I')
         if (iv(1).eq.3.and.ifct.eq.1) then 
            if (icopt.ne.7.and.iv(2).ne.3) call error (111,r,i,'I')
         end if 
         if (vmin(iv(1)).ge.vmax(iv(1)).and.icopt.ne.9
     *                                 .and.icopt.ne.7.and.icont.lt.3) 
     *                                 call error (112,r,i,'I')
     
         if (icopt.ne.5.and.icopt.ne.7.and.icopt.ne.9) then 
            if (dv(iv(1)).eq.0d0) call error (113,r,i,'INPUT1')
            dip = dv(iv(1))/(vmax(iv(1))-vmin(iv(1)))
            if (dip.lt.0.01) call warn (114,dip,i,'INPUT1')
            if (dip.lt.0.001) call error (115,dip,i,'I')
         end if 
         if (vname(iv(1)).eq.blank) call error (116,dip,i,'I')
      end if
c                             now check iv(2):
      if (icopt.eq.1) then
         if (iv(2).eq.3.and.ifyn.eq.1) call error (110,r,i,'INPUT1')
         if (iv(2).eq.3.and.ifct.eq.1) call error (111,r,i,'INPUT1')
         if (vmin(iv(2)).ge.vmax(iv(2))) call error (112,r,i,'I')
         if (icopt.ne.5) then 
            if (dv(iv(2)).eq.0d0) call error (113,r,i,'INPUT1')
            dip = dv(iv(2))/(vmax(iv(2))-vmin(iv(2)))
            if (dip.lt.0.01d0) call warn (114,r,i,'INPUT1')
            if (dip.lt.0.001d0) call error (115,r,i,'INPUT1')
         end if 
         if (vname(iv(2)).eq.blank) call error (116,r,i,'INPUT1')
      end if
c                             if a chemical potential is specified as an
c                             independent variable (iv(1-3)), check if
c                             the variable is defined:
      kct = 0
      do i = 1, 3
         if (iv(i).gt.3) kct = kct + 1
      end do 
c                             identify the variable used to determine
c                             which phases lie on the left hand side
c                             of a reaction equation.
      if (icopt.eq.3) then
         ivfl = iv(1)
      else if (iv(1).eq.2.or.iv(2).eq.2) then
c                             choose T
         ivfl = 2
      else if (iv(1).eq.1.or.iv(2).eq.1) then
c                             no T, so choose P
         ivfl = 1
      else
c                             no P or T, choose independent V
         if (iv(2).ne.3) then
            ivfl = iv(2)
         else
            ivfl = iv(1)
         end if
      end if
c                             ok, now find out which variables are
c                             dummies and story the indexes of the
c                             non-dummy variables in jv.
      ipot = 0
      do i = 1, 5
c                             variables v(1) (p) and v(2) (t) are
c                             only dummies if idep is set.
         if ((iv(i).ne.idep.or.icopt.eq.7.or.icopt.eq.9).and.
     *       (iv(i).eq.1.or.iv(i).eq.2)) then
            ipot = ipot+1
            jv(ipot) = iv(i)
c                             variable v(3) is a dummy if ifyn = 1:
         else if ((iv(i).eq.3).and.(ifyn.eq.0)) then
            ipot = ipot+1
            jv(ipot) = iv(i)
c                             variables v(4) and v(4) are dummies if
c                             imyn = 1:
         else if (jmct.ne.0) then
            if (iv(i).eq.4) then
               ipot = ipot+1
               jv(ipot) = iv(i)
            else if (iv(i).eq.5.and.jmct.eq.2) then
               ipot = ipot+1
               jv(ipot) = iv(i)
            end if
         end if
      end do 
c                             compute interval range of variables
      do i = 1, 5
         delv(i) = vmax(i) - vmin(i)
      end do  
c                              if jtest = 4, write a list of reactions for
c                              stefano
      if (icopt.eq.1.and.jtest.eq.3) then 
         open (69,file='reaction_list.dat')
         write (*,1250)
      end if 
c                              set convergence criteria for routine
c                              univeq: (why bother with this conditional?)
      if (icopt.ne.7) call concrt
      goto 99

998   call error (27,r,i,n2name)

1000  format (a,a)
1010  format (a2,a5,a1)
1040  format ('**error ver191** FOPEN cannot find file:',/,a,/,
     *        'Try again (y/n)?')
1050  format ('Enter computational option file name (i.e. the file',
     *        ' created',/,'with BUILD), left justified: ')
1120  format (a,1x,i2)
1170  format (/,'Reading thermodynamic data from file: ',a)
1180  format (/,'Writing print output to file: ',a)
1190  format (/,'Writing plot output to file: ',a)
1200  format (/,'Writing pseudocompound glossary to file: ',
     *          'pseudocompound_glossary.dat')
1210  format (/,'Reading solution models from file: ',a)
1220  format (/,'Writing bulk composition plot output to file: ',a)
1240  format (/,'Initializing polygon output file: ',a,/)
1250  format (/,'Writing complete reaction list to file: ',
     *          'reaction_list.dat')

99    if (icopt.ne.0) close (n1)

      end

      subroutine input2 (icmpn)
c----------------------------------------------------------------------
c                     input2
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*5 cname, cmpnt, dname*40, zname, name*8, exname*8

      double precision twt(k5), cst
 
      integer icout(k0)
 
      common/ cst10 /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst1  /thermo(k4,k10),uf(2),us(h5)
     *      / cst208 /ifct,idfl/ cst12 /cp(k5,k1)
     *      / cst3  /ctot(k1)/ cst36 /exname(h8)
     *      / cst209 /iwt/ cst209a /zname/ cst6 /icomp,istct,iphct,icp
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst5  /p,t,xco2,u1,u2,tr,pr,r,ps/ cst113 /ifproj
     *      / cst37 /ixct,iexyn/ csta4 /cname(k5)
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / csta5 /dname(8),cmpnt(k0)/ cst42 /ic(k5),idbase
     *      / cst60 /ipoint,imyn/ csta6 /name/ cst18 /jvol(2),ivol
     *      / cst43 /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont
     *      / cst202 /tm(m7,m6),td(m8),ilam,idiso,lamin,idsin
c-----------------------------------------------------------------------
c                               initialization for each data set
 
c                               for k1 phases:
      do i = 1, k1
         ikp(i) = 0
      end do 
c                               other counters and flags:
      do i = 1, h5
         isct(i) = 0
      end do 
c                               counters for bounds
      iphct = 0
      lamin = 0 
      idsin = 0 
      idfl = 0
      ivol = 0
c                               read data base header, do component
c                               transformations
      call topn2 (0,icmpn)
c                               general input data for main program

c                               reorder thermodynamic components
c                               if the saturated phase components are 
c                               present
      ict = 0 
c                               first component 1st
      do i = 1, icp
         if (cname(i).eq.cmpnt(idh2o)) then 
            ict = 1
            if (i.eq.1) goto 1
            cname(i) = cname(1)

            do j = 1, 3
               cst = dblk(j,i)
               dblk(j,i) = dblk(j,1) 
               dblk(j,1) = cst
            end do 

            cname(1) = cmpnt(idh2o)
            ict = 1
            goto 1            
         end if 
      end do 
c                               now check second component
1     do i = 1, icp
         if (cname(i).eq.cmpnt(idco2)) then 
            if (i.eq.ict+1) goto 2
            cname(i) = cname(ict+1)

            do j = 1, 3
               cst = dblk(j,i)
               dblk(j,i) = dblk(j,ict+1) 
               dblk(j,ict+1) = cst
            end do 

            cname(ict+1) = cmpnt(idco2)
            ict = 1
            goto 1            
         end if 
      end do 
c                              load the old cbulk array
2     if (ifyn.ne.1) iphct = 2
c                               identify nonzero components.
c                               initialize icout(i) = 0
      do i = 1, icmpn
         icout(i) = 0
      end do

      do i = 1, icomp
         im = 0
         do j = 1, icmpn
            if (cname(i).eq.cmpnt(j)) then 
               twt(i) = atwt(j)
               ic(i) = j
               icout(j) = 1
               if (j.eq.idh2o) then
                  iff(1) = i
                  idfl = idfl + 1
                  jvol(idfl) = i
               else if (j.eq.idco2) then
                  iff(2) = i
                  idfl = idfl + 1
                  jvol(idfl) = i
               end if 
               im = 1
            end if 
         end do 

         ivol = idfl
c                               write error message if a component
c                               was not found:
         if (im.eq.0) then 
            write (n3,1230) cname(i), (cmpnt(k), k = 1, icmpn)
            write (n3,1240)
            pause
            stop
         end if 
 
      end do 
cjc                            check if a fluid is in the thermodynamic
cjc                            composition space:
      ifproj = 0
      if (ifct.eq.0) then 
c                              special cases, the first component is not
c                              H2O
           do i = 1, icp

            if (cname(1).eq.cmpnt(idh2o)) then
               if (cname(2).eq.cmpnt(idco2)) then
c                              both components present:
                  ifproj = 3
               else 
c                              only water present
                  ifproj = 1
               end if 
            else if (cname(1).eq.cmpnt(idco2)) then
c                              only co2 present
               ifproj = 2
            end if
         end do  
      end if 
c                              this segment is to check if
c                              a possible saturated phase component
c                              has been made a mobile component,
c                              if there is also a saturated phase
c                              component idfl is the identity of the
c                              mobile component otherwise idfl = 0.
      if (ifct.eq.1.and.idfl.eq.2) then
         if (zname.eq.cmpnt(idh2o)) then
            idfl = 1
         else if (zname.eq.cmpnt(idco2)) then
            idfl = 2
         end if
      else 
         idfl = 0
      end if
c                               load atwts in updated order
      do i = 1, icomp
         atwt(i) = twt(i)
      end do 
c                               convert weight to molar amounts
      if (jbulk.ne.0) then 

         if (iwt.eq.1) then 
            do i = 1, jbulk
               do j = 1, 3
                  dblk(j,i) = dblk(j,i)/atwt(i)
               end do 
            end do 
         end if 

         do i = 1, jbulk
            cblk(i) = dblk(1,i)
         end do   

      end if 

      
c                               begin first read loop for data on
c                               saturated components.
      if (isyn.ne.0.and.ifyn.ne.0) goto 40
 
      ifer = 0
 
50    call getphi (name,ibase,icmpn,*59)
 
      call chkphi (0,name,tot,icout,ibase,icmpn,*50)
 
      if (ifyn.eq.0) then
c                               check for fluid species data
         if (name.eq.cmpnt(idco2)) then
            j = 2
            ifer = ifer + 1
         else if (name.eq.cmpnt(idh2o)) then
            j = 1
            ifer = ifer + 1
         else
            goto 70
         end if
 
         call loadit (j)
 
         goto 50
      end if
 
70    if (isyn.eq.1) goto 50
c                               check for saturated phases:
c                               reject the phase if it contains
c                               a thermodynamic component:
      do j = 1, icp
         if (comp(ic(j)).ne.0d0) goto 50
      end do
c                               now load the phase if it has
c                               the saturated component idc:
      do j = isat, 1, -1
         idc = icp + j
         if (comp(ic(idc)).ne.0d0) then
            isct(j) = isct(j) + 1
            if (isct(j).gt.h6) call error (17,r,h6,'INPUT2')
            iphct = iphct + 1
            if (iphct.gt.k1) call error (180,r,k1,'INPUT2')
            ids(j,isct(j)) = iphct
            call loadit (iphct)
            goto 50
         end if
      end do 
 
      goto 50
c                               check that there is data for
c                               every fluid component.
59    if (ifyn.eq.0.and.ifer.ne.ifct) call error (36,r,i,'INPUT2')
c                               check that there is one phase
c                               for each saturation constraint
40    do i = 1, isat
         if (isct(i).lt.1) call error (15,r,i,cname(icp+i))
      end do
c                               read data for the remaining
c                               phases of appropriate composition.
      istct = iphct + 1
c
      rewind n2
c                               read till end of header
      call eohead (n2)
c                               begin second read loop:
220   call getphi (name,ibase,icmpn,*999)
c                               check if valid phase:
      call chkphi (1,name,tot,icout,ibase,icmpn,*220)
c                               acceptable data, count the phase:
      iphct = iphct + 1
c                               for normalized composition:
      ctot(iphct) = tot
c                               store thermodynamic parameters:
      call loadit (iphct)
c                               next phase:
      goto 220
1230  format ('**error ver013** ',a,' is an incorrect component'
     *       ,' name, valid names are:',/,12(1x,a))
1240  format ('check for upper/lower case matches or extra blanks',/)

999   close (n2)
      end

      subroutine outchm
c-------------------------------------------------------------------
c outchm writes new chemographies to unit n3 as generated.
c flag ichem (1) determines if outchm is called, no output will be 
c generated if flag io3 = 1 (see introductory comments).
c-------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 names,vname,xname
 
      common/ cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst5  /v(l2),tr,pr,r,ps/ cst6 /icomp,istct,iphct,icp
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ cst79 /isoct   
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst8  /names(k1)/ cst96 /idcf(k5,j9),icfct
     *      / cst60 /ipoint,imyn/ cst27 /jasmbl(j9),iasmbl(j9)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / csta2 /xname(k5),vname(l2)
c-----------------------------------------------------------------------
c
      if (io3.eq.1) goto 999
c                             output independent potential variable
c                             constraints:
      write (n3,1160)
      write (n3,1000)
      write (n3,1010) (vname(jv(i)), v(jv(i)), i = 1, ipot)
      write (n3,1015)
c                             next the stable assemblages:
      if (icp.gt.4) then 
c                             output high order chmographies
         do i = 1, icfct
            write (n3,7020) iasmbl(i), (names(idcf(j,i)), j = 1, icp)
         end do
      else if (icp.eq.1) then  
c                             output unary chemographies.
         write (n3,1020) names(idcf(1,1))
      else if (icp.eq.2) then 
c                             output binary chemographies.
         write (n3,2020) ((names(idcf(j,i)), j = 1, icp), iasmbl(i), 
     *                 i = 1, icfct)
      else if (icp.eq.3) then 
c                             output ternary chemographies:
         write (n3,3020) ((names(idcf(j,i)), j = 1, icp), iasmbl(i), 
     *                i = 1, icfct)
      else if (icp.eq.4) then 
c                             output quaternary chemographies:
         write (n3,4020) ((names(idcf(j,i)), j = 1, icp), iasmbl(i), 
     *                i = 1, icfct)
      end if 
c                             output phases consistent with component
c                             saturation constraints:
      if (isyn.eq.1) goto 9000

      write (n3,6000)
      write (n3,2030) (names(idss(i)), i = 1, isat)
c
9000  if (icp.eq.1) goto 999

      write (n3,*)
c                             write miscibility flag:
      if (isoct.eq.0) goto 999

      if (imyn.eq.1) then
         write (n3,6010) 
      else
         write (n3,6030) 
      end if
 
1000  format ('the stable assemblages at:',/)
1010  format (25x,a8,' = ',g12.6)
1015  format (/,'are (variance flag in parentheses):',/)
1020  format (25x,a8)
1160  format (/,80('-'),/)
2020  format (3(a8,'-',a8,'(',i1,')',3x))
2030  format (6(1x,a8))
3020  format (2(2(a8,'-'),a8,'(',i1,')',2x))
4020  format (2(3(a8,'-'),a8,'(',i1,')'))
6000  format (/,'these assemblages are compatible with the followi',
     *        'ng phases or species',/,'determined by component ',
     *        'saturation or buffering constraints:',/)
6010  format ('** no immiscibility occurs in the stable solution ',
     *        'phases **',/)
6030  format ('** immiscibility occurs in one or more of the ',
     *        'stable solution phases **',/)
7020  format ('(',i1,')',12(1x,a8))
 
999   end
 
      subroutine outhed
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 names,vname,xname,title*72,fname*10
 
      common/ cst3   /ctot(k1)/ cst8 /names(k1)/ cst12 /cp(k5,k1)
     *      / csta2  /xname(k5),vname(l2)
     *      / cst40  /ids(h5,h6),isct(h5),icp1,isat,io2/ csta8 /title
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst6   /icomp,istct,iphct,icp/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst60  /ipoint,imyn/ cst96 /idcf(k5,j9),icfct
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst103 /isec,icopt,ifull,imsg,io3p/ cst79 /isoct
     *      / cst27  /jasmbl(j9),iasmbl(j9)/ csta7 /fname(h9)
c-----------------------------------------------------------------------
c                             number of components, phase counters,
c                             assemblage counter, fluid saturation flag,
c                             component saturation flag, ipot = number
c                             of independent potential variables:
      write (n4,1020) icopt
c
      write (n4,1020) icp,istct,iphct,ipoint,ifyn,isyn,ipot,isoct
c                             write graphics code variable names:
      write (n4,1000) (vname(jv(i)), i = 1, ipot)
c                             write a blank record as left caption
      write (n4,1000) title
c                             write phase names
      write (n4,1015) (names(i), i = 1, iphct)
c                             write phase coordinates
      write (n4,1025) ((cp(j,i)/ctot(i), j = 2, icp), i = istct, iphct)
c                             write solution phase flags:
      write (n4,1090) (ikp(i), i = 1, iphct)
c                             solution names
      if (isoct.ne.0) write (n4,1080) (fname(i),i = 1, isoct)
c                             write graphics code names for x variables:
      write (n4,1015) (xname(i), i = 1, icp)

1000  format (a)
1015  format (10a8)
1020  format (20(i8,1x))
1025  format (9(f8.6,1x))
1080  format (8a10)
1090  format (20(i4,1x))
 
      end

      subroutine outgrf
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst40  /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst6   /icomp,istct,iphct,icp/ cst5  /v(l2),tr,pr,r,ps
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst96  /idcf(k5,j9),icfct/ cst27  /jasmbl(j9),iasmbl(j9)
     *      / cst24 /ipot,jv(l2),iv(l2)
c-----------------------------------------------------------------------
c                             stable configurations, phases are
c                             labelled by the index 'i' in the
c                             list of phases.
c                             write values of the independent potentials
      write (n4,1040) (v(jv(i)), i = 1, ipot)

      if (icp.ne.2) then
         write (n4,*) icfct
      else
         write (n4,*) icfct+1
      end if 

      if (icp.eq.2) then 
c                             binary is a special case (1-d)
         write (n4,1020) (idcf(1,j), j = 1, icfct),idcf(2,icfct)

      else if (icp.eq.1) then 
         goto 99
      else 
c                             higher order:
         write (n4,1020) ((idcf(j,i), j = 1, icp), i = 1, icfct)
      end if 
c                             write assemblage flags
      if (icp.gt.2) write (n4,1020) (iasmbl(j), j = 1, icfct)

      if (isyn.eq.1) goto 99

      write (n4,1020) isat
      write (n4,1020) (idss(i), i = 1, isat)

1020  format (20(i8,1x))
1040  format (5(g12.6,1x))

99    end
 
      subroutine prtpot
c---------------------------------------------------------------------
c prtpot outputs the dependent potentials for phase assemblages
c determined by the combin subroutine. it is called if the flag
c jpot, in common debug, is not equal 1.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names

      common/ cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst23 /a(k5*k5),u(k5+1),iar(2*k5+4)/ cst8 /names(k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst3 /ctot(k1)
     *      / cst12 /cp(k5,k1)
     *      / cst52 /h,id(k5+1),ic(k5)/ debug /jtest,jpot

      call abload (*99)

      if (jpot.ne.0) goto 100

      if (icp.gt.5) then 
         write (n3,1060) (names(id(i)),i=1,icp)
         write (n3,1070) (u(i),i=1,icp)
      else if (icp.eq.1) then 
         write (n3,1010) names(id(1)),u(1)
      else if (icp.eq.2) then 
         write (n3,1020) (names(id(i)),i=1,icp),(u(i),i=1,icp)
      else if (icp.eq.3) then 
         write (n3,1030) (names(id(i)),i=1,icp),(u(i),i=1,icp)
      else if (icp.eq.4) then 
         write (n3,1040) (names(id(i)),i=1,icp),(u(i),i=1,icp)
      else if (icp.eq.5) then 
         write (n3,1050) (names(id(i)),i=1,icp),(u(i),i=1,icp)
      end if 

      goto 99 

100   do i = 1, icp
            jd = id(i) 
            g = 0d0
            do j = 1, icp
               g = g + cp(j,jd) * u(j)/ctot(jd)
            end do 
         write (n3,1080) names(id(i)),g,(cp(j,jd)/ctot(jd),j=1,icp) 
      end do 

1010  format (1x,a8,1x,g14.7)          
1020  format (2(1x,a8),2(1x,g14.7))
1030  format (3(1x,a8),3(1x,g14.7))
1040  format (4(1x,a8),4(1x,g14.7))
1050  format (5(1x,a8),5(1x,g14.7))
1060  format (16(1x,a8))
1070  format (16(1x,g14.7))
1080  format (1x,a8,16(1x,g14.7))

99    end 

      subroutine newhld 
c-----------------------------------------------------------------------
c newhld replaces nohold.  newhld is designed to do boundary traverses
c for individual equilibrium configurations and in certain cases,
c specifically for systems with fewer than 4 components, is 
c slower than nohold. in contrast to newhld, nohold simultaneously
c monitored all assemblages on the thermodynamic surface of the system.
c a copy of the nohold code is stored in check.fortran on tape xyw003.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 vname,xname*8,cname*5

      double precision value, vt 

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ csta4 /cname(k5)
     *      / cst6 /icomp,istct,iphct,icp/ cst96 /idcf(k5,j9),icfct
     *      / cst5  /v(l2),tr,pr,r,ps/ cst7 /iflag/ cst38 /iedge
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)/ cst39 /ichem
     *      / csta2 /xname(k5),vname(l2)/ cst102 /ivfl
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ cst80 /irchk(k2)
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst31 /vn(k2,k7),irct,ird/ cst49 /value,ismax,igot
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst65 /vt(j9),vti(j9),jok(j9),kok(j9),index
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b 
     *      / cst87 /delt(l2),dtol,utol,ptol     
c-----------------------------------------------------------------------
c                                  initialization:
      irend = 0
      irct = 0
 
      do i = 1, k2
         irchk(i) = 0
      end do 
c                                  set starting values for search
      v(iv1) = vmin(iv1)
      call incdep (iv1,v(iv1))
      v(iv2) = vmin(iv2)
      call incdep (iv2,v(iv2))

c                                  determine the stable chemography.
      call gall 
      call combin

      if (icopt.eq.3) then 
         iedge= 1
         if (io4.eq.0) then 
c                             write chemographies for mixed variable
c                             diagrams:
            call outgrf
c                             write min and max values for the
c                             independent potentials:
            write (n4,*) vmin(iv1),vmax(iv1)
         end if 
      end if
c                                  output chemography if requested.
c                                  this option is archaic with newhld.
      if (ichem.eq.0) call outchm  
c                                  write tier header to print file:
      if (io3.eq.0) then 
      write (n3,1160)
      write (n3,1170) vname(iv1),vname(iv2)
c                                  write potential variable sectioning
c                                  constraints:
      if (ipot.gt.2) then
         write (n3,1180)
         write (n3,1190) (vname(jv(i)),v(jv(i)), i = 3, ipot)
      end if
c                                  write saturated and buffered
c                                  component names:
      if (isyn.eq.0) write (n3,1200) (cname(i+icp), i= 1, isat)
c                                  write dependent extensities blurb
      write (n3,1210)
      write (n3,1230) vname(ivfl)

      write (n3,1160)

      if (io3p.eq.1) then
         write (n3,1240)
         write (n3,1160)
      end if
      end if 
c                                  initialize start parms:
      do i = 1, icfct
         vt(i)  = vmin(iv1)
         vti(i) = vmin(iv2)
         kok(i) = 1
         jok(i) = 1
      end do 
      
      write (*,1000) icfct
c                                  look for reactions involving each
c                                  assemblage.
      knct = 1
40    jnct = icfct
c
      do 50 i = knct, jnct
c                                  save the current position for chkass
         index = i 

         if (imsg.eq.0) write (*,1220) i,icfct-i
c                                  load id's into array idv:
         do j = 1, icp
            idv(j) = idcf(j,i)
         end do 
c                                  get the lower/upper decomposition
c                                  of the transpose of the concentration
c                                  matrix:
         call pivots (ier)
         if (ier.eq.1) then
c                                  error, a degenerate assemblage in
c                                  pivots due to a programming error.
            call warn (29,v(1),ier,'NEWHLD')
            goto 50
         end if 
c                                  set flag for maxend:
         ismax = jok(i) 
         igot = 0
         if (ismax.eq.1) then 
            value = vmin(iv1)
         else if (ismax.eq.2) then 
            value = vmin(iv2)
         else if (ismax.eq.3) then
            value = vmax(iv1) 
         else 
            value = vmax(iv2)
         end if 
c                                  test stability along an edge of the
c                                  diagrams coordinate frame:
30       kok(i) = jok(i)
         call search (vt(i),vti(i),jok(i),iste,ivi,ivd,div,ier)
c                                  ier =2 error from search.
c                                  ier =1 stable at all conditions.
         if (ier.eq.1.or.ier.eq.2) goto 50
c                                  call coface to delineate all the
c                                  equilibria that are cofacial with
c                                  the simplex defined by the vertices
c                                  idv.
         call coface (ivd,ivi,div,iste,irend)
c                                  after tracing the net attempt
c                                  to resume testing the stability
c                                  of the assemblage:
         if (igot.eq.1) then
            if (ismax.eq.1) then
c                                  resume search of travserse 1:
               vt(i) = value + 1d1*delt(iv1)
               vti(i) = vmin(iv2)
            else if (ismax.eq.2) then
c                                  resume search of travserse 2:
               jok(i) = 2
               vt(i) = value + 1d1*delt(iv2)
               vti(i) = vmax(iv1)
            else if (ismax.eq.3) then
c                                  resume search of travserse 3:
               jok(i) = 3
               vt(i) = value - 1d1*delt(iv1)
               vti(i) = vmax(iv2)
            else
c                                  resume search of travserse 4:
               jok(i) = 4
               vt(i) = value - 1d1*delt(iv2)
               vti(i) = vmin(iv1)
            end if
c                                  test for stability before search? 
            do j = 1, icp
               idv(j) = idcf(j,i)
            end do 
            call pivots (ier)
            igot = igot + 1
            goto 30 
         end if 
50    continue
      if (icfct.eq.jnct) goto 90
      knct = jnct + 1
      goto 40
c                                  return for icopt=1
90    if (icfct.eq.j9) call warn (205,r,j9,'NEWHLD')
c                                  for icopt=3 sort and output
c                                  reactions
      if (icopt.eq.3) call onedim
      
1000  format ('Initial number of',
     *        ' divariant assemblages to be tested is:',i6)
1160  format (/,80('-'),/)
1170  format ('The ',a,'-',a,' loci of (pseudo-) univariant fields'
     *        ,' follow:')
1180  format (/,'the fields are subject to the constraint(s):'/)
1190  format (25x,a,'=',g12.6)
1200  format (/,'These fields are consistent with saturation or ',
     *            'buffering constraints',/,'on the component(s):',
     *            7(a,2x))
1210  format (/,'NOTE: For each field the values of the dependent ',
     *          ' extensities are output',/,'for the first',
     *          ' equilibrium condition, in general these properties',
     *          ' vary with',/,'the independent potentials.',/)
1220  format ('Testing divariant assemblage ',i6,', ',i6,
     *        ' assemblages remaining to be tested.')
1230  format (/,'Reaction equations are written such that the high ',
     *          a,/,'assemblage is on the right of the = sign',/)
1240  format (//,'You have chosen the short form print file. ',/,
     *           'If you want equilibrium coordinates output select',
     *           /,'the long form when you run BUILD.',//)
      end
 
      subroutine satsrt 
c---------------------------------------------------------------------
c routine to sort pseudocompounds consisting entirely of saturated
c components.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
  
      include 'perplex_parameters.h'
 
      common/ cst40 /ids(h5,h6),isct(h5),icp1,isat,io2/ cst12 /cp(k5,k1)
     *      / cst42 /ic(k5),idbase/ cst6 /icomp,istct,iphct,icp
 
      do j = isat, 1, -1
         idc = icp + j
         if (cp(idc,iphct).ne.0d0) then
            isct(j) = isct(j) + 1
            if (isct(j).gt.h6) call error (17,cp(1,1),h6,'SATSRT')
            if (iphct.gt.k1) call error (180,cp(1,1),k1,'SATSRT')
            ids(j,isct(j)) = iphct
            goto 99
         end if
      end do 
99    end
 
      function gproj (id)
c-----------------------------------------------------------------------
c gproj computes the projected molar free energy of the
c phase with index id.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'

      common/ cst1 /thermo(k4,k10),uf(2),us(h5)/ cst12 /cp(k5,k1)
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst6  /icomp,istct,iphct,icp/ cst208 /ifct,idfl
c-----------------------------------------------------------------------
      call gphase (id,gph)
c                                 this is a screw up solution
c                                 necessary cause uf(1) and uf(2)
c                                 are allocated independent of ifct!
      if (ifct.gt.0) then 
         do j = 1, 2
            if (iff(j).ne.0) gph = gph - cp(iff(j),id)*uf(j)
          end do 
      end if 

      do j = 1, isat
         gph = gph - cp(icp+j,id) * us(j)
      end do 

      gproj = gph

      end

      subroutine grxn (gval) 
c-----------------------------------------------------------------------
c grxn computes the free energy of univariant equilibria
c defined by the data in commonn block cst21 which is initialized
c in the subprogram balanc.  grxn is partially redundant with
c the function gphase but because of the frequency that these
c these routines are used a significant increase in efficiency is
c gained by maintaining separate functions.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst60 /ipoint,imyn/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst11 /fh2o,fco2
     *      / cst201 /vuf(2),vus(h5),iffr,isr
     *      / cst6  /icomp,istct,iphct,icp/ cst25 /vnu(k7),idr(k7),ivct
c-----------------------------------------------------------------------
c                                 compute potentials of saturated phases
c                                 and components, note that in this
c                                 version of vertex the stoichiometry of
c                                 such components may vary.

c                                 no saturated phase components and no
c                                 saturated components:
      if (iffr.eq.1.and.isyn.eq.1) goto 10
c                                 note that this call to uproj makes a
c                                 subsequent call in gall redundant if
c                                 sfol1 is used to trace a univariant
c                                 curve.
      call uproj
c                                 compute free energy change of the rxn
10    gval = 0d0

      do j = 1, ivct
         gval = gval + vnu(j) * gproj(idr(j))
      end do 

      end

      subroutine soload (z,isoct,icpct,ixct,tname,icky)
c--------------------------------------------------------------------------
c soload - loads/requires solution properties: 

c   jend(h9,k12)  - h9 is the maximum number of solutions
c                   k12 is the maximum number of endmembers pers
c                   solution plus two.
c   jend(i,1)     - is the number of endmembers in solution i.
c   jend(i,2)     - is the number of pseudocompounds of solution i.
c   jend(i,3-3+j) - are the indices of the j endmembers in solution i.
c   sxs(k13)      - contains the mole fractions of the endmembers
c                   in the pseudocompounds.
c   ixp(i)        - a pointer that locates the first mole fraction (- 1)
c                   of the ith pseudocompound, the remaining mole fractions
c                   follow sequentially (as in jend(i,3-3+j)).
c   ikp(i)        - the index of the solution corresponding to pseudocompound i.
c   ifp(i)        - flag, 0  - margules/ideal
c                         1  - internal eos fluid                         
c                         3  - van laar
c                         23 - toop, internal
c   exces(j,i)    - the excess function of pseudocompound i, accounts for 
c                   excess properties and configurational entropy as a function
c                   of pressure and temperature:

c                       gexces(i) = exces(1) + exces(2)*T + exces(3)*P 
c--------------------------------------------------------------------------

      implicit double precision (a-g,o-z),integer (h-n)
  
      include 'perplex_parameters.h'

      character*10 tname, names*8
 
      double precision z(mst,msp),zp(m4),zpt(m4),zpr,y2p,pa(m4),nu,dqf

      integer mdep,jdep,idep,indq,idqf

      common/ cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst222 /dqf(m3,m4),indq(m4),idqf
     *      / cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *     iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),isp(mst),
     *     isite,iterm,iord,istot,jstot,kstot
     *      / cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst1  /thermo(k4,k10),uf(2),us(h5)
     *      / cst6  /icomp,istct,iphct,icp/ cst8 /names(k1)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst5  /p,t,xco2,u1,u2,tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b/ cst3 /ctot(k1)
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst319 /emod(k15,k10),iemod(k1),kmod
     *      / cst221 /vlaar(m3,m4),jsmod/ cst140 /inew
     *      / cst142 /jmsol(m4,mst),kdsol(m4)
     *      / cst146 /nu(m15,3),y2p(m4,m15),mdep,jdep(m15),idep(m15,3)
     *      / cst141 /depvnu(2),depenth,iddeps(2)
c----------------------------------------------------------------------
c                              eliminate end-member compositions 
      do l = 1, istot
         pa(l) = 0d0
         zpt(l) = 1d0
         do m = 1, isite
c                              check for invalid compositions
            call zchk (z(m,jmsol(l,m)),tname)
            zpt(l) = zpt(l)*z(m,jmsol(l,m))
         end do
c                              zp is the mole fraction of endmember l
         if (zpt(l).gt.0.9999d0.and.kdsol(l).gt.0) goto 999
c                              reject the compositions if missing
c                              endmember
         if (kdsol(l).eq.0.and.zpt(l).ne.0d0) goto 999
      end do   
c                              the composition is acceptable.

c                              move site fractions into array indexed 
c                              only by present endmembers:
      do i = 1, jstot
         zp(i) = zpt(insp(i))
      end do 

      iphct = iphct + 1
      icpct = icpct + 1 

      if (iphct.gt.k1) call error (41,z(1,1),k1,'SOLOAD')

      ikp(iphct) = isoct
      ixp(iphct) = ixct
      icky = 0 

      do i = 1, m3
         exces(i,iphct) = 0d0
      end do
c                               inew = 0 new format
      if (inew.eq.0) then 
c                               classify model:
      if (jsmod.eq.0) then
c                               fluid uses internal eos
         ifp(iphct) = 1
c                               don't allow gonzoids to treat fluid
c                               as a solution if the user has also 
c                               specified fluid saturation
         if (ifyn.eq.0) call error (43,r,i,tname)  
      else if (jsmod.eq.1.or.jsmod.eq.2.or.jsmod.eq.7.or.jsmod.eq.24.or.
     *         jsmod.eq.25) then
c                               no special treatment in computational
c                               routines.
         ifp(iphct) = 0 
      else if (jsmod.eq.26) then 
c                               andreas's H2O-CO2-NaCl internal EoS
         ifp(iphct) = 26
c                               other special cases (internal solution
c                               models)
      else if (jsmod.gt.2) then
c                               Toop, internal EoS
c                               Van Laar ala HP.
         ifp(iphct) = jsmod    
      else 
c                               Ideal or Margules.
         ifp(iphct) = 0
      end if 

      else 
c                               check if the solution is a fluid:
      if (thermo(3,kdsol(insp(1))).eq.0d0) then
         ifp(iphct) = 1
c                               don't allow gonzoids to treat fluid
c                               as a solution if the user has also 
c                               specified fluid saturation
         if (ifyn.eq.0) call error (43,r,i,tname)  
c                               other special cases (internal solution
c                               models)
      else if (tname.eq.'Toop-Melt') then
         ifp(iphct) = 23    
      else
         ifp(iphct) = 0
      end if 
      end if 
c                                encode a name
      if (isite.eq.2.and.istot.eq.4.and.jstot.eq.4) then
c                                special case 1, bin-bin reciprocal solution
          write (names(iphct),1020) tname,
     *                              (idint(1d2*z(j,1)), j = 1, 2)
      else if (isite.eq.2.and.istot.eq.6.and.jstot.eq.6.and.
     *                                           isp(1).eq.3) then
c                                special case 2, tern-bin reciprocal solution
          write (names(iphct),1060) tname,
     *                              (idint(1d2*z(1,j)), j = 1, 2),
     *                               idint(1d2*z(2,1))
      else if (isite.eq.2.and.istot.eq.6.and.jstot.eq.6.and.
     *                                           isp(1).eq.2) then
c                                special case 3, bin-tern reciprocal solution
          write (names(iphct),1060) tname,
     *                               idint(1d2*z(1,1)),
     *                              (idint(1d2*z(2,j)), j = 1, 2)
      else if (isite.eq.2.and.istot.eq.9.and.jstot.eq.9) then
c                                special case 4, tern-tern reciprocal solution
          write (names(iphct),1010) (idint(1d2*z(1,j)), j = 1, 2),
     *                              (idint(1d2*z(2,j)), j = 1, 2)

      else if (jstot.eq.2) then 
c                                binary solutions
         if (zp(1).ge.0.04d0.and.zp(1).le.0.96d0) then
              write (names(iphct),1030) names(kdsol(insp(1))),
     *                                  idint(1d2*zp(1))
         else   
              write (names(iphct),1070) names(kdsol(insp(1))), 1d2*zp(1)
         end if
      else if (jstot.eq.3) then 
c                                ternary solutions
         write (names(iphct),1040) (names(kdsol(insp(j))),
     *                                idint(1d2*zp(j)), j = 1, 2)
      else if (jstot.eq.4) then 
c                                quaternary solutions
         icky = 1
         write (names(iphct),1060) tname,
     *                    (idint(1d2*zp(j)), j = 1, 3)
      else
c                                all the rest:
         icky = 1

         if (iphct.lt.1000000) then 
            write (names(iphct),1080) tname, iphct
         else if (iphct.lt.10000000) then
            write (names(iphct),1100) tname, iphct
         else
            write (names(iphct),1110) iphct
         end if 

      end if 
c                                get blanks out of name:
      if (istot.lt.4) then 
         call unblnk (names(iphct)) 
      else 
         call reblnk (names(iphct)) 
      end if   
c                                initialize constants:
      smix = 0d0
      esum = 0d0
      atw(iphct) = 0d0

      do i = 1, icomp
         cp(i,iphct) = 0d0
      end do 
c                              load constants:
      ctotal = 0d0
      p0ord = 0d0
      ktic = 0
      jtic = 0 
      kstx = 0 

      do h = 1, kstot
c                               do not count the mole
c                               fractions of absent endmembers
         id = kdsol(insp(h)) 

         ixct = ixct + 1
         if (ixct.gt.k13) call error (40,zp,k13,'SOLOAD')

         sxs(ixct) = zp(h)

         if (jsmod.eq.7.or.jsmod.eq.8) then
c                               convert y's to p's
            do j = 1, mdep
               sxs(ixct) = sxs(ixct) + y2p(h,j) * zpt(jdep(j))
            end do 
               
            pa(insp(h)) = sxs(ixct)

            p0ord = p0ord + sxs(ixct)

         end if 

         if (sxs(ixct).ne.0d0) then
c                              composition vector
            do l = 1, icomp
               cp(l,iphct) = cp(l,iphct) + sxs(ixct) * cp(l,id)
               if (l.le.icp) ctotal = ctotal + sxs(ixct) * cp(l,id)
            end do 
c                              accumulate endmember configurational entropy
            esum = esum + sxs(ixct) * ecoef(insp(h))
c                              atomic wieght
            atw(iphct) = atw(iphct) + sxs(ixct) * atw(id)
c                              count elastic props in endmember:
            if (iemod(id).ne.0) ktic = ktic + 1

            if (iemod(id).eq.2) kstx = 1

            jtic = jtic + 1
         end if  
      end do  

      if (jsmod.eq.8) then 
c                              reciprocal solutions with speciation:
c                              expect that the fraction of the ordered endmember 
c                              is non-zero (insp(kstot+2)), first get the fraction
c                              by difference:
         kd = mdep + kstot + 1
c                              changed index to kd!!! 11/1/05.
         pa(insp(kd)) = 1d0 - p0ord
c                              now split this fraction into the fractions of the
c                              consituent disordered species:
         do j = 1, 2
            x = depvnu(j)*pa(insp(kd))
            id = kdsol(iddeps(j))
c                              composition vector
            do l = 1, icomp
               cp(l,iphct) = cp(l,iphct) + x * cp(l,id)
               if (l.le.icp) ctotal = ctotal + x * cp(l,id)
            end do 
c                              accumulate endmember configurational entropy
            esum = esum + x * ecoef(iddeps(j))
c                              atomic wieght
            atw(iphct) = atw(iphct) + x * atw(id)

         end do   
c                              restore the species fractions to sxs
         ixct = ixct - kstot
         do i = 1, jstot
            ixct = ixct + 1
            sxs(ixct) = zp(i)
         end do 
      end if 

      do l = 1, icomp
         if (cp(l,iphct).gt.-1d-8.and.cp(l,iphct).lt.0d0) then 
            cp(l,iphct) = 0d0 
         else if (cp(l,iphct).lt.0d0) then
            call error (228,cp(l,iphct),l,tname)
         end if 
      end do 
c                              if shear modulii for all endmembers:
      if (ktic.eq.jtic) then 
c                              shear modulus is of form mu(p,t)
         iemod(iphct) = 1
c                              shear modulus is of form mu(v,t)
         if (kstx.eq.1) iemod(iphct) = 2
      end if 
c                              check if the phase consists
c                              entirely of saturated components:
      if (ctotal.eq.0d0) then
         call satsrt 
      else
         ctot(iphct) = ctotal
      end if 
c                              compute ideal configurational negentropy:
      if (jsmod.eq.1.and.nsite.eq.0.and.ist(1).ne.0) then  

            do i = 1, isite 
               do j = 1, isp(i)
                  if (z(i,j).ne.0d0) smix = smix + 
     *                            float(ist(i))*z(i,j)*dlog(z(i,j)) 
               end do 
            end do     
      else if (jsmod.eq.6.or.jsmod.eq.8) then
c                               save the endmember configurational entropies

         smix = esum

      else if (nsite.ne.0) then

         if (jsmod.eq.1) then 
            call omega0 (z,dlnw,tname)
         else 
            call omega1 (zpt,dlnw,tname)
         end if 

         smix = -dlnw + esum
      else if (jsmod.eq.24) then 
c                              hp melt model, use internal routine to get entropy
         call hpmelt (zpt,dlnw)

         smix = -dlnw

      else if (jsmod.eq.25) then 
c                              ghiorso melt model, use internal routine to get entropy
         call gmelt (zpt,dlnw)

         smix = -dlnw

      end if 
c                              save it:
      exces(2,iphct) = r * smix
c                              dqf corrections are also be saved in the
c                              exces array this implies that speciation
c                              does not effect the amount of the dqf'd
c                              endmembers.
      do i = 1, idqf
c                              index points to the endmember in the full
c                              model:
         index = indq(i)
         if (kdsol(index).eq.0) cycle
c                              user has made a dqf to an ordered species
c                              or a dependent endmember
         if (kdsol(index).lt.0) call error (227,exces(1,1),index,tname)

         if (jsmod.eq.7.or.jsmod.eq.8) then
            do j = 1, m3
               exces(j,iphct) = exces(j,iphct) + pa(index)*dqf(j,i)
            end do 
         else if (jsmod.ne.1) then
            do j = 1, m3
               exces(j,iphct) = exces(j,iphct) + zpt(index)*dqf(j,i)
            end do 
         end if 
      end do 
c                              load excess terms (if not Laar format):
      if (iterm.ne.0) then
         if (jsmod.ne.3.and.jsmod.ne.6.and.jsmod.ne.8) then 
            do i = 1, iterm
               zpr = 1d0

               do j = 1, iord
                  if (jsmod.eq.1) then 
c                                        use bw site fraction expansion
                     if (isub(i,j,1).ne.0) zpr = 
     *                                  zpr * z(isub(i,j,1),isub(i,j,2))
                  else if (jsmod.eq.2.or.jsmod.eq.24.or.jsmod.eq.25) 
     *                 then 
c                                        use endmember fraction expansion
                     if (isub(i,j,1).ne.0) zpr = zpr * zpt(isub(i,j,1))
                  else if (jsmod.eq.7) then 
c                                        use independent endmember expansion
                     if (isub(i,j,1).ne.0) zpr = zpr * pa(isub(i,j,1))
                  end if 
               end do  

               do j = 1, m3
                  exces(j,iphct) = exces(j,iphct) + zpr * wg(i,j)
               end do 

            end do  
         end if 
      end if 

1000  format ('icky name ',a8,' = ',4(4(f5.3,1x,a4,1x),/))
1010  format (i2,i2,i2,i2)
1020  format (a2,i2,'_',i2)
1030  format (a2,i2)
1040  format (a1,i2,a1,i2)
1060  format (a2,i2,i2,i2)
1070  format (a2,f4.1)
1080  format (a2,i6)
1090  format (i2,'_',i2,'_',i2)
1100  format (a1,i7)
1110  format (i8)

999   end

      subroutine input9 (icmpn)
c-----------------------------------------------------------------------
c given a list of solution phase names (fname(h9)) input9 searches a
c data file (on unit n9) for the relevant data and subdivides the
c solutions into pseudo-compounds.
c-----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer istot,jstot,iend,isub,imd,jmsol,kdsol,ntot,ksp,
     *        ist,i,j,iord,iterm,isite,isp,insp,kstot,npairs,iphct,
     *        h,ms1,im,isoct,n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,hsub,
     *        io4,io5,io9,icmpn,icopt,ixp,icky,ikp,ivarip,ivarrx,
     *        icp,icomp,istct,nttyp,nsub2,nsub1,nterm,nspm1,id,ipoint,
     *        msite,mstot,knsp,io3p,imsg,ifull,isec,ifp,
     *        ivar,isudo,imyn,jcpct,icpct,jsmod,jend,nsite,inew,idsol,
     *        jm,idnm,ixct,jdep,idep,idnam,mdep,inid,ideps,jsub,jterm,
     *        lterm
  
      parameter (ms1=msp-1) 
 
      character*10 tname, uname(2)*8, sname(h9), fname, names*8, new*3,
     *             dnames*8, mname*8

      double precision y2p,nu,smult,a0,acoef,xy,xnc,xmx,dydy,deph,dvnu,
     *                 xmn,sxs,wg,y,us,uf,thermo,ecoef,zt,vlaar,exces,
     *                 vlar,wgl,sdzdy,x,w,dcoef,d0,qmult,z(mst,msp)

      common/ cst108 /wg(m1,m3),xmn(mst,msp),xmx(mst,msp),xnc(mst,msp),
     *      iend(m4),isub(m1,m2,2),imd(mst),insp(m4),ist(mst),
     *      isp(mst),isite,iterm,iord,istot,jstot,kstot
     *      / cst86 /xy(mdim,k1),y(k1,mst,ms1),ntot,npairs
     *      / cst107 /a0(m10,m11),acoef(m10,m11,m0),smult(m10),
     *     ecoef(m4),nsite,nspm1(m10),nterm(m10,m11),
     *     nsub1(m10,m11,m0,m12),nsub2(m10,m11,m0,m12),nttyp(m10,m11,m0)
     *      / cst60 /ipoint,imyn/ csta7 /fname(h9)/ cst149a /dnames(h8)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst79 /isoct/ cst6  /icomp,istct,iphct,icp
     *      / cst1  /thermo(k4,k10),uf(2),us(h5)/ cst18a /mname(m4)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst8  /names(k1)/ cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst221 /vlaar(m3,m4),jsmod/ cst140 /inew
     *      / cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst142 /jmsol(m4,mst),kdsol(m4)/ cst149 /idnam 
     *      / cst146 /nu(m15,3),y2p(m4,m15),mdep,jdep(m15),idep(m15,3)

      save uname

      data uname/' ',' '/
c-----------------------------------------------------------------------
c                               set compound counter ipoint.
      ipoint = iphct
      ixct = 0 
      idnm = 0 
c                               open pseudocompund list file
      open (n8,file="pseudocompound_glossary.dat")
c                               initialize match flag, and internal
c                               model counter
      im = 0
      jm = 0 
c                               no request for solutions, goto 999:
      if (io9.eq.1)  goto 999
c                               format test line
      read (n9,1030) new
      inew = 0 
      if (new.ne.'new'.and.new.ne.'NEW') then
         write (*,1050) 
         inew = 1
      end if 

      write (*,1090) 
c                               read the solution name
10    call rmodel (tname)

      if (istot.eq.0) goto 99
c                               check the solution model:
      call cmodel (im,idsol,tname,0,uname(1),uname(2))

      if (jstot.lt.2) goto 10 
c                               save found solutions:
      sname(im) = tname
c                               if nsite ne 0 get "normalization"
c                               constants for entropy model:
      if (nsite.ne.0) then 
         if (jsmod.eq.1) then
c                               bragg-williams sconf model
            call snorm0 (tname)
         else
            call snorm1 (tname)
         end if
      end if 
c                               generate pseudocompound compositions.
c                               subdiv returns the total
c                               number of pseudocompounds (ipcps) and 
c                               array y, of which element y(h,i,j) is
c                               the site fraction of the jth species on
c                               the ith site of the hth pseudocompound.
      call subdiv (tname)       
c                               subdiv generates ntot compositions,
c                               generate the compound data for
c                               each solution:
c                               save the identities of the endmembers
      jend(im,1) = 0
      icpct = 0
      jcpct = 0
c                               counter of number of real endmembers 
      if (jsmod.eq.8) then 
c                               dependent endmembers with speciation,
c                               save all species fractions: 
         jend(im,1) = jstot
      else 
         jend(im,1) = kstot
      end if 

      if (jstot+2.gt.k12) call error (39,zt,k12,'INPUT9')    

      do i = 1, kstot
c                               if the endmember has not been
c                               "killed" by a solution model
c                               set the solution identifier flag.
         id = kdsol(insp(i))

         if (ikp(id).ne.-1) ikp(id) = im
c                               pointer to endmember
         jend(im,2+i) = id

      end do           

      if (jsmod.ne.8) then
         write (n8,1060) tname, (names(jend(im,2+i)), i =1, jend(im,1))
      else
         write (n8,1060) tname, (mname(insp(i)), i = 1, jend(im,1)),'*'
      end if 
      
      do h = 1, ntot
c                               load the composition into
c                               a the site fraction array:
         do i = 1, isite
            zt = 0d0
            do j = 1, isp(i) - 1
               z(i,j) = y(h,i,j)
               zt = zt + z(i,j)
            end do 
            z(i,isp(i)) = 1d0 - zt
         end do 
c                               generate the pseudocompound:
         call soload (z,im,icpct,ixct,tname,icky)

      end do 

      if (icky.eq.1) call warn (41,wg(1,1),i,tname)

      if (icpct.eq.0) then
         call warn (30,wg(1,1),i,tname)
c                               reset endmember flags
         do i = 1, i point
            if (ikp(i).eq.im) ikp(i) = 0 
         end do 
         im = im - 1
         goto 10 
      else
         write (*,1100) icpct, tname

         do i = iphct-icpct+1, iphct
 
            write (n8,1070) names(i),(sxs(ixp(i)+j), j = 1, jend(im,1))

         end do

      end if 

      if (jsmod.eq.6.or.jsmod.eq.8) write (n8,1120)

      if (icopt.lt.10) then 
c                               save global model parameters
c                               if required
         if (jsmod.gt.2) call pmodel (im,jm)
      else 
c                               save all model parameters
c        call qmodel (im,jm)
      end if 

      jend(im,2) = icpct

      if (im.eq.isoct) goto 999
c                               next solution
      goto 10 
c                               at least one solution phase referenced
c                               in the input is not included in the
c                               solution phase data file, write warning:
99    call warn (43,zt,isoct-im,'INPUT9')
c                               scan for "killed endmembers"
999   do i = 1, ipoint
c                               reset ikp
         if (ikp(i).lt.0) ikp(i) = 0
      end do 

      write (*,1110) iphct - ipoint

      if (isoct.gt.0) then 
         if (io3.eq.0) then 
            write (n3,1020)
            write (n3,1010) (fname(i), i = 1, isoct)
            if (im.gt.0) then 
               write (n3,1000)
               write (n3,1010) (sname(i), i = 1, im)
            else 
               write (n3,1040) 
            end if 
         end if 

         do i = 1, im
            fname(i) = sname(i)
         end do 
      end if 

      isoct = im
c                              close pseudocompound list
      close (n8)
c                              close solution model file
      close (n9)
c                              output thermo data file if requested
      if (icopt.eq.4) then 
	   call swash (icmpn)
	else if (icopt.eq.8) then 
	   call gwash 
	end if 

1000  format (/,'the following solution models will be considered:',/)
1010  format (7(2x,a10))
1020  format (/,'Of the requested solution models:',/)
1030  format (a)
1040  format (/,'no models will be considered.',/)
1050  format (//,'WARNING, you are using a solution model file with',
     *         /,'the old Perple_X format, this format will not be ',
     *         /,'supported in future versions. You should correct ',
     *         /,'the file format of your file, or copy a new version',
     *         /,'from the Perple_X web site.',//)
1060  format (/,'Solution: ',a,/,12x,'Endmember fractions:',
     *        /,12x,20(a,1x))
1070  format (a,2x,20(1x,f6.3,2x))
1080  format (/,'Refer to file pseudocompound_glossary.dat for ',
     *          'endmember proportions',/)
1090  format (/,'Generating pseudocompounds, ',
     *          'this may take a while...',/)
1100  format (/,i8,' pseudocompounds generated for: ',a,/)
1110  format (/,'Done generating pseudocompounds (total:',i8,')',/)
1120  format (/,'*Although the bulk composition of any pseudocompound'
     *        ,' for this solution is fixed,',/,' the proportions of'
     *        ,' its endmembers may vary due to respeciation.',/)

      end 

      subroutine gwash 
c-----------------------------------------------------------------------
c swash outputs thermodynamic data for pseudocompounds. 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      character*8 names, fname*10, cname*5

      include 'perplex_parameters.h'

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst8 /names(k1)
     *      / cst60 /ipoint,imyn/ cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst6 /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
     *      / csta7 /fname(h9)/ cst2 /g(k1)/ csta4 /cname(k5)

      p = vmin(1)
	t = vmin(2)
	xco2 = vmin(3)

	call gall

	open (n2,file='components.dat')
      write (n2,'(a)') (cname(i),i=1,icp)
	close (n2)

      open (n2,file='names.dat')
      write (n2,'(a)') (names(i),i=1,iphct)
	close (n2)

      open (n2,file='g.dat')
	do i = 1, iphct
	   write (n2,*) g(i)
	end do 
	close (n2)

      open (n2,file='comp.dat')
	do i = 1, iphct
	   write (n2,'(15(g15.7,1x))') (cp(j,i),j=1,icp)
	end do 
	close (n2)

      open (n2,file='solution_name.dat')
	do i = 1, iphct
	   if (ikp(i).eq.0) then 
	      write (n2,*) names(i)
	   else 
	      write (n2,*) fname(ikp(i))
	   end if 
	end do 
	close (n2)

      stop

      end

      subroutine swash (icmpn)
c-----------------------------------------------------------------------
c swash outputs thermodynamic data for pseudocompounds. 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      character*8 names

      include 'perplex_parameters.h'

      double precision tm(m7,m6) 

      common/ cst1 /thermo(k4,k10),uf(2),us(h5)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst203 /therdi(m8,m9),therlm(m7,m6,k9)
     *      / cst204 /ltyp(k1),lmda(k1),idis(k1)/ cst8 /names(k1)
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst60 /ipoint,imyn/ cst42 /ic(k5),idbase
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst6 /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
     *      / cst43 /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind

      open (n2,file='swash.dat')

      iwarn9 = 0
      iwarn49 = 0 
      iwarn48 = 0
      iwarn46 = 0
      
      do 10 jd = ipoint + 1, iphct
c                              skip fluid cpds
         if (ifp(jd).eq.1) iwarn48 = 1
c                              create mock-entry iphct + 1 for output
c                              initialize paramaters
         id = k1
         idis(k1) = 0
         ltyp(k1) = 0
         jdis = 0
         klam = 0
         names(k1) = names(jd)  
 
         do i = 1, k4
            therm(i) = 0d0
         end do 

         do i = 1, k0
            comp(i) = 0d0  
         end do 
 
         imurg = 0
c                                    now combine endmember props

         do j = 1, jend(ikp(jd),1)
c                                    endmember index
            kd = jend(ikp(jd),2+j)
            xkd = sxs(ixp(jd)+j) 
c                                    test for mughnahan EoS
            if (thermo(18,kd).lt.0d0) then
               iwarn49 = 1 
               goto 10
            else if (thermo(18,kd).gt.0d0) then
               imurg = 1
               cmurg = thermo(18,kd)
            end if             

            do i = 1, 18
               therm(i) = therm(i) + xkd * thermo(i,kd)
            end do 

            do i = 1, icomp
               comp(ic(i)) = comp(ic(i)) + xkd * cp(i,kd)
            end do 

            if (idis(kd).ne.0) then

               idis(k1) = m9
               jdis = jdis + 1
               ld = idis(kd) 
               if (jdis.gt.1) goto 91
               do i = 1, 7      
                  therdi(i,m9) = xkd * therdi(i,ld)
               end do 
               therdi(8,m9) = therdi(8,ld)
               therdi(9,m9) = therdi(9,ld)

            end if

            if (ltyp(k1).ne.0) then

               ltyp(k1) = ltyp(kd)
               lmda(k1) = k9
               ld = lmda(kd)
               klam = klam + 1

               if (ltyp(kd).eq.10) then
                  if (klam.gt.3) goto 91
                  ltyp(k1) = 9 + klam
                  therlm(1,klam,k9) = therlm(1,1,ld)
                  therlm(2,klam,k9) = xkd * therlm(2,1,ld)
               else if (ltyp(kd).lt.4) then
                  if (klam.gt.1) goto 91
                  do i = 1, ltyp(kd)
                     therlm(1,i,k9) = xkd * therlm(1,i,ld)
                     therlm(2,i,k9) = xkd * therlm(2,i,ld)
                     therlm(5,i,k9) = xkd * therlm(5,i,ld)
                     therlm(6,i,k9) = xkd * therlm(6,i,ld)
                     therlm(3,i,k9) = therlm(3,i,ld)
                     therlm(4,i,k9) = therlm(4,i,ld)
                     therlm(7,i,k9) = therlm(7,i,ld)
                     therlm(8,i,k9) = therlm(8,i,ld)
                  end do
               else if (ltyp(kd).lt.8) then

                  if (klam.gt.1) goto 91

                  do k = 1, ltyp(kd) - 3

                     therlm(1,k,k9) = therlm(1,k,ld)
                     therlm(2,k,k9) = therlm(2,k,ld)

                     do h = 3, 12
                        therlm(h,k,k9) = xkd * therlm(h,k,ld) 
                     end do
                  end do 
               end if
            end if
         end do

         call unlam (tm,id,lct)
c                                 for mughnahan EoS
         if (imurg.eq.1) then
            iwarn46 = 1
            therm(18) = cmurg
         end if 

         call unver (therm(1),therm(2),therm(3),therm(4),
     *               therm(5),therm(6),therm(7),therm(8),
     *               therm(9),therm(10),therm(11),
     *               therm(12),therm(13),therm(14),
     *               therm(15),therm(16),therm(17),
     *               therm(18),tr,pr)
c                                 output the data 
         write (n2,1100) names(k1),idbase,ikp(jd),ltyp(k1),idis(k1)
         write (n2,1080) (comp(i),i=1,icmpn)
         write (n2,1090) (therm(i),i=1,18)
         call outlam (tm,idis(k1),lct)

         goto 10 

91       iwarn9 = 1

10    continue  

      if (iwarn46.eq.1) call warn (46,thermo(18,kd),iphct,names(kd))

      if (iwarn49.eq.1) call warn (49,thermo(18,kd),iphct,names(kd))

      if (iwarn48.eq.1) call warn (48,thermo(18,kd),iphct,names(kd))

      if (iwarn9.eq.1) call warn (9,t,ifyn,names(k1))

      close (n2)

      stop

1080  format (12(f7.4,1x))
1090  format (5(g13.7,1x))
1100  format (a8,i2,i2,i2,i2)

      end

      subroutine abload (*)
c---------------------------------------------------------------------
c abload assembles the matrix 'a' and vector 'b' for icp component
c sytems and then solves the equation ax = b, the vector x is
c returned in 'b'.
 
c   referenced by: combin
c   references to: subst,factor
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),iophi,idphi,
     *               iiphi,iflg1
     *      / cst2  /g(k1)/ cst12 /cp(k5,k1)/ cst52 /hcp,id(k5+1),ic(k5)
c-----------------------------------------------------------------------
      do i = 1, hcp
         do j = 1, hcp
            a(i,j) = cp(j,id(i))
         end do
      end do

      call factor (a,hcp,ipvt,ier)
 
      if (ier.eq.1) goto 99
 
      do i = 1,  hcp
         b(i) = g(id(i))
      end do 
 
      call subst (a,ipvt,hcp,b)
 
      return
99    return 1
      end

      subroutine asschk
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),iophi,idphi,
     *               iiphi,iflg1/ cst87 /delt(l2),dtol,utol,ptol
     *      / cst6 /icomp,istct,iphct,icp/ cst62 /blim(l2),ulim(l2),dgr
     *      / cst2 /g(k1)/ cst12 /cp(k5,k1)/ cst7 /iflag
c-----------------------------------------------------------------------
c                                 determine chemical potentials
      iflag = 0
      do i = 1, icp
         b(i) = g(idv(i))
      end do
c
      call subst (a,ipvt,icp,b)
c                                 test phases against the
c                                 assemblage idv
      do 20 i = istct, iphct
         gphi = 0d0
         do j = 1, icp
            gphi = gphi + cp(j,i) * b(j)
         end do 
         dg = g(i) - gphi
         if (dg.gt.dtol) goto 20
c                                check that a phase is not metastable
c                                with respect to itself, this
c                                could be remedied by changing the 
c                                value of dtol.
         do j = 1, icp
            if (idv(j).eq.i) goto 20
         end do 
c                                assemblage is metastable with respect
c                                to phase idphi.
         iflag = iflag + 1
         idphi = i
c                                this would force a refinement
c                                of condtions by the calling
c                                routine. since utol << dtol
c        if (dabs(dg).gt.utol) iflag = iflag + 1
         if (iflag.gt.1) goto 99
20    continue

99    end
 
      block data
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst5  /p,t,xco2,u1,u2,tr,pr,r,ps/ cst85 /pp,tt,yy,rr
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst32 /ptx(l5),ipt2
     *      / cst1  /thermo(k4,k10),uf(2),us(h5)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
c-----------------------------------------------------------------------
      data iff/2*0/,ipt2/0/
c
      data us, uf/ h5*0d0, 2*0d0/
c
      data r,rr/8.3144126d0,83.14d0/
c                              the following data statement assigns
c                              logical unit numbers for i/o.
      data n1,n2,n3,n4,n5,n6,n7,n8,n9/21,22,23,24,25,26,27,28,29/

      end

      subroutine assir 
c----------------------------------------------------
c irct - counter of univariant rxns.
c ivarrx(irct) - variance flag of the iirct(th) rxn (set in balanc as
c                 ivar in cst61.
c irchk(irct) - a flag indicating if the iirct(th) rxn occurs on the
c                edge of a diagram (initialized in newhld and set in
c                coface or sfol1).
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps/ cst80 /irchk(k2)
     *      / cst31 /vn(k2,k7),irct,ird/ cst25 /vnu(k7),idr(k7),ivct
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst35 /irv(k2)/ csta1 /irnms(k2,k7)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
c-----------------------------------------------------------------------
c                         check if the equilibrium is already known
      do 70 i = 1, irct
         if (ivct.ne.irv(i)) goto 70
         do 80 j = 1, ivct
            do k = 1, ivct
               if (irnms(i,k).eq.idr(j)) goto 80
            end do 
c                         no match with irnms(i,k) try i + 1.
         goto 70
80       continue
c                         match found with irnms(i,k)
         ird = i
         goto 99
70    continue
c                         the reaction is new:
      irct = irct + 1

      ird = irct
      if (irct.gt.k2) call error (181,r,k2,'ASSIR')
      irv(irct) = ivct
      ivarrx(irct) = ivar

      do i = 1, ivct
         vn(irct,i) = vnu(i)
         irnms(irct,i) = idr(i)
      end do 

99    end

      subroutine assri (ier)
c----------------------------------------------------------------------
c irct         - reaction counter
c irv(irct)    - number of phases in the irct(th) rxn
c                (set in balanc as ivct cst25).
c ivarrx(irct) - variance flag of the iirct(th) rxn (set in balanc as
c                 ivar in cst61.
c irchk(irct)  - a flag indicating if the iirct(th) rxn occurs on the
c                edge of a diagram (initialized in newhld and set in
c                coface or sfol1).
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (k8=k5+2)

      common/ cst5  /v(l2),tr,pr,r,ps/ cst80 /irchk(k2)/ cst35 /irv(k2)
     *      / cst31 /vn(k2,k7),irct,ird/ cst87 /delt(l2),dtol,utol,ptol
     *      / cst25 /vnu(k7),idr(k7),ivct/ csta1 /irnms(k2,k7)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5      
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst29 /vip(l2,k2),ipid(k2,k8),ipct
c-----------------------------------------------------------------------
c                         check if the reaction is known
      do 70 i = 1, irct
         if (ivct.ne.irv(i)) goto 70
         do 80 j = 1, ivct
            do k = 1, ivct
               if (irnms(i,k).eq.idr(j)) goto 80
            end do 
c                         no match with irnms(i,k) try i + 1.
            goto 70
80       continue
c                         match found with irnms(i,k), now check
c                         that the conditions are different, if
c                         ier = 1, univeq failed, skip the check
         if (ier.eq.0) then 
            if (dabs(vip(iv1,i)-v(iv1)).le.delt(iv1)) then 
c                         since there mat be multiple occurences
c                         of the same reaction, go through the entire
               ird = i
               ier = 1
               goto 99 
            end if 
         end if 

70    continue
c                         the reaction is new:
40    irct = irct + 1
      ier = 0
      if (irct.gt.k2) call error (181,r,k2,'ASSRI')
      ird = irct
      irv(ird) = ivct
      ivarrx(ird) = ivar

      do i = 1, ivct
         vn(ird,i) = vnu(i)
         irnms(ird,i) = idr(i)
      end do 

99    end

      subroutine balanc (b,idv,idphi,ier)
c-----------------------------------------------------------------------
c balanc balances reactions between the phases in idv and idphi
c if the system is saturated with any phases these are taken
c into account.  once the reaction is determined balanc determines
c the thermodynamic parameters needed for subroutine univeq and
c saves them in the array 'rxn'.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      dimension a(k5,k5), ipvt(k5), idv(k5), b(k5+1)

      common/ cst1  /thermo(k4,k10),uf(2),us(h5)
     *      / cst6  /icomp,istct,iphct,icp/ cst25 /vnu(k7),idr(k7),ivct
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst12 /cp(k5,k1)
     *      / cst201 /vuf(2),vus(h5),iffr,isr
c-----------------------------------------------------------------------
c                                 assemble transpose of the composition
c                                 matrix for an assemblage of icp phases
      ivar =1
                                                
      do i = 1, icp
         do j = 1, icp
            a(j,i) = cp(j,idv(i))
         end do 
         idr(i) =idv(i)
      end do 
c                                 assemble the composition vectore
c                                 for the icp+1 th phase tangent to
c                                 the free energy plane.
      ip=icp+1
      do i = 1, icp
         b(i) =cp(i,idphi)
      end do 
      idr(ip) =idphi
      b(ip) = -1d0
c                                  balance the equilibrium
      call factor (a,icp,ipvt,ier)
      if (ier.eq.1) goto 999

      call subst (a,ipvt,icp,b)
c                                  eliminate phases with vnu= 0
      ivct= 0
      do i = 1, ip
         if (dabs(b(i)).gt.1d-05) then 
            ivct=ivct + 1
            vnu(ivct) =b(i)
            idr(ivct) =idr(i)
         end if 
      end do 
c                                 test to determine if the reaction
c                                 is pseudo-univariant, and set flag
c                                 ivar =2 if so, and =1 otherwise.
      iv1 = ivct-1
      do i = 1, iv1
         if (ikp(idr(i)).ne.0) then 
            i1=i + 1
            do j =i1,ivct
               if (ikp(idr(i)).eq.ikp(idr(j))) then 
                  ivar =2
c                                pseudo-univariant reject rxn if isudo=1
                  if (isudo.eq.1) goto 999
                  goto 170
c                                note that this test will fail to
c                                distinguish between critical equilibria
c                                and pseudo-univariant equilibria.
               end if
            end do
         end if 
      end do 

170   if (isyn.eq.1) goto 40
c                                determine stoichiometric coefficients
c                                of saturated components:
c                                (these aren't used for anything real)
      isr = 1
      do j = 1, isat
         vus(j) = 0d0
         do i = 1, ivct
            vus(j) = vus(j) + vnu(i) * cp(icp+j,idr(i))
         end do 
c                                flag isr is not used.
         if (vus(j).ne.0d0) isr = 0   
      end do 
c                                ok, now vus has the total deltas
   
c                                determine stoichiometric coefficients
c                                of saturated phase components:
      iffr = 1
40    do j = 1, 2
         vuf(j) = 0d0
         if (iff(j).ne.0d0) then 
            do i = 1, ivct
               vuf(j) = vuf(j) + vnu(i) * cp(iff(j),idr(i))
            end do 
c                                this is only the total of saturated
c                                phase components for the reaction
c                                (doesn't include saturated phases).
c                                flag iffr is used
            if (vuf(j).ne.0d0) iffr = 0
         end if
      end do 
c                                et finis
999   end

      subroutine combin
c----------------------------------------------------------------------------
c combinatorial algorithm for mfes. 

c for constrained bulk compositions (jbulk.ge.icp):

c   if jtest = 1 switches to smplx (lp optimization)

c   if jtest = 0 allows user to specify initial assemblage (tertex) 

c   if jtest = 2 output thermo data
  
c----------------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst6   /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)   
     *      / cst96  /idcf(k5,j9),icfct/ cst27 /jasmbl(j9),iasmbl(j9)
     *      / cst52  /hcp,id(k5+1),ic(k5)/ cst60 /ipoint,imyn
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / debug  /jtest,jpot
c                                   initialization
      imyn = 1
      icfct = 0
      jcp = icp - 1

      iclose = 0
c                                   output thermo data 
      if (jtest.eq.2) call outdat
c                                   switch to simples or
c                                   allow the user to specify the
c                                   initial assemblage: 
c      if (jbulk.ge.icp.and.jtest.ne.0) then
      if (jbulk.ge.icp) then
         call smplex 
         goto 99 
      else if (jbulk.ge.icp.and.icopt.ne.0) then
         call tertex (icase)
         if (icase.eq.1) goto 99 
      end if
c                                   identify the seed stable assemblage
      call findas 
c                                   save the assemblage
      call assdc (icase)
c                                   finished if one component
      if (icp.eq.1.or.icase.eq.2.and.jbulk.ge.icp) goto 91
c                                   now use the c-1 phase simplicial
c                                   facets generated from the seed
c                                   assemblage to identify the remaining
c                                   assemblages.
      istart = 1
      jstart = 0
      jend = 0
      igen = 1

40    iend = icfct 
      write (*,*) 'cycle ',igen,istart,iend
      igen = igen + 1

      do 50 i = istart, iend
c                             each facet may generate icp-1 new psf's
         do 60 j = 1, jcp
c                             set vertices identities:
            itic = 0
c                             this loop generates the psf consisting of
c                             the icp-1 phases, saving the icp'th phase
            do k = 1, icp 
               if (k.eq.j) then 
                  idno = idcf(k,i) 
               else 
                  itic = itic + 1
                  id(itic) = idcf(k,i)
               end if 
            end do
c                              check that the assemblage is not degenerate
c                              a valid psf must span the icp dimensional 
c                              composition space, count the components:
            do k = 1, icp
               sum = 0d0
               do l = 1, jcp
c                             sum the amounts of component k in the psf 
                  sum = sum + cp(k,id(l))
               end do 
c                             if the component is 0, the psf is degenerate
               if (sum.lt.1d-5) goto 60
            end do
c                             next check if the psf matches any permutation
c                             of icp-1 phases in a facet tested earlier:
c                             this test is not essential, but it should
c                             save time. 
            do 80 k = 1, i-1
c           do 80 k = jstart+1, i-1
c                             unless there is some weird hyperdimensional
c                             effect that i don't understand, the index k
c                             should only need to run over the facets identified
c                             in the last cycle up to i-1. but tests seem to
c                             give more assemblages if k runs over all the
c                             assemblages, a problem to be investigated.
               do 70 l = 1, jcp
                  do m = 1, icp
                     if (idcf(m,k).eq.id(l)) goto 70
                  end do
c                             no match with assemblage k:
                  goto 80
70             continue
c                             matched assemblage k, reject the psf:
               goto 60
80          continue
c                             now identify the first potential vertice:

            do 10 k = istct, iphct
c                             don't allow phases of the original simplex.
               do l = 1, icp
                  if (k.eq.idcf(l,i)) goto 10
               end do 
c                             try to find a phase that is stable
c                             with respect to phase idno:
               id(icp) = k

               call abload (*10)

               if (dgphc(idno).gt.-1d-05) then
c                              the plane is stable
                  lastk = k + 1
                  goto 20   
               end if 
10          continue
c                               if no acceptable phase is found
c                               the assemblage must lie on an edge
c                               of the compositional space.
            if (iclose.eq.0) call warn (28,sign,1,'COMBIN')
            iclose = 1
            goto 60
c                               now do thermodynamic testing
20          if (lastk.gt.iphct) then
               call assdc (icase)
               if (icase.eq.2.and.jbulk.ge.icp) goto 91
               goto 60
            end if          

            do k = lastk, iphct
c                                   checkd replaces the trial vertice
c                                   with phase j, if the assemblage is 
c                                   found to be metastable with respect 
c                                   to phase j. given the speed of this
c                                   test, i doubt geometric testing would
c                                   be worthwhile here.
c                                   don't allow return of the vertice
c                                   idpsf(icp,i)
               if (k.ne.idno) call checkd (k)
            end do 
c                                   save the new assemblage:
            call assdc (icase)
            if (icase.eq.2.and.jbulk.ge.icp) goto 91
60       continue
50    continue 

      if (icfct.gt.iend) then
c                                    some new assemblages have been
c                                    identified in the present cycle.
         if (jbulk.gt.0.and.jend.gt.0) then
c                                    eliminate blind assemblages if 
c                                    constrained bulk compositions
            itic = 0
            do i = jstart, jend
               if (jasmbl(i).ne.0) then
                  itic = itic + 1
                  do j = 1, icp
                     idcf(j,itic) = idcf(j,i)
                  end do
                  jasmbl(itic) = jasmbl(i)
               end if
            end do
c                                     now shift all other assemblages
            jtic =  jend-jstart+1-itic

            do i = jstart+itic, icfct-jtic

               k = i + jtic
               jasmbl(i) = jasmbl(k)
               do j = 1, icp
                  idcf(j,i) = idcf(j,k)
               end do
            end do
c                                     reset counters for next cycle
c                                     accounting for eliminated assemblages:
            jstart = istart - jtic
            jend   = iend - jtic
            istart = jend + 1
            icfct  = icfct - jtic  
            goto 40 
         else
c                                     reset counters for next cycle
            jstart = istart
            jend = iend
            istart = iend + 1
            goto 40
         end if 
      end if
c                                    check that something was found.
      if (icfct.eq.0) then
         if (iclose.eq.1.and.jbulk.ne.0) call error (42,sign,2,'COMBIN')
         call error (999,sign,2,'COMBIN')
      end if 
c                                    if constrained bulk composition
c                                    eliminate irrelevant assemblages
91    if (jbulk.ge.icp) then
         do j = 1, icp
            idcf(j,1) = idcf(j,icfct)
         end do
         icfct = 1
      else if (jbulk.ne.0) then
         itic = 0
         do i = 1, icfct
            if (jasmbl(i).eq.1) then
               itic = itic + 1
               do j = 1, icp
                  idcf(j,itic) = idcf(j,i)
               end do
            end if
         end do
      end if 

99    end 

      subroutine assdc (icase)
c-----------------------------------------------------------------------
c assdc counts (icfct) and assigns (idcf) c-component facets. the
c vertices are identified by the array id(icp).

c   icase = 0 => no new assemblage counted.
c   icase = 1 => new assemblage.
c   icase = 2 => new assemblage that bounds a specified bulk composition.

c   referenced by: findas
c   references to: bulkck, error
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst96  /idcf(k5,j9),icfct/ cst6 /icomp,istct,iphct,icp
     *      / cst27 /jasmbl(j9),iasmbl(j9)
     *      / cst12  /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / debug  /jtest,jpot/ cst52 /ht,id(k5+1),ic(k5) 
c-----------------------------------------------------------------------
      icase = 0
c                             test for equivalence with earlier
c                             assemblage:
      do 10 i = 1, icfct
         do 20 j = 1, icp
            do k = 1, icp
              if (idcf(j,i).eq.id(k)) goto 20
            end do 
            goto 10
20       continue
c                             matches earlier assemblage, return.
         return
10    continue
c                             unique (and bounding) assemblage:   
      icfct = icfct + 1
      if (icfct.gt.j9) call error (204,sum,j9,'ASSDC')
      jasmbl(icfct) = 0
      icase = 1
c                             check if assmblage bounds a constrained
c                             composition:      
      if (jbulk.ne.0) then
         call bulkck (id,jbfg,kdbulk,1,1)
         jasmbl(icfct) = jbfg
         if (jbfg.ne.0) icase = 2
      end if 
c                             assign the assemblage: 
      do i = 1, icp
         idcf(i,icfct) = id(i)
      end do 
c                             do miscibility check:
      call miscib (id,ivar,imyn)
      iasmbl(icfct) = ivar

      if (jpot.eq.1) goto 99
c                             print potentials:
      call prtpot

99    end

      subroutine chkass (jdv,ivd,ivi,iste)
c-----------------------------------------------------------------------
c chkass determines if a divariant assemblage (jdv) generated by
c newass has already been identified. there are three possible cases:
c in the reaction as determined by routine balanc.
c (1) the assemblage is unique: the indices of the phases, the initial
c     conditions of stability, and the coordinate frame edge are recorde
c (2) the assemblage has already been defined and it's stability field
c     determined: quit
c (3) the assemblage has already been defined but it's stability field
c     has not been determined: reset the initial conditions of stability
c     to shorten traverses by the routine search, if possible. 

c input:  jdv(icp) - a vector containing the indices of phases in a
c                    divariant assemblage.
c         ipart    - 1 if testing original divariant assemblages, 2 if
c                    testing new divariant assemblages.
c         index    - index of the divariant assemblage in array idcf
c                    (ipart=1) or idns (ipart=2) currently being tested.
  
c referenced by: newass
c references to: no external references
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      integer jdv(k5+1), icrap, kdv(k5), idsol(k5), kdsol(k5),
     *        ncpd, np

      common/ cst6 /icomp,istct,iphct,icp/ cst5 /v(l2),tr,pr,r,ps
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst65 /vt(j9),vti(j9),jok(j9),kok(j9),index
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9    
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)/ cst96 /idcf(k5,j9),icfct
     *      / cst74 /idbulk(k5,k2),iap(k2),ibulk,iaid(2)
     *      / cst87 /delt(l2),dtol,utol,ptol

      save icrap

      data icrap /0/
c-----------------------------------------------------------------------

      do 40 i = 1, icfct
         do 50 j = 1, icp
            do k = 1, icp
               if (jdv(k).eq.idcf(j,i)) goto 50
            end do 
c                                 no match with assemblage i
            goto 40
c                                 
50       continue
c                                 jdv matches the ith assemblage =>
         if (jbulk.ge.icp) then
            do k = 1, icp
               kdv(k) = jdv(k)
            end do
c                                 if constrained bulk composition 
c                                 calculation call blkmat to get
c                                 the index of the divariant assemblage:
c           call blkmat (kdv,kdbulk,igo)
            call blkmat (kdv,idsol,kdsol,ncpd,np,icp,kdbulk,igo)
            if (igo.ne.1) call error (999,atw(1),igo,'CHKASS')
            iaid(2) = kdbulk
         end if 
c                                 first check if within known stability
c                                 field of the assemblage
         if (iste.lt.kok(i)) then 
            goto 99
         else if (iste.eq.kok(i)) then 
            if (iste.lt.3) then
               if (vt(i).ge.v(ivd)-0.01*dv(ivd)) goto 99
            else 
               if (vt(i).le.v(ivd)+0.01*dv(ivd)) goto 99
            end if 
c                                 it extends known stability field:
            if (i.gt.index) then
c                                 the assemblage hasn't been tested
c                                 then reset start conditions 
               call sreset (jok(i),iste,vt(i),v(ivd),vti(i),v(ivi))
               goto 99
            else 
c                                 the assemblage has been tested, so 
c                                 skip the check and look for another
c                                 occurence, if no other occurence the
c                                 assemblage will be saved as new. 
               goto 40

            end if 

         else if (i.gt.index) then 
            call sreset (jok(i),iste,vt(i),v(ivd),vti(i),v(ivi))
            goto 99
         end if
c                                 otherwise save as a new assemblage
         goto 100
  
40    continue
c                                 the assemblage is unique, store
c                                 parameters:
100   if (icrap.ne.1) then 
c                                 do bulk composition check
         if (jbulk.ne.0) then
            call bulkck (jdv,jbfg,kdbulk,1,1)
            if (jbfg.eq.0) goto 99
            if (jbulk.ge.icp) iaid(2) = kdbulk
         end if 

         icfct = icfct + 1

         if (icfct.gt.j9) then
            call warn (205,r,j9,'CHKASS')
            icfct = j9
            icrap = 1
         end if 

         do i = 1, icp
            idcf(i,icfct) = jdv(i)
         end do 

         vt(icfct) = v(ivd)
         vti(icfct) = v(ivi)
         jok(icfct) = iste
         kok(icfct) = iste

      end if 

99    end

      subroutine coface (iovd,iovi,odiv,iste,irend)
c----------------------------------------------------------------------
c once nohold has established that a stable equilibrium occurs on
c the boundary of a diagram, coface traces the equilibrium condiitons
c within the diagram.  if the equilibrium terminates at an
c invariant point coface traces the equilibria which emanate
c from the invariant point and any invariant points encountered
c subsequently.  coface does not detect indifferent points.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (k8=k5+2)

      common/ cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst29  /vip(l2,k2),ipid(k2,k8),ipct/ cst32 /ptx(l5),ipt2
     *      / cst7   /iflag/ cst80 /irchk(k2)/ cst5 /v(l2),tr,pr,r,ps
     *      / cst23  /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst31  /vn(k2,k7),irct,ird/ cst25 /vnu(k7),idr(k7),ivct
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9         
     *      / cst105 /ibug(k2)/ cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst49  /value,ismax,igot
c-----------------------------------------------------------------------
      ivi =iovi                                                  
      ivd=iovd
      jflg= 1

      call balanc (b,idv,idphi,ier)
c                                 singular concentration matrix
      if (ier.eq.1) call error (68,v(1),ier,'COFACE')
c                                 call newass to define new (pseudo-)
c                                 divariant assemblages generated by
c                                 the reaction defined by balanc:
      call newass (b,idv,idphi,ivd,ivi,iste)
c                                 if the rxn is not univariant
c                                 continue only if isudo ne 1.
      if (ivar.ne.1.and.isudo.eq.1) goto 9999
c                                 call assir to determine if the
c                                 reaction has already been identified
c                                 and if so quit, there is a small
c                                 possibility that as a result of
c                                 quitting at this stage some equilibria
c                                 will not be identified, if this is a
c                                 significant concern the following
c                                 computed goto should be deleted.
      if (icopt.ne.1) then  
c                                 allow duplicated reactions in 
c                                 mixed variable diagrams:
         call univeq (ivd,ier)
c                                 assign reaction:
         call assri (ier)

         if (ier.eq.1) goto 9999

         call delrxn 

         vip(1,ird) = v(1)
         vip(2,ird) = v(2)
         vip(3,ird) = v(3)
         vip(4,ird) = v(4)
         vip(5,ird) = v(5)

         call outrxn (0,0)

         goto 9999

      end if 

      call assir 
c                                 skip equilibria that have already
c                                 been defined on an edge, there is some
c                                 possibility that this may result in an
c                                 an incomplete diagram, but it saves
c                                 some time and redundancy in output.
c                                 check if the univariant rxn has alread
c                                 been found on the edge of a diagram.
c                                 set flags irchk and ier:
c      if (irchk(ird).eq.1) goto 9999
c                                 call svrend to see if the end point 
c                                 has already been found, if it has ier
c                                 is 1, and traverse can be skipped. 
      call svrend (ird,irend,ier)
      if (ier.eq.1) goto 9999

      irchk(ird) = 1
      call univeq (ivd,ier)
c                                 get deltas for dependent extensities:
      call delrxn
c                                 if univeq fails on a bounding edge
c                                 write error message and return:
      if (ier.eq.1.or.ier.eq.2) goto 9000
c                                 save the index of the c+1th phase
c                                 as iophi.
      iophi =idphi
c                                 factor the concentration matrix:
      call pivots (ier)
      if (ier.eq.1) call error (69,v(1),ier,'COFACE')
c                                 set the increment for the iv
      div = odiv
c                                 initialize counters
      ipt2 = 0
      icter = 0
c                                 assign the 1st point
      call assptx
c                                 follow the equilibrium
60    ikwk = 0 
      jflg = 0
      call sfol1 (ivd,ivi,ier,div,ikwk,irend)
c                                 sfol1 returns ier = 1
      if (ier.eq.1.or.ier.eq.2) goto 70

      ivi = iovi
      ivd = iovd

      if (iflag.eq.0) then 
c                                 iflag = 0 the univariant
c                                 curve was traced from one
c                                 edge to another, reset
c                                 the starting conditions
c                                 and set flag jer for newhld
         goto 9999
      else if (iflag.eq.1) then
c                                 an ip was found:
         goto 10
      end if 

70    call switch (div,ivi,ivd,jer)
      if (jer.eq.1) goto 75

      icter = icter + 1
      if (icter.lt.4) goto 60
      
75    call warn (10,v(1),ier,'COFACE')

      call outrxn (ipct,2)
      ibug(irct) = 1
      ivi =iovi
      ivd=iovd
c                                 return on error
      goto 9999                
c                                 a new invariant point has been
c                                 encountered, generate the new 
c                                 univariant fields.
c                           iopct=the total number of ip's encountered
c                                 before each call to sfol2.
c                           inpct=the total number after each call to 
c                                 sfol2.
10    iopct = ipct
      call sfol2 (iovi,iovd,iopct,irend)
      if (iopct.eq.ipct) goto 9999
      iopct = iopct + 1
      inpct = ipct
c                                 loop for new invariant points
30    do i = iopct, inpct
         call sfol2 (iovi,iovd,i,irend)
      end do 
      if (ipct.eq.inpct) goto 9999
      iopct = inpct + 1
      inpct = ipct
      goto 30
c                                 error in univeq:
9000  call warn (79,v(1),ird,'COFACE')
      ipt2 = 0
      call outrxn (ipct,2)
      ibug(irct) = 1

9999  iflg1= 0 

      if (jflg.eq.1.or.io3p.eq.1) goto 999

      write (n3,1020) 
      
1020  format ('Network traced, resuming boundary search.',/)

999   end

      subroutine delvar (dv,iflag,iflg1)
c-----------------------------------------------------------------------
c delvar determines if a variable increment 'dv' should
c be incremented or decremented on a traverse to locate
c to locate conditions where an equilibrium is metastable with respect
c to exactly one phase (iflag = 1). the flag iflg1 indicates if the
c equilibrium was stable (iflg1 = 0) or metastable (iflg1 = 1) at the
c last conditions tested.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

c                            the equilibrium is stable (iflag = 0):
      if (iflag.eq.1.or.iflag.eq.2) then 
c                            the equilibrium is metastable (iflag = 2):
        if (iflg1.ne.1) then
c                            and was stable before (iflg1 = 0) then
c                            switch sign and halve dv:
            dv = -dv/2d0
            iflg1 = 1
         end if 

      else 
c                            and was stable before (iflg1 = 0) then
c                            continue looking in the same direction
         if (iflg1.eq.1) then
c                            and was metastable before (iflg1 = 1) then
c                            switch sign and halve dv:
            dv = -dv/2d0
            iflg1 = 0
         end if 
      end if 

      end
 
      subroutine factor (a,n,ipvt,ier)
c-----------------------------------------------------------------------
c factor is a subroutine which calculates the triangular
c decompositions of the matrix 'a'. factor is modified from
c the subroutine of the same name given by conte and de boor
c in 'elementary numerical analysis', mcgraw-hill, 1980.
c factor uses scaled partial pivoting.
c
c input     a- an n by n array containing the elements of matrix a.
c           n- the dimension of the matrix a.
c output    a- an n by n array containing the upper, u, and lower, l,
c              triangular decompositions of input matrix a.
c        ipvt- a vector indicating that row ipvt(k) was used to
c              eliminate the a(n,k).
c         ier- a flag, zero if a is of rank = n, and 1 if a is of
c              lower rank.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      dimension a(k5,k5),d(k5),ipvt(k5)
c-----------------------------------------------------------------------
      ier = 0
c                            initialize ipvt,d
      do i = 1, n
         ipvt(i) = i
         rmax = 0d0
         do j = 1, n
            rmax = dmax1(rmax,dabs(a(i,j)))
         end do 
c                            ax = b is singular if rmax = 0
         if (dabs(rmax).lt.1d-5) goto 9000
         d(i) = rmax
      end do 
c                            begin decomposition: 
      nm1 = n-1
c
      do i = 1, nm1
         ip1 = i+1
c                            determine pivot row (istr).
         rmax = dabs(a(i,i))/d(i)
         istr = i
         do j = ip1, n
            tmax = dabs(a(j,i))/d(j)
            if (tmax.gt.rmax) then
               rmax = tmax
               istr = j
            end if 
         end do

         if (dabs(rmax).lt.1d-5) goto 9000
c                            if istr gt i, make i the pivot row
c                            by interchanging it with row istr.
         if (istr.gt.i) then 
            j = ipvt(istr)
            ipvt(istr) = ipvt(i)
            ipvt(i) = j
            temp = d(istr)
            d(istr) = d(i)
            d(i) = temp

            do j = 1, n
               temp = a(istr,j)
               a(istr,j) = a(i,j)
               a(i,j) = temp
            end do 
         end if 
c                            eliminate x(k) from rows k+1,...,n.
         do j = ip1,n
         a(j,i) = a(j,i)/a(i,i)
         ratio = a(j,i)
            do k = ip1,n
               a(j,k) = a(j,k)-ratio*a(i,k)
            end do 
         end do
      end do 

      if (dabs(a(n,n)).lt.1d-5) ier = 1

      return
c                           algoritmic singularity.
9000  ier = 1
 
      end

      subroutine flipit (ddv,vst,ivd,ist,inow,jer)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst5 /v(l2),x(4)/ cst7 /iflag/ cst6 /icomp,istct,iphct,icp
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ cst38 /iedge
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst9 /vmax(l2),vmin(l2),dv(l2)/ cst8 /names(k1)
c-----------------------------------------------------------------------
c                                 initialization
      call gall 
      call asschk
c                                 the goto 20 is a dummy to avoid
c                                 compiler errors, change 90 to 20 to
c                                 get backup search, ha ha
      if (iflag.ne.0.and.v(ivd).eq.vst.and.ist.eq.inow) goto 90

      goto (10,20,20),iflag
c                                 the assemblage is stable, return
      jer = 0
      return
c                                 the assemblage is metastable with
c                                 respect to only one phase:
10    jer = 1
      return
c                                 the assemblage is metastable with
c                                 respect to >1 phase, therefore
c                                 it must be stable at some condition
c                                 between the present condition and the
c                                 conditions of the univariant
c                                 equilibrium that generated the
c                                 assemblage. flipit does a reverse
c                                 traverse to find this condition.
c                                 initialize:
20    iflg1 = 1
      ddv = -ddv
c                                 begin search:
100   v(ivd) = v(ivd) + ddv
      call incdep (ivd,v(ivd)) 
c                                 out of range?:
      if (ist.lt.3) then 
         if (v(ivd).lt.vmin(ivd)) v(ivd) = vmin(ivd)
         if (v(ivd).lt.vst) goto 130
         ddv = -dabs(ddv)/2d0
         v(ivd) = vst
         call incdep (ivd,v(ivd)) 
         iflg1 = 0
         goto 100
      else 
         if (v(ivd).gt.vmax(ivd)) v(ivd) = vmax(ivd)
         if (v(ivd).gt.vst) goto 130
         ddv = dabs(ddv)/2d0
         v(ivd) = vst
         call incdep (ivd,v(ivd)) 
         iflg1= 0
         goto 100
      end if 
c                                 calculate phase energies:
130   call gall 
c                                 test stability of the assemblage:
      call asschk
c                                 check if search is in range:
      if (ist.lt.3) then 
         if (v(ivd).le.vmin(ivd).and.iflag.gt.0) goto 60
      else
         if (v(ivd).ge.vmax(ivd).and.iflag.gt.0) goto 60
      end if 
c                                 iflag=1, found a stable equilibrium
c                                 in bounds:
      if (iflag.eq.1) write (n3,*) 'flipit worked please tell me!'
c
160   if (iflag.eq.1) goto 10
c                                 refine increment if necessary:
      call delvar (ddv,iflag,iflg1)
c                                 check if increment has been refined
c                                 beyond the tolerance which is
c                                 arbitrarily set at 1.d-8.
      if (dabs(ddv).lt.1d-8) goto 9000
c                                 increment conditions again:
      goto 100
c                                 next traverse.
60    call warn (76,vst,ist,'FLIPIT')

      goto 90
c                                 write warning message:
9000  write (n3,1010) ivd,dv(ivd),(names(idv(j)),j = 1, icp)
90    jer = 2
c                                 done:
1010  format (/,'**warning ver045** FLIPIT: > 1 equilibrium',
     *          ' occurs within the',/,'minimum search increment for',
     *          ' variable: ',i1,', this often occurs as YCO2 => 1',
     *          ' or => 0, you may be able to correct this',/,
     *          'by reducing the default increment for this variable',
     *          ' (',g12.3,') in the computational option file.',/,
     *          'Equilibria involving the following assemblage may',
     *          ' not be delineated:',/,7(1x,a8)) 
      end

      subroutine outdel
c-----------------------------------------------------------------------
c output deltas of dependent extensities for a reaction
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*5 cname,vname*8,xname*8,exten(2)*7,names*8

      common/ csta2  /xname(k5),vname(l2)/ csta4 /cname(k5)
     *      / cst21  /du(2),dv(2),jds(h5),ifr 
     *      / cst6   /icomp,istct,iphct,icp
     *      / cst201 /vuf(2),vus(h5),iffr,isr/ cst12 /cp(k5,k1)
     *      / cst25  /vnu(k5+1),idr(k5+1),ivct/ cst8 /names(k1)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct

      save exten 
c                                     this is a bullshit trick and
c                                     will cause errors of someone
c                                     uses a function other than G.
      data exten/'-V(j/b)','S(j/k)'/
c                                     composant stoichiometry:
 
      do i = 1, isat 
         write (n3,1000) cname(icp+i),vus(i),names(jds(i))
      end do 
c                                     fluid stoichiometry:
      if (ifyn.eq.0) then 
         do i = 1, 2
            if (iff(i).ne.0) write (n3,1010) names(i),vuf(i)
         end do 
      end if 
c                                     mobile components:
      do i = 1, jmct
         write (n3,1020) cname(jprct+i),du(i),vname(3+i)
      end do
c                                     dependent extensities 1 and 2, 
c                                     normally entropy and negative 
c                                     volume:
      do i = 1, 2
         write (n3,1030) exten(i), dv(i),vname(i)
      end do

1000  format (10x,'Delta(',a7,') =',g9.3,1x,
     *            '(saturated composant=',a8,')')
1010  format (10x,'Delta(',2x,a5,') =',g9.3,1x,
     *            '(saturated phase component)')
1020  format (10x,'Delta(',a7,') =',g9.3,1x,
     *            '(dependent conjugate of ',a8,')')
1030  format (10x,'Delta(',a7,') =',g9.3,1x,
     *            '(dependent conjugate of ',a8,')')
      end 

      subroutine delrxn
c-----------------------------------------------------------------------
c delrxn computes the change in the dependent extensities for a reaction
c these are returned in cst21, delrxn is always called after UNIVEQ, so
c the free energy at the present conditions is zero, if called 
c independently, then grxn must be called first.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5   /v(5),tr,pr,r,ps/ cst21 /du(2),dv(2),jds(h5),ifr 
     *      / cst87  /delt(l2),dtol,utol,ptol/ cst102 /ivfl
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40  /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst201 /vuf(2),vus(h5),iffr,isr/ cst208 /ifct,idfl
     *      / cst25  /vnu(k7),idr(k7),ivct/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst31  /vn(k2,k7),irct,ird
     *      / cst1   /thermo(k4,k10),uf(2),us(h5)
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct
     *      / cst6   /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
c-----------------------------------------------------------------------
c                                         save dependent extensities

c                                         it is necessary to calculate
c                                         the initial G cause iptol may
c                                         be large relative to the 
c                                         change in G due to delt
      call grxn (og)

      do i = 1, 2
         v(i) = v(i) + delt(i)
         call grxn (gval)   
         dv(i) = (og - gval) / delt(i)
         v(i) = v(i) - delt(i) 
      end do 
c                                         get deltas on mobile comps
      if (jmct.gt.0) then
         do i = 1, jmct
            du(i) = 0d0
         end do

         do i = 1, ivct 
            do j = 1, jmct 
               du(j) = du(j) + vnu(i) * cp(jprct+j,idr(i))
            end do 
         end do
      end if 
c                                         do inverse projection to 
c                                         get stoichiometries of 
c                                         constrained components:
      if (isat.gt.0) then
         if (isat.gt.1) then  
            do j = 1, isat - 1
               do i = j + 1, isat
                  vus(j) = vus(j) - vus(i) * cp(icp+j,idss(i))
               end do 
            end do
         end if 
c                                         now do the effects of 
c                                         the inverse projection on
c                                         saturated phase components:
         if (ifct.ne.0) then 
            do i = 1, isat
               do j = 1, ifct
                  vuf(j) = vuf(j) - vus(i) * cp(jfct+j,idss(i))
               end do
            end do 
         end if 
c                                         now do the effects of 
c                                         the inverse projection on
c                                         mobile components:
         if (jmct.ne.0) then 
            do i = 1, isat 
               do j = 1, jmct
                  du(j) = du(j) - vus(i) * cp(jprct+j,idss(i))
               end do 
            end do 
         end if 
      end if 
c                                         see if coefficients should 
c                                         change sign:
      invert = 1
      if (ivfl.eq.1.and.dv(1).gt.0d0) then 
         invert = 0
      else if (ivfl.eq.2.and.dv(2).gt.0d0) then 
         invert = 0
      else if (ivfl.gt.2) then 
         if (du(ivfl-3).gt.0d0) then 
c                                         diagram must have a mu variable 
            invert = 0
         else if (du(ivfl-3).eq.0d0) then
            if (iv(2).eq.3) then
               if (vuf(2).lt.0d0) invert = 0
            else if (iv(1).eq.3) then
               if (vuf(2).lt.0d0) invert = 0
            else if (du(iv(1)-3).gt.0d0) then
               invert = 0
            end if
         end if
      end if

      if (invert.eq.1) then 
         do i = 1, ivct 
            vnu(i) = -vnu(i)
c                                         why different signs?
            vn(ird,i) = vnu(i)
         end do

         do i = 1, isat
            vus(i) = -vus(i)
         end do 

         do i = 1, ifct
            vuf(i) = -vuf(i)
         end do

         do i = 1, jmct 
            du(i) = -du(i)
         end do

         dv(1) = -dv(1)
         dv(2) = -dv(2)

      end if   
c                                         save id's of saturated comps
      do i = 1, isat
         jds(i) = idss(i)
      end do

      end     

      subroutine header (icopt)
c-----------------------------------------------------------------------
c header writes the graphics file header for schreinemakers diagrams
c to lun n4
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names,vname,xname,title*72,fname*10,cname*5

      common/ cst8   /names(k1)/ cst6 /icomp,istct,iphct,icp/ cst79 /iso
     *      / csta2  /xname(k5),vname(l2)
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b/ cst18 /jvol(2),ivol
     *      / cst60  /ipoint,imyn/ cst12 /cp(k5,k1)
     *      / cst9   /vmax(l2),vmin(l2),dv(l2)/ csta7 /fname(h9)
     *      / cst24  /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ csta8 /title
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst102 /ivfl/ cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn/ csta4 /cname(k5)
     *      / cst43 /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont
     *      / cst316 /c0,c1,c2,c3,c4,iind,idep
     *      / cst319 /emod(k15,k10),iemod(k1),kmod
c-----------------------------------------------------------------------
c                              value to be read as icopt
c      junk = icopt
c      if (icopt.eq.7) junk = 5
      write (n4,1020) icopt
c                              phase and solution count:
      write (n4,1020) iphct, iso, icomp  
c                              component and volatile counters:
      write (n4,1020) jbulk, ivol, icp, jvol, kmod
c                              write component names
      write (n4,1090) (cname(i), i = 1, icomp)
c                              component at wts:
      write (n4,1100) (atwt(i), i = 1, icomp)
c                              phase names:
      write (n4,1010) (names(i), i = 1, iphct)
c                              phase compositions:
      write (n4,1100) ((cp(j,i), j = 1, icomp), i = 1, iphct)
c                              solution phase flags:
      write (n4,1060) (ikp(i), i = 1, iphct)  
c                              solution names:
      if (iso.ne.0) write (n4,1040) (fname(i), i = 1, iso)
c
      write (n4,1000) title                                
c                               saturated and buffered component names:
      if (isyn.eq.0) then 
         write (n4,1070) (cname(i+icp), i= 1, isat)
      else 
         write (n4,1000) ' '
      end if 
c                               find out which variables are p and t:
      ipres = 0 
      itemp = 0 
      jind = 0 

      do i = 1, ipot
         if (jv(i).eq.1) then
            ipres = i
         else if (jv(i).eq.2) then
            itemp = i
         end if
      end do 

      if (idep.eq.1) then 
         jind = itemp
      else if (idep.eq.2) then
         jind = ipres
      end if 
         

      write (n4,1080) vname(ivfl)

      if (icont.le.1) then 
         write (n4,1020) ipot, (jv(i), i = 1, ipot), ipres, itemp
         write (n4,1110) 0,jind,idep,c0,c1,c2,c3,c4
         write (n4,1030) (vmax(jv(i)), vmin(jv(i)), i= 1, ipot)
         write (n4,1000) (vname(jv(i)), i= 1, ipot)
      else 
c                               gridded minimization with compositions
         j = icont - 1
         write (n4,1020) ipot+j, (i,i=1,j),
     *                   (j+jv(i), i = 1, ipot), ipres+j, itemp+j
         write (n4,1110) j, jind+j, idep, c0, c1, c2, c3, c4
         write (n4,1030) (1d0,0d0, i = 1, j)
         write (n4,1030) (vmax(jv(i)), vmin(jv(i)), i= 1, ipot)
         write (n4,1050) 
         if (j.eq.2) write (n4,1055)
         write (n4,1000) (vname(jv(i)), i= 1, ipot)
         write (n4,1030) ((dblk(i,j),j=1,jbulk),i=1,icont)
      end if 
     

1000  format (a)
1010  format (10a8)
1020  format (20(i8,1x))
1030  format (6(d11.5,1x))
1040  format (8a10)
1050  format ('X(C2)')
1055  format ('X(C3)')
1060  format (26(i2,1x))
1070  format ('Component saturation hierarchy: ',7(a5,1x))
1080  format ('Reaction equations are written such that the high ',
     *         a,/,'assemblage is on the right of the = sign')
1090  format (20a5)
1100  format (20(g9.3,1x))
1110  format (3(i1,1x),5(g12.6,1x))

      end

      subroutine lchk (lphi,lchkl)
c-----------------------------------------------------------------------
c a function subprogram to determine if a phase, lphi, lies
c below a g-x plane defined by the vertices idv.
c lchk has a value of 0 if the plane is stable, and 1 if not.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      integer i, lphi, lchkl, idv, iophi, idphi, j

      double precision a, g, gproj, b, gphi, cp, dtol, utol, ptol, delt

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst62 /blim(l2),ulim(l2),dg/ cst12 /cp(k5,k1)
     *      / cst6 /icomp,istct,iphct,icp/ cst2 /g(k1)
     *      / cst87 /delt(l2),dtol,utol,ptol
c-----------------------------------------------------------------------
      call uproj

      do i = 1, icp
        b(i) = gproj(idv(i))
      end do 

      g(lphi) = gproj(lphi)
      lchkl = 0

      call subst (a,ipvt,icp,b)

      gphi = 0d0
      do j = 1, icp
         gphi = gphi + cp(j,lphi) * b(j)
      end do

      if (gphi.lt.g(lphi)) goto 99

      lchkl = 1

99    end

      subroutine newass (b,idv,idphi,ivd,ivi,iste)
c-----------------------------------------------------------------------
c newass determines the (pseudo-) divariant assemblages generated by
c a reaction based on the stoichiometric coefficients of the phases
c in the reaction as determined by routine balanc.

c input:  idv(icp) - a vector containing the indices of phases in a
c           divariant assemblage.
c         idphi - the index of the phase which is involved with the
c           the divariant assemblage (idv) in a univariant equilibrium.
c         b(icp) - the stoichiometric coefficient of the phases inedxed
c           by idv in the reaction of these phases to form the phase
c           idphi. by convention the stoichiometric coeffeicient of
c           idphi is -1.0.

c output: jdv(icp) - a vector containing the indices of phases in a
c           divariant assemblage generated by the reaction between the
c           phases idphi and idv this vector is used by the routine
c           chkass. this assemblage is stable on the opposite 'side' of
c           the univariant equilibrium from the assemblage idv.

c referenced by: coface
c references to: chkass

c algorithm: phases with null or negative coefficients are always
c   compatible with eachother on the 'side' of a univariant reaction
c   which idv is metastable. the k phases with positive coefficients
c   define a chemographic simplex surrounding phases with null or
c   negative coefficients. consequently a univariant reaction will
c   generate k new divariant assemblages consisting of k-1 of the phases
c   in the original assemblage (idv) and all the phases with null or
c   negative coefficients. if only one phase has a positive coefficient
c   this phase is unstable and only one new assemblage will be generated
c   which consists of all the other phases (including idphi).
c-----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer idv(k5),idpos(k5),jdv(k5+1),idsol(k5),kdsol(k5),io3b,
     *        jbulk,icomp,istct,iphct,icp,ncpd,np,kdbulk,igo,iaid,
     *        ipos,i,j,ineg,idphi,jtic,k,ivd,ivi,iste,idbulk,iap,ibulk
     
      double precision b(k5+1),cblk,atw

      common/ cst6   /icomp,istct,iphct,icp
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2)
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b 
c-----------------------------------------------------------------------
c                                 if fully constrained bulk compositions
c                                 get identity of first bounding assemblage
      if (jbulk.ge.icp) then 
c        call blkmat (idv,kdbulk,igo)
         call blkmat (idv,idsol,kdsol,ncpd,np,icp,kdbulk,igo)
         if (igo.ne.1) call error (999,atw(1),igo,'NEWASS')
         iaid(1) = kdbulk
         iaid(2) = 0
      end if 
c                                 initialize counter:
      ipos = 0

      do i = 1, icp
c                                 is the b(i) coefficient zero/negative?
         if (b(i).ge.1d-5) then 
c                                 no, save in the array idpos and count.
            ipos = ipos + 1
            idpos(ipos) = idv(i)
         else 
c                                 yes, load into the array jdv
           jdv(i-ipos) = idv(i)
         end if 
      end do 
c                                 load idphi
      ineg = icp - ipos+1
      jdv(ineg) = idphi
c                                 generate the divariant assemblages
      do j = 1, ipos
c                                 make ipos permutations of ipos - 1 phases
         jtic = 0
         do k = 1, ipos
            if (k.ne.j) then
               jtic = jtic + 1
               jdv(ineg+jtic) = idpos(k)
            end if 
         end do            
c                                 send to the routine chkass to store th
c                                 the assemblage if unique.
         call chkass (jdv,ivd,ivi,iste)
      end do
c                                 done
      end

      subroutine onedim
c----------------------------------------------------------------------
c for phase diagrams with one extrinsic variable (iv1) onedim sorts
c equilibria, identified by newhld and located by coface, by veq(iv1).
c the sorted equilibria are then output to units n3 and n4 as requested.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      parameter (k8=k5+2)

      common/ cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst29 /vip(l2,k2),ipid(k2,k8),ipct/ cst13 /ir(k2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst31 /vn(k2,k7),irct,ird
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9    
c-----------------------------------------------------------------------
c                             any rxns to be sorted?
      if (irct.le.1) goto 30
c                             initialize:
      ist = 2

      do i = 1, irct
         ir(i) = i
      end do 
c                             begin sort loop:
20    ist1 = ist - 1
      itemp = ir(ist1)
      vst = vip(iv1,itemp)

      do i = ist, irct
         if (vip(iv1,ir(i)).le.vst) then
            ir(ist1) = ir(i)
            ir(i) = itemp
            itemp = ir(ist1)
            vst = vip(iv1,itemp)
         end if 
      end do

      ist = ist + 1
      if (ist.gt.irct) goto 30
      goto 20
c                             end of sort loop
30    if (irct.eq.0) goto 99
c                             output invariant reactions and conditions:
      call outirn 

99    end
            
      subroutine outier
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character vname*8, xname*8, text(400)*1

      common/ cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst29 /vip(l2,k2),ipid(k2,k5+2),ipct
     *      / cst24 /ipot,jv(l2),iv(l2)/ cst6 /icomp,istct,iphct,icp
     *      / csta2 /xname(k5),vname(l2)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst81 /ipchk(k2),ip2(k2),icp2,ifake
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2) 
     *      / cst75  /idasls(k5,k3),iavar(2,k3),iasct
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b 
c-----------------------------------------------------------------------
c                                 write end of ptx data for univariant
c                                 equilibria marker to graphics file.
      if (io4.ne.1) then
         write (n4,1090)
         write (n4,1000) ipct,icp2
         do j = 1, ipct
c                                 n4:
            write (n4,1000) j, ivarip(j), (ipid(j,i),i = 1, icp2)
            write (n4,1060) (vip(jv(i),j), i = 1, ipot)
         end do     
      end if 
c                                 write ip names and locations to
c                                 units n6 and n3.
      if (ipct.eq.0) goto 99

      if (io3.ne.1) then 
         write (n3,1050)
         write (n3,1030)
         if (io3p.ne.1) then 
            do j = 1, ipct
         
               call iptext (text,iend,j,icp2)
               write (n3,1080) j, ivarip(j), (text(i), i= 1, iend)
               write (n3,1020)
               write (n3,1040) (vname(jv(i)), vip(jv(i),j), i= 1, ipot)
            end do 
         end if 
      end if
    
1000  format (14(i6,1x))
1020  format (15x,'occurs at:')
1030  format ('(pseudo-) invariant points are summarized below:')
1040  format (25x,a8,'=',g12.6)
1050  format (/,80('-'),/)
1060  format (5(1x,g12.6))
1080  format (/,' (',i6,'-',i1,') ',380a1)
1090  format (' 1 1 1 1 1 1 1 1 1 EOR',/,' 1.0 EOR')

99    end

      subroutine outirn
c-----------------------------------------------------------------------
c outirn writes the identity and equilibrium conditions of invariant
c equilibria to units n3 and n4 after they have been sorted by onedim.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 vname, xname*8, rxnstr*(kd2)

      common/ cst31 /vn(k2,k7),irct,ird
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst6  /icomp,istct,iphct,icp/ cst104 /rxnstr(k2)
     *      / cst29 /vip(l2,k2),ipid(k2,k5+2),ipct/ cst5  /v(l2),q(4)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst35 /irv(k2)/ cst13 /ir(k2)/ csta1 /irnms(k2,k7)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / csta2 /xname(k5),vname(l2)
c-----------------------------------------------------------------------
c                             write invariant reactions:
      do i = 1, irct
         j = ir(i)
c                             the index on these two guys used to be j
c                             must be wrong? if so why did it work 
c                             before? changed to i. you know sometimes
c                             i'm so stupid, i just can't believe it.
         k = irv(j)
         l = ivarrx(j)
         if (io3.ne.1) then 
c                             output to print file:
            if (l.eq.1) then
               write (n3,1030) j, l, rxnstr(j)
            else
               write (n3,1040) j, l, rxnstr(j)
            end if 
            write (n3,1070) vname(iv1), vip(iv1,j), vname(iv2), 
     *                                  vip(iv2,j)
            write (n3,1160)
         end if 
c                             output to graphics file
         if (io4.ne.1) then 
            write (n4,1000) j, k, l, vip(iv1,j), (irnms(j,l), l= 1, k)
            write (n4,1050) (vn(j,l), l = 1, k)
         end if 
      end do 

1000  format (i6,1x,2(i6,1x),g12.5,1x,6(i6,1x))
1030  format ('The equilibrium of the invariant reaction:',//,
     *       ' (',i6,'-',i1,') ',a)
1040  format ('The equilibrium of the pseudoinvariant reaction:',//,
     *       ' (',i6,'-',i1,') ',a)
1050  format (12x,6(g9.2,1x))
1070  format (/,'occurs at ',a,'=',g12.6,' and ',a,'=',g12.6)
1160  format ('      ----------------------------------------')

      end

      subroutine outlst
c----------------------------------------------------------------------
c outlst writes cumulative lists of the univariant and invariant
c equilibria identified during a single execution of the program
c to the graphics and print files. outlst also calls the subroutine
c out5 to write a permanent list (on n5) of equilibria for multiple
c executions.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names,cname*5,rxnstr*(kd2)

      common/ cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst81 /ipchk(k2),ip2(k2),icp2,ifake
     *      / cst31 /vn(k2,k7),irct,ird/ cst105 /ibug(k2)
     *      / cst8  /names(k1)/ cst29 /vip(l2,k2),ipid(k2,k5+2),ipct
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst37 /ixct,iexyn/ csta4 /cname(k5)
     *      / cst80 /irchk(k2)/ cst104 /rxnstr(k2)
c------------------------------------------------------------------------
c                             printing to n4 is suppressed for the
c                             erlanger system 
      jbug = 0

      do i = 1, irct
         if (ibug(i).ne.0) jbug = 1
      end do    

      if (jbug.ne.0) then 
         write (*,1160)
         write (*,1190)
         do i = 1, irct
            if (ibug(i).ne.0) write (*,1000) i,ivarrx(i),rxnstr(i)
         end do 
         write (*,1160)
      end if 

      if (io3.eq.1) goto 99
c                             invariant point lists:
      if (ipct.ne.0) then 
c                             output print file heading:
c                             output ip lists:
         write (n3,1030)
         do i = 1, ipct
            write (n3,1150) i,ivarip(i),(names(ipid(i,k)),k=1,icp2)
         end do 
      end if 
c                              write end of invariant point list
c                              marker to graphics file.
      write (n3,1160)
c                              write list of stable univariant
c                              equilibria to units n3 and n4.
      write (n3,1070)

      do i = 1, irct
         write (n3,1000) i,ivarrx(i),rxnstr(i)
      end do    
c                              if itic= 0 no equilibria were identified
c                              goto 9000 to write no eq mess.
      if (irct.eq.0) goto 9000

      write (n3,1160)
c                              jbug ne 0, one or more equilibria
c                              may not have completely defined, write
c                              list of possible failures:
      if (jbug.ne.0) then 
         write (n3,1160)
         write (n3,1190)
         do i = 1, irct
            if (ibug(i).ne.0) write (n3,1000) i,ivarrx(i),rxnstr(i)
         end do 
         write (n3,1160)
      end if 

      goto 99
c                              write no equilibrium messages:
9000  write (n3,1180)

1000  format (' (',i6,'-',i1,') ',a)
1030  format ('(pseudo-) invariant points are summarized below:',/)
1070  format ('(pseudo-) univariant equilibria are summarized ',
     *       'below:',/)
1150  format (' (',i6,'-',i1,') ',12(a,1x))
1160  format (//,80('-'),//)
1180  format ('no univariant or invariant equilibria occur.')
1190  format ('WARNING!! The stability fields of the following',
     *        ' equilibria may',/,'have been entirely or',
     *        ' partially skipped in the calculation: ',/)
99    end

      subroutine iptext (text,iend,jp,icp2)
c-----------------------------------------------------------------------
c subprogram to write a text label for an invariant point 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names, text(400)*1, fname*10

      common/ cst8  /names(k1)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst29 /vip(l2,k2),ipid(k2,k5+2),ipct/ csta7 /fname(h9)

c                             dump the names into the array text
      ist = 1

      do 20 i = 1, icp2
         id = ipid(jp,i)
         if (ikp(id).eq.0) then
c                             simple compound:
            read (names(id),1000) (text(j), j = ist, ist + 7)
            ist = ist + 8
         else  
c                             solution phases:
            read (fname(ikp(id)),1000) (text(j), j = ist, ist + 9)
            text(ist + 10) = '('
            read (names(id),1000) (text(j), j = ist + 11, ist + 18)
            text(ist + 19) = ')'
            ist = ist + 20
         end if 
         text(ist) = ' '
20       ist = ist + 1 
c                             now filter out blanks
      iend = 1
      do 40 i = 2, ist - 1
         if (text(i).eq.' '.and.text(i + 1).eq.' ') then 
            goto 40 
         else if (text(i).eq.' '.and.text(i + 1).eq.')') then 
            goto 40
         else if (text(i).eq.' '.and.text(i + 1).eq.'(') then 
            goto 40 
         end if 
      iend = iend + 1
      text(iend) = text(i)
40    continue

1000  format (20a1)

      end 

      subroutine rxntxt (iend,jend,text,alpha)
c-----------------------------------------------------------------------
c subprogram to write a text label for a reaction 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names, text(kd2)*1, fname*10,
     *            alpha(130)*1, string*(kd2), rxnstr*(kd2)

      integer iplus(k5), iminus(k5), jplus(k5), jminus(k5), 
     *        ione(k5+1), jone(k5+1)

      common/ csta7 /fname(h9)/ cst8 /names(k1)
     *      / cst25 /vnu(k7),idr(k7),ivct
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst31 /vn(k2,k7),irct,ird
     *      / cst80 /irchk(k2)/ cst104 /rxnstr(k2)

      ip = 0 
      im = 0
      jend = 1
c                             load id's of phases with + or - coeffs 
      do i = 1, ivct 
         if (vnu(i).gt.0d0) then 
            ip = ip + 1
            iplus(ip) = idr(i)
            jplus(ip) = i
         else 
            im = im + 1
            iminus(im) = idr(i)
            jminus(im) = i
         end if 
      end do 

      do i = 1, im 
         jone(i) = jminus(i)
         ione(i) = iminus(i)
      end do 

      do i = 1, ip
         j = i + im
         jone(j) = jplus(i)
         ione(j) = iplus(i)
      end do 
c                             dump coefficients into string:
      write (string,1010) (vnu(jone(i)),i = 1, ivct)
c                             read alpha from string:
      jst = 6 + ivct * 11
      read (string,1000) (alpha(i), i = 1, jst)
c                             filter blanks and add right )
      do 50 i = 1, jst 
         if (alpha(i).eq.' '.and.alpha(i + 1).eq.' ') then 
            goto 50
         else if (alpha(i).eq.' '.and.alpha(i + 1).eq.',') then
            goto 50
         end if 
         alpha(jend) = alpha(i)
         jend = jend + 1
50    continue

      if (alpha(jend-1).eq.' '.or.alpha(jend-1).eq.',') 
     *                                       jend = jend - 1  
      alpha(jend) = ')'
c                             now dump the names into the array text
      ist = 1
      is = 1
80    do i = is, im 
         id = ione(i)
         if (ikp(id).eq.0) then
c                             simple compound:
            read (names(id),1000) (text(j), j = ist, ist + 7)
            text(ist+8) = ' '
            ist = ist + 9
         else  
c                             solution phases:
            read (fname(ikp(id)),1000) (text(j), j = ist, ist + 9)
            text(ist + 10) = '('
            read (names(id),1000) (text(j), j = ist + 11, ist + 18)
            text(ist + 19) = ')'
            text(ist+20) = ' '
            ist = ist + 21
         end if 
      end do 

      if (is.eq.1) then 
         text(ist) = '='
         text(ist + 1) = ' '
         ist = ist + 2
         is = im + 1
         im = ivct 
         goto 80 
      else 
         text(ist) = ' ' 
      end if 
c                             now filter out blanks
      iend = 1
      do 40 i = 2, ist
         if (text(i).eq.' '.and.text(i + 1).eq.' ') then 
            goto 40 
         else if (text(i).eq.' '.and.text(i + 1).eq.')') then 
            goto 40
         else if (text(i).eq.' '.and.text(i + 1).eq.'(') then 
            goto 40 
         end if 
         iend = iend + 1
         text(iend) = text(i)
40    continue
c                             this is redundant if the 
c                             reaction was found before. 
      write (rxnstr(ird),1000) (text(i),i=1,iend)

1000  format (434a1)
1010  format ('Alpha(',13(g9.3,', '))

      end 

      subroutine outrxn (ip,ier)
c-----------------------------------------------------------------------
c outrxn writes the identity and v1-v2 loci of univariant equilibria
c to units n3 and n4.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character text(kd2)*1, alpha(130)*1, fname*10

      common/ cst31 /vn(k2,k7),irct,ird/ cst7 /iflag/ csta7 /fname(h9)
     *      / cst25 /vnu(k7),idr(k7),ivct/ cst32 /ptx(l5),ipt2
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst105 /ibug(k2)/ cst103 /isec,icopt,ifull,imsg,io3p   
      common/ cst6   /icomp,istct,iphct,icp/ debug /jtest,jpot
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2) 

      do i = 1, ivct 
         if (ikp(idr(i)).lt.0) goto 99
      end do 
c                                set bug flag to indicate that
c                                the equilibrium was at least 
c                                partially located.
      if (ier.lt.2) ibug(irct) = 0
c                                write stefano's list to 
c                                reaction_list.dat
      if (ird.eq.irct.and.jtest.eq.3) call stetxt

      if (ifull.eq.0) then 
         call rxntxt (iend,jend,text,alpha)
      else  
         call fultxt (iend,text)
      end if 

      if (icopt.eq.3) goto 99
         
      if (imsg.eq.0) write (*,1110) ird,(text(i),i = 1, iend)

      if (io3p.eq.1) goto 10

      write (n3,1120) ird,ivarrx(ird),(text(i),i = 1, iend)

      if (ifull.ne.0) goto 80

      write (n3,1130) (alpha(i),i = 1,jend)

      if (ipt2.lt.3) then 
         write (n3,*)
         goto 99
      end if 

      call outdel 

80    write (n3,*)
      write (n3,1000) (ptx(i), i = 1, ipt2)
      write (n3,*)

      if (ier.ne.0) goto 10
      if (iflag.eq.1) goto 40 
      goto 10

40    write (n3,1070) ip        
      write (n3,1040)

10    if (io4.eq.1) goto 99

      if (jbulk.lt.icp) then 
         write (n4,1080) ipt2,ird,ivar,ivct,(idr(i),i=1,ivct),0,0,0,0
      else 
         if (iaid(2).eq.0) then 
            write (*,*) 'Sommat wot? ',ird,iaid
            iaid(2) = iaid(1)
         end if
         write (n4,1080) ipt2,ird,ivar,ivct,(idr(i),i=1,ivct),iaid,
     *                   iap(iaid(1)),iap(iaid(2))
      end if 

      write (n4,1090) (vnu(i),i = 1, ivct)
      write (n4,1000) (ptx(i),i = 1, ipt2)

1000  format (1x,g12.6,1x,g12.6,2x,g12.6,1x,g12.6,2x,g12.6,1x,g12.6)
1040  format (/)
1070  format ('the equilibrium extends to invariant point (',i6,')')
1080  format (16(i6,1x))
1090  format (6(g12.4,1x)) 
1110  format ('finished with equilibrium (',i6,') ',434a1)
1120  format (' (',i6,'-',i1,') ',434a1)
1130  format (/,10x,90a1)

99    end

      subroutine pchk (dg,igo)
c-----------------------------------------------------------------------
c a subprogram which returns the difference in free energy
c beteen a g-x plane, defined by the assemblage idv, and a phase lphi.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst6  /icomp,istct,iphct,icp/ cst2 /g(k1)/ cst7 /iflag
     *      / cst12 /cp(k5,k1)/ cst87 /delt(l2),dtol,utol,ptol
c-----------------------------------------------------------------------
c                                 compute energies:
      igo = 0

      call uproj

      do i = 1, icp
         b(i) =gproj(idv(i))
      end do 

      dg=gproj(idphi)
c                                 solve for chemical potentials:
      call subst (a,ipvt,icp,b)
c                                 compute energy difference:
      do j = 1, icp
         dg = dg - cp(j,idphi)*b(j)
      end do 

      if (dabs(dg).lt.ptol) then
        igo = 1
        iflag = 1

        call ssaptx 
      else
        if (dg.gt.dtol) then
          iflag= 0

          call ssaptx
        else
          iflag= 1
        end if
      end if

      end

      subroutine pivots (ier)
c--------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst6 /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
c--------------------------------------------------------------------
c                              calculate pivots for the matrix 'a'
      do k = 1, icp
         do j = 1, icp
            a(k,j) = cp(j,idv(k))
         end do 
      end do  

      call factor (a,icp,ipvt,ier)

      end

      subroutine schk (lphi)
c-----------------------------------------------------------------------
c schk my ___
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),u(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst6 /icomp,istct,iphct,icp/ cst62 /blim(l2),ulim(l2),dg
     *      / cst87 /delt(l2),dtol,utol,ptol
     *      / cst2 /g(k1)/ cst12 /cp(k5,k1)/ cst7 /iflag
c-----------------------------------------------------------------------
c                                 determine chemical potentials
      iflag= 0
      do i = 1, icp
         u(i) = g(idv(i))
      end do 

      call subst (a,ipvt,icp,u)
c                                 test phases, not=iophi, against the
c                                 assemblage idv(i),i = 1, icp
      do 20 i = istct, iphct
         if (i.eq.iophi.or.i.eq.lphi) goto 20
         dg = g(i)

         do j = 1, icp
            dg = dg - cp(j,i) * u(j)
         end do 

         if (dg.gt.dtol) goto 20
c                                check that a phase is not metastable
c                                with respect to itself, this
c                                could be remedied by changing the 
c                                value of dtol.
         do j = 1, icp
            if (idv(j).eq.i) goto 20
         end do 

         iflag = iflag + 1
         idphi = i
         goto (20,40,40), iflag

20    continue

40    end

      subroutine nschk 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),u(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst6 /icomp,istct,iphct,icp/ cst62 /blim(l2),ulim(l2),dg
     *      / cst87 /delt(l2),dtol,utol,ptol
     *      / cst2 /g(k1)/ cst12 /cp(k5,k1)/ cst7 /iflag
c-----------------------------------------------------------------------
c                                 determine chemical potentials
      iflag = 0

      do i = 1, icp
         u(i) = g(idv(i))
      end do 

      call subst (a,ipvt,icp,u)
c                                 test phases, not=iophi, against the
c                                 assemblage idv(i),i = 1, icp
      do 20 i = istct, iphct
         if (i.eq.iophi) goto 20

         dg = g(i)

         do j = 1, icp
            dg = dg - cp(j,i) * u(j)
         end do 

         if (dg.gt.dtol) goto 20
c                                check that a phase is not metastable
c                                with respect to itself, this
c                                could be remedied by changing the 
c                                value of dtol.
         do j = 1, icp
            if (idv(j).eq.i) goto 20
         end do 

         iflag = iflag + 1
         idphi = i
         goto (20,40,40), iflag

20    continue

40    end

      subroutine search (vst,vsti,ist,iste,ivi,ivd,div,ier)
c-----------------------------------------------------------------------
c search looks for an equilibrium around the periphery of the coordinate
c frame specified for a spd calculation. if a stable equilibrium is
c found ier = 0, if an error occurs ier =2, and if no equilibrium is
c found ier =1.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1     
     *      / cst6 /icomp,istct,iphct,icp/ cst5 /v(l2),x(4)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ cst38 /iedge
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)/ cst8 /names(k1)
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b/ cst7 /iflag
     *      / cst49 /value,ismax,igot
c-----------------------------------------------------------------------
c                                 initialization
      iflg1 = 0
      ier = 0
      iugh = 0

      v(iv1) = vst
      call incdep (iv1,v(iv1))
      v(iv2) = vst
      call incdep (iv2,v(iv2))

      do i = ist, iedge

         iste = i
c                                 set default dependent and independent
c                                 variables and increments for each edge
c                                 to be searched.
         if (i.eq.1) then 
c                                 traverse 1.
            ivi = iv2
            ivd = iv1
            ddv = dv(ivd)
            div = dv(ivi)
            v(ivi) = vmin(ivi)

         else if (i.eq.2) then 
c                                 traverse 2.
            ivi = iv1
            ivd = iv2
            ddv = dv(ivd)
            div = -dv(ivi)
            v(ivi) = vmax(ivi)

         else if (i.eq.3) then 
c                                 traverse 3.
            ivi = iv2
            ivd = iv1
            ddv = -dv(ivd)
            div = -dv(ivi)
            v(ivi) = vmax(ivi)

         else 
c                                 traverse 4.
            ivi = iv1
            ivd = iv2
            ddv = -dv(ivd)
            div = dv(ivi)
            v(ivi) = vmin(ivi)

         end if 

         call incdep (ivi,v(ivi)) 

c                                 in some cases newass may generate
c                                 a metastable assemblage call flipit
c                                 to check. if the assemblage is
c                                 stable flipit will return jer = 0,
c                                 if it is metastable flipit tries a
c                                 reverse traverse and returns jer =1
c                                 if it is succesful, and jer =2 if not.
         if (i.eq.ist) then
            v(ivd) = vst
            v(ivi) = vsti
            call incdep (ivd,v(ivd)) 
            call incdep (ivi,v(ivi)) 
         end if 

         call flipit (ddv,vst,ivd,ist,i,jer)

         if (jer.eq.2) then 
            write (*,1000) (names(idv(k)), k= 1, icp)
            write (*,1010) v
            ier = 1
            goto 99
         end if 

         vlast = v(ivd)

         if (jer.eq.1) goto 99
c                                 begin search:
100      v(ivd) = v(ivd) + ddv
         call incdep (ivd,v(ivd)) 
c                                 out of range?:
         if (i.lt.3) then 

            if (v(ivd).gt.vmax(ivd)) then
               v(ivd) = vmax(ivd)
               ddv = dabs(vmax(ivd) - vlast)
            else if (i.eq.ist) then
               if (v(ivd).gt.vst) goto 130
               ddv = dabs(ddv)/2d0
               v(ivd) = vst
               iflg1= 0
               goto 100
            end if

         else 

            if (v(ivd).lt.vmin(ivd)) then
               v(ivd) = vmin(ivd)
               ddv = -dabs(vlast - vmin(ivd))
            else if (i.eq.ist) then
               if (v(ivd).lt.vst) goto 130
               ddv = -dabs(ddv)/2d0
               v(ivd) = vst
               iflg1 = 0
               goto 100
            end if

         end if 

         call incdep (ivd,v(ivd))
c                                 calculate phase energies:
130      call gall 
c                                 test stability of the assemblage:
         call asschk

         if (iflag.eq.0) vlast = v(ivd)
c                                 iflag= 0 still stable
c                                 iflag=1 metastable with respect to one
c                                         phase indexed by idphi.
c                                 iflag=2 metastable with respect to
c                                         multiple phases refine the
c                                         search increment.
         if (iflag.eq.1) goto 99
c                                 check if search is in range:
         if (i.lt.3) then 
            if (v(ivd).ge.vmax(ivd).and.iflag.eq.0) cycle
         else 
            if (v(ivd).le.vmin(ivd).and.iflag.eq.0) cycle
         end if 
c                                 refine increment if necessary:
         call delvar (ddv,iflag,iflg1)
c                                 check if increment has been refined
c                                 beyond the tolerance which is
c                                 arbitrarily set at 1.d-8.
         if (dabs(ddv).lt.1d-8) then

            iugh = iugh + 1

            if (i.lt.3) then 
c                                 traverses 1,2.
               ddv = dv(ivd)
            else  
c                                 traverses 3,4.
               ddv = -dv(ivd)
            end if

            v(ivd) = vlast
            call incdep (ivd,v(ivd)) 
            ddv = ddv/10.
            iflg1 = 0
   
            if (iugh.gt.3) then
               write (n3,1020) ivd,dv(ivd),(names(idv(j)),j = 1, icp)
               write (n3,*) ' '
               ier = 2
               goto 99
            end if 
      
            goto 100 

         end if 
c                                 increment conditions again:
         goto 100
c                                 next traverse.
      end do 

      ier = 1

1000  format ('Metastable assemblage in FLIPIT, ',
     *        'the assemblage is:',/,4x,12(1x,a))
1010  format ('v =',5(g12.6,1x))
1020  format (/,'**warning ver047** SEARCH: > 1 equilibrium',
     *         ' occurs within the minimum search',/,'increment for',
     *         ' variable: ',i1,'. This often occurs as YCO2 =>',
     *         ' 1 or => 0.',/,'You may be able to correct this by',
     *         ' assigning slightly different limits',/,'for YCO2 or',
     *         ' by reducing the default increment for this variable',
     *         ' (',g12.3,')',/,'in the computational option file.',/,
     *         'Equilibria involving the following assemblage may',
     *         ' not be delineated:',/,7(1x,a8)) 
99    end

      subroutine sfol1 (ivd,ivi,ier,dv,ikwk,irend)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst5  /v(l2),tr,pr,r,ps/ cst80 /irchk(k2)/ cst7 /iflag
     *      / cst24 /ipot,jv(l2),iv(l2)/ cst9 /vmax(l2),vmin(l2),ddv(l2)
     *      / cst6  /icomp,istct,iphct,icp/ cst62 /blim(l2),ulim(l2),dg
     *      / cst31 /vn(k2,k7),irct,ird/ cst87 /delt(l2),dtol,utol,ptol     
c-----------------------------------------------------------------------
      if (ikwk.eq.1) then
         irchk(ird) = 1
         goto 9000
      end if

      iswtch = 0
      jswtch = 0 
      iflag = 0 
c                                 begin traverse:
10    v(ivi) = v(ivi) + dv
c                                 is search in range?
      if (v(ivi).gt.vmax(ivi)) then
         v(ivi) = v(ivi) - dv
         dv = (vmax(ivi) - v(ivi))
         v(ivi) = vmax(ivi)
      else if (v(ivi).lt.vmin(ivi)) then
         v(ivi) = v(ivi) - dv
         dv = (vmin(ivi) - v(ivi))
         v(ivi) = vmin(ivi)
      end if

      call incdep (ivi,v(ivi)) 
c                                 solve for the equilibrium conditions:
      call univeq (ivd,ier)
c                                 on error return:
c                                 calling routine will switch variables.
      goto (999,999),ier

      if (jswtch.eq.0.and.
     *   (v(ivd).gt.vmax(ivd).or.v(ivd).lt.vmin(ivd))) then 
         call switch (dv,ivi,ivd,jer)
         jswtch = 1
         if (jer.eq.0) goto 10
      end if 
c                                 test if the equilibrium is stable:
      call gall 
      call nschk 
c                                 iflag= 0 stable.
c                                 iflag=1 metastable w/respect a phase.
c                                 iflag=2 metastable w/respect to more
c                                         than one phase.
      goto (20,70),iflag
c                                 iflag= 0:
c                                 dependent v in range? if
c                                 greater than the maximum value for v
c                                 or less than the minimum value for v
c                                 reset conditions, and
c                                 switch independent/dependent variables
      if (v(ivd).gt.vmax(ivd)) then
            v(ivd) = vmax(ivd)
            call incdep (ivd,v(ivd)) 
         else if (v(ivd).lt.vmin(ivd)) then
            v(ivd) = vmin(ivd)
            call incdep (ivd,v(ivd))  
         else
            call assptx
c                                 if independent variable has reached a
c                                 limit of the diagram output traverse
c                                 otherwise increment the variable and
c                                 continue the traverse
         if (v(ivi).eq.vmax(ivi).or.v(ivi).eq.vmin(ivi)) then
c                                 flag irchk indicates that the
c                                 traverse ended on an edge.
            irchk(ird) = 1

            call maxend
            goto 9000
         end if
c                                 try for next point
         goto 10
c                                 no route out of this construction
      end if
c                                 solve for the equilibrium with
c                                 the switched variables:
      call univeq (ivi,ier)
c                                 flag irchk indicates that the
c                                 traverse ended on an edge.
      irchk(ird) = 1
c                                 on error output traverse anyway
      goto (9000,9000),ier
c                                 assign final value
      call assptx
      goto 9000
c                                 iflag=2 reset to last stable condition
c                                 and redefine increment dv
70    call reptx

      dv = dv/2d0
      if (dabs(dv).gt.delt(ivi)) goto 10 
c                                 refined increment down to dtol, try
c                                 switching iv's, set iswitch flag
      if (iswtch.eq.0) then 
         call switch (dv,ivi,ivd,jer)
         iswtch = 1
         if (jer.eq.0) goto 10
      end if 

      call warn (73,delt(ivi),ivi,'SFOL2 ')
      iflag = 0 

      goto 9000
c                                 iflag=1 (an invariant equilibrium):
c                                 iterate independent variable to refine
c                                 the conditions of the invariant point:
20    call findjp (ivi,ivd,dv,ip)                              
c                                 output the traversed equilibrium:
9000  ier = 0

      call outrxn (ip,ier)    
c                                 save conditions of the endpoint
      call svrend (ird,irend,jer)

999   end                         

      subroutine sfol2 (iovi,iovd,ip,irend)
c----------------------------------------------------------------------
c sfol2 generates and computes the loci of stable equilibria
c emanating from an ip indexed by the array ipid and the
c arguement (i).  since one of these equilibria is calculated
c by sfol1 or by previous calls to sfol2 only c+1 (at most)
c curves are calibrated.
c
c the phases identified by idv define a plane in g-x space, idphi
c is the phase which is tangent to this plane, and lphi is the
c ,or one of the, phases which is present at the ip but absent
c along the univariant point.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names, text(400)*1, alpha(130)*1

      common/ cst8  /names(k1)/ cst6  /icomp,istct,iphct,icp
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1     
     *      / cst29 /vip(l2,k2),ipid(k2,k5+2),ipct
     *      / cst31 /vn(k2,k7),irct,ird/ cst5 /v(l2),tr,pr,r,ps
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst24 /ipot,jv(l2),iv(l2)/ cst25 /vnu(k7),idr(k7),ivct
     *      / cst103 /isec,icopt,ifull,imsg,io3p/ cst32 /ptx(l5),ipt2
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b/ cst105 /ibug(k2)
c-----------------------------------------------------------------------
      jchk = 0 
      icp1 = icp + 1
      icp2 = icp + 2
      icp3 = icp + 3
      ivi = iovi
      ivd = iovd
      ivarip(ip) = 1

      goto (150), io3
      goto (150), io3p

      call iptext (text,iend,ip,icp2)

      write (n3,1040)
      write (n3,1020) ip, (text(i),i = 1, iend)
      write (n3,1050)

150   do j = 2, icp2
         jtic = 0
         idno  = icp3 - j
         lphi  = ipid(ip,idno)
         idphi = ipid(ip,icp2)

         do i = 1, icp1
            if (i.ne.idno) then 
               jtic = jtic + 1
               idv(jtic) = ipid(ip,i)
            end if 
         end do 
c                            balance the equilibrium, ier = 1
c                            if the plane idv is degenerate
         itic= 0
c                            set invariant point conditions
         v(ivd) = vip(ivd,ip)
         v(ivi) = vip(ivi,ip)
         call incdep (ivi,v(ivi)) 
         call incdep (ivd,v(ivd))  
c                            check if assemblage sees bulk comp
         if (jbulk.ne.0) then
c                            put idphi into idv for bulkun
            call bulkun (idv,idphi,jbfg)
            if (jbfg.eq.0) cycle
         end if

110      call balanc (b,idv,idphi,ier)
         if (ier.eq.1) goto 80
         if (ivar.eq.1) goto 130
         ivarip(ip) = 2
         if (isudo.eq.1) cycle
c                            call pivots for lchk.
130      call pivots (ier)
         if (ier.eq.0) goto 90
c                            in some cases singularity of the
c                            transpose of the concentration matrix
c                            may not be detected by balanc, but,
c                            will be detected for the concentration
c                            matrix by pivots, in which case the
c                            columns are permuted and the stoichiometry
c                            of the reaction is redetermined.
80       if (itic.eq.icp-1) cycle
         icter =idphi
c                              if a degenerate facet has been
c                              generated by permutation permute
c                              idphi with the vertices idv until
c                              balanc succeeds.  for a c component
c                              system no more than c-1 permutations
c                              will be necessary.
         idphi = idv(icp-itic)
         idv(icp-itic) = icter
         itic = itic + 1
         goto 110

90       call assir 
c                                TESTING 
c                                test endpoint.
         call svrend (ird,irend,ier)
         if (ier.eq.1) cycle
c                                calculate the loci of the equilibia,
         ipt2 = 0
         call delrxn 
         call assptx
c                            determine direction of traverse and
c                            dependent and independent variables
         call wway (div,lphi,ivi,ivd,ikwk,ier)
c                            begin traverse
         if (ier.ne.0) then 
c                            call rxntext to save the equilibrium
c                            in the list in case it is not found 
c                            again:
            call rxntxt (iend,jend,text,alpha)
c                            set the bug flag:
            ibug(irct) = 1
            cycle 
         end if 

         icter = 0

60       call sfol1 (ivd,ivi,ier,div,ikwk,irend)
         jchk = jchk + 1

         if (ier.eq.0) then 
            ivd = iovd
            ivi = iovi
            cycle
         end if 

         call switch (div,ivi,ivd,jer)  
  
         if (jer.ne.1) then 
            icter = icter + 1
            if (icter.lt.4) goto 60
         end if 
      
         call warn (20,v(ivi),ivi,'SFOL2 ')

         call outrxn (ip,1)
         ivi =iovi
         ivd=iovd
      end do 

      if (jchk.eq.0) call warn (74,v(ivi),ivi,'SFOL2 ')
      if (io3p.eq.0) write (n3,1040)

1020  format ('equilibria about invariant point (',i6,'):',//,
     *        3x,200a1)
1040  format ('                       ------')
1050  format (/,'are listed below:',/)

      end

      subroutine sreset (jok,iste,vt,v,vti,vi)
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
c                                 if the assemblages conditions
c                                 of stability are already known (jok=5)
c                                 or are known to extend beyond the
c                                 current condition (jok>iste) quit:
      if (jok.gt.iste) goto 99
c                                 if the known conditions of stability
c                                 have been extened, i.e., either:
c                                 jok=iste  or v>vt
      if (jok.ne.iste) then
         jok=iste
      else if (iste.lt.3) then
         if (vt.gt.v) goto 99
      else
         if (vt.lt.v) goto 99
      end if

      vt = v
      vti = vi

99    end  

      subroutine maxend 
c----------------------------------------------------------------------
c save the most advanced condition for the stability of an 
c assemblage along the periphery of a diagram. 
c ismax - is the traverse
c value - is the dependent variable
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst49 /value,ismax,igot
     *      / cst5  /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),ddv(l2)
c-----------------------------------------------------------------------
      if (igot.eq.0) igot = 1

      if (v(iv2).eq.vmin(iv2).and.ismax.eq.1) then
c                                  traverse 1
c                                  now don't allow backing up:
         if (value.lt.v(iv1)) value = v(iv1)

      else if (v(iv1).eq.vmax(iv1).and.ismax.lt.3) then
c                                  traverse 2
         if (ismax.eq.1) then

            ismax = 2
            value = v(iv2)

         else if (value.lt.v(iv2)) then

            value = v(iv2)

         end if

      else if (v(iv2).eq.vmax(iv2).and.ismax.lt.4) then
c                                  traverse 3
         if (ismax.lt.3) then

            ismax = 3
            value = v(iv1)

         else if (value.gt.v(iv1)) then

            value = v(iv1)

         end if 

      else if (v(iv1).eq.vmin(iv1)) then
c                                  traverse 2
         if (ismax.lt.4) then
            ismax = 4
            value = v(iv2)
         else if (value.gt.v(iv1)) then
            ismax = 4
            value = v(iv2)
         end if 

      end if 

      end 

      subroutine svrend (ird,irend,ier)
c----------------------------------------------------------------------
c save end points of univariant equilibria and 
c check if the intersection has already been recorded (ier =1).

c two cases:

c 1) reaction occurs on an edge, if the intersection has already 
c    occurred then ignore future occurrences.
c 2) reaction occurs at an IP only ignore occurences being traced in 
c    the same direction if the reaction is degenerate.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      dimension rends(2,16,k2),irends(k2)

      common/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst35 /irv(k2)/ cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst5  /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),ddv(l2)

      save rends, irends
c-----------------------------------------------------------------------
      ier = 0 
c                                    if isec = 1, don't do any checks
      if (isec.eq.1) goto 99

      dx =  ddv(iv(1))
      dy =  ddv(iv(2))

      x = v(iv(1))
      y = v(iv(2))

      if (x.ne.vmin(iv(1)).and.x.ne.vmax(iv(1)) .and.
     *    y.ne.vmin(iv(2)).and.y.ne.vmax(iv(2))) then
c                                    the endpoint is not on an edge:
          if (isec.eq.2) then 
c                                    if isec = 2, don't check unless
c                                    curve intersects at an edge.
             goto 99
          else if (isec.eq.3) then 
c                                    if isec = 3, don't do a check for 
c                                    degenerate interior endpoints:
             if (irv(ird).lt.icp1) goto 99
          end if 
      else 
c                                    the endpoint is on an edge:
c                                    if isec = 4, don't do a check for
c                                    degenerate equilibria.
          if (isec.eq.4.and.irv(ird).lt.icp1) goto 99
      end if 
c                                    isec = 5, do check on all ends
      if (ird.le.irend) then 
c                                    reaction has been found already
c                                    look to see if its at the same 
c                                    endpoint (tolerance +/- dv).
         jct = irends(ird)
         do j = 1, jct 
            xx = rends(1,j,ird)
            yy = rends(2,j,ird)
            if (xx.gt.x-dx.and.xx.lt.x+dx.and.
     *          yy.gt.y-dy.and.yy.lt.y+dy) then 
c                                    condition match has occurred
               ier = 1
               goto 99
            end if 
         end do 
c                                    no condition match has occurred
         jct = jct + 1
         if (jct.gt.16) then 
            call warn (999,x,ier,'SVREND')
            jct = 16 
         end if  

      else 
c                                    reaction has not been recorded
         jct = 1
         irend = irend + 1

      end if 

      if (ird.gt.k2) call error (206,x,k2,'SVREND')
      irends(ird) = jct 
      rends(1,jct,ird) = x
      rends(2,jct,ird) = y

99    end 

      subroutine ufluid (fo2)
c----------------------------------------------------------------------
c subroutine ufluid computes the potential of the components
c of a saturated fluid phase. if the mole fraction of a component is les
c less than 1.d-38 the chemical potential is set to -9.9d09.
c ufluid may call one of three molecular fluid equations of state, or
c alternatively users may supply their own routines, however,
c the routines currently in use return the log of a components fugacity
c which is then added to the reference state potential computed by the
c function gphase.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      double precision xf(2)
 
      common/ cst1 /thermo(k4,k10),uf(2),us(h5)
     *      / cst5 /p,t,xco2,u1,u2,tr,pr,r,ps/ cst11 /f(2)
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst208 /ifct,idfl
c-----------------------------------------------------------------------
c                           compute the chemical potentials of
c                           fluid components in fluid saturated
c                           systems.
      call cfluid (fo2,fs2)

      if (idfl.ne.0) then
         call gphase (idfl,gph)
         uf(idfl) = gph + r * t * f(idfl)
      else
         xf(1) = 1d0 - xco2
         xf(2) = xco2
 
         do i = 1, 2
            if (iff(i).ne.0) then 
               if (xf(i).lt.1d-38) then 
                  uf(i) = -1d10
               else 
                  call gphase (i,gph)
                  uf(i) = gph + r * t * f(i)
               end if
            end if 
         end do

      end if 
 
      end

      subroutine uproj
c----------------------------------------------------------------------
c subroutine uproj computes the potential of mobile
c or saturated components.  the potentials of
c saturated non-volatile components are projected through
c saturated volatile components.
c
c uproj is always called before the routines gphase or gproj, in
c therefore in order to increase program efficiency the values
c log(t),dsqrt(t),(p-pr)*dexp(-t/300d0), and dexp(-p/35000d0)
c are maintained in common block 'cst44'.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      double precision uss(h6)
 
      common/ cst1  /thermo(k4,k10),uf(2),us(h5)
     *      / cst5  /p,t,xco2,u1,u2,tr,pr,r,ps/ cst12 /cp(k5,k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst208 /ifct,idfl
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst44 /dlogt,dsqrtt/ cst2 /g(k1)
c-----------------------------------------------------------------------
c                           compute isobaric isothermal constants.
      if (t.lt.0d0.or.tr.eq.0d0) then
        dlogt = 0d0
        dsqrtt = 0d0
      else
        dlogt = dlog(t)
        dsqrtt = dsqrt(t)
      end if

      fo2 = 0d0
c                           compute the chemical potentials of
c                           fluid components in fluid saturated
c                           systems.
      if (ifyn.ne.1) call ufluid (fo2)
c                           determine stable saturated composants
c                           and the corresponding chemical potentials
c                           (these are projected through the fluid
c                           components).
      do i = 1, isat

         ict = isct(i)

         ll = icp+i

         do j = 1, ict

            k = ids(i,j)
            call gphase (k,gph)
            
            if (ifct.gt.0) then 
               do l = 1, 2
                  if (iff(l).ne.0) gph = gph - cp(iff(l),k)*uf(l)
               end do 
            end if 

            uss(j) = gph 
c                           if multiple component saturation constraints
c                           project by saturation hierarchy:
            if (i.ne.1) then 
               i1 = i-1
               do l = 1, i1
                  uss(j) = uss(j)-cp(icp+l,k)*us(l)
               end do
            end if 

            g(k) = uss(j)
            uss(j) = uss(j)/cp(ll,k)
         end do 
c                           if O2, check if fo2 has been 
c                           determined by a fluid phase routine,
c                           if so do the projection:
         if (io2.eq.i) then 
            do j = 1, ict 
               uss(j) = uss(j) + r * t * fo2
            end do 
         end if 
c                           now find stable "composant":

         u = uss(1)

         id = 1

         if (ict.ne.1) then 
            do j = 2, ict
               if (uss(j).le.u) then 
                 id = j
                  u = uss(j)
               end if 
            end do 
         end if 
c                               save the id of the stable composant.
         idss(i) = ids(i,id)
c                               and its chemical potential.
         us(i) = u
c                               in case a phase in the component
c                               saturation space is an endmember of
c                               a solution correct the endmember
c                               properties:
c        if (i.ne.1) then 
c                               why wasn't this originally done for
c                               i = 1? commented if, nov 27, 2003
            do j = 1, ict
               k = ids(i,j)
               g(k) = g(k) - cp(icp+i,k)*u
            end do 
c         end if 
      end do 

      end

      subroutine wway (div,lphi,ivi,ivd,ikwk,ier)
c-----------------------------------------------------------------------
c wway determines from which side of an invariant point a specific
c univariant equilibrium emanates. the appropriate sign for the
c increment of the independent variable (dv) is also determined.
c where necessary wway also refines dv and/or may switch the
c independent and independent intensive variables (ivi,ivd). vo1
c and vo2 are the values of the intensive variables iv1 and iv2 at
c the invariant point. lphi is the id of the absent phase.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names

      dimension vo(l2),vvi(2),vvd(2),ddv(2)

      common/ cst25 /vnu(k7),idr(k7),ivct/ cst8 /names(k1)
     *      / cst31 /vn(k2,k7),irct,ird/ cst6 /icomp,istct,iphct,icp
     *      / cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst5  /v(l2),tr,pr,r,ps/ cst9 /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ cst7 /iflag
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
c-----------------------------------------------------------------------
      iophi =idphi
      ikwk = 0
      icter = 0
      ier = 0
      vo(iv1) = v(iv1)
      vo(iv2) = v(iv2)
c                                 first try iv1 as the iv
      ivi = iv2
      ivd = iv1

      ifail= 0

5     call univeq (ivd,ier)

      if (ier.eq.0) then
c                                 is the equilibrium condition
c                                 for this reaction consistent
c                                 with the invariant point?
         if ( dabs( (v(ivd)-vo(ivd))/dv(ivd) ).gt.0.1d0) then 
            vo(iv1) = v(iv1)
            vo(iv2) = v(iv2)
            call warn (18,v(ivd),ird,'WWAY')
         end if 

      else
         goto (9800),ifail
         ivi = iv1
         ivd = iv2
c                                 added april 2001 without check!!
         div = dv(ivi)

         v(iv1) = vo(iv1)
         call incdep (iv1,v(iv1))
         v(iv2) = vo(iv2)
         call incdep (iv2,v(iv2))

         ifail = ifail + 1
         goto 5
      end if
c                                 test that the ip isn't on an edge
      if (v(iv1).eq.vmin(iv1).or.v(iv1).eq.vmax(iv1).or.
     *    v(iv2).eq.vmin(iv2).or.v(iv2).eq.vmax(iv2)) goto 9600

      jswit = 0

190   order = 1d0
c                                 set iv increment
60    div = dv(ivi) / order

      ifail = 0

      do i = 1, 2
c                                 because of numerical slop 
c                                 an equilibrium may appear stable
c                                 on its metastable extension close to
c                                 an ip, because of this wway always
c                                 tests for stability on both extensions
c                                 if wway cannot solve for equilibrium
c                                 conditions on both sides of an ip
c                                 the increment is decreased and if this
c                                 fails the independent and dependent
c                                 variables are switched.
         jfail = 0

15       v(ivi) = vo(ivi) + div
c                                 check if the increment is too large
         if (v(ivi).gt.vmax(ivi)) then
            v(ivi) = vmax(ivi)
         else if (v(ivi).lt.vmin(ivi)) then 
            v(ivi) = vmin(ivi)
         end if 
c                                 set the dependent variable to ip:
         v(ivd) = vo(ivd)

         call incdep (ivi,v(ivi)) 
         call incdep (ivd,v(ivd))  
c                                 solve for the equilibrium
         call univeq (ivd,ier)

         if (jfail.le.5.and.ier.eq.0.and.(v(ivd).gt.vmax(ivd)
     *       .or.v(ivd).lt.vmin(ivd))) then
c                                 increment to big:
            div = div / 10.
            jfail = jfail + 1
            goto 15
  
         else if (jswit.eq.0.and.ier.gt.0) then 
c                                 on error switch dependent and
c                                 independent variables
            iv  = ivi
            ivi = ivd
            ivd = iv
            jswit = 1
            jfail = 0

            goto 190

         else if (jswit.gt.0.and.ier.gt.0) then

            call warn (999,dv(1),ivi,'WWAY')
            ier = 1
            return

         end if 

         vvi(i) = v(ivi)
         vvd(i) = v(ivd)
         ddv(i) = div
c                                 determine if the equilibrium is
c                                 metastable with respect to lphi
c                                 (lchkl=1):
         call lchk (lphi,lchkl)
         if (lchkl.eq.0) then
            ifail = ifail + 1
            isign = i
         end if
c                                 end of looop
         div = -div
      end do 
c                                 at this stage there are three
c                                 possibilities indicated by ifail
      if (ifail.eq.1) goto 50
c                                 ifail= 0 or 2 the equilibrium was stable
c                                 or metastable on both sides, the most 
c                                 plausible scenario is the increment is
c                                 too big and overshoots the stability,
c                                 in this scenario ifail = 2; however the
c                                 ip could be located inside of, rather than
c                                 on the edge of the stability field, in 
c                                 which case there is a small chance the
c                                 increment is too small to cross the border.
c                                 
c                                 so first decrease the increment:
         if (order.ge.1d0) then 
            order = order * 10d0 
c                                 stop decreasing after four orders 
c                                 of magnitude, and try increasing
c                                 increment:
            if (order.gt.1d4) order = 0.1
         else 
            order = order / 10d0
c                                 stop increasing after two orders 
c                                 of magnitude, cause there should 
c                                 never be a default increment much less 
c                                 than 1 % of the variable range.
            if (order.lt.1d-2) goto 9500
         end if 
c                                  why the fuck was this here?
c         ivi = iv1
c         ivd = iv2

      goto 60
c                                 ifail=1, stable extension identified.
c                                 determine the sign of the increment
c                                 and its magnitude:
50    if (isign.eq.1) then
         div = dv(ivi)
         del= vmax(ivi)-vo(ivi)
         if (del.lt.div) div = del/3.d0
      else
         div = -dv(ivi)
         del= vo(ivi)-vmin(ivi)
         if (del.lt.-div) div = -del/3.d0
      end if
c                                 reset variable values and save
      v(ivi) = vvi(isign)
      v(ivd) = vvd(isign)
      call incdep (ivi,v(ivi)) 
      call incdep (ivd,v(ivd))  
      odiv = ddv(isign)
      icter = 0
c                                 now check the stability with respect
c                                 to all phases
      order = 1d0
70    call gall 
      call schk (lphi)
c                                 stable if iflag= 0 from schk
c                                 now check if dependent variable is in
c                                 range.
      if (iflag.eq.0) goto 9999
c                                 metastable try refining the increment
      order = order * 5.d0
      div = div / order

      v(ivi) = vo(ivi) + div
      call incdep (ivi,v(ivi)) 
      v(ivd) = vo(ivd)
      call incdep (ivd,v(ivd))  
 
100   call univeq (ivd,ier)

      if (ier.eq.0) goto 90
c                                 error from univeq, more than once quit
      goto (9400), icter
      div =odiv
      icter = icter + 1
      goto 100
c                                 no error from univeq
90    if (order.gt.10.d6) goto 9500
      goto 70
c                                 all systems go
9999  call assptx
      if (dabs(div).lt.dv(ivi)) then
         div = dv(ivi)*div/dabs(div)
      end if 

      if (v(ivi).eq.vmax(ivi).or.v(ivi).eq.vmin(ivi)) then
         ikwk= 1
         call maxend
      end if 

      return

9400  call warn (87,dv(1),ivi,'WWAY')
      ier = 1
      return  
       
9500  call warn (24,dv(ivi)/order,ivi,'WWAY')
      ier = 1
      return

9600  call warn  (63,dv(1),ivi,'WWAY')     
      ier = 1
      return
  
9800  call warn (58,dv(1),ier,'WWAY')
      if (ivct.gt.4) goto 9820
      write (n3,1050) ird,(vnu(l),names(idr(l)), l= 1, ivct)
      goto 9830
9820  write (n3,1050) ird,(vnu(l),names(idr(l)), l= 1, 4)
      write (n3,1060) (vnu(l),names(idr(l)), l= 5, ivct)
9830  write (n3,*) 
      ier = 1

1050  format (1x,'(',i6,')',4(1x,g9.3,1x,a))
1060  format (6x,4(1x,g9.3,1x,a),/,6x,4(1x,g9.3,1x,a))
      end

      subroutine fultxt (nchar,rtxt)
c-----------------------------------------------------------------------
c subprogram to write text for complete reaction reactions
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names, text(kd2,2)*1, rtxt(kd2)*1,
     *            ppart(kd1)*31, rxnstr*(kd2), char8, 
     *            mpart(kd1)*31,cname*5, exten(2)*8, 
     *            ptext*(kd2), mtext*(kd2)

      integer ichar(2)

      common/ csta4 /cname(k5)/ cst21  /du(2),dv(2),jds(h5),ifr 
     *      / cst201 /vuf(2),vus(h5),iffr,isr/ cst104 /rxnstr(k2)
     *      / cst25  /vnu(k7),idr(k7),ivct/ cst8 /names(k1)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst31 /vn(k2,k7),irct,ird/ cst80 /irchk(k2)
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst6 /icomp,istct,iphct,icp
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct

      save exten 
c                                     this is a bullshit trick and
c                                     will cause errors if someone
c                                     uses a function other than G.
      data exten/'-V(j/b) ','S(j/k) '/

      ip = 0 
      im = 0

      do i = 1, 8
         mpart(i) = '                              '
         ppart(i) = '                              '
      end do
c                             load id's of phases with + or - coeffs 
      do i = 1, ivct 
c           
         id = idr(i)   
         if (vnu(i).lt.0d0) then
            im = im + 1
            call wrpart (-vnu(i),ikp(id),names(id),mpart(im))
         else 
            ip = ip + 1
            call wrpart (vnu(i),ikp(id),names(id),ppart(ip))
         end if
      end do
c                             composant stoichiometry: 
      do i = 1, isat 
         if (vus(i).gt.0d0) then
            im = im + 1
            call wrpart (vus(i),0,names(jds(i)),mpart(im))
         else if (vus(i).lt.0) then
            ip = ip + 1
            call wrpart (-vus(i),0,names(jds(i)),ppart(ip))
         end if 
      end do 
c                                     fluid stoichiometry:
      if (ifyn.eq.0) then 
         do i = 1, 2
            if (iff(i).ne.0) then
               if (vuf(i).gt.0d0) then
                  im = im + 1
                  call wrpart (vuf(i),0,names(i),mpart(im))
               else if (vuf(i).lt.0d0) then
                  ip = ip + 1
                  call wrpart (-vuf(i),0,names(i),ppart(ip))
               end if 
            end if 
         end do
      end if 
c                                     mobile components: 
      do i = 1, jmct
         char8 = cname(jprct+i)
         if (du(i).gt.0d0) then
            im = im + 1
            call wrpart (du(i),0,char8,mpart(im))
         else if (du(i).lt.0d0) then
            ip = ip + 1
            call wrpart (-du(i),0,char8,ppart(ip))
         end if
      end do

      if (ifull.eq.1.or.ifull.eq.3) goto 55

      do i = 1, 2
         if (dv(i).gt.0d0) then
            im = im + 1
            call wrpart (dv(i),0,exten(i),mpart(im))
         else if (dv(i).lt.0d0) then
            ip = ip + 1
            call wrpart (-dv(i),0,exten(i),ppart(ip))
         end if
      end do 

55    if (ip.gt.k5+2.or.im.gt.k5+2) call error (997,du(1),ip,'FULTXT')

      write (ptext,1000) (ppart(i), i = 1, ip)
      write (mtext,1000) (mpart(i), i = 1, im)

      ichar(1) = im * 31
      read (mtext,1010) (text(i,1), i = 1, ichar(1))
      ichar(2) = ip * 31
      read (ptext,1010) (text(i,2), i = 1, ichar(2))
c                             filter out blanks:
      do i = 1, 2
         nchar = 1
         do j = 2, ichar(i)
            if (text(j-1,i).eq.' '.and.text(j,i).eq.' ') cycle
            nchar = nchar + 1
            text(nchar,i) = text(j,i)
         end do 
c                             filter out blanks before parentheses:
         ichar(i) = nchar
         nchar = 0
         do j = 1, ichar(i)-1
            if ((text(j,i).eq.' '.and.text(j+1,i).eq.')').or.
     *          (text(j,i).eq.' '.and.text(j+1,i).eq.'(')) cycle
            nchar = nchar + 1
            text(nchar,i) = text(j,i)
         end do  
         ichar(i) = nchar
      end do
c                             concatenate parts into rxnstr:
      if (ichar(1)+ichar(2).gt.(k5+2)*31-3) 
     *                    call error (208,du(1),ip,'FULTXT')
 
      nchar = 0 

      do i = 1, 2
         do j = 1, ichar(i)
            nchar = nchar + 1
            rtxt(nchar) = text(j,i)
         end do
         if (i.eq.2) cycle
         nchar = nchar + 3
         rtxt(nchar-2) = ' '
         rtxt(nchar-1) = '='
         rtxt(nchar)   = ' '
      end do 
c                             this is redundant if the reaction 
c                             was found before
      write (rxnstr(ird),1010) (rtxt(i),i=1,nchar)

1000  format (14(a,1x))
1010  format (434a1)

      end 


      subroutine stetxt 
c-----------------------------------------------------------------------
c subprogram to write complete reactions to a file called 
c "reaction_list.dat" for import into excel, etc. 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      double precision rcoef(kd1)

      character*8 rname(kd1), exten(2), names, cname*5

      common/ csta4 /cname(k5)/ cst21  /du(2),dv(2),jds(h5),ifr 
     *      / cst201 /vuf(2),vus(h5),iffr,isr
     *      / cst25  /vnu(k7),idr(k7),ivct/ cst8 /names(k1)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst31 /vn(k2,k7),irct,ird
     *      / cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst6 /icomp,istct,iphct,icp
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct

      save exten 
c                                     this is a bullshit trick and
c                                     will cause errors if someone
c                                     uses a function other than G.
      data exten/'-V(j/b) ','S(j/k) '/
c                                     ouput data as follows
c                                     V, S, mobile components, fluid components, 
c                                     saturated component, thermodynamic components
c                                     extensities:
      do i = 1, 2
         rcoef(i) = -dv(i)
         rname(i) = exten(i)
      end do 

      ict = 2
c                                     mobile components:
      do i = 3, 2 + jmct
         rcoef(i) = -du(i-2)
         rname(i) = cname(jprct+i-2)
      end do

      ict = ict + jmct
c                                     saturated phase components:
      if (ifyn.eq.0) then 
         do i = 1, 2
            if (iff(i).ne.0) then
               ict = ict + 1
               rcoef(ict) = -vuf(i)
               rname(ict) = names(i) 
            end if 
         end do
      end if 
c                                     saturated components: 
      do i = 1, isat 
         if (vus(i).ne.0d0) then 
            ict = ict + 1
            rcoef(ict) = -vus(i)
            rname(ict) = names(jds(i))
         end if 
      end do
c                                     thermodynamic components
      do i = 1, ivct         
            
         ict = ict + 1
         rcoef(ict) = vnu(i)
         rname(ict) = names(idr(i))
      end do
 
      write (69,1000) irct,ivar,(rname(i), rcoef(i), i = 1, ict)

1000  format (i5,1x,i1,1x,20(a,1x,g14.8,1x))

      end 

      subroutine wrpart (vnu,ikp,name,part)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 name, fname*10, part*30

      common/ csta7 /fname(h9)/ cst103 /isec,icopt,ifull,imsg,io3p

      if (ikp.eq.0) then
         if (ifull.gt.2) then  
            write (part,1000) vnu,name
         else 
            write (part,2000) name
         end if 
      else 
         if (ifull.gt.2) then 
            write (part,1010) vnu,fname(ikp),name
         else 
            write (part,2010) fname(ikp),name
         end if 
      end if 

1000  format (g9.3,1x,a)
1010  format (g9.3,1x,a,'(',a,')')
2000  format (a)
2010  format (a,'(',a,')')

      end

      subroutine bulkun (kdv,idum,jbfg)
c----------------------------------------------------------------------
c check if any permutation of a c+1 phase assemblage sees a 
c specified bulk composition, if not jbfg = 0.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      integer kdv(k5),idv(k5+1),iaid

      common/ cst6   /icomp,istct,iphct,icp
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2)
c-----------------------------------------------------------------------
      icp1 = icp + 1
      iact = 0
      iaid(1) = 0
      iaid(2) = 0

      do j = 1, icp1
c                                      do c permutations of the
c                                      phases to assure that 
c                                      degeneracy is not a problem
         k = 1

         do i = 1, icp1
            if (i.eq.j) then
               cycle
            else if (i.lt.icp1) then
               idv(k) = kdv(i)
            else 
               idv(k) = idum
            end if 
            k = k + 1
         end do 
c                                      check the assemblage
         call bulkck (idv,jbfg,kdbulk,1,1)
c                                      jbfg ne 0, bounds a comp.
         if (jbfg.ne.0) then
            if (jbulk.lt.icp) then
               goto 99
            else 
c                                      save the assemblage indices:     
               iact = iact + 1
               iaid(iact) = kdbulk
               if (iact.eq.2) goto 99
            end if 
         end if 
      end do 

      if (iact.eq.1) write (*,*) 'huh?'

99    end 

      subroutine blkmat (kdv,idsol,kdsol,ncpd,np,ntot,kdbulk,igo)
c----------------------------------------------------------------------
c blkmat compares divariant (ntot) assemblages to those already defined
c----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer idsol(k5),kdv(k5),idbulk,i,j,ntot,ncpd,np,igo,ibulk,
     *        jtest,jpot,ikp,ivarip,isudo,ivar,ivarrx,iap,iaid,iasct,
     *        idasls,iavar,isec,icopt,imsg,io3p,kdsol(k5),k,kdbulk,
     *        nst,ifull
 
      common/ debug  /jtest,jpot
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2)
     *      / cst75  /idasls(k5,k3),iavar(2,k3),iasct
     *      / cst103 /isec,icopt,ifull,imsg,io3p
c                                do a check to prevent the data
c                                for the same assemblage being
c                                output more than once:
      igo = 0
c                                if jtest = 3, skip check, output
c                                all data (before 10/28/04)
      if (jtest.eq.3.and.icopt.ne.1) goto 100 

      do 40 i = 1, ibulk
         do 50 j = 1, ntot
            do k = 1, ntot
               if (kdv(j).eq.idbulk(k,i)) goto 50
            end do
c                                no match on the i'th assemblage
            goto 40 
50       continue
c                                if here a match on the i'th ass.
         kdbulk = i
         igo = 1

         goto 99
40    continue
c                                the assemblage is new
100   ibulk = ibulk + 1
      kdbulk = ibulk
c                                sort pseudocompounds into phases
      call srtblk (kdv,idsol,kdsol,ntot,ncpd,np)

      if (ibulk.gt.k2) call error (183,0d0,k2,'BLKMAT')

      iap(ibulk) = 0

      do i = 1, ntot
         idbulk(i,ibulk) = kdv(i)
      end do

      if (np.eq.0) then 
c                                assemblage consists entirely
c                                of compounds, figure out if it
c                                is a new assemblage:
         do 110 i = 1, iasct

            do 120 j = 1, ncpd
               do k = 1, ncpd
                  if (idasls(k,i).eq.kdv(j)) goto 120
               end do
c                                no match with compound j:
c                                do next assemblage
               goto 110 

120         continue  
c                                everything must have matched
c                                exit 
            iap(ibulk) = i
            goto 99 

110      continue 

      else 
c                                now compare to known assemblages:
c                                counter for compounds:
         nst = ntot - ncpd + 1

         do 20 i = 1, iasct
            if (np.eq.iavar(1,i).and.ncpd.eq.iavar(2,i)) then
c                                same number of compounds and phases:
               do 60 j = 1, np
                  do k = 1, np
                     if (idasls(j,i).eq.kdsol(k)) goto 60
                  end do
c                                no match with solution k:
                  goto 20
60             continue
c                                the foregoing loop assures that
c                                every phase in the old list is 
c                                present in the new list, but not 
c                                that every phase in new list is 
c                                present in the old list, so switch
c                                loop orders for a second check:
               do 61 j = 1, np
                  do k = 1, np
                     if (idasls(k,i).eq.kdsol(j)) goto 61
                  end do
c                                no match with solution k:
                  goto 20
61             continue

               do 80 j = 1, ncpd
                  do k = nst, ntot
                     if (idasls(np+j,i).eq.kdv(k)) goto 80
                  end do
c                                no match with compound k:
                  goto 20
80             continue
c                                everything must have matched
               iap(ibulk) = i
               goto 99 
            end if
20       continue 

      end if 
c                                the assemblage must be new:
      iasct = iasct + 1
      if (iasct.gt.k3) call error (184,0d0,k3,'BLKMAT')

      if (np.gt.0) then 
         do i = 1, np
            idasls(i,iasct) = kdsol(i)
         end do
         j = np
         do i = nst, ntot
            j = j + 1
            idasls(j,iasct) = kdv(i)
         end do
      else 
         do i = 1, ncpd
            idasls(i,iasct) = kdv(i)
         end do 
      end if

      iavar(1,iasct) = np
      iavar(2,iasct) = ncpd
      iap(ibulk) = iasct 

99    end 

      subroutine detest (ier)
c----------------------------------------------------------------------
c subroutine used by lpopt to test if an assemblage is non-degenerate
c ier = 0 if non-degenerate, otherwise 1.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst6   /icomp,istct,iphct,icp
     *      / cst301 /a(k5,k5),b(k5),ipvt(k5)/ cst12 /cp(k5,k1) 
     *      / cst52  /hcp,idv(k5+1),ic(k5)  
c----------------------------------------------------------------------
      ier = 0
c                                load the transpose of the
c                                concentration matrix of the pseudo-
c                                invariant assemblage.
      do i = 1, icp
         do j = 1, icp
            a(j,i) = cp(j,idv(i))
         end do
      end do  
c                                factor the matrix
      call factr1 (icp,ier)

      end 

      subroutine bulkck (jdv,jbfg,kdbulk,ico,jco)
c----------------------------------------------------------------------
c bulkck determines if the phases identified by jdv bound
c any of the specified bulk compositions.

c jbfg - 0 if the phases do not span the composition, else 1
c kdblk - pointer to the assemblage in the array idbulk(k5,kdblk)

c the test for bounded compositions is redundant when 
c this routine is called by smplex or grdmin.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'

      integer jdv(k5+1),kdv(k5),iflug(k5),idsol(k5),kdsol(k5),
     *        ncpd,np

      double precision props(i8,k5)
 
      common/ cst6   /icomp,istct,iphct,icp/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst10  /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst5   /v(l2),tr,pr,r,ps/ cst301 /a(k5,k5),b(k5),ipvt(k5)
     *      / cst43  /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst18  /jvol(2),ivol/ cst12 /cp(k5,k1)   
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1) 
     *      / cst19  /jpoly/ cst103 /isec,icopt,ifull,imsg,io3p
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2) 
c----------------------------------------------------------------------
      jbfg = 0
c                                load the transpose of the
c                                concentration matrix of the pseudo-
c                                invariant assemblage.
      do i = 1, jbulk
         if (i.le.icp) then
            k = jdv(i)
            kdv(i) = k
         else
            k = idss(i-icp)
            kdv(i) = k
         end if
         do j = 1, jbulk
            a(j,i) = cp(j,k)
         end do
      end do  
c                                factor the matrix
      call factr1 (jbulk,ier)
      if (ier.eq.1) goto 99 
c                                test for bounded compositions
      do i = 1, jbulk
c                                load composition vector, the alpha
c                                vector is returned in the same array
         b(i) = cblk(i)
      end do
c                                solve for the alpha vector
      call subst1 (jbulk)
c                                a negative coefficient indicates
c                                the composition is not bounded
      do i = 1, icp
         if (b(i).lt.-1d-5) goto 99
      end do 
c                                the composition is bounded 
      jbfg = 1
c                                now check if it's a new or old assemblage:
      if (jbulk.ge.icp) then
c                                do a check to prevent the data
c                                for the same assemblage being
c                                output more than once, blkmat
c                                also sorts pseudcompounds.
         call blkmat (kdv,idsol,kdsol,ncpd,np,jbulk,kdbulk,igo)
         if (igo.eq.1) goto 99
         jd = ibulk
      else 
         jd = jpoly
      end if     
c                                the remainder of this routine is
c                                for output purposes:
      iflu = 0
      jflu = 0

      do i = 1, jbulk
         iflug(i) = 0
      end do 
c                              now do totals:
20    do i = 1, jbulk
         k = kdv(i)
         if (b(i).ge.0d0) then 
            props(1,i) = b(i)
         else 
            props(1,i) = 0d0
         end if
         props(2,i) = atw(k) 

         call dphase (k,props(3,i),props(4,i),props(5,i),props(6,i),
     *                  props(7,i),props(8,i),props(9,i),props(10,i),
     *                  props(11,i))
c                              identify phases that are considered to 
c                              be fluids; what happens for pure h2o
c                              and co2???       
         if (ifp(k).eq.1.or.ifp(k).eq.26) then
            iflu = 1
            jflu = jflu + 1 
            iflug(i) = 1
         end if 
      end do 
c                              check if all fluid:
      if (jflu.eq.jbulk) iflu = 0
c                              get true variance
      if (icopt.eq.0) then 
         jvar = jbulk-np-ncpd
      else if (icopt.eq.3) then 
         jvar = jbulk-np-ncpd + 1
      else if (icopt.eq.1.or.icopt.eq.5) then 
         jvar = jbulk-np-ncpd + 2
      end if        

      if (jbulk.ge.icp) 
     *    call outblk (jd,jvar,kdv,props,iflu,iflug,iap(jd),ico,jco,
     *                 idsol,kdsol,ncpd,np)               

99    end


      subroutine outblk (jd,jvar,kdv,props,iflu,iflug,iap,ico,jco,
     *                   idsol,kdsol,ncpd,np) 

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
 
      character cname*5, pname(k5)*20, vname*8, xname*8, names*8, 
     *          fname*10

      integer kdv(k5), iflug(k5), jfract, idsol(k5), kdsol(k5)

      double precision fbulk(k5), fbulk1(k5), props(i8,k5), ptot(7),
     *                 ptot1(7), emods(k15,k5), dcomp, dmole
 
      common/ cst6   /icomp,istct,iphct,icp/ cst24 /ipot,jv(l2),iv(l2)
     *      / csta4  /cname(k5)/ cst5   /v(l2),tr,pr,r,ps
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b/ cst12 /cp(k5,k1)
     *      / csta2  /xname(k5),vname(l2)/ cst18 /jvol(2),ivol  
     *      / cst103 /isec,icopt,ifull,imsg,io3p/ cst8  /names(k1) 
     *      / cst43  /therm(k4),comp(k0),atwt(k0),idh2o,idco2,ikind
     *      / cst319 /emod(k15,k10),iemod(k1),kmod/ frct2 /dcomp(k5)
     *      / csta7 /fname(h9)/ frct1 /ifrct,ifr(5),jfr(5)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
c                                      initialize
      ptot(1) = 0d0
      ptot1(1) = 0d0
      do i = 1, icomp
         fbulk(i) = 0d0
         fbulk1(i) = 0d0
      end do
      btot = 0d0
      gtot = 0d0
      btot1 = 0d0
      gtot1 = 0d0
c                                      compute molar weighted totals
      do j = 1, jbulk
         ptot(1) = ptot(1) + cblk(j)
         if (iflu.eq.1.and.iflug(j).eq.0) 
     *   ptot1(1) = ptot1(1) + cblk(j) 
      end do 

      do i = 2, 7  
         ptot(i) = 0d0
         ptot1(i) = 0d0
         do j = 1, jbulk
            if (i.lt.6) then 
c                                      sys prop = sum phase prps
               ptot(i) = ptot(i) + props(1,j)*props(i,j)
               if (iflu.eq.1.and.iflug(j).eq.0) 
     *            ptot1(i) = ptot1(i) + props(1,j)*props(i,j) 
            else 
c                                      expansivity and compressibility
c                                      weighted by molar volume

c                                      3/27/04 removed weighting by
c                                      molar volume. JADC.

               ptot(i) = ptot(i) + props(1,j)*props(i,j)
               if (iflu.eq.1.and.iflug(j).eq.0) 
     *            ptot1(i) = ptot1(i) + props(1,j)*props(i,j)
            end if 
         end do 
      end do 

      do i = 6, 7
         ptot(i) = ptot(i)/ptot(4)
         if (iflu.eq.1) ptot1(i) = ptot1(i)/ptot1(4)
      end do 

      if (io3b.eq.0) then 
         write (n3,1000) jd 
         write (n3,1120) (vname(jv(i)), v(jv(i)), i = 1, ipot)
         write (n3,1020) (cname(i), i = 1, icomp)

         do i = 1, jbulk
            k = kdv(i)
            call blktxt (pname(i),k)
            write (n3,1030) pname(i), props(1,i)/ptot(1)*1d2, 
     *                      props(1,i)*props(2,i)/ptot(2)*1d2,
     *                      props(1,i)*props(4,i)/ptot(4)*1d2,
     *                      (cp(l,k), l = 1, icomp)
         end do 

         write (n3,1160)
c                                     phase/system summary:
         do i = 1, jbulk
            write (n3,1170) pname(i), (props(j,i), j = 2, 7)
         end do
         write (n3,1170) 'System              ',ptot(2),ptot(3),ptot(4),
     *                                       ptot(5),ptot(6),ptot(7)
         if (iflu.eq.1) 
     *   write (n3,1170) 'System - Fluid      ',
     *                                       ptot1(2),ptot1(3),ptot1(4),
     *                                       ptot1(5),ptot1(6),ptot1(7) 
      end if 
c                                     compute globally weighted totals
      do i = 1, jbulk
         k = kdv(i)
         do l = 1, icomp
            fbulk(l) = fbulk(l) + props(1,i) * cp(l,k)
            if (iflu.eq.1.and.iflug(i).eq.0) 
     *         fbulk1(l) = fbulk1(l) + props(1,i) * cp(l,k) 
         end do 
      end do

      do i = 1, icomp
         btot = btot + fbulk(i)*atwt(i)
         gtot = gtot + fbulk(i)
         btot1 = btot1 + fbulk1(i)*atwt(i)
         gtot1 = gtot1 + fbulk1(i)
      end do 

      if (io4.eq.0.and.icopt.ne.0) then
c                                graphics output  
         write (n5,1130) jd,jvar,iap,iflu,(kdv(i), i = 1, jbulk)
         write (n5,1130) ico,jco,np,ncpd,(kdsol(i), i = 1, np),
     *                   (idsol(i),i = 1, np)
         write (n5,1150) (v(jv(i)), i = 1, ipot)
         write (n5,1150) ((props(j,i), j = 1, 11), i = 1, jbulk)
         write (n5,1150) ptot(1),ptot(2),btot,gtot,
     *                      (fbulk(j), j = 1, icomp)
         if (iflu.eq.1) then 
            write (n5,1130) (iflug(j), j = 1, jbulk)
            write (n5,1150) ptot1(1),ptot1(2),btot1,gtot1,
     *                        (fbulk1(j), j = 1, icomp)
         end if 
c                                shear moduli if present:
         if (kmod.ne.0) then 
            call moduli (kdv,jmod,emods) 
            write (n5,1130) jmod
            if (jmod.gt.0) write (n5,1150) 
     *                           ((emods(j,i), j= 1, 3), i = 1, jbulk)
         end if 
      end if 
c                                fractionation effects:
      do i = 1, ifrct
         if (ifr(i).ne.0) then 
            do j = 1, jbulk
               dcomp(j) = 0d0
            end do 
            dmole = 0d0
            jfract = 0
c                                fractionating a solution
            do j = 1, jbulk
               if (ikp(kdv(j)).eq.ifr(i)) then 
                  dmole = dmole + props(1,j)
                  jfract = 1
                  do k = 1, jbulk 
                     dcomp(k) = dcomp(k) + props(1,j)*cp(k,kdv(j))
                  end do 
               end if
            end do 
c                                do output even if 0 mode:            
            if (jfract.eq.1) then 
c                                change bulk
               do k = 1, jbulk 
                  cblk(k) = cblk(k) - dcomp(k) 
               end do 
c                                write to console
               write (*,1185) vname(iv(2)),v(iv(2)),
     *                        vname(iv(1)),v(iv(1))
               write (*,1190) dmole,fname(ifr(i)),
     *                        (dcomp(k),k=1,jbulk)
c                                write to file
               write (100+i,1200) v(iv(2)),dmole,fname(ifr(i)),
     *                           (dcomp(k),k=1,jbulk)
            else 
               write (*,1185) vname(iv(2)),v(iv(2)),
     *                        vname(iv(1)),v(iv(1))
               write (*,1210) fname(ifr(i))
            end if 
         else
c                                fractionating a compound
            jfract = 0 
            do j = 1, jbulk
               if (kdv(j).eq.jfr(i)) then 
                  jfract = 1
                  do k = 1, jbulk 
                     cblk(k) = cblk(k) - props(1,j)*cp(k,kdv(j))
                  end do 
c                                write to console
                  write (*,1185) vname(iv(2)),v(iv(2)),
     *                           vname(iv(1)),v(iv(1))
                  write (*,1190) props(1,j),names(jfr(i)),
     *                          (props(1,j)*cp(k,kdv(j)),k=1,jbulk)
c                                write to file
                  write (100+i,1200) v(iv(2)),
     *                               props(1,j),names(jfr(i)),
     *                              (props(1,j)*cp(k,kdv(j)),k=1,jbulk)
               end if
            end do 

            if (jfract.eq.0) then
               write (*,1185) vname(iv(2)),v(iv(2)),
     *                        vname(iv(1)),v(iv(1))
               write (*,1210) names(jfr(i))
            end if 
         end if 

      end do 

      if (io3b.eq.1) goto 99 

      if (iflu.eq.0) then 
         write (n3,1220)
         write (n3,1040)
         do i = 1, icomp
            write (n3,1110) cname(i),fbulk(i), 
     *                      fbulk(i)/gtot*1d2, fbulk(i)*atwt(i)/btot*1d2                            
         end do
         write (n3,1230)
         write (n3,1060) ptot(2)/ptot(4)*1d2,ptot(3)/ptot(4)*1d5,
     *                   ptot(5)/ptot(4)*1d5
      else 
         write (n3,1220)
         write (n3,1080)
         do i = 1, icomp
            write (n3,1090) cname(i),fbulk(i),
     *                  fbulk(i)/gtot*1d2,fbulk(i)*atwt(i)/btot*1d2,
     *                  fbulk1(i)/gtot1*1d2,fbulk1(i)*atwt(i)/btot1*1d2          
         end do
         write (n3,1230)
         write (n3,1060) ptot(2)/ptot(4)*1d2,ptot(3)/ptot(4)*1d5,
     *                   ptot(5)/ptot(4)*1d5
         write (n3,1180) ptot1(2)/ptot1(4)*1d2,ptot1(3)/ptot1(4)*1d5,
     *                   ptot1(5)/ptot1(4)*1d5
      end if 

      if (icopt.eq.0) then 
         write (n3,1070) 0, jvar 
      else if (icopt.eq.3) then 
         write (n3,1070) 1, jvar
      else if (icopt.eq.1.or.icopt.eq.5) then 
         write (n3,1070) 2, jvar
      end if            

1000  format ('---------',/,'(',i6,') Stable phases at:')
1020  format (54x,'Molar Phase Compositions:',
     *        /,24x,'mol %',6x,'wt %',5x,'vol %',5x,20(a5,2x))
1030  format (a20,3x,3(f6.2,4x),20(f5.2,2x))
1040  format (/,13x,'mol',7x,'mol %',6x,'wt %')
1050  format (/,'WARNING - negative mass indicates the bulk'
     *         ,' composition is',/,'inconsistent with the'
     *         ,' saturation constraints',/)
1060  format (/,'Density (kg/m3) = ',f7.1,/, 
     *          'Specific Enthlpy (J/m3) = ',g12.6,/,
     *          'Specific Heat Capacity (J/K/m3) = ',g12.6)
1070  format (/,'Variance (c-p+',i1,') = ',i2,/,'---------',/)
1080  format (/,16x,'Complete Assemblage',15x,'Solids Only',
     *        /,13x,'mol',7x,' mol %',6x,'wt %',9x,' mol %',6x,'wt %')
1090  format (a8,3x,f6.2,5x,2(f6.2,4x),5x,2(f6.2,4x))
1110  format (a8,3x,3(f6.2,4x))
1120  format (28x,a,' = ',g12.6)
1130  format (20(i8,1x))
1150  format (12(g14.6,1x))
1160  format (/,24x,'Molar Properties:'
     *        /,26x,'N(g)',8x,'H(J)',5x,'V(J/bar)',6x,'Cp(J/K)'
     *         ,6x,'Alpha(1/K)',3x,'Beta(1/bar)')
1170  format (a20,1x,f9.2,3x,5(g12.5,1x))
1180  format (/,'Solid Density (kg/m3) = ',f7.1,/, 
     *          'Solid Specific Enthlpy (J/m3) = ',g12.6,/,
     *          'Solid Specific Heat Capacity (J/K/m3) = ',g12.6)
1185  format (/,'At ',a,'=',g12.6,' and ',a,'=',g12.6)
1190  format ('fractionating ',g12.6,' moles of ',a,'; changes bulk by:'
     *        ,/,15(1x,g12.6))
1200  format (g12.6,1x,g12.6,1x,a,15(1x,g12.6))
1210  format (a,' is not stable.')
1220  format (/,'Bulk Composition:',/)
1230  format (/,'Bulk Properties:',/)
99    end 

      subroutine ftext (text,iend)
c-----------------------------------------------------------------------
c subprogram to filter blanks from text 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      character*1 text(20)

      itic = 0 

      do i = 1, iend

         if (text(i).ne.' ') then 
            itic = itic + 1
            text(itic) = text(i)
         end if 

      end do 

      iend = itic 
 
      end

      subroutine blktxt (pname,id)
c-----------------------------------------------------------------------
c subprogram to write a text labels for bulk composition output 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*8 names, text(20)*1, fname*10, pname*20

      common/ cst8  /names(k1)/ csta7 /fname(h9)
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar    
c                             dump the names into the array text
         if (ikp(id).eq.0) then
c                             simple compound:
            iend = 8
            read (names(id),1000) (text(j), j = 1, iend)
            call ftext (text,iend)
            write (pname,1000) (text(j), j = 1, iend)
         else  
c                             solution phases:
            iend = 20
            read (fname(ikp(id)),1000) (text(j), j = 1, 10)
            text(11) = '('
            read (names(id),1000) (text(j), j = 12, 19)
            text(20) = ')'
            call ftext (text,iend)
            write (pname,1000) (text(j), j = 1, iend)
         end if 

1000  format (20a1)

      end 

      subroutine tset 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst44 /dlogt,dsqrtt

      if (t.lt.0d0.or.tr.eq.0d0) then
        dlogt = 0d0
        dsqrtt = 0d0
      else
        dlogt = dlog(t)
        dsqrtt = dsqrt(t)
      end if

      end 

      subroutine subst1 (n)
c-----------------------------------------------------------------------
c subst uses the lu decomposition of the matrix 'a' contained
c in the array 'a' to solve ax = b for x. subst is modified from the
c the subroutine of the same name listed by conte and de boor
c in 'elementary numerical analysis', mcgraw-hill, 1980.
c factor uses scaled partial pivoting.
 
c input     a- an n by n array containing the non-zero elements of
c              the u and l decompositions of a, as output by factor.
c           n- the dimension of the matrix a.
c        ipvt- a vector indicating that row ipvt(k) was used to
c              eliminate the coefficient a(n,k).
c           b- the vector b.
c output    b- the solution vector x.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      double precision x(k5)

      common / cst301 /a(k5,k5),b(k5),ipvt(k5)

c                            solve ly = b for y:
      ip = ipvt(1)
      x(1) = b(ip)

      do i = 2, n
         sum = 0d0
         im1 = i-1
         do j = 1, im1
            sum = a(i,j)*x(j)+sum
         end do 
         ip = ipvt(i)
         x(i) = b(ip)-sum
      end do 
c                            solve ux = y for x:
      x(n) = x(n)/a(n,n)
      nm1 = n-1

      do ii = 1, nm1
         i = n-ii
         ip1 = i+1
         sum = 0.
         do j = ip1, n
            sum = a(i,j)*x(j)+sum
         end do
         x(i) = (x(i)-sum)/a(i,i)
         b(i) = x(i)
      end do 

      b(n) = x(n)
 
      end

      subroutine factr1 (n,ier)
c-----------------------------------------------------------------------
c factr1 is a subroutine which calculates the triangular
c decompositions of the matrix 'a'. factor is modified from
c the subroutine of the same name given by conte and de boor
c in 'elementary numerical analysis', mcgraw-hill, 1980.
c factor uses scaled partial pivoting.
c
c input     a- an n by n array containing the elements of matrix a.
c           n- the dimension of the matrix a.
c output    a- an n by n array containing the upper, u, and lower, l,
c              triangular decompositions of input matrix a.
c        ipvt- a vector indicating that row ipvt(k) was used to
c              eliminate the a(n,k).
c         ier- a flag, zero if a is of rank = n, and 1 if a is of
c              lower rank.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst301 /a(k5,k5),d(k5),ipvt(k5)
c-----------------------------------------------------------------------
      ier = 0
c                            initialize ipvt,d
      do i = 1, n
         ipvt(i) = i
         rmax = 0d0
         do j = 1,n
            rmax = dmax1(rmax,dabs(a(i,j)))
         end do 
c                            ax = b is singular if rmax = 0
         if (dabs(rmax).lt.1d-5) goto 9000
         d(i) = rmax
      end do 
c                            begin decomposition:
      do i = 1, n - 1
c                            determine pivot row (istr).
         rmax = dabs(a(i,i))/d(i)
         istr = i
         ip1 = i + 1

         do j = ip1, n
            tmax = dabs(a(j,i))/d(j)
            if (tmax.le.rmax) cycle
            rmax = tmax
            istr = j
         end do 

         if (dabs(rmax).lt.1d-5) goto 9000
c                            if istr gt i, make i the pivot row
c                            by interchanging it with row istr.
         if (istr.gt.i) then 
            j = ipvt(istr)
            ipvt(istr) = ipvt(i)
            ipvt(i) = j
            temp = d(istr)
            d(istr) = d(i)
            d(i) = temp
            do j = 1, n
               temp = a(istr,j)
               a(istr,j) = a(i,j)
               a(i,j) = temp
            end do 
         end if 
c                            eliminate x(k) from rows k+1,...,n.
         do j = ip1,n
            a(j,i) = a(j,i)/a(i,i)
            ratio = a(j,i)
            do k = ip1, n
               a(j,k) = a(j,k)-ratio*a(i,k)
            end do 
         end do 
 
      end do 
     
      if (dabs(a(n,n)).lt.1d-5) ier = 1

      return
c                           algoritmic singularity.
9000  ier = 1
 
      end

      subroutine money
c
      open (97, file = 'money', iostat = ierr, status = 'old')
      if (ierr.ne.0) then
c                                 system could not find the file
         write (*,1000) 
         pause
         stop
1000     format (/,'There is no money file, make one and',
     *             ' deposit at least a dime.',/)
 
      end if

      read (97,*) imoney

      rewind (97)

      if (imoney.le.0) then

         write (*,1010) 
         stop
1010     format (/,'You are out of money, deposit at least a dime'/)

      else 

         imoney = imoney - 1
         write (97,*) imoney

      end if

      close (97)

      end

      subroutine subst (a,ipvt,n,b)
c-----------------------------------------------------------------------
c subst uses the lu decomposition of the matrix 'a' contained
c in the array 'a' to solve ax = b for x. subst is modified from the
c the subroutine of the same name listed by conte and de boor
c in 'elementary numerical analysis', mcgraw-hill, 1980.
c factor uses scaled partial pivoting.
 
c input     a- an n by n array containing the non-zero elements of
c              the u and l decompositions of a, as output by factor.
c           n- the dimension of the matrix a.
c        ipvt- a vector indicating that row ipvt(k) was used to
c              eliminate the coefficient a(n,k).
c           b- the vector b.
c output    b- the solution vector x.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'
 
      dimension a(k5,k5),ipvt(k5),b(k5+1),x(k5)
c                            solve ly = b for y:
      ip = ipvt(1)
      x(1) = b(ip)
      do i = 2, n

         sum = 0d0

         do j = 1, i - 1
            sum = a(i,j)*x(j)+sum
         end do 

         ip = ipvt(i)
         x(i) = b(ip)-sum

      end do 
c                            solve ux = y for x:
      x(n) = x(n)/a(n,n)

      do ii = 1, n - 1

         i = n-ii

         sum = 0d0

         do j = i + 1, n
            sum = a(i,j)*x(j)+sum
         end do 

         x(i) = (x(i)-sum)/a(i,i)
         b(i) = x(i)

      end do 
      b(n) = x(n)
 
      end

      subroutine testit (ier)
c-----------------------------------------------------------------------
c ier = 0 the plane is stable
c ier = 1 the plane is metastable
c ier = 2 the assemblage is degenerate
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 names
 
      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),iophi,idphi,
     *               iiphi,iflg1
     *      / cst8  /names(k1)/ cst2 /g(k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst52 /h,id(k5+1),ic(k5)/ cst87 /delt(l2),dtol,utol,ptol
c                              optional routine for testing
c                              for metastable g-x planes.
      ier = 0

      call abload (*991)
 
      do j = istct, iphct
 
         dg = g(j)
 
         do i = 1, icp
            dg = dg - cp(i,j) * b(i)
         end do
 
         if (dg.gt.dtol) cycle
c                                check that a phase is not metastable
c                                with respect to itself, this
c                                could be remedied by changing the 
c                                value of dtol.
         do k = 1, icp
            if (id(k).eq.j) goto 10
         end do
c                                the assemblage is metastable with
c                                respect to phase j
         ier = 1
 
10    end do 

      goto 99

991   ier = 2
 
99    end

      subroutine outtit (n)
c-----------------------------------------------------------------------
c outtit writes title information and a brief description of the
c chemical system for each calculation requested.
 
c the subroutine arguments are:
c   'm'- specifies the logical unit for the output.
c   'h'- indexes the calculation.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      character*8 cname*5,cmpnt*5,names,exname,
     *             dname*40,vname,xname,title*72
 
      common/ cst208 /ifct,idfl/ cst36 /exname(h8)
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn
     *      / cst3  /ctot(k1)/ cst8 /names(k1)
     *      / cst6  /icomp,istct,iphct,icp/ csta8 /title
     *      / cst37 /ixct,iexyn/ csta4 /cname(k5)
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst41 /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / csta5 /dname(8),cmpnt(k0)/ cst42 /ic(k5),idbase
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / csta2 /xname(k5),vname(l2)/ cst113 /ifproj
     *      / cst27 /jasmbl(j9),iasmbl(j9)/ cst12 /cp(k5,k1)
c----------------------------------------------------------------------
      if (n.eq.n3) goto (999),io3
      if (n.eq.n4) goto (999),io4
 
      write (n,1000)
c                          title:
      write (n,1190) title
c                          data base
      write (n,1210) dname(idbase)
c                          fluid
      if (ifyn.eq.0.or.ifproj.ne.0) call rfluid (2,ifug,n) 
c                          independent potentials:
      write (n,1070) (vname(jv(j)), j = 1, ipot)
c                          saturated phase components:
      if (ifyn.eq.0) then 
         j = icp + isat
         write (n,1200) (cname(j+i), i = 1, ifct)
      end if 
c                          saturated components:
      if (isyn.eq.0) then 
         j = icp + isat
         write (n,1180) (cname(i), i = icp1, j)
      end if 
c                          unconstrained components
      write (n,1080) (cname(i), i = 1, icp)
c                          phases
      if (icp.gt.3) then 
         write (n,1150) (cname(i), i = 1, icp)
         do i = istct, iphct
            write (n,1160) names(i), (cp(j,i)/ctot(i), j = 1, icp)
         end do 
      else if (icp.eq.3) then 
         write (n,1090) (cname(i), i = 2, 3)
         write (n,1100) (names(i), cp(2,i)/ctot(i), cp(3,i)/ctot(i),
     *                                             i = istct, iphct)
      else if (icp.eq.2) then 
         write (n,1040) cname(2)
         write (n,1030) (names(i), cp(2,i)/ctot(i), i = istct, iphct)
      else if (icp.eq.1) then 
         write (n,1130)
         write (n,1110) (names(i), i = istct, iphct)
      end if 
c                          saturation composant phases,
c                          load the indexes into iasmbl:
      if (isat.gt.0) then 
         itot = 0
         do i = 1, isat
            ic11 = isct(i)
c                          do a check that there is at least
c                          one composant for each saturated component:
c                          why here? well, why not?
            if (ic11.eq.0) call error (279,cp(1,1),i,'OUTTIT')
 
            do j = 1, ic11
               iasmbl(itot+j) = ids(i,j)
            end do 
            itot = itot+ic11
         end do 
 
         write (n,1170)
         write (n,1110) (names(iasmbl(i)), i = 1, itot)
      end if 
c                          excluded phases
      if (iexyn.eq.0) then 
         write (n,1140)
         write (n,1110) (exname(i), i = 1, ixct)
      end if 
 
      write (n,1000)
 
1000  format (/,80('-'),/)
1030  format (4(2x,a8,1x,f5.3))
1040  format (/,'Phases and (projected) mol fraction ',a,':',/)
1070  format (/,'Independently constrained potentials:',//,3x,8(a,1x))
1080  format (/,'Components with unconstrained potent'
     *       ,'ials:',//,3x,10(a5,3x))
1090  format (/,'Phases and (projected) composition with respect to '
     *         ,a5,' and ',a5,':',/)
1100  format (3(1x,a,2x,f5.3,2x,f5.3,5x))
1110  format (7(1x,a,1x))
1130  format (/,'Phases:',/)
1140  format (/,'Excluded phases:',/)
1150  format (/,'Phases and (projected) compositions:',//,
     *        11x,12(1x,a5,1x),/)
1160  format (3x,a8,12(1x,f5.3,1x))
1170  format (/,'Phases on saturation and buffering surfaces:',/)
1180  format (/,'Saturated or buffered components:',//,3x,7(a,3x))
1190  format (/,'Problem title: ',a,/)
1200  format (/,'Saturated phase components:',//,3x,5(a,3x))
1210  format ('Thermodynamic data base from: ',a)
999   end

      subroutine tertex (icase)
c--------------------------------------------------------------------
c allows a user to input the initial assemblage for a fixed
c bulk composition calculation. 

c returns 2 cases:

c icase = 0 -> do a normal calculation.
c icase = 1 -> the user has input a valid initial assemblage

c   referenced by: combin
c   references to: testit, assdc
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*1 y, names*8, name*8

      common/ cst52 /hcp,idv(k5+1),ic(k5)/ cst96 /idcf(k5,j9),icfct
     *      / cst6  /icomp,istct,iphct,icp/ cst8  /names(k1)
c-----------------------------------------------------------------------

      write (*,1020) 
      read (*,1000) y  

      icase = 0

      if (y.eq.'y') then

         icase = 1

         hcp = icp

         do i = 1, icp
            ic(i) = i 
         end do 

         write (*,1030) 
         read (*,1000) y
         if (y.eq.'y'.or.y.eq.'Y') then
            do i = 1, iphct, 5
               k = i + 5
               if (k.gt.iphct) k = iphct + 1
               write (*,1010) (j, names(j), j = i, k-1)
            end do
         end if
c                                             get cpds
         write (*,1080)
         read (*,1000) y
         if (y.eq.'y'.or.y.eq.'Y') then
c                                             by index 
            write (*,1040) 
            read (*,*) (idv(i), i = 1, icp)
         else 
            write (*,1090)
            do i = 1, icp 
30             read (*,1100) name
               do j = 1, iphct
                  if (name.eq.names(j)) then
                     idv(i) = j
                     goto 20  
                  end if 
               end do
               write (*,1110) name
               goto 30
20          end do 
         end if 
c                                             check stability
         call testit (ier) 
         if (ier.eq.1) then
            call error  (107,r,ier,'TERTEX')
         else if (ier.eq.2) then
            call error (108,r,ier,'TERTEX')
         end if 
c                                             assign assemblage
c                                             and check composition
         call assdc (ier)
         if (ier.ne.2) call error (109,r,ier,'TERTEX')

         write (*,1060) (idv(i), i = 1, icp)
         write (*,1070)

      end if    

1000  format (a)
1010  format (1x,6(i8,1x,a8,2x))
1020  format (/,'specify the initital assemblage (y/n)?')
1030  format (/,'list current phase indices (y/n)?')
1040  format (/,'enter the indices of the initial stable phases:')
1060  format (/,'the indices of the stable phases are:',//,10(2x,i8))
1070  format (/)
1080  format (/,'identify phases by numeric index (y/n)?')
1090  format ('enter pseudocompound names (one per line):')
1100  format (a)
1110  format (a,' is not a valid pseudocompound name, try again:')

      end

      subroutine smplex 
c--------------------------------------------------------------------
c smplx computes the stable pseudocompound assemblage for fully
c constrained problems by linear programming.  

c   referenced by: combin
c   references to: lpnag, assdc
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      integer liw, lw

      character*8 names

      parameter (liw=2*k1+3,lw=2*(k5+1)**2+7*k1+5*k5)  

      double precision  a(k5,k1), ax(k5), bl(k5), 
     *                  clamda(k1+k5), cvec(k1), w(lw), x(k1)

      integer is(k1+k5), iw(liw)

      common/ cst52 /hcp,idv(k5+1),ic(k5)/ cst3  /ctot(k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst2 /g(k1)/cst8/names(k1)
     *      / cst12 /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
c-----------------------------------------------------------------------

      hcp = icp
      ctotal = 0d0
c                                             normalize mole fractions
c                                             lp problem could be reformulated
c                                             as single bound problem to save
c                                             here.
      do i = 1, icp
         ic(i) = i 
         ctotal = ctotal + cblk(i)
      end do 
c                                             load arrays for lp solution
c                                             this is necessary 'cause of
c                                             the pointer strategy used in 
c                                             perplex, lpnag could be modified
c                                             to account for this and save 
c                                             on some massive arrays.
      jphct = iphct - istct + 1
c                                             initial guess of phase amounts
      do i = 1, jphct
         cvec(i) = g(istct-1+i)/ctot(istct+i-1)
      end do
c                                              composition constraint
      do i = 1, icp
         bl(i) = cblk(i)/ctotal
      end do 

      do i = 1, jphct
         do j = 1, icp
            a(j,i) = cp(j,istct+i-1)/ctot(istct+i-1)
         end do 
      end do
c                                             cold start 
      istart = 0
c                                             optimize by nag
      call lpnag (jphct,icp,a,k5,bl,cvec,is,x,ax,
     *            clamda,iw,liw,w,lw,ifail,l6,istart)
c                                             error status
      if (ifail.eq.1) then 
c                                             non-unique solution
         call warn (88,ctotal,ifail,'SMPLEX') 
      else if (ifail.eq.2.or.ifail.gt.4) then 
c                                             unbounded solution, or
c                                             other programming error.
         call warn (189,ctotal,ifail,'SMPLEX') 
      else if (ifail.eq.3) then 
c                                             no feasible solution
         call warn (42,ctotal,ifail,'SMPLEX')
      else if (ifail.eq.4) then 
c                                             iteration count exceeded,
c                                             probable cause no feasible
c                                             solution.
         call warn (190,ctotal,ifail,'SMPLEX') 
      end if
c                                             load results in to vertex
c                                             array idv
      ict = 0 
      do k = 1, jphct
         if (is(k).ne.1) then 
            ict = ict + 1
c                                             programming error ict > icp
            if (ict.gt.icp) call error (89,ctotal,ifail,'SMPLEX') 
            idv(ict) = k + istct - 1 
c           write (*,*) k,x(k),names(k),is(k),idv(ict)
         end if 
      end do

         call testit (ier) 
         if (ier.eq.1) then
            call error  (107,r,ier,'TERTEX')
         else if (ier.eq.2) then
            call error (108,r,ier,'TERTEX')
         end if 
c                                             assign assemblage
c                                             and (unnecessarily) check composition
      call assdc (ier)
      if (ier.ne.2) call error (209,r,ier,'SMPLEX')

      end

      subroutine findas

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      character*5 cname

      integer kmax(k1)

      common/ cst52  /hcp,id(k5+1),ic(k5)/ cst2 /g(k1)
     *      / cst23  /a(k5,k5),u(k5+1),ipvt(k5),idv(k5),iophi,idphi,
     *                iiphi,iflg1
     *      / cst6   /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
     *      / csta4  /cname(k5)

      do i = istct, iphct
c                                   sort phases into subcompositions
         do j = 1, icp
            if (cp(j,i).gt.1d-5) kmax(i) = j
         end do
      end do

      do hcp = 1, icp

         ic(hcp) = hcp
c                             make the first guess for the
c                             stable hcp component phase:
         do i = istct, iphct
            if (kmax(i).eq.hcp) then
               id(hcp) = i
               goto 20
            end if
         end do
c                             missing composant error
         call error (15,r,i,cname(hcp))
c
20       call abload (*992)
c                             start test loop:
         do j = istct, iphct 
            if (kmax(j).gt.hcp) cycle
            dg = g(j)
            do i = 1, hcp
               if (id(i).eq.j) goto 10
               dg = dg - cp(i,j)*u(i)
            end do

            if (dg.lt.-1d-8) then
c                             plane is metastable
c                             replace the most recently
c                             added test phase:
               id(hcp) = j
               call abload (*992)
            end if
10       end do 

      end do
c                                   this is to reset the component
c                                   counter in ABLOAD via cst52. 
      hcp = icp

      goto 99

992   call error (999,dg,i,'FINDAS')

99    end 
         

      subroutine checkd (jd)

      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common / cst52 /hcp,id(k5+1),ic(k5)/ cst6 /icomp,istct,iphct,icp
c                           istabl = 1 if the original
c                           assemblage was metastable, istable = 0
c                           otherwise.
      do i = 1, icp 
         if (id(i).eq.jd) goto 99
      end do

      if (dgphc(jd).gt.-1d-05) goto 99

      idold = id(icp)
      id(icp) = jd 

      call abload (*110)

      goto 99
c                           restore old vertex
110   id(icp) = idold

      call abload (*9000)
      
      goto 99

9000  call error (19,1d0,jd,'CHECKD')

99    end 

      function dgphc (ld)
c-----------------------------------------------------------------------
c dgphc is the g difference between a phase
c identified by 'ld', and the plane defined by 'u'.
c   referenced by: 
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst23 /a(k5*k5),u(k5+1),is(2*k5+4)/ cst2 /g(k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
 
      dg = g(ld)

      do i = 1, icp 
         dg = dg - cp(i,ld)*u(i)
      end do 

      dgphc = dg

      end

      subroutine miscib (jdv,jvar,imyn)
c----------------------------------------------------------------------
c miscib determines the variance [ivar] of the assemblage of c phases
c identified by the array jdv. miscib also checks for immiscibility 
c in solution phases.
c----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      integer iok(k5),jdv(k5+1)
 
      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),iophi,idphi,
     *               iiphi,iflg1      
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst6  /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
c----------------------------------------------------------------------     
c                                initialize flags and counters
      do i = 1, icp
         iok(i) = 0
      end do 
      ihom = 1
      im = 0
      jvar = 1
      ii = 1
c                                test if the phase idv(ii) is a
c                                a pseudo-compound not previously
c                                matched
50    ikt = ikp(jdv(ii))
      if (ikt.eq.0) goto 20
      do i = 1, icp - 3
         if (ikt.eq.iok(i)) goto 20
      end do 
c                                look for a match
      jj = ii+1

      do j = jj, icp
         if (ikp(jdv(j)).ne.ikt) cycle
c                                test for possibility of a homogeneous
c                                phase
         if (jj.ne.2) goto 40
         ihom = ihom + 1
      end do
 
      if (ihom.eq.1) goto 20
      goto 40
c                                no match or previously found
20    ii = ii+1

      if (ii.gt.icp) goto 99
      goto 50
c                                match found, set flags. note that the
c                                maximum number of matches in a five
c                                component system is two, increase the
c                                dimension of iok for larger systems.
40    im = im + 1
      iok(im) = ikt
      jvar = 2
c                                load and factor the transpose of the
c                                concentration matrix of the pseudo-
c                                invariant assemblage.
      do i = 1, icp
         do j = 1, icp
            a(j,i) = cp(j,jdv(i))
         end do
      end do 
 
      call factor (a,icp,ipvt,ier)
      if (ier.eq.1) then
         call warn (106,b(1),i,'MISCIB')
         write (n3,*) 'jdv=',jdv
         write (n3,*) 'ii, iok, jvar:',ii,iok,jvar
         goto 99
      end if 
c                                test for bounded metastable pseudo-cpds
c                                of the solution ikt
      do 60 j = istct, iphct

         if (ikp(j).ne.ikt) goto 60

         do i = 1, icp
c                                load composition vector, the alpha
c                                vector is returned in the same array
            b(i) = cp(i,j)
         end do 
c                                solve for the alpha vector
         call subst (a,ipvt,icp,b)
c                                a negative coefficient indicates
c                                the phase is not bounded
         do i = 1, icp
            if (b(i).lt.1d-5) goto 60
         end do 
c                                the phase is bounded, exsolution has
c                                occurred, test to determine if degen-
c                                erate solvus has been found
         if (ihom.eq.icp) goto 110

         do i = 1, icp
            if (ikp(jdv(i)).ne.ikt) goto 60
         end do 
c
110      jvar = 4
         imyn = 0
         goto 99
 
60    continue
c                                 if no immiscibility and the assemblage
c                                 is a homogeneous phase set asmbl flag
      if (ihom.eq.icp) jvar = 3
 
99    end 

      subroutine findjp (ivi,ivd,dv,ip)
c-----------------------------------------------------------------------
c findjp - called by sfol1 to iteratitvely refine the location of 
c an invariant point, the invariant point is located within an 
c energy tolerence specified by the variable ptol in routine pchk
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst63 /delv(l2)/ cst6  /icomp,istct,iphct,icp/ cst7 /iflag
     *      / cst87 /delt(l2),dtol,utol,ptol     
c                                 iterate independent variable to refine
c                                 the conditions of the invariant point:
      iwarn = 0 
      ier = 0 
      ip = 0
c                                 step back to stable condition
20    call reptx
      dv = dv / 2d0
c                                 check if the point is already known
      call sameip (ip)

      if (ip.ne.0) goto 99

      if (dabs(dv).lt.delt(ivi)) goto 30
 
10    v(ivi) = v(ivi) + dv 
      call incdep (ivi,v(ivi))   

      call univeq (ivd,ier)
      if (ier.ne.0) goto 30 

      call pchk (dg,igo)

      if (igo.eq.1) then
         goto 40 
      else if (iflag.eq.1) then
         goto 20
      else  
         goto 10
      end if 
c                                 univeq blew up, assign ip with 
c                                 warning:
30    call reptx 
      iwarn = 1 
c                                 assign the invariant assemblage:
40    call assip (ip)
      
      if (iwarn.eq.1) call warn (47,ptol,ip,'FINDJP')

99    end 

      subroutine ssaptx
c---------------------------------------------------------------------
c ssaptx assigns the current value of v1 and v2 to the array ptx and
c increments the ordinate counter ipt2 if the value of either variable
c has increased by more than 1% of the default increment. 
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)
 
      include 'perplex_parameters.h'
 
      common/ cst32 /ptx(l5),ipt2/ cst5  /v(l2),tr,pr,r,ps
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5/ cst63 /delv(l2)
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)

      if (dabs((v(iv1)-ptx(ipt2-1))/dv(iv1)).gt.1d-2.or.
     *    dabs((v(iv2)-ptx(ipt2))/dv(iv2)).gt.1d-2) then 
  
         ipt2 = ipt2 + 2

         if (ipt2.gt.l5) ipt2 = l5

         ptx(ipt2-1) = v(iv1)
         ptx(ipt2)   = v(iv2)

      else 

         ptx(ipt2-1) = v(iv1)
         ptx(ipt2)   = v(iv2)

      end if 
 
      end


      subroutine assip (ip)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst5  /v(l2),tr,pr,r,ps/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst29 /vip(l2,k2),ipid(k2,k5+2),ipct
     *      / cst6  /icomp,istct,iphct,icp
c-----------------------------------------------------------------------
      idv(icp+1) = iophi
      idv(icp+2) = idphi
c                                  assign the new elements of ipid
      ipct = ipct + 1
      if (ipct.gt.k2) call error (182,r,k2,'ASSIP')

      do j = 1, icp + 2
         ipid(ipct,j) =idv(j)
      end do 

      vip(1,ipct) = v(1)
      vip(2,ipct) = v(2)
      vip(3,ipct) = v(3)
      vip(4,ipct) = v(4)
      vip(5,ipct) = v(5)

      ip = ipct

      end

      subroutine sameip (ip)
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst23 /a(k5,k5),b(k5+1),ipvt(k5),idv(k5),
     *               iophi,idphi,iiphi,iflg1
     *      / cst5 /v(l2),tr,pr,r,ps/ cst24 /ipot,jv(l2),iv(l2)
     *      / cst7 /iflag/ cst29 /vip(l2,k2),ipid(k2,k5+2),ipct
     *      / cst9 /vmax(l2),vmin(l2),ddv(l2)
     *      / cst6 /icomp,istct,iphct,icp
c-----------------------------------------------------------------------
      ip = 0
c
      if (ipct.eq.0) goto 99
 
      icp2 = icp + 2
      idv(icp+1) = iophi
      idv(icp2) = idphi
c                                 determine if a new ip was found
      jps = 1
5     ips = jps 

      do 10 i = ips, ipct

         do 20 j = 1, icp2
            do 30 k= 1, icp2
30             if (idv(k).eq.ipid(i,j)) goto 20
c                                 check if invariant point is equivalent
c                                 to the ith ip already found
               goto 10
20       continue
c                      the new ip assemblage is equivalent to one found 
c                      earlier, check if the conditions match:
         do 25 j = 1, 5
            if (dabs(vip(j,i)-v(j)).gt.ddv(j).and.
     *                                 ddv(j).ne.0d0) then
               if (i.lt.ipct) then 
                  jps = i + 1
                  goto 5
               end if 
               goto 99 
            end if 
25       continue 
c                                 conditions and assemblage match
         v(iv(1)) = vip(iv(1),i)
         call incdep (iv(1),v(iv(1)))
         v(iv(2)) = vip(iv(2),i)
         call incdep (iv(2),v(iv(2)))
         call assptx
c                                 reset iflag so sfol2 doesn't try 
c                                 anything stupid
         iflag = 0
         ip = i
         goto 99

10    continue

99    end

      subroutine initlp (jphct,istart)
c--------------------------------------------------------------------
c initial arrays for lp minimizarion
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst52 /hcp,idv(k5+1),ic(k5)/ cst3  /ctot(k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst2 /g(k1)
     *      / cst12 /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst313 /a(k5,k1),b(k5)
c-----------------------------------------------------------------------
c                                 load arrays for lp solution
      hcp = icp
      ctotal = 0d0

      do i = 1, icp
         ic(i) = i 
         ctotal = ctotal + cblk(i)
      end do 
c                                         load arrays for lp solution
      jphct = iphct - istct + 1
c                                         composition constraint
      do i = 1, icp
         b(i) = cblk(i)/ctotal
      end do 

      do i = 1, jphct
         do j = 1, icp
            a(j,i) = cp(j,istct+i-1)/ctot(istct+i-1)
         end do 
      end do
c                                          cold start istart = 0
      istart = 0

      end 

      subroutine lpopt (i,j,jphct,istart,idead)
c-----------------------------------------------------------------------
c lpopt - calls lp minimization after a call to initlp. lpopt
c does the minimization, writes error messages if necessary and
c outputs results.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      integer liw, lw, ier

      parameter (liw=2*k1+3,lw=2*(k5+1)**2+7*k1+5*k5)  

      double precision  ax(k5), x(k1),
     *                  clamda(k1+k5), cvec(k1), w(lw)

      integer is(k1+k5), iw(liw), jdv(100)

      common/ cst52 /hcp,idv(k5+1),ic(k5)/ cst3  /ctot(k1)
     *      / cst6  /icomp,istct,iphct,icp/ cst2 /g(k1)
     *      / cst311/igrd(l7,l7)/ cst313 /a(k5,k1),b(k5)

      save ax, x, clamda, cvec, w, is, iw, ier

      data ier/0/
c-----------------------------------------------------------------------

      call gall

      do k = 1, jphct
         cvec(k) = g(istct-1+k)/ctot(istct+k-1)
      end do
 
      idead = 0 
c                                             optimize by nag
      call lpnag (jphct,icp,a,k5,b,cvec,is,x,ax,
     *            clamda,iw,liw,w,lw,ifail,l6,istart)
c                                             do a cold start on next
c                                             try if there was a problem
      if (ifail.ne.0) istart = 0 
c                                             look for errors                                            
      if (ifail.eq.1.and.ier.eq.0) then 
c                                             non-unique solution
         ier = 1
         call warn (88,ctotal,ifail,'SMPLEX') 

      else if (ifail.eq.2.or.ifail.gt.4) then 
c                                             unbounded solution, or
c                                             other programming error.
         call warn (91,ctotal,ifail,'SMPLEX') 

      else if (ifail.eq.3) then 
c                                             no feasible solution
         call warn (42,ctotal,ifail,'SMPLEX')

      else if (ifail.eq.4) then 
c                                             iteration count exceeded,
c                                             probable cause no feasible
c                                             solution.
         call warn (90,ctotal,ifail,'SMPLEX') 

      end if
c
      if (ifail.gt.1) then 
         idead = 1
         goto 10 
      end if 
c                                             load results in to vertex
c                                             array idv
      ict = 0 
      do k = 1, jphct
         if (is(k).ne.1) then 
            ict = ict + 1
c                                             programming error ict > icp
            if (ict.gt.100) then 
               call warn (89,ctotal,ifail,'SMPLEX') 
               idead = 1
               goto 10 
            end if 
            jdv(ict) = k
         end if 
      end do

      if (ict.eq.icp) then 

         do k = 1, icp
            idv(k) = jdv(k) + istct - 1
         end do  
c                                             check for degnerate comps
      else if (ict.gt.icp) then 
c                                             first move real active 
c                                             constraints into front of 
c                                             pointer array jdv
         jct = 0 
         do k = 1, ict 
            if (is(jdv(k)).ne.0.and.is(jdv(k)).ne.1) then
               jct = jct + 1
               iold = jdv(jct)
               jdv(jct) = jdv(k)
               idv(jct) = jdv(k) + istct - 1
               jdv(k) = iold
            end if 
         end do 
c                                             allow for only 1 false
c                                             active constraint
         if (jct.ne.icp-1) then 
            call warn (88,r,ifail,'LPOPT')
            write (*,*) jct, ict
            idead = 1
            goto 10
         end if 
c                                             now find one of the false
c                                             active constraints that      
         do k = jct+1, ict
            idv(icp) = jdv(k) + istct - 1    
            call detest (ier)
            if (ier.eq.0) goto 10   
         end do 
        
         call warn (88,r,ifail,'LPOPT')
         idead = 1

      else if (ict.lt.icp) then
         call warn (999,r,ifail,'LPOPT')
         idead = 1

      end if               
c                                             assign and output assemblage
10    if (idead.eq.0) then
c                                             
         call bulkck (idv,jbfg,igrd(i,j),i,j) 

      else if (i.gt.1.or.j.gt.1) then 

         istart = 0 
c                                             failed, set grid point to 
c                                             1st known assemblage
         if (i.gt.1) then
c                                             moving on i is preferable 
c                                             for 2-d fractionations
            igrd(i,j) = igrd(i-1,j)
         else 
            igrd(i,j) = igrd(i,j-1)
         end if 

         write (*,1000) i,j

      else

         istart = 0 

         write (*,1010) 
         igrd (i,j) = 0 

      end if 

1000  format (/,'WARNING: minimization failed at node i=',i6,' j= ',i6,/
     *         'last known assemblage will be assigned to this node',/)
1010  format (/,'ERROR: minimization failed at first grid node.',/)

      end 

      integer function jgrd(i,j)
c--------------------------------------------------------------------
c if imode = 0 returns pseudocompound assemblage index at node i,j
c "    "     1  "      phase assemblage index at node i,j
c--------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst311/igrd(l7,l7)/ cst312 /imode,jlow,jlev,loopx
     *      / cst74  /idbulk(k5,k2),iap(k2),ibulk,iaid(2)

      id = igrd(i,j)
      if (id.ne.0.and.imode.eq.1) id = iap(id)
      jgrd = id
      end 

      subroutine outdat
c----------------------------------------------------------------------------
c output data for special uses
c----------------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      character*8 names

      include 'perplex_parameters.h'

      common/ cst6   /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)   
     *      / cst52  /hcp,id(k5+1),ic(k5)/ cst60 /ipoint,imyn
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst8  /names(k1)/ cst2 /g(k1)/ cst3  /ctot(k1)

c           
1000  format (12(g16.8,1x))  
2000  format (a,16(g16.8,1x)) 
                     
      open (123,file='xsystem.dat')

      do j = 1, icp
         write (123,1000) cblk(j)
      end do 

      close (123)
      
      open (123,file='phase.dat')

      do i = 1, iphct
         write (123,1000) names(i),g(i)/ctot(i),
     *                    (cp(j,i)/ctot(i),j=1,icp)
      end do

      close (123)
      
      stop
      end 


      subroutine setvar (loopx,loopy)
c--------------------------------------------------------------------
c setvar initializes the variables for gridded minimization, three
c cases:

c icont = 1 -> independent variables are the 1st and 2nd potentials
c icont = 2 -> 1st independent variable is a composition variable,  
c              2nd independent variable is the 1st potential (iv1)
c icont = 3 -> independent variables are compositional variables
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont

      rloopy = float(loopy-1)
      rloopx = float(loopx-1)
      if (rloopx.eq.0) rloopx = 1d0

      do i = 1, ipot
         v(jv(i)) = vmin(jv(i))
         call incdep (jv(i),v(jv(i)))
      end do

      if (icont.eq.1) then 
c                               max resolution is
         dv(iv2) = (vmax(iv2) - vmin(iv2))/rloopy
         dv(iv1) = (vmax(iv1) - vmin(iv1))/rloopx
      else 
         dx = 1d0/rloopx
         dy = 1d0/rloopy 
         cx(1) = 0d0
         cx(2) = 0d0
c                                if icont = 2, the potential iv1
c                                is on y
         if (icont.eq.2) dv(iv1) = (vmax(iv1) - vmin(iv1))/rloopy 
      end if 
c                                set the bulk composition:
      do j = 1, jbulk
         cblk(j) = dblk(1,j)
      end do 

      end 

      subroutine incvr2 (jinc)
c--------------------------------------------------------------------
c incvar increments the inner loop variable (y) and for gridded 
c minimization, three cases:

c icont = 1 -> independent variables are the 1st and 2nd potentials
c icont = 2 -> 1st independent variable is a composition variable,  
c              2nd independent variable is the 1st potential (iv1)
c icont = 3 -> independent variables are compositional variables

c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont

      if (icont.eq.1) then 
c                               the variable is a potential
         v(iv2) = vmin(iv2) + dfloat(jinc)*dv(iv2)
         call incdep (iv2,v(iv2))


      else if (icont.eq.2) then 
c                               the variable is a potential
         v(iv1) = vmin(iv1) + dfloat(jinc)*dv(iv1)
         call incdep (iv1,v(iv1))

      else 
c                               the variable is a composition
         cx(2) = dfloat(jinc)*dy
         call setblk

      end if 

      end 

      subroutine incvr1 
c--------------------------------------------------------------------
c incvr1 increments the outer loop variable (x) and initializes
c the inner loop variable for gridded minimization, three cases:

c icont = 1 -> independent variables are the 1st and 2nd potentials
c icont = 2 -> 1st independent variable is a composition variable,  
c              2nd independent variable is the 1st potential (iv1)
c icont = 3 -> independent variables are compositional variables

c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst5  /v(l2),tr,pr,r,ps
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont

      if (icont.eq.1) then 

         v(iv1) = v(iv1) + dv(iv1)
         call incdep (iv1,v(iv1))
         v(iv2) = vmin(iv2)
         call incdep (iv2,v(iv2))

      else if (icont.eq.2) then 

         v(iv1) = vmin(iv1)
         call incdep (iv1,v(iv1))
         cx(1) = cx(1) + dx
         call setblk 

      else 

         cx(1) = cx(1) + dx
         cx(2) = 0d0
         call setblk

      end if 

      end 

      subroutine wavgrd 
c--------------------------------------------------------------------
c wavgrd does constrained minimization on a 1 dimensional multilevel
c grid (ith column of a 2-d grid), lowest resolution is the default, 
c and phase boundaries are located at the highest level

c the phase assemblage at node(i,j) of the grid is identified by the 
c pointer jgrd(i,j), jgrd is an integer function that returns a pointer
c to a compound or phase asseemblage depending on the paramter imode.
c the compound grid data igrd is output to the "plot" file.
c---------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      integer igrd, kinc, jinc(l8)

      common/ cst52 /hcp,idv(k5+1),ic(k5)/ cst5  /v(l2),tr,pr,r,ps
     *      / cst6  /icomp,istct,iphct,icp/ cst311/igrd(l7,l7)
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst312 /imode,jlow,jlev,loopx/ frct1 /ifrct,ifr(5),jfr(5)
c-----------------------------------------------------------------------
      ifrct = 0
c                               load arrays for lp solution
      call initlp (jphct,istart)
c                               jlow is the number of nodes
c                               at the lowest level, the number of
c                               nodes is
      klow = jlow - 1
c                               first level:
      loopy = klow * 2**(jlev-1) + 1 

      if (loopy.gt.l7) then
         call warn (92,v(iv1),loopy,'Y-AXIS')
         klow = (l7 - 1)/2**(jlev-1)
         loopy = klow * 2**(jlev-1) + 1 
      end if          

      if (loopx.gt.l7) then
         call warn (92,v(iv1),loopx,'X-AXIS')
         loopx = l7
      end if     
c                                increments
      do j = 1, jlev
         jinc(j) = 2**(jlev-j)
      end do 
c                                initialize variables
      call setvar (loopx,loopy)

      if (io4.eq.0) write (n4,1000) loopx, loopy 
c                                initialize grid vars
      kinc = jinc(1)
      klev = 1
      ktic = 0 

      do i = 1, loopx

         j = 1
         jmax = 1 
         jmin = 0
         jtic = 0 

         do while (j.le.loopy) 

            if (igrd(i,j).eq.0) then
               call lpopt (i,j,jphct,istart,idead)
               jtic = jtic + 1
            end if 

            kd = jgrd(i,j)
c                                jmax should always be the last known node
c                                at lowest level of resolution
            if (j.gt.jmax) then 
               jmax = j
               jmin = j - jinc(1) 
               kmax = kd
            end if 

            if (j.eq.1) then
 
               klast = kd
               
            else if (kd.ne.klast.and.klev.ne.jlev) then
c                                crossed a boundary and not
c                                at max resolution, increment resolution
               klev = klev + 1
               kinc = -jinc(klev)

            else if (kd.eq.klast.and.klev.ne.jlev) then 
c                                backed into field?
               if (kd.eq.kmax) then
c                                go to next low res node
                  kinc = jinc(1)
                  j = jmax
                  klev = 1
                  goto 10
               end if 
            
               klev = klev + 1
               kinc = jinc(klev)

            else if (klev.eq.jlev) then
c                                boundary has been located at max
c                                resolution
              
c                                jmin is the minimum value at which 
c                                subsequent searchs can go
               do k = j, jmax - 1
                  if (igrd(i,k).eq.0) then 
                     jmin = k - 1
                     goto 20 
                  end if 
               end do 
          
20             klast = jgrd(i,jmin)
c                                check if there is a grid point with
c                                this assemblage at a higher level
               do k = jmin, jmax - 1
                  if (jgrd(i,k).eq.klast) jmin = k
               end do 

               if (klast.eq.kmax.or.j.eq.jmax-1) then
c                                the grid can be filled in between j and jmax
c                                go on to the next low level grid point:
                  klast = jgrd(i,jmax)
                  kinc = jinc(1)
                  klev = 1
                  j = jmax
                  
               else 
c                                find the level to search for next boundary
c                                must be > level 1
                  do k = 2, jlev
                     if (jmax-jmin.gt.jinc(k)) then
                        j = jmax
                        klev = k
                        kinc = -jinc(k)
                        goto 10 
                     end if
                  end do 
               end if 
            end if 

10          j = j + kinc

            if (j.lt.jmin+1) then 
               j = j - kinc 
               klev = klev + 1
               kinc = -jinc(klev)
               goto 10 
            end if 

            call incvr2 (j-1)

         end do 

         ktic = ktic + jtic 
         write (*,1030) ktic,i*loopy,float(i)/float(loopx)*1e2

         if (io4.eq.0) then 
c                                             output graphics data
            kst = 1

40          jst = kst
            kd = igrd(i,jst)
            ltic = -1

            do j = jst, loopy
               if (igrd(i,j).eq.0.or.igrd(i,j).eq.kd) then
                  ltic = ltic + 1
                  if (j.eq.loopy) then 
                     write (n4,1000) ltic,kd
                     kst = 1
                  end if 
               else 
                  write (n4,1000) ltic,kd
                  kst = j
                  goto 40 
               end if 
            end do 
         end if 
         
         call incvr1 
         
      end do 

1000  format (20(i6,1x))
1010  format (2(g14.6,1x))
1030  format (i6,' minimizations required out of ',i6,' possible. ',
     *        f4.1,'% done.')

99    stop

      end 


      subroutine setblk
c-----------------------------------------------------------------------
c for gridded minimization setblk computes the bulk composition
c and initializes the arrays for lpopt.
c-----------------------------------------------------------------------
      implicit double precision (a-g,o-z),integer (h-n)

      include 'perplex_parameters.h'

      common/ cst6  /icomp,istct,iphct,icp
     *      / cst12 /cp(k5,k1)/ cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst313 /a(k5,k1),b(k5)
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont
c-----------------------------------------------------------------------

      if (icont.eq.2) then 
         x1 = 1d0 - cx(1)
         x2 = cx(1)
c                                 bulk composition computed as
c                                 c = (1-cx(1))*dblk(1) + cx(1)*dblk(2)
         do j = 1, jbulk
            cblk(j) = x1*dblk(1,j) + x2*dblk(2,j)
         end do 
      else
         x1 = cx(1)
         x2 = cx(2)
c                                 bulk composition computed as
c                                 c = dblk(1) + cx(1)*dblk(2) + cx(2)*dblk(3)  
         do j = 1, jbulk
            cblk(j) = dblk(1,j) + x1*dblk(2,j) + x2*dblk(3,j)
         end do 
      end if 

c                                 modify cblk here to change the 
c                                 composition before minimization.
      ctotal = 0d0
c                                 get total moles to compute mole fractions             
      do i = 1, icp
         ctotal = ctotal + cblk(i)
      end do

      do i = 1, icp 
         b(i) = cblk(i)/ctotal
      end do

      end 

      subroutine moduli (idv,jmod,emods) 
c-----------------------------------------------------------------------
c subroutine moduli determines if an assemblage contains a cpd with
c special elastic coefficients (iemod(id) ne 0), if so 
c moduli computes elastic coefficients for all compounds present 
c (compounds with no coefficients are assigned zero).

c jmod is the number of cpds for which it is possible to calculate coeffs.
c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      double precision emods(k15,k5), emod, sxs, exces, cblk, atw, y2pg, 
     *                 p0a(m4), dvnu, p0ord, x, vlar, deph, dydy, 
     *                 qmult, d0, dcoef, w, xx, wgl, sdzdy, mods(k15)

      integer idv(k5), iemod, kmod, ipoint, imyn, ixp, ifp, jend, io3b,
     *        ndep, im2jm, k, i, j, ivar, ivarip, isudo, jmod, l,ivarrx,
     *        jm, mstot, ixpt, knsp, jd, ids, ikp, jbulk, id, inid, 
     *        jsub, hsub, lterm, jterm, ksp, msite, ideps

      common/ cst319 /emod(k15,k10),iemod(k1),kmod/ cst60  /ipoint,imyn
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)
     *      / cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),xx(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
c-----------------------------------------------------------------------
  
      jmod = 0 
      do i = 1, jbulk
         do j = 1, k15
            emods(j,i) = 0d0
         end do
      end do 

      do i = 1, jbulk

         id = idv(i)
         if (id.le.ipoint.and.iemod(id).ne.0) then
c                                 we have a cpd with moduli 
            jmod = jmod + 1

            call shearm (emods(1,i),emods(2,i),emods(3,i),id)

         else if (iemod(id).ne.0) then 

            jmod = jmod + 1
c                                 we have a pseudocpd with moduli
c                                 for reciprocal solutions with speciation
c                                 (ifp(id)=8) we need to compute endmember
c                                 fractions:
            if (ifp(id).ne.8) then 
c                                 for each endmember cpd:
               do j = 1, jend(ikp(id),1) 
c                                 the endmember is:
                  jd = jend(ikp(id),2+j)
c                                  get endmember modulus
                  call shearm (mods(1),mods(2),mods(3),jd)
c                                 and its mole fraction is:
                  x = sxs(ixp(id)+j)
                  do k = 1, k15
                     emods(k,i) = emods(k,i) + x*mods(k)
                  end do 
               end do     
       
            else 

               ids = ikp(id)
               jm = im2jm(ids)
               jd = mstot(jm) + 1
c                                 initialize p0's
               do j = 1, mstot(jm)
                  p0a(j) = 0d0
               end do 
c                                 convert y's to p's, this loop 
c                                 could be eliminated by saving the
c                                 p's.
               ixpt = ixp(id) + jend(ids,1) - ndep(jm)
               p0ord = 0d0

               do k = 1, jend(ids,1) - ndep(jm)

                  p0a(knsp(jm,k)) =  sxs(ixp(id)+k)

                  do l = 1, ndep(jm)
                     p0a(knsp(jm,k)) =  p0a(knsp(jm,k)) 
     *                           + y2pg(jm,k,l) * sxs(ixpt+l)
                  end do 

                  p0ord = p0ord + p0a(knsp(jm,k))

               end do 
c                                 the initial fraction of the ordered species
               p0ord = 1d0 - p0ord
               p0a(knsp(jm,jend(ids,1)+1)) = p0ord
c                                 get gmech
               do k = 1, jend(ids,1) - ndep(jm)
                  jd = jend(ids,2+k)
c                                  get endmember modulus
                  call shearm (mods(1),mods(2),mods(3),jd)
c                                 check if these are gonna total to one
                  x = p0a(knsp(jm,k)) + dvnu(jm,k)*p0ord
                  do l = 1, k15
                     emods(l,i) = emods(l,i) + x*mods(l)
                  end do 
               end do 

            end if 
         end if 
c
      end do 

      end

      subroutine dphase (id,e,v,cp,alpha,beta,alphat,alphap,betap,s)
c-----------------------------------------------------------------------
c dphase returns the props of the phase identified by id as 
c computed by centered finite differences from the Gibbs energy. 

c the difference increments are

c dt0, dp0 for 1st order derivatives (entropy,volume and enthalpy)
c dt1, dp1 for 2nd order derivatives (heat capacity, expansivity*, 
c          compressibility*)
c dt2, dp2 for 3rd order derivatives (alphat, alphap = betat, betap
c          alphtt), and the fourth order derivative alphtt, this 
c          derivative was used by werami prior to 5/31/04. 

c *expansivity (alpha) as returned here is dv/dt
c *compressibility (beta) as returned here is dv/dp

c corrected to check for negative pressure, in which case 
c forward differences are used. june 22, 2004, JADC.
c-----------------------------------------------------------------------
      implicit none

      integer id

      double precision p,t,xco2,u1,u2,tr,pr,r,ps,dt0,dt1,dt2,g0,
     *                 dp0,dp1,dp2,e,alpha,v,ginc,beta,betap,betat,
     *                 cp,alphat,alphap,g1,g2,g3,g4,s,fbetap

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps

      save dt0,dt1,dt2,dp0,dp1,dp2

      data dt0,dt1,dt2,dp0,dp1,dp2/0.5,5.,50.,5.,50.,500./

      dp0 = 0.5d-3 * p 
      dp1 = 0.5d-2 * p
      dp2 = 0.5d-1 * p
            
      g0 = ginc(0d0,0d0,id)
c                                  straight derivatives:
c                                  first order
      if (p-dp0.le.0d0) then 

         v = (ginc(0d0,dp0,id) - ginc(0d0,0d0,id))/dp0
         if (v.lt.0d0.or.dabs(v).gt.1d9)  
c                                  expand increment if invalid v
     *   v = (ginc(0d0,dp1,id) - ginc(0d0,0d0,id))/dp1
         if (v.lt.0d0.or.dabs(v).gt.1d9)  
c                                  expand increment more if invalid v
     *   v = (ginc(0d0,dp2,id) - ginc(0d0,0d0,id))/dp2

      else 

         v = (ginc(0d0,dp0,id) - ginc(0d0,-dp0,id))/dp0/2d0
         if ((v.lt.0d0.or.dabs(v).gt.1d9).and.p-dp1.gt.0d0)  
c                                  expand increment if invalid v
     *   v = (ginc(0d0,dp1,id) - ginc(0d0,-dp1,id))/dp1/2d0
         if ((v.lt.0d0.or.dabs(v).gt.1d9).and.p-dp2.gt.0d0)  
c                                  expand increment more if invalid v
     *   v = (ginc(0d0,dp2,id) - ginc(0d0,-dp2,id))/dp2/2d0

      end if 

      s = (ginc(-dt0,0d0,id) - ginc(dt0,0d0,id))/dt0/2d0
      e = g0 + t * s
c                                  second order
      cp = -t*(ginc(dt1,0d0,id) + ginc(-dt1,0d0,id) - 2d0*g0)/dt1/dt1
      if (cp.lt.0d0.or.dabs(cp).gt.1d9)  
c                                  expand increment if invalid cp
     *   cp = -t*(ginc(dt2,0d0,id) + ginc(-dt2,0d0,id) - 2d0*g0)/dt2/dt2
      if (cp.lt.0d0.or.dabs(cp).gt.1d9)  
c                                  shrink increment if invalid cp
     *   cp = -t*(ginc(dt0,0d0,id) + ginc(-dt0,0d0,id) - 2d0*g0)/dt0/dt0
   
      if (p-dp1.le.0d0) then 
c                                  use forward difference at small p's
         beta = (ginc(0d0,2d0*dp1,id) + g0 - 2d0*ginc(0d0,dp1,id))
     *          /dp1/dp1
         if (dabs(beta).gt.v)
c                                  expand increment if invalid beta
     *   beta = (ginc(0d0,2d0*dp2,id) + g0 - 2d0*ginc(0d0,dp2,id))
     *          /dp2/dp2                                 
         if (dabs(beta).gt.v)
c                                  shrink increment if invalid beta
     *   beta = (ginc(0d0,2d0*dp0,id) + g0 - 2d0*ginc(0d0,dp0,id))
     *          /dp0/dp0   

         alpha = ( ginc( dt1,dp1,id) - ginc( dt1,0d0,id)
     *            -ginc(-dt1,dp1,id) + ginc(-dt1,0d0,id))/dp1/dt1/2d0
         if (dabs(alpha).gt.v.or.alpha.lt.0d0)
c                                  expand increment if invalid alpha
     *   alpha = ( ginc( dt2,dp2,id) - ginc( dt2,0d0,id)
     *            -ginc(-dt2,dp2,id) + ginc(-dt2,0d0,id))/dp2/dt2/2d0
         if (dabs(alpha).gt.v.or.alpha.lt.0d0)
c                                  shrink increment if invalid alpha
     *   alpha = ( ginc( dt0,dp0,id) - ginc( dt0,0d0,id)
     *            -ginc(-dt0,dp0,id) + ginc(-dt0,0d0,id))/dp0/dt0/2d0

      else 
         beta = (ginc(0d0,dp1,id) + ginc(0d0,-dp1,id) - 2d0*g0)/dp1/dp1
         if (dabs(beta).gt.v.and.p-dp2.gt.0d0)
c                                  expand increment if invalid beta
     *   beta = (ginc(0d0,dp2,id) + ginc(0d0,-dp2,id) - 2d0*g0)/dp2/dp2                                
         if (dabs(beta).gt.v)
c                                  shrink increment if invalid beta
     *   beta = (ginc(0d0,dp0,id) + ginc(0d0,-dp0,id) - 2d0*g0)/dp0/dp0

         alpha = ( ginc( dt1,dp1,id) - ginc( dt1,-dp1,id)
     *            -ginc(-dt1,dp1,id) + ginc(-dt1,-dp1,id))/dp1/dt1/4d0
         if (dabs(alpha).gt.v.or.alpha.lt.0d0)
c                                  expand increment if invalid alpha
     *   alpha = ( ginc( dt2,dp2,id) - ginc( dt2,-dp2,id)
     *            -ginc(-dt2,dp2,id) + ginc(-dt2,-dp2,id))/dp2/dt2/4d0
         if (dabs(alpha).gt.v.or.alpha.lt.0d0)
c                                  shrink increment if invalid alpha
     *   alpha = ( ginc( dt0,dp0,id) - ginc( dt0,-dp0,id)
     *            -ginc(-dt0,dp0,id) + ginc(-dt0,-dp0,id))/dp0/dt2/4d0      
     
      end if 
       
      if (dabs(alpha).gt.v.or.dabs(beta).gt.v.or.dabs(cp).gt.1d4.or.
     *    v.lt.0d0.or.cp.lt.0d0.or.alpha.lt.0d0) then
c         write (*,*) 'ugga bugga'
      end if 
c                                  third order
      if (p-2d0*dp2.le.0d0) then 
         betap = ((ginc(0d0,4d0*dp2,id) - g0)/2d0
     *           - ginc(0d0,3d0*dp2,id) + ginc(0d0,dp2,id))/dp2**3
      else 
         betap = fbetap(dp2,id)
         if (dabs(betap).gt.v) then 
            betap = fbetap(dp1,id)
            if (dabs(betap).gt.v) then 
               betap = fbetap(dp0,id)
            end if 
         end if 
      end if 
c                                  third order
      if (p-dp2.le.0d0) then 
         betat = 0d0 
         alphat = 0d0
         g1 = ginc(-dt2,0d0,id)
         g2 = ginc( dt2,2d0*dp2,id)
         g3 = ginc( dt2,0d0,id)
         g4 = ginc(-dt2,2d0*dp2,id)

         betat = (g2 + g3 + 2d0*(ginc(-dt2, dp2,id)-ginc(dt2,dp2,id))
     *           -g4 - g1)/dp2/dp2/dt2/2d0

         alphat = (g2 + g4  + 2d0*(g0 - ginc(0d0,2d0*dp2,id))
     *            -g3 - g1)/2d0/dp2/dt2/dt2
      else 

         g1 = ginc(-dt2,-dp2,id)
         g2 = ginc( dt2,dp2,id)
         g3 = ginc( dt2,-dp2,id)
         g4 = ginc(-dt2,dp2,id)

         betat = (g2 + g3 + 2d0*(ginc(-dt2, 0d0,id)-ginc(dt2,0d0,id))
     *           -g4 - g1)/dp2/dp2/dt2/2d0
         alphat = (g2 + g4  + 2d0*(ginc(0d0,-dp2,id) - ginc(0d0,dp2,id))
     *            -g3 - g1)/2d0/dp2/dt2/dt2
      end if 

      if (dabs(beta).gt.v) beta = 0d0
      if (dabs(alpha).gt.v) alpha = 0d0
      if (dabs(betat).gt.v) betat = 0d0
      if (dabs(alphat).gt.v) alphat = 0d0
      if (dabs(betap).gt.v) betap = 0d0
      if (dabs(cp).gt.1d9.or.cp.lt.0d0) cp = 0d0

      alphap = betat 
c                                  fourth order, no longer used in WERAMI, 5/04, JADC
c      alphtt = ((ginc(2d0*dt2,dp2,id) - ginc(-2d0*dt2,dp2,id))/2d0
c     *        - g2 + g4
c     *        -(ginc(2d0*dt2,-dp2,id) - ginc(-2d0*dt2,-dp2,id))/2d0
c     *        + g3 - g1)/dt2**3/dp2/2d0

      end 

      double precision function fbetap (dp,id)
c-----------------------------------------------------------------------
c fbetap returns centered difference estimator for d3v/dp/dp/dp
c-----------------------------------------------------------------------
      implicit none

      integer id

      double precision ginc, dp 

      fbetap = ((ginc(0d0,2d0*dp,id) - ginc(0d0,-2d0*dp,id))/2d0
     *         - ginc(0d0,+dp,id) + ginc(0d0,-dp,id))/dp**3

      end 

      subroutine shearm (mu,mup,mut,id)
c-----------------------------------------------------------------------
c shearm returns a linear model for the adiabatic shear modulus
c relative to the current pressure and temperature.

c two cases:

c iemod = 1, linear model is input

c iemod = 2, shear modulus is known as a function of (V,T), then
c computed by centered finite differences.
c-----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer id, kmod, iemod

      double precision p,t,xco2,u1,u2,tr,pr,r,ps,dt,dp,smu,mut,mup,mu,
     *                 g,ginc,emod

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst319 /emod(k15,k10),iemod(k1),kmod
     *      / cst323 /smu

      save dt, dp

      data dt, dp/50d0,500d0/

      dp = p * 1d-3

      if (iemod(id).eq.1) then 

         mup = emod(2,id)
         mut = emod(3,id)
         mu  = emod(1,id) + (p-pr)*mup + (t-tr)*mut

      else if (iemod(id).eq.2) then 
c                                  by calling ginc a call to
c                                  stixrudes EoS for the adiabatic
c                                  shear modulus is implicit (cst323)
         g = ginc(0d0,0d0,id)
         mu = smu
c                                  pressure derivative:
         g = ginc(0d0,dp,id)
         mup = smu

         if (p-dp.le.0) then 
c                                  use 1/2 forward difference at low P
            mup = (mup - mu)/dp
         else
            g = ginc(0d0,-dp,id)
            mup = (mup - smu)/dp/2d0
         end if 
c                                  temperature derivative
         g = ginc(dt,0d0,id)
         mut = smu       
         g = ginc(-dt,0d0,id)
         mut = (mut - smu)/dt/2d0

      end if             

      end 


      double precision function ginc (dt,dp,id)

      implicit none

      double precision dt,dp,p,t,xco2,u1,u2,tr,pr,r,ps,gee

      integer id

      common/ cst5 /p,t,xco2,u1,u2,tr,pr,r,ps

      p = p + dp 
      t = t + dt 

      call tset
      call gphase(id,gee)

      p = p - dp 
      t = t - dt

      ginc = gee 

      end 

      subroutine matchi (unnown,itis,icpd)
c----------------------------------------------------------------
 
c matchi - subroutine to determine if the string unnown is a valid
c          solution or compound name.
 
c   itis = 0 if compound, = ikp if solution, and =-1 if invalid
c   icpd = the id of the compound if itis = 0

      implicit none
 
      character*10 fname, names*8, unnown
 
      include 'perplex_parameters.h'

      integer ivar,ikp,ivarrx,ivarip,isudo,icomp,istct,iphct,icp,
     *        isoct,ipoint,imyn,icpd,itis,i

      common/ cst60 /ipoint,imyn/ csta7 /fname(h9)/ cst8  /names(k1)
     *      / cst79 /isoct/ cst6  /icomp,istct,iphct,icp
     *      / cst61 /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
 
      if (isoct.ne.0) then
         do i = 1, isoct
            if (unnown.eq.fname(i)) then
               itis = i
               goto 99
             end if
         end do
      end if
 
      do i = 1, iphct
         if (unnown.eq.names(i)) then
            itis = 0
            icpd = i
            goto 99
         end if
      end do 
 
      itis = -1
99    end


      subroutine frname 
c---------------------------------------------------------------
c frname - prompt for names of phases to be fractionated

c          ifrct - number of phases to be fractionated
c          ifr(ifract) - 0 if cpd, else ikp (solution pointer)
c          jfr(ifract) - cpd pointer

      implicit none

      character*10 phase, fname*14

      integer ifrct, ifr, jfr
 
      include 'perplex_parameters.h'

      common/ frct1 /ifrct,ifr(5),jfr(5)

      ifrct = 1

110   write (*,1040)
      read (*,1020) phase

      if (phase.eq.'') goto 210

      call matchi (phase,ifr(ifrct),jfr(ifrct))

      if (ifr(ifrct).eq.-1) then
         write (*,1100) phase
         goto 110
      end if

      write (fname,1000) phase,'.dat'
      call unblnk (fname)

      write (*,1010) phase, fname

      open (unit=100+ifrct,file=fname,status='unknown')

      ifrct = ifrct + 1

      goto 110
 
210   ifrct = ifrct - 1

1000  format (a,a)
1010  format (/,'The fractionated amount and composition of ',a,/,
     *          'will be written to file: ',a,/)
1020  format (a)
1040  format (/,'Enter the name of a phase to be fractionated',
     *        /,'(left justified, <cr> to finish): ')
1100  format (/,'No such entity as ',a,', try again: ')

      end 

      subroutine setvr0 (i,j)
c--------------------------------------------------------------------
c setvr1 computes nodal variables for node ij, three cases:

c icont = 1 -> independent variables are the 1st and 2nd potentials
c icont = 2 -> 1st independent variable is a composition variable,  
c              2nd independent variable is the 1st potential (iv1)
c icont = 3 -> independent variables are compositional variables

c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer ipot,jv,iv1,iv2,iv3,iv4,iv5,icont,i,j

      double precision v,tr,pr,r,ps,vmax,vmin,dv,dblk,cx,dx,dy

      common/ cst5  /v(l2),tr,pr,r,ps
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst314 /dblk(3,k5),cx(2),dx,dy,icont

      if (icont.eq.1) then 

         v(iv1) = vmin(iv1) + (i-1)*dv(iv1)
         call incdep (iv1,v(iv1))
         v(iv2) = vmin(iv2) + (j-1)*dv(iv2)
         call incdep (iv2,v(iv2))

      else if (icont.eq.2) then 

         v(iv1) = vmin(iv1) + (j-1)*dv(iv1)
         call incdep (iv1,v(iv1))

         cx(1) =  (i-1)*dx
         call setblk 

      else 

         cx(1) = (i-1) * dx
         cx(2) = (j-1) * dy
         call setblk

      end if 

      end

      subroutine amihot (i,j,jhot,jinc)
c--------------------------------------------------------------------
c check if cell with lower left corner at node ij is homogeneous
c---------------------------------------------------------------------
      implicit none

      integer jgrd, jinc, i, j, jhot

      jhot = 1

      if (jgrd(i,j).eq.jgrd(i,j+jinc).and.
     *    jgrd(i,j).eq.jgrd(i+jinc,j+jinc).and.
     *    jgrd(i,j).eq.jgrd(i+jinc,j)) jhot = 0
     
      end  

      subroutine filler (i,j,kinc)
c--------------------------------------------------------------------
c fill nodes between identical edges or diagonals, this could
c be modified to a less biased (symmetrical) strategy as in 
c routine aminot. 
c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer kinc, i, j, igrd, ii, jj, k

      common/ cst311/igrd(l7,l7)

      if (kinc.eq.1) return 

      if (igrd(i,j).eq.igrd(i+kinc,j+kinc)) then
c                                right diagonal
         do k = 1, kinc-1
c                                the conditional prevents 
c                                overwriting of identities if 
c                                compression is off and searching
c                                for true boundaries.
            if (igrd(i+k,j+k).eq.0) igrd(i+k,j+k) = igrd(i,j)
         end do 
      else if (igrd(i+kinc,j).eq.igrd(i,j+kinc)) then
c                                left diagonal
         do k = 1, kinc-1
c                                the conditional prevents 
c                                overwriting of identities if 
c                                compression is off and searching
c                                for true boundaries.
            if (igrd(i+k,j+kinc-k).eq.0) 
     *          igrd(i+k,j+kinc-k) = igrd(i,j+kinc)
         end do 

c        write (*,*) 'left diagonal'

      end if
c                                bottom and top edge
      do jj = j, j+kinc, kinc
         if (igrd(i,jj).eq.igrd(i+kinc,jj)) then 
            do k = 1, kinc-1
c                                the conditional prevents 
c                                overwriting of identities if 
c                                compression is off and searching
c                                for true boundaries.
               if (igrd(i+k,jj).eq.0) igrd(i+k,jj) = igrd(i,jj)
            end do 
         end if
      end do 
c                                left and right edges
      do ii = i, i+kinc, kinc
         if (igrd(ii,j).eq.igrd(ii,j+kinc)) then 
            do k = 1, kinc-1
c                                the conditional prevents 
c                                overwriting of identities if 
c                                compression is off and searching
c                                for true boundaries.
               if (igrd(ii,j+k).eq.0)  igrd(ii,j+k) = igrd(ii,j)
            end do 
         end if
      end do 

      end

      subroutine wav2d1
c--------------------------------------------------------------------
c wav2d does constrained minimization on a 2 dimensional multilevel
c grid (ith column of a 2-d grid), lowest resolution is the default, 
c and phase boundaries are located at the highest level

c the phase assemblage at node(i,j) of the grid is identified by the 
c pointer jgrd(i,j), jgrd is an integer function that returns a pointer
c to a compound or phase asseemblage depending on the paramter imode.
c the compound grid data igrd is output to the "plot" file.
c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer igrd, kinc, jinc(l8), iind(4), jind(4), iiind(4,2),htic,
     *        jjind(4,2), hotij(l7*l7,2), kotij(l7*l7,2), icind(4),
     *        jcind(4), lhot(4), ieind(5), jeind(5), jfr, ifr, imode,
     *        n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9,i,ihot,jlev,
     *        iv1,iv2,iv3,iv4,iv5,jv,ipot,io3b,jbulk,icp,iphct,istct,
     *        icomp,ic,idv,hcp,ltic,kd,jst,kst,iil,jjl,ll,je,ie,icell,
     *        ii,jj,kk,jgrd,hh,hhot,jcent,icent,jjc,iic,h,jtic,khot,k,
     *        ktic,j,jhot,loopy,jlow,klow,istart,jphct,loopx,ifrct,
     *        kinc2,kinc21,ktest,jtest,jpot,idead

      double precision dv,vmin,vmax,atw,cblk,ps,r,pr,tr,v

      common/ cst52 /hcp,idv(k5+1),ic(k5)/ cst5  /v(l2),tr,pr,r,ps
     *      / cst6  /icomp,istct,iphct,icp/ cst311/igrd(l7,l7)
     *      / cst300 /cblk(k5),atw(k1),jbulk,io3b
     *      / cst9  /vmax(l2),vmin(l2),dv(l2)/ debug  /jtest,jpot
     *      / cst24 /ipot,jv(l2),iv1,iv2,iv3,iv4,iv5
     *      / cst41  /n1,n2,n3,n4,n5,n6,n7,n8,n9,io3,io4,io5,io9
     *      / cst312 /imode,jlow,jlev,loopx/ frct1 /ifrct,ifr(5),jfr(5)

      save iind, jind, iiind, jjind, icind, jcind, istart

      data iind, jind   /0,0,1,1, 0,1,1,0/
      data iiind, jjind /0,0,1,1,1,1,2,2, 1,1,2,0,0,2,1,1/
      data icind, jcind /0,1,2,1, 1,2,1,0/
      data ieind, jeind /0,0,2,2,0, 0,2,2,0,0/
            
c-----------------------------------------------------------------------
c                               call old routine for 1d grid
      if (loopx.eq.1) call wavgrd

      ifrct = 0
c                               load arrays for lp solution
      call initlp (jphct,istart)
c                               jlow is the number of nodes
c                               at the lowest level, the number of
c                               nodes is
      klow = jlow - 1
c                               first level:
      loopy = klow * 2**(jlev-1) + 1 

      loopx = (loopx-1) * 2**(jlev-1) + 1 

      write (*,1040)

      if (loopy.gt.l7) then
         call warn (92,v(iv1),loopy,'Y-AXIS')
         klow = (l7 - 1)/2**(jlev-1)
         loopy = klow * 2**(jlev-1) + 1 
      end if          

      if (loopx.gt.l7) then
         call warn (92,v(iv1),loopx,'X-AXIS')
         loopx = l7
      end if  

c                               increments at each level
      do j = 1, jlev
         jinc(j) = 2**(jlev-j)
      end do 

      kinc = jinc(1)
c                               reset jtest to make sure 
c                               the low level grid is uncompressed
      ktest = jtest
      jtest = 3

      call setvar (loopx,loopy)
c                               do all points on lowest level
      do i = 1, loopx, kinc
         do j = 1, loopy, kinc
            call setvr0 (i,j)
            call lpopt (i,j,jphct,istart,idead)
         end do
c                               progress info
         write (*,1030) float(i)/float(loopx)*1e2
      end do 
c                               restore jtest
      jtest = ktest
c                               get hot points
      ihot = 0 
      kinc2 = kinc/2
      kinc21 = kinc2 + 1

      do i = 1, loopx-1, kinc
         do j = 1, loopy-1, kinc
            call amihot (i,j,jhot,kinc)
            if (jhot.ne.0) then 
               ihot = ihot + 1
               hotij(ihot,1) = i
               hotij(ihot,2) = j 
c                               cell is heterogeneous
c                               fill in homogeneous diagonals
c                               and edges
               if (kinc.gt.1) call filler (i,j,kinc)
            else 
c                               cell is homogeneous
c                               fill in entire cell
               if (kinc.gt.1) call aminot (i,j,kinc,kinc2,kinc21)
            end if 
         end do 
      end do

      if (ihot.eq.0) goto 10 

      ktic = (loopx/kinc+1)*(loopy/kinc+1)

      write (*,1050) 
      htic = 0 
c                              now refine on all higher levels:
      do k = 2, jlev
c                              set new hot cell counter
         khot = 0 
         jtic = 0 
c                              now working on new level
         kinc = jinc(k)
         kinc2 = kinc/2
         kinc21 = kinc2 + 1
c
         write (*,1060) ihot,k
c                              compute assemblages at refinement
c                              points
         do h = 1, ihot
c                              cell corner
            iic = hotij(h,1) 
            jjc = hotij(h,2)
c                              first compute the central node of
c                              the hot cell
            icent = iic + kinc
            jcent = jjc + kinc
            if (igrd(icent,jcent).eq.0) then 
               call setvr0 (icent,jcent)
               call lpopt (icent,jcent,jphct,istart,idead)                       
               jtic = jtic + 1
               htic = htic + 1
            end if 
c                              now determine which of the diagonals
c                              has a change
            hhot = 0 

            do hh = 1, 4
            
               i = iic + iind(hh)*2*kinc
               j = jjc + jind(hh)*2*kinc
               lhot(hh) = 0 

               if (jgrd(i,j).ne.jgrd(icent,jcent)) then 
c                              cell is hot
                  khot = khot + 1
                  hhot = hhot + 1
                  kotij(khot,1) = iic + iind(hh)*kinc
                  kotij(khot,2) = jjc + jind(hh)*kinc
                  lhot(hh) = 1
c                              compute assemblages at new nodes
                  do kk = 1, 2
                     ii = iic + iiind(hh,kk)*kinc
                     jj = jjc + jjind(hh,kk)*kinc
                     if (igrd(ii,jj).eq.0) then 
                        call setvr0 (ii,jj)
                        call lpopt (ii,jj,jphct,istart,idead)
                        jtic = jtic + 1
                        htic = htic + 1
                     end if 
                  end do 
               end if 
            end do 
c                              if less than 3 hot sub-cells check
c                              edges
            if (hhot.lt.4.and.hhot.gt.1) then 

               do hh = 1, 4
c                              index the edge node
                  ii = iic + icind(hh)*kinc
                  jj = jjc + jcind(hh)*kinc
                  if (igrd(ii,jj).ne.0) then 
c                              could have a second hot cell, check
c                              both corners
                     icell = hh

                     do kk = 1, 2
                        ie = iic + ieind(hh+kk-1)*kinc
                        je = jjc + jeind(hh+kk-1)*kinc
                        icell = icell + kk - 1
                        if (icell.gt.4) icell = 1
                        
                        if (jgrd(ii,jj).ne.jgrd(ie,je).and.
     *                     lhot(icell).eq.0) then 
c                               new cell
                           khot = khot + 1
                           hhot = hhot + 1
                           lhot(icell) = 1
c                                cell index is 
                           ii = iic + iind(icell)*kinc
                           jj = jjc + jind(icell)*kinc
                           kotij(khot,1) = ii
                           kotij(khot,2) = jj
c                                compute assemblage at cell nodes
                           do ll = 1, 4
                              iil = ii + iind(ll)*kinc
                              jjl = jj + jind(ll)*kinc
                              if (igrd(iil,jjl).eq.0) then              
                                 call setvr0 (iil,jjl)
                                 call lpopt (iil,jjl,jphct,istart,idead)
                                 jtic = jtic + 1
                                 htic = htic + 1
                              end if 
                           end do  
                        end if
                     end do 
                  end if 
               end do 
            end if      

            do hh = 1, 4

               i = iic + iind(hh)*kinc
               j = jjc + jind(hh)*kinc
               if (i.lt.loopx.and.j.lt.loopy) then 
                  if (lhot(hh).eq.0) then 
c                                fill cold cells
                     call aminot1 (icent,jcent,i,j,kinc)
                  else 
c                                fill hot cells
                     call filler (i,j,kinc)
                  end if 
               end if 
            end do 
         
            
            if (htic.gt.500) then 
               write (*,1090) jtic
               htic = 0 
            end if 
         end do

         write (*,1070) k,jtic
         ktic = ktic + jtic
 
         write (*,1080) ktic,(loopx/kinc+1)*(loopy/kinc+1)

         if (khot.eq.0.or.k.eq.jlev) goto 10 
c                             now switch new and old hot list
         ihot = khot
         do i = 1, khot
            hotij(i,1) = kotij(i,1)
            hotij(i,2) = kotij(i,2)
         end do 

      end do 
c                             no more hot points or at highest level
10    if (io4.eq.0) then 

          write (n4,1000) loopx, loopy, jinc(1) 
c                                             output graphics data
         do i = 1, loopx

            if (i.ne.1.and.igrd(i,1).eq.0) igrd(i,1) = igrd(i-1,1)

            kst = 1

20          jst = kst
            if (i.ne.1.and.igrd(i,jst).eq.0) igrd(i,jst) = igrd(i-1,jst)
            kd = igrd(i,jst)
            ltic = -1

            do j = jst, loopy

               if (i.ne.1.and.igrd(i,j).eq.0) igrd(i,j) = igrd(i-1,j)

               if (igrd(i,j).eq.0.or.igrd(i,j).eq.kd) then
                  ltic = ltic + 1
                  if (j.eq.loopy) then 
                     write (n4,1000) ltic,kd
                     kst = 1
                  end if 
               else 
                  write (n4,1000) ltic,kd
                  kst = j
                  goto 20 
               end if 
            end do 
         end do         
      end if

1000  format (20(i6,1x))
1010  format (2(g14.6,1x))
1030  format (f5.1,'% done with low level grid.')
1040  format (/)
1050  format (/,'Beginning grid refinement stage.',/)
1060  format (i6,' grid cells to be refined at grid level ',i1)
1070  format (7x,'refinement at level ',i1,' involved ',i6,
     *        ' minimizations')
1080  format (i6,' minimizations required of the ',
     *        'theoretical limit of ',i6)
1090  format (7x,'...working (',i6,' minimizations done)')

99    stop

      end 

      subroutine aminot (i,j,kinc,kinc2,kinc21)
c--------------------------------------------------------------------
c fill in nodes of a homogeneous cell, called only for lowest level
c grid, aminot1 is called for higher level cells. kinc is always
c even and > 1. modfied to fill from each corner if kinc > 2 in case
c compression is off. JADC sept '04. 

c kinc   - the current grid increment
c kinc2  - kinc/2 (integer division)
c kinc21 - kinc2 + 1
c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer igrd, i, j, kinc, ii, jj, kinc2, kinc21

      common/ cst311/igrd(l7,l7)
c                                 the commented version fills
c                                 with just assemblage i,j this
c                                 would be fine if compression is
c                                 on, but is biased otherwise
c      do ii = i, i + kinc - 1
c         do jj = j, j + kinc - 1
c                                 the conditional prevents 
c                                 overwriting of identities if 
c                                 compression is off and searching
c                                 for true boundaries.
c            if (igrd(ii,jj).eq.0) igrd(ii,jj) = igrd(i,j)
c         end do
c      end do 
c                                 first fill from lower left (i,j)
      do ii = i, i + kinc2 
         do jj = j, j + kinc2
            if (igrd(ii,jj).eq.0) igrd(ii,jj) = igrd(i,j)
         end do
      end do
c                                 then from lower right (i+kinc,j)
      do ii = i + kinc21, i + kinc
         do jj = j, j + kinc2
            if (igrd(ii,jj).eq.0) igrd(ii,jj) = igrd(i+kinc,j)
         end do
      end do
c                                 then from upper left (i,j+kinc)
      do ii = i, i + kinc2
         do jj = j + kinc21, j + kinc
            if (igrd(ii,jj).eq.0) igrd(ii,jj) = igrd(i,j+kinc)
         end do
      end do
c                                 then from upper right (i+kinc,j+kinc)
      do ii = i + kinc21, i + kinc
         do jj = j + kinc2 + 1, j + kinc
            if (igrd(ii,jj).eq.0) igrd(ii,jj) = igrd(i+kinc,j+kinc)
         end do
      end do
     
      end  

      subroutine aminot1 (icent,jcent,i,j,kinc)
c--------------------------------------------------------------------
c fill in nodes of a homogeneous cell, called only for high level
c grids, aminot is called for lowest level cells. kinc is always
c even and > 1.  JADC sept '04. 

c kinc   - the current grid increment
c---------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      integer igrd, i, j, ii, jcent, icent, jj, kinc

      common/ cst311/igrd(l7,l7)

      do ii = i, i + kinc
         do jj = j, j + kinc
c                                 the conditional prevents 
c                                 overwriting of identities if 
c                                 compression is off and searching
c                                 for true boundaries.
            if (igrd(ii,jj).eq.0) igrd(ii,jj) = igrd(icent,jcent)
         end do
      end do 
     
      end  

      subroutine gall 
c-----------------------------------------------------------------------
c subroutine gall computes molar free energies of all phases.
c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      integer i,jterm,knsp,istct,ipoint,j,ifct,iff,isat,icp,isoct,jm,
     *        mstot,ideps,msite,lterm,ksp,inid,ifp,jend,k,ixp,jd,im2jm,
     *        hsub,id,ndep,jsub,ixpt,l,icomp,ifug,ifyn,
     *        isyn,icp1,io2,iphct,idss,ids,isct,imyn,ifproj,jsat,ksat,
     *        jfct,kfct,jmct,jprct,idfl,ikp,ivarrx,ivarip,isudo,ivar,knd

      double precision wgl,r,vlar,deph,dydy,y2pg,y,pa,p0a,dg1,
     *                 qmult,d0,dcoef,sdzdy,w,x,dvnu,
     *                 tr,pr,ps,gval,cp,uf,us,g,sxs,gzero,
     *                 tphi,dg,exces,thermo,p,t,xco2,u1,u2,
     *                  g0(k5),phi(m4),p0ord,alpha(m4)

      common/ cst1  /thermo(k4,k10),uf(2),us(h5)
     *      / cst2  /g(k1)/ cst6  /icomp,istct,iphct,icp
     *      / cst10 /iff(2),idss(h5),ifug,ifyn,isyn/ cst12 /cp(k5,k1)
     *      / cst40 /ids(h5,h6),isct(h5),icp1,isat,io2
     *      / cst304 /sxs(k13),exces(m3,k1),jend(h9,k12),ixp(k1),ifp(k1)
     *      / cst60 /ipoint,imyn/ cst79 /isoct/ cst113 /ifproj
     *      / cst307 /jsat,ksat,jfct,kfct,jmct,jprct/ cst208 /ifct,idfl
     *      / cst144 /knsp(l1,m4),mstot(l1),msite(l1),ksp(l1,m10),
     *                lterm(l1,m10,m11),hsub(l1,m10,m11,m0),
     *                jsub(l1,m1,m2),jterm(l1),ideps(l1,2),inid(l1,m4)
     *      / cst145 /qmult(l1,m10),
     *                d0(l1,m10,m11),dcoef(l1,m10,m11,m0),w(m1),x(m4),
     *                sdzdy(l1,m10,m11),wgl(l1,m1,m3),
     *                vlar(l1,m4,m3),dvnu(l1,m4),deph(l1),dydy(l1,m4)
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
     *      / cst5   /p,t,xco2,u1,u2,tr,pr,r,ps
     *      / cst147 /y2pg(l1,m4,m15),ndep(l1),im2jm(h9)
     *      / cst148 /p0a(m4),pa(m4),y(m4)
c-----------------------------------------------------------------------
c                                 compute the chemical potential
c                                 of the projected components.
      call uproj
c                                 first do the endmembers:
      do id = istct, ipoint

         call gcpd(id,gval)
c                                 this is a screw up solution
c                                 necessary cause uf(1) and uf(2)
c                                 are allocated independent of ifct!
         if (ifct.gt.0) then 
            do j = 1, 2
               if (iff(j).ne.0) gval = gval - cp(iff(j),id)*uf(j)
            end do 
         end if 

         do j = 1, isat
            gval = gval - us(j) * cp(icp+j,id)
         end do

         g(id) = gval 

      end do 
c                                 now do solutions:
      do i = 1, isoct
c                                 check if normal solution:
         if (ifp(id).eq.0) then 
c                                 it's normal margules or ideal:
            do j = 1, jend(i,2)
c                                 initialize with excess energy, dqf,
c                                 and configurational entropy terms
               call gexces (id,g(id))

               do k = 1, jend(i,1) 
                  if (jend(i,2+k).ne.0)  g(id) = g(id) 
     *                                 + g(jend(i,2+k)) * sxs(ixp(id)+k)
               end do 

               id = id + 1

            end do 

         else if (ifp(id).eq.1) then
c                                 it's a fluid compound, the way 
c                                 things are now it must have two
c                                 components.
c                                 first get the reference state 
c                                 free energy:
            
            do j = 1, jend(i,1)
               g0(j) = gzero(jend(i,2+j))
            end do 

            do j = 1, jend(i,2)

               call fexces (id,gval)

               g(id) = g0(1) * sxs(ixp(id)+1) + 
     *                 g0(2) * sxs(ixp(id)+2) + gval
               id = id + 1

            end do 

         else if (ifp(id).eq.3) then 
c                            it's van laar ala hp
c                            first compute "volumes"
            jm = im2jm(i)

            do k = 1, jend(i,1)
               alpha(knsp(jm,k)) = vlar(jm,k,1) + t * vlar(jm,k,2) 
     *                                          + p * vlar(jm,k,3)
            end do 
c                             because the hp van laar may have p-t
c                             dependent volumes, the full expression
c                             must be evaluated here:                   
            do j = 1, jend(i,2)

               tphi = 0d0
c                                 initialize with dqf,
c                                 and configurational entropy terms
               call gexces (id,g(id))

c                                 compute margules coefficients
               do k = 1, jterm(jm)
                  w(k) = wgl(jm,k,1) + t * wgl(jm,k,2) + p * wgl(jm,k,3)
               end do 

               do k = 1, jend(i,1) 
                  g(id) = g(id) + g(jend(i,2+k)) * sxs(ixp(id)+k)
c                             pointer to original indices
                  knd = knsp(jm,k)
c                             these phi's are the numerator of
c                             holland & powells "phi's"
                  phi(knd) = alpha(knd) * sxs(ixp(id)+k)
c                             tphi is the denominator of holland & powell's
c                             phi's
                  tphi = tphi + phi(knd)
               end do 
c                             rearrange tphi
               tphi = 2d0/tphi

               do k = 1, jterm(jm)
c                             assume holland powell form, all terms regular
                  g(id) = g(id) + w(k) 
     *                    * tphi * phi(jsub(jm,k,1)) * phi(jsub(jm,k,2))
     *                    / (alpha(jsub(jm,k,1)) + alpha(jsub(jm,k,2))) 

               end do  

               id = id + 1

            end do 

         else if (ifp(id).eq.6) then

            jm = im2jm(i)
c                                 solution with 1 order parameter 
c                                 first initialize x's
            jd = mstot(jm) + 1

            do j = 1, mstot(jm)
               x(j) = 0d0
            end do 
c                                 compute margules coefficients
            do j = 1, jterm(jm)
               w(j) = wgl(jm,j,1) + t * wgl(jm,j,2) + p * wgl(jm,j,3)
            end do 
c                                 now for each compound:
            do j = 1, jend(i,2)
c                                 here excess gets the endmember entropys
c                                 and dqf corrections
               call gexces (id,g(id))

               do k = 1, jend(i,1) 
                  x(knsp(jm,k)) = sxs(ixp(id)+k)
                  g(id) = g(id) + g(jend(i,2+k)) * sxs(ixp(id)+k)
               end do 
c                                 get the speciation energy effect
               call specis (dg,jm,jd)

               g(id) = g(id) + dg 

               id = id + 1

            end do 
         else if (ifp(id).eq.8) then

            jm = im2jm(i)
c                                 reciprocal solution with 1 order 
c                                 parameter first zero x's and p's
            jd = mstot(jm) + 1
c                                 zero all values cause we don't
c                                 know which are in use
            do j = 1, mstot(jm)
               x(j) = 0d0
               p0a(j) = 0d0
            end do
c                                 compute margules coefficients
            do j = 1, jterm(jm)
               w(j) = wgl(jm,j,1) + t * wgl(jm,j,2) + p * wgl(jm,j,3)
            end do 
c                                 now for each compound:
            do j = 1, jend(i,2)
c                                 assign x's
               do k = 1, jend(i,1) 
                  x(knsp(jm,k)) = sxs(ixp(id)+k)
               end do 
c                                 convert y's to p's, this loop 
c                                 could be eliminated by saving the
c                                 p's.
               ixpt = ixp(id) + jend(i,1) - ndep(jm)

               p0ord = 0d0

               do k = 1, jend(i,1) - ndep(jm)

                  p0a(knsp(jm,k)) =  sxs(ixp(id)+k)

                  do l = 1, ndep(jm)
                     p0a(knsp(jm,k)) =  p0a(knsp(jm,k))
     *                                + y2pg(jm,k,l) * sxs(ixpt+l)
                  end do 

                  p0ord = p0ord + p0a(knsp(jm,k))

               end do 
c                                 the initial fraction of the ordered species
               p0ord = 1d0 - p0ord
               p0a(knsp(jm,jend(i,1)+1)) = p0ord
c                                 get the speciation energy effect,
c                                 this includes the enthalpy necessary to form
c                                 the ordered species
               call specip (dg,jm,jd)
c                                 and endmember sconfg/dqf
               call gexces (id,dg1)

               g(id) = dg + dg1
c                                 add in g from real endmembers, this
c                                 must include the g for the disordered equivalent
c                                 of the ordered species
               do k = 1, jend(i,1) - ndep(jm)

                  g(id) = g(id) + g(jend(i,2+k)) * 
     *                           (p0a(knsp(jm,k)) + dvnu(jm,k)*p0ord)

               end do 

               id = id + 1

            end do 

         else if (ifp(id).eq.23) then
c                                 toop melt:
            do j = 1, jend(i,1)
               g0(j) = gzero(jend(i,2+j))
            end do 

            do j = 1, jend(i,2)
               call toop (id,g(id))
               do k = 1, jend(i,1) 
                  g(id) = g(id) + g(jend(i,2+k)) * sxs(ixp(id)+k)
               end do 
               id = id + 1
            end do 
         else if (ifp(id).eq.26) then
c                                 H2O-CO2-Salt:
            do j = 1, jend(i,2)
        
               call hcneos (g(id),sxs(ixp(id)+1),
     *                      sxs(ixp(id)+2),sxs(ixp(id)+3))

               do k = 1, jend(i,1) 
                  g(id) = g(id) + g(jend(i,2+k)) * sxs(ixp(id)+k)
               end do 

               id = id + 1

            end do
         end if 
      end do 

      end

      logical function solvus (id1,id2)
c-----------------------------------------------------------------------
c function to test if a solvus separates two pseudocompounds .
c-----------------------------------------------------------------------
      implicit none
 
      include 'perplex_parameters.h'

      double precision cp, ctot, dx, nopt

      integer icomp, iphct, icp, i, id1, id2, istct, iopt

      common/ cst6 /icomp,istct,iphct,icp/ cst12 /cp(k5,k1)
     *      / cst3 /ctot(k1)/ opts /iopt(10),nopt(10)
c-----------------------------------------------------------------------

      dx = 0d0

      do i = 1, icp
         dx = dx + ( cp(i,id1)/ctot(id1) 
     *                     - cp(i,id2)/ctot(id2))**2
      end do 

      if (dx.ne.0) dx = dsqrt(dx)

      if (dx.gt.nopt(8)) then
         solvus = .true.
      else
         solvus = .false.
      end if 

      end 

      subroutine srtblk (kdv,idsol,kdsol,ntot,ncpd,np)
c----------------------------------------------------------------------
c srtblk sorts the pseudocompounds of a pseudodivariant assemblage
c in kdv according to phase identities. the arrays b and kdv are 
c reordered accordingly, with 

c     np  - is the number of solutions, 
c     ncpd - is the number of true compounds
c     idsol(np) - is the number of pseudocompounds in phase np
c     kdsol(np) - is the identity flag of solution np
c     ntot - the total number of pseudocompounds for sorting, 
c            icp in newass and chkass, jbulk in bulkun

c idsol is not properly initialized for the case ncpd = jbulk.
c----------------------------------------------------------------------
      implicit none

      include 'perplex_parameters.h'

      logical solvus

      integer idsol(k5),kdv(k5),jdsol(k5,k5),kdsol(k5),ntot,np,ncpd,i,
     *        ikp,ivarrx,ivarip,isudo,ivar,ipvt,j,itic

      double precision bsol(k5,k5),a,b
 
      common/ cst301 /a(k5,k5),b(k5),ipvt(k5)
     *      / cst61  /ikp(k1),ivarrx(k2),ivarip(k2),isudo,ivar
c-----------------------------------------------------------------------
c                                first check if any solutions
      do i = 1, ntot
         if (ikp(kdv(i)).ne.0) goto 10
      end do 

      np = 0
      ncpd = ntot
c                                the assemblage consists entirely 
c                                of true compounds, nothing more to do.
      goto 99
c                                figure out how many solutions
c                                are present:
10    np = 0
      ncpd = 0

      do 30 i = 1, ntot
         if (ikp(kdv(i)).eq.0) then
c                                 the pseudocompound is a true compound
            ncpd = ncpd + 1 
            idsol(ntot) = ncpd
            jdsol(ntot,ncpd) = kdv(i)
            bsol(ntot,ncpd) = b(i)

         else 
            do j = 1, np
               if (kdsol(j).eq.ikp(kdv(i))) then 
                  if (.not.(solvus(kdv(i),jdsol(j,1)))) then
c                                 the pseudocompound matches a solution
c                                 found earlier.
                     idsol(j) = idsol(j) + 1
                     jdsol(j,idsol(j)) = kdv(i)
                     bsol(j,idsol(j)) = b(i)
                     goto 30 
                  end if 
               end if 
            end do
c                                 the pseudocompound is a new solution 
c                                 phase.
            np = np + 1
            idsol(np) = 1
            jdsol(np,1) = kdv(i)
            kdsol(np) = ikp(kdv(i))
            bsol(np,1) = b(i)

         end if    
30    continue  
c                                now reform the arrays kdv and b
      itic = 0

      do i = 1, np
         do j = 1, idsol(i)
            itic = itic + 1
            b(itic) = bsol(i,j)
            kdv(itic) = jdsol(i,j)
         end do
      end do

      do j = 1, ncpd
         itic = itic + 1
         b(itic) = bsol(ntot,j)
         kdv(itic) = jdsol(ntot,j)
      end do 

99    end 
c----------------------------------------------------------------------
c verbatim from plib.f with name change, typing for "nopt", and
c name change for readcd to redcd1

      subroutine redop1 
c----------------------------------------------------------------------
c redop1 - redop1 looks for the perplex_option.dat file, if it finds
c the option file it scans the file for keywords and sets options
c accordingly.
c----------------------------------------------------------------------
      implicit none

      integer lun, ier, iopt, i

      double precision nopt

      character*3 key, val, value(10), nval*12, nval1*12

      save lun

      data lun/13/

      common / opts /iopt(10),nopt(10)

c                                   default option values:
      iopt(1) = 1
      value(1) = 'on '
      iopt(2) = 0 
      value(2) = 'mol'
      iopt(3) = 0 
      value(3) = 'vol'
c                                   interpolation keyword
      iopt(4) = 0
      value(4) = 'on '
      nopt(4) = 2d0
c                                   extrapolation keyword
      iopt(5) = 0
      value(5) = 'on'
c                                   vrh_weighting keyword
      nopt(6) = 0.5d0
c                                   bad_number keyword
      nopt(7) = 0d0
c                                   solvus_tolerance keyword
      nopt(8) = 0.10
c                                   zero_mode (<0 off)
      nopt(9) = 1d-6
c                                   look for file
      open (lun, file = 'perplex_option.dat', iostat = ier, 
     *      status = 'old')
c                                   no option file, set defaults
      if (ier.ne.0) goto 99
c                                   read cards to end of 
c                                   option file
      do while (ier.eq.0) 

         call redcd1 (lun,ier,key,val,nval,nval1)
c                                   ier ne 0 bad record or eof
         if (ier.ne.0) exit 
c                                   if here we should have a keyword and
c                                   value
         if (key.eq.'ave') then
c                                   average pseudocompounds key
            if (val.eq.'on') then
               iopt(1) = 1
            else if (val.eq.'off') then 
               iopt(1) = 0 
            else 
               iopt(1) = 2
            end if 
            value(1) = val
         else if (key.eq.'com') then 
c                                   phase composition key
            if (val.eq.'wt') then
               iopt(2) = 1
            else 
               iopt(2) = 0
            end if 
            value(2) = val
         else if (key.eq.'pro') then 
c                                   phase proportion key
            if (val.eq.'wt') then
               iopt(3) = 1
            else if (val.eq.'mol') then 
               iopt(3) = 2 
            else 
c                                   volume is default
               iopt(3) = 0
            end if 
            value(3) = val
         else if (key.eq.'int') then 
c                                   interpolation key
            if (val.eq.'on ') then
               iopt(4) = 0
            else 
               iopt(4) = 1
            end if 
            value(4) = val
            read (nval1,*) nopt(4)
         else if (key.eq.'ext') then 
c                                   extrapolation key
            if (val.eq.'on ') then
               iopt(5) = 0
            else 
               iopt(5) = 1
            end if 
            value(5) = val
         else if (key.eq.'vrh') then 
c                                   vrh weighting key
            read (nval,*) nopt(6)
         else if (key.eq.'bad') then 
c                                   bad number key
            read (nval,*) nopt(7)
         else if (key.eq.'sol') then 

            read (nval,*) nopt(8)

         else if (key.eq.'zer') then
c                                   zero_mode key
            read (nval,*) nopt(9)
         end if 
      end do 
                
99    close (lun)

      
      write (*,1000) 
      write (*,1010) (value(i),i=1,5),nopt(6),nopt(7),nopt(8),nopt(9)
      write (*,1020)

1000  format (/,'Output options are currently set as:',//,
     *          '    Keyword:         Value:     Permitted values ',
     *          '[default]:')
1010  format (4x,'average          ',a3,8x,'off [on ] both',/,
     *        4x,'compositions     ',a3,8x,'wt  [mol]',/,
     *        4x,'proportions      ',a3,8x,'wt  [vol]',/,
     *        4x,'interpolation    ',a3,8x,'off [on ]',/,
     *        4x,'extrapolation    ',a3,8x,'off [on ]',/,
     *        4x,'vrh_weighting    ',f3.1,8x,'0->1 [0.5]',/,
     *        4x,'bad_number       ',f7.0,4x,'[0.0]',/,
     *        4x,'solvus_tolerance ',f5.3,6x,'0->1 [0.10], = 1 => off',/
     *        4x,'zero_mode        ',e7.1,4x,'0->1 [1e-6], < 0 => off',/
     *)

1020  format ('To change these options edit or create',
     *        ' the file perplex_option.dat',/,'See: ',
     *        'www.perplex.ethz.ch/perplex_options.html',/)

      end 

      subroutine redcd1 (lun,ier,key,val,nval,nval1)
c----------------------------------------------------------------------
c readcd - read 240 column card image from unit lun, strip out unwanted
c characters. ier = 1 -> eof.
c this routine seeks a card containg a keyword and as many as 
c two values (char variables nval, nval1), the
c first 3 letters of each is returned in char(1:6)
c----------------------------------------------------------------------    
      implicit none

      integer len, ier, iscan, i, iscnlt, ibeg, lun, iend, ist

      character chars(240)*1, card*240, key*3, val*3, nval*12,
     *          nval1*12
 
      ier = 0 

      ibeg = 0
  
      len = 0 

      card = ''

      do while (ibeg.ge.len) 

         read (lun,'(a)',end=90) card

         if (card.ne.'') then 
            read (card,'(240a)') chars
c                                    find end of data marker '|'
            len = iscan (1,240,chars,'|') - 1
c                                    find a non blank character
            ibeg = iscnlt (1,len,chars,' ')
         end if 

      end do 

      iend = ibeg + 2
c                                    ok, we've got the keyword
c                                    load 3 chars of keyword into
      write (key,'(3a1)') (chars(i), i = ibeg, iend)
c                                    find next blank 
      iend = iscan (iend,240,chars,' ') 
c                                    now locate the value:
      ibeg = iscnlt (iend,len,chars,' ')
c                                    now find trailing blank
      iend = iscan (ibeg,240,chars,' ') 
c                                    now save a longer version in case
c                                    the value is numeric
      if (iend-ibeg.gt.11) iend = ibeg + 11
c                                    should check for error
      write (val,'(3a1)') (chars(i), i = ibeg, ibeg + 2)
c                                    this is done in case the first
c                                    value is numeric
      write (nval,'(12a1)') (chars(i), i = ibeg, ibeg+11)
c                                    now look for a second value
      ist = ibeg + 3
c                                    find a non blank character
      ibeg = iscnlt (ist,len,chars,' ')
c                                    if no blank exit
      if (ibeg.gt.len) then 
         nval1 = '0'
      else 
         iend = iscan (ibeg,len,chars,' ')
         if (iend-ibeg.gt.11) iend = ibeg + 11 
         write (nval1,'(12a1)') (chars(i), i = ibeg, ibeg+11)
      end if 

      goto 99

90    ier = 1

99    end
